

#ifndef _GenMat_Array3D
#define _GenMat_Array3D

/*
Template Numerical Toolkit (TNT): Three-dimensional numerical array

Mathematical and Computational Sciences Division
National Institute of Technology,
Gaithersburg, MD USA

This software was developed at the National Institute of Standards and
Technology (NIST) by employees of the Federal Government in the course
of their official duties. Pursuant to title 17 Section 105 of the
United States Code, this software is not subject to copyright protection
and is in the public domain. NIST assumes no responsibility whatsoever for
its use by other parties, and makes no guarantees, expressed or implied,
about its quality, reliability, or any other characteristic.
*/

/*
Tempplated three-dimensional, numerical array which
looks like a conventional C multiarray. 
Storage corresponds to conventional C ordering.
That is the right-most dimension has contiguous
elements.  Indexing is via the A[i][j][k] notation. 
	
<p>
Array assignment is by reference (i.e. shallow assignment).
That is, B=A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.

<p>
The indexing and layout of this array object makes
it compatible with C and C++ algorithms that utilize
the familiar C[i][j][k] notation.  This includes numerous
textbooks, such as Numercial Recipes, and various
public domain codes.

<p>
This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.
*/

template <class T>
class Array3D 
{
	private:
	
	T* p;				// Pointer to underlying array
    int d1;				// Size of first dimension
    int d2;				// Size of second dimension
	int d3;				// Size of third dimension
	int f;				// ...
	int len;			// Size of underlying array
    int *RefCount;		// Count references

	void Initialize();
	void Destroy();
	
	//void Copy_(T* p, const T*  q, int len) const;
    //void Set_(const T& val);
    //inline const T* Begin_() const;
	//inline T* Begin_();

	public:

	string Name;		// Name of the Array3D

	       Array3D();
	       Array3D(int d1, int d2, int d3);
	       Array3D(int d1, int d2, int d3, T *a);
	       Array3D(int d1, int d2, int d3, const T &a);
    inline Array3D(const Array3D &A);
	inline Array3D & operator=(const T &a);
	inline Array3D & operator=(const Array3D &A);
	inline Array3D & Ref(const Array3D &A);
	       Array3D Copy() const;
		   Array3D & Inject(const Array3D &A);
	inline T& operator[](int i);
	inline const T& operator[](int i) const;
	inline T& operator()(int i1, int i2, int i3);
	inline const T& operator()(int i1, int i2, int i3) const;
	inline int Dim1() const;
	inline int Dim2() const;
	inline int Dim3() const;
	inline int Length() const;
	inline Vector<int> Size() const;
	inline int GetRefCount() const;
     	   T* Begin() const;
		   T* End()   const;
		   ~Array3D();
	
	inline void operator += (const T &c);
    inline void operator -= (const T &c);
    inline void operator *= (const T &c);
    inline void operator /= (const T &c);		   
};


/*

Copy constructor. Array data is NOT copied, but shared.
Thus, in Array3D B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
Array3D B(A.Copy()), or B = A.Copy(), instead.

*/
template <class T>
Array3D<T>::Array3D(const Array3D<T> &A)
	: p(A.p), d1(A.d1), d2(A.d2), d3(A.d3), f(A.f), len(A.len), Name(A.Name), RefCount(A.RefCount)
{
	(*RefCount)++;
}



/*

Create a new (m x n x k) Array3D, WITHOUT initializing array elements.
This version avoids the O(m*n*k) initialization overhead and
is used just before manual assignment.

@param m	-	The first dimension of the new Array3D.
@param n	-	The second dimension of the new Array3D.
@param k	-	The third dimension of the new Array3D.

*/

template <class T>
Array3D<T>::Array3D(int d1, int d2, int d3)
	: p(0), d1(d1), d2(d2), d3(d3)
{
	f = d2 * d3;
	len = d1 * d2 * d3;
	Name = "NoName";
	Initialize();
	RefCount = new int;
	*RefCount = 1;
}

/*

Create a new (m x n x k) Array3D, initializing elements to
constant specified by argument.

@param m	-	The first dimension of the new Array3D.
@param n	-	The second dimension of the new Array3D.
@param k	-	The third dimension of the new Array3D.
@param val	-	The constant value to set all elements of the new Array3D to.

*/

template <class T>
Array3D<T>::Array3D(int d1, int d2, int d3, const T &Val)
	: p(0), d1(d1), d2(d2), d3(d3)
{
	f = d2 * d3;
	len = d1 * d2 * d3;
	Name = "NoName";
	Initialize();
	BlasSet(len,Val,p,1);
	RefCount = new int;
	*RefCount = 1;
}

/*

Create a new (m x n x k) array,  as a view of an existing one-dimensional
array stored in <b>C order</b>, i.e. right-most dimension varying fastest.  
(Often referred to as "row-major" ordering.)
Note that the storage for this pre-existing array will
never be garbage collected by the Array3D class.

@param m the first dimension of the new matrix.
@param n the second dimension of the new matrix.
@param k the third dimension of the new matrix.
@param a the one dimensional C array to use as data storage for
the array. 

*/
template <class T>
Array3D<T>::Array3D(int d1, int d2, int d3, T *a)
	: p(a), d1(d1), d2(d2), d3(d3)
{
	f = d2 * d3;
	len = d1 * d2 * d3;
	//p = a;
	Name = "NoName";
	RefCount = new int;
	*RefCount = 2;	// This avoids destorying original data.
}

template <class T>
inline T& Array3D<T>::operator[](int i)
{
	return p[i];
}

template <class T>
inline const T& Array3D<T>::operator[](int i) const
{
	return p[i];
}



/**
	Assign all elemnts of A to a constant scalar.
*/
/*
template <class T>
Array3D<T> & Array3D<T>::operator=(const T &a)
{
	set_(a);
	return *this;
}
*/

// Operator () allows for bounds checking on all indices of 3D Arrays
template <class T>
inline T& Array3D<T>::operator()(int i1, int i2, int i3)
{
	#ifdef _CheckBounds
		if(i1 < 0 || i1 >= d1 || i2 < 0 || i2 >= d2 || i3 < 0 || i3 >= d3)
		{
			Print("Error in Array3D: Index Out of Bounds");
			Print("Name",Name);
			Print("i1",i1);
			Print("i2",i2);
			Print("i3",i3);
			Print("d1",d1);
			Print("d2",d2);
			Print("d3",d3);
			HaltExecution();
		}
	#endif
	
	return p[f * i1 + d3 * i2 + i3];
}

// Operator () allows for bounds checking on all indices of 3D Arrays
template <class T>
inline const T& Array3D<T>::operator()(int i1, int i2, int i3) const
{
	#ifdef _CheckBounds
		if(i1 < 0 || i1 >= d1 || i2 < 0 || i2 >= d2 || i3 < 0 || i3 >= d3)
		{
			Print("Error in Array3D: Index Out of Bounds");
			Print("i1",i1);
			Print("i2",i2);
			Print("i3",i3);
			Print("d1",d1);
			Print("d2",d2);
			Print("d3",d3);
			HaltExecution();
		}
	#endif
	
	return p[f * i1 + d3 * i2 + i3];
}



/*

Create a new of existing matrix.  Used in B = A.copy()
or in the construction of B, e.g. Array3D B(A.copy()), 
to create a new array that does not share data.

*/

template <class T>
Array3D<T> Array3D<T>::Copy() const
{
	Array3D A(d1,d2,d3);
	BlasCopy(len,p,1,A.p,1);
	return A;
}


/**
	Copy the elements to from one array to another, in place.
	That is B.inject(A), both A and B must conform (i.e. have
	identical dimensions).

	This differs from B = A.copy() in that references to B
	before this assignment are also affected.  That is, if
	we have 
	<pre>
	Array3D A(m,n,k);
	Array3D C(m,n,k);
	Array3D B(C);        // elements of B and C are shared. 

</pre>
	then B.inject(A) affects both and C, while B=A.copy() creates
	a new array B which shares no data with C or A.

	@param A the array from elements will be copied
	@return an instance of the modifed array. That is, in B.inject(A),
	it returns B.  If A and B are not conformat, no modifications to 
	B are made.

*/

/*
template <class T>
Array3D<T> & Array3D<T>::Inject(const Array3D &A)
{
	if (A.m_ == m_ &&  A.n_ == n_)
		copy_(begin_(), A.begin_(), m_*n_);
	return *this;
}
*/


/*

Create a reference (shallow assignment) to another existing array.
In B.ref(A), B and A shared the same data and subsequent changes
to the array elements of one will be reflected in the other.
<p>
This is what operator= calls, and B=A and B.ref(A) are equivalent
operations.

@return The new referenced array: in B.ref(A), it returns B.

*/

template <class T>
Array3D<T> & Array3D<T>::Ref(const Array3D<T> &A)
{
	if(this != &A)
	{
		(*RefCount)--;
		if(*RefCount < 1) Destroy();
		d1   = A.d1;
		d2   = A.d2;
		d3   = A.d3;
		f    = A.f;
		len  = A.len;
		p    = A.p;
		Name = A.Name;
		RefCount = A.RefCount;
		(*RefCount)++ ;
		
	}
	return *this;
}

/* B = A is shorthand notation for B.Ref(A). */
template <class T>
Array3D<T> & Array3D<T>::operator=(const Array3D<T> &A)
{
	return Ref(A);
}

/* Return the size of the first dimension of the array. */
template <class T>
inline int Array3D<T>::Dim1() const
{
	return d1;
}

/* Return the size of the second dimension of the array. */
template <class T>
inline int Array3D<T>::Dim2() const
{
	return d2;
}

/* Return the size of the third (right-most) dimension of the array. */
template <class T>
inline int Array3D<T>::Dim3() const
{
	return d3;
}

/* Return the size of the underlying array */
template <class T>
inline int Array3D<T>::Length() const
{
	return len;
}


/* Return the dimensions of the array in a vector */
template <class T>
inline Vector<int> Array3D<T>::Size() const
{
	Vector<int> Ret(3);
	Ret[0] = d1;
	Ret[1] = d2;
	Ret[2] = d3;
	return Ret;
}


/*

@return the number of arrays that share the same storage area
as this one.  (Must be at least one.)

*/

template <class T>
inline int Array3D<T>::GetRefCount() const
{
	return *RefCount;
}

template <class T>
Array3D<T>::~Array3D()
{
	(*RefCount)--;
	if(*RefCount < 1) Destroy();
}

/* private internal functions */

template <class T>
void Array3D<T>::Initialize()
{
	try
	{
		if(len > 0) p = new T[len];
	}
	catch(bad_alloc ba)
	{
		Print("Error in Array3D: Error Allocating Memory");
		Print("Size",len);
		Print("Memory per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",len * sizeof(T) / 1e6);
		HaltExecution();
	}
}

template <class T>
void Array3D<T>::Destroy()
{
	if(len > 0) if(p != 0) delete[](p);
	if(RefCount != 0) delete RefCount;
}

/* Create a null (0 x 0 x 0) Array3D obeject. */  
template <class T>
Array3D<T>::Array3D() : p(0), d1(0), d2(0), d3(0), f(0), len(0), Name("NoName")
{
	RefCount = new int;
	*RefCount = 1;
}

/* Begin */
template <class T>
T* Array3D<T>::Begin() const
{
	return &p[0];
}

/* End */
template <class T>
T* Array3D<T>::End() const
{
	return &p[0] + len;
}

template <class T>
inline void Array3D<T>::operator += (const T &c)
{
	for(int i=0; i<len; i++) p[i] += c;
}

template <class T>
inline void Array3D<T>::operator -= (const T &c)
{
	for(int i=0; i<len; i++) p[i] -= c;
}

template <class T>
inline void Array3D<T>::operator *= (const T &c)
{
	for(int i=0; i<len; i++) p[i] *= c;
}

template <class T>
inline void Array3D<T>::operator /= (const T &c)
{
	for(int i=0; i<len; i++) p[i] /= c;
}

#endif
