
#ifndef _BlasGemm
#define _BlasGemm

	/* Perform one of the matrix-matrix operations, C := Alpha * op(A) * op(B) + Beta * C */
	template <class T>
	void BlasGemm(bool TransA, bool TransB, int m, int n, int r,const T &Alpha, const T *A, int lda,
					const T *B, int ldb, const T &Beta, T *C, int ldc);

/*

Perform one of the matrix-matrix operations

     C := Alpha * A   * B   + Beta * C
     C := Alpha * A   * B^T + Beta * C
     C := Alpha * A^T * B   + Beta * C
     C := Alpha * A^T * B^T + Beta * C
     
where Alpha and Beta are scalars, and A, B and C are matrices, with op( A )
an m x k matrix, op( B ) a  k x n matrix and  C an m x n matrix.

This routine is loosely based on the Level-3 Blas routine DGEMM,
although the loops have been reordered in all cases. In addition,
we link to the Level-1 Blas routines BlasSet, BlasScal, BlasDot,
and BlasAxpy, in order to implement unrolling of loops.

Parameters
----------

TransA (boolean)	-	Specifies the form of op( A ) to be used in
	           			the matrix multiplication as follows:
TransB (boolean)	-	Specifies the form of op( B ) to be used in
						the matrix multiplication as follows:

m (integer)			-	Specifies  the number  of rows  of the matrix
						op( A ) and of the  matrix C. m must  be at least
						zero.

n (integer)			-	Specifies the number  of columns of the matrix
						op( B ) and the number of columns of the matrix C.
						n must be at least zero.
						
k (integer)			-	Specifies  the number of columns of the matrix
						op( A ) and the number of rows of the matrix op( B ). K must
						be at least  zero.
						
Alpha (Real)		-	Specifies the scalar alpha.

A (array)			-	Array of DIMENSION ( LDA, ka ), where ka is
						k  when  TRANSA = 'N' or 'n',  and is  m  otherwise.
						Before entry with  TRANSA = 'N' or 'n',  the leading  m by k
						part of the array  A  must contain the matrix  A,  otherwise
						the leading  k by m  part of the array  A  must contain  the
						matrix A.

lda (integer)		-	Specifies the first dimension of A as declared
						in the calling (sub) program. When  TRANSA = 'N' or 'n' then
						LDA must be at least  max( 1, m ), otherwise  LDA must be at
						least  max( 1, k ).
						
						
B (array)			-	Array of DIMENSION ( LDB, kb ), where kb is
						n  when  TRANSB = 'N' or 'n',  and is  k  otherwise.
						Before entry with  TRANSB = 'N' or 'n',  the leading  k by n
						part of the array  B  must contain the matrix  B,  otherwise
						the leading  n by k  part of the array  B  must contain  the
						matrix B.

ldb (integer)		-	Specifies the first dimension of B as declared
						in the calling (sub) program. When  TRANSB = 'N' or 'n' then
						LDB must be at least  max( 1, k ), otherwise  LDB must be at
						least  max( 1, n ).

Beta (Real)			-	Specifies the scalar  beta.  When  BETA  is
						supplied as zero then C need not be set on input.
						Unchanged on exit.

C (array)			-	Array of DIMENSION ( LDC, n ).
						Before entry, the leading  m by n  part of the array  C must
						contain the matrix  C,  except when  beta  is zero, in which
						case C need not be set on entry.
						On exit, the array  C  is overwritten by the  m by n  matrix
						( alpha*op( A )*op( B ) + beta*C ).
						
ldc (integer)		-	Specifies the first dimension of C as declared
						in  the  calling  (sub)  program.   ldc  must  be  at  least
						Max(1,m).


Adapted from the Level 3 Blas subroutine,

	SUBROUTINE DGEMM ( TRANSA, TRANSB, M, N, K, ALPHA, A, LDA, B, LDB, BETA, C, LDC )

	Written on 8-February-1989.
	Jack Dongarra, Argonne National Laboratory.
	Iain Duff, AERE Harwell.
	Jeremy Du Croz, Numerical Algorithms Group Ltd.
	Sven Hammarling, Numerical Algorithms Group Ltd.
	
*/

template <class T>
void BlasGemm(bool TransA, bool TransB, int m, int n, int r, const T &Alpha, const T *A, int lda,
				const T *B, int ldb, const T &Beta, T *C, int ldc)
{
	int nRowA,nColA,nRowB,nColB;
	
	if(!TransA)
	{
		nRowA = m;
		nColA = r;
	}
	else
	{
		nRowA = r;
		nColA = m;
	}
	
	if(!TransB)
	{
		nRowB = r;
		nColB = n;
	}
	else
	{
		nRowB = n;
		nColB = r;
	}
	
	/* Test the input parameters. */
	if(m < 0)
	{
		Print("Error in BlasGemm");
		Print("m < 0");
		Print("m",m);
		HaltExecution();
	}
	else if(n < 0)
	{
		Print("Error in BlasGemm");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(r < 0)
	{
		Print("Error in BlasGemm");
		Print("k < 0");
		Print("r",r);
		HaltExecution();
	}
	else if(lda < nColA)
	{
		Print("Error in BlasGemm");
		Print("lda < Max(1,nColA)");
		Print("lda",lda);
		Print("nColA",nColA);
		HaltExecution();
	}
	else if(ldb < nColB)
	{
		Print("Error in BlasGemm");
		Print("m",m);
		Print("n",n);
		Print("r",r);
		Print("ldb < Max(1,nRowB)");
		Print("ldb",ldb);
		Print("nRowB",nRowB);
		HaltExecution();
	}
	else if(ldc < n)
	{
		Print("Error in BlasGemm");
		Print("ldc < Max(1,n)");
		Print("ldc",ldc);
		Print("n",n);
		HaltExecution();
	}

	/* Quick return if possible. */
	if((m == 0) || (n == 0) || (((Alpha == Zero) || (r == 0)) && (Beta == One))) return;

	if(Beta == Zero) for(int i=0; i<m; i++) BlasSet (n,GetZero<T>(),&C[ldc*i],1);
	else		     for(int i=0; i<m; i++) BlasScal(n,Beta,&C[ldc*i],1);
	
	/* And if Alpha is zero. */
	if(Alpha == Zero) return;
    
	/* Start the operations. */
	if(!TransB)
	{
		if(!TransA)
		{
			/* Form  C := Alpha * A * B + Beta * C. */
			for(int i=0; i<m; i++) for(int k=0; k<r; k++)
			{
				if(A[lda*i+k] != Zero)
				{
					T Temp = Alpha * A[lda*i+k];
					BlasAxpy(n,Temp,B+ldb*k,1,C+ldc*i,1);
				}
			}
		}
		else
		{
			/* Form  C := Alpha * A^T * B + Beta * C */
			for(int i=0; i<m; i++) for(int k=0; k<r; k++)
			{
				if(A[lda*k+i] != Zero)
				{
					T Temp = Alpha * A[lda*k+i];
					BlasAxpy(n,Temp,B+ldb*k,1,C+ldc*i,1);
				}
			}
		}
	}
	else
	{
     	if(!TransA)
     	{
     		/* Form  C := Alpha * A * B^T + Beta * C */
			for(int i=0; i<m; i++) for(int j=0; j<n; j++)
			{
				T Temp = BlasDot(r,A+lda*i,1,B+ldb*j,1);
				C[ldc*i+j] += Alpha * Temp;
			}
		}
		else
		{
			/*
			
			Form  C := Alpha * A^T * B^T + Beta * C
			
			Note that this is the only case where increment
			of 1 are not used in the innermost loop. I think
			that doing so is not possible. In order to use
			loops with icrements of 1, we need that either j
			is the last index on A or B, or that l is the last
			index in both A and B
			
			 */
	
			for(int i=0; i<m; i++) for(int j=0; j<n; j++)
			{
				T Temp = BlasDot(r,A+i,lda,B+ldb*j,1);
				C[ldc*i+j] += Alpha * Temp;
			}		
		}
	}
}

#endif
