
#ifndef _BlasGemv
#define _BlasGemv


	/* BlasGemv performs one of the matrix-vector operations

     y := Alpha * A * x + Beta * y,   or   y := Alpha * A^T * x + Beta * y */

	template <class T>
	void BlasGemv(bool Trans, int m, int n, const T &Alpha, const T *A, int lda, const T *x, int incx,
		const T &Beta, T *y, int incy);

/*

BlasGemv performs one of the matrix-vector operations

     y := Alpha * A * x + Beta * y,   or   y := Alpha * A^T * x + Beta * y,

where Alpha and Beta are scalars, x and y are vectors and A is an
m x n matrix.

Parameters
----------

Trans (boolean)	-	Specifies where to solve the normal or transposed
					equation.

m (integer)		-	Specifies the the number of rows of the matrix A
					(must be at least zero).

n (integer)		-	Specifies the the number of columns of the matrix A
					(must be at least zero).

Alpha (real)	-	Specifies the scalar Alpha.

A (array) 	    -	An array of dimension lda x n.

lda (integer)	-	Specifies the first dimension of A as declared
					in the calling (sub) program (must be at least
					Max(1,m).

x (array)		-	An incremented array of dimension n or m which contains
					the vector x.

incx (integer)	-	Specifies the increment for the elements of x (must not
					be zero).

Beta (real)		-	Specifies the scalar Beta. When Beta is supplied as zero
					then Y need not be set on input.

y (array)		-	An incremented array of dimension m or n which contains
					the vector y.

incy (integer)	-	Specifies the increment for the elements of y (must not
					be zero).

This routine is adapted from the Level 2 Fortran Blas subroutine,

	SUBROUTINE DGEMV( TRANS, M, N, ALPHA, A, LDA, X, INCX, BETA, Y, INCY )

	Written on 22-October-1986.
    Jack Dongarra, Argonne National Lab.
    Jeremy Du Croz, Nag Central Office.
    Sven Hammarling, Nag Central Office.
    Richard Hanson, Sandia National Labs.

Adapted by MP.

*/

template <class T>
void BlasGemv(bool Trans, int m, int n, const T &Alpha, const T *A, int lda, const T *x, int incx,
	const T &Beta, T *y, int incy)
{

	int kx,ky,lenx,leny;

	/* Test the input parameters. */

	if(m < 0 || n < 0)
	{
		Print("Error in BlasGemv");
		Print("m and n must be positive");
		Print("m",m);
		Print("n",n);
		HaltExecution();
	}
	//else if(lda < Max(1,n))
	//{
	//	Print("Error in BlasGemv");
	//	Print("lda < Max(1,n)");
	//	Print("lda",lda);
	//	Print("n",n);
	//	HaltExecution();
	//}
	else if(incx == 0 || incy == 0)
	{
		Print("Error in BlasGemv");
		Print("Increments must not be zero");
		Print("incx",incx);
		Print("incy",incy);
		HaltExecution();
	}

	/* Quick return if possible. */
	if((m == 0) || (n == 0) || ((Alpha == Zero) && (Beta == One))) return;

    /*

    Set  lenx  and  leny, the lengths of the vectors x and y, and set
	up the start points in  x  and  y.

	*/

	if(!Trans)
	{
		lenx = n;
        leny = m;
    }
    else
    {
    	lenx = m;
        leny = n;
    }

    if(incx > 0) kx = 0;
    else         kx = -(lenx - 1) * incx;

    if(incy > 0) ky = 0;
    else         ky = -(leny - 1) * incy;

	/*

	Start the operations. In this version the elements of A are
	accessed sequentially with one pass through A.
	
	*/
	
	/* First form  y := Beta * y. */

	if(Beta != One)
	{
		if(Beta == Zero) BlasSet(leny,GetZero<T>(),y,incy);
		else             BlasScal(leny,Beta,y,incy);
    }

    if(Alpha == Zero) return;

	if(!Trans)
	{
		/* Form, y = Alpha * A * x + y */
	
		int jy = ky;
 		for(int j=0; j<leny; j++)
		{
			T Temp = BlasDot(n,&A[lda*j],1,&x[0],incx);
			y[jy] += Alpha * Temp;
			jy += incy;
		}
	}
	else
	{
		/* Form, y = Alpha * A^T * x + y */
		
		int jx = kx;
		for(int j=0; j<m; j++)
		{
			T Temp = Alpha * x[jx];
			BlasAxpy(n,Temp,&A[lda*j],1,&y[0],incy);
			jx += incx;
		}
	}
}

#endif
