
#ifndef _BlasSpr
#define _BlasSpr


	/* Preform the symmetric rank 1 operation, A := Alpha * x * x^T + A */
	template <class Real>
	void BlasSpr(bool Upper, int n, const Real &Alpha, Real *x, int incx, Real *A);

/*

Preform the symmetric rank 1 operation,

     A := Alpha * x * x^T + A,

where Alpha is a real scalar, x is an n element vector
and A is an n by n symmetric matrix, supplied in packed
form.

Parameters
----------

Upper (boolean)	-	Specifies where the matrix A is upper or lower
					triangular.
					
n (interger)	-	Specifies the dimension of the matrix A. Must
					be at least zero.
					
Alpha (real)	-	Specifies the scalar Alpha.

x (array)		-	Array of dimension at least (1 + (n - 1) * abs(incx)).

incx (integer)	-	Specifies the increment for the elements of x. Must not
					be zero.
					
A (array) 	    -	An array of dimension n * (n + 1) / 2, which
					represents a lower or upper-triangular matrix
					in packed form.

This routine is adapted from the Level 2 Fortran Blas subroutine,

	DSPR(UPLO, N, ALPHA, X, INCX, AP)
	
	Written on 22-October-1986.
    Jack Dongarra, Argonne National Lab.
    Jeremy Du Croz, Nag Central Office.
    Sven Hammarling, Nag Central Office.
    Richard Hanson, Sandia National Labs.

*/

template <class Real>
void BlasSpr(bool Upper, int n, const Real &Alpha, Real *x, int incx, Real *A)
{

	/* Test the input parameters. */
	if(n < 0)
	{
		Print("Error in BlasSpr");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(incx == 0)
	{
		Print("Error in BlasSpr");
		Print("incx = 0");
		Print("incx",incx);
		HaltExecution();
	}
	
	/* Quick return if possible. */
	if((n == 0) || (Alpha == Zero)) return;
	
	/* Set the start point in x if the increment is not unity. */
	int kx;
	if(incx <= 0) kx = -(n - 1) * incx;
	else          kx = 0;

	/*
	
	Start the operations. In this version the elements of the array A
    are accessed sequentially with one pass through A.

	*/
	
    int kk = 0;
    if(Upper)
    {
    	/* Form  A  when upper triangle is stored in A. */
    	if(incx == 1)
    	{
    		for(int j=0; j<n; j++)
    		{
    			if(x[j] != Zero)
    			{
    				Real Temp = Alpha * x[j];
    				BlasAxpy(n-j,Temp,&x[j],1,&A[kk],1);
    			}
    			kk += n - j;
			}   
        }
		else
		{
			int jx = kx;
            for(int j=0; j<n; j++)
            {
            	if(x[jx] != Zero)
            	{
            		Real Temp = Alpha * x[jx];
                	BlasAxpy(n-j,Temp,&x[jx],incx,&A[kk],1);
				}
				jx += incx;
				kk += n - j;
            }
		}
  	}
  	else
  	{
    	/* Form  A  when lower triangle is stored in A. */
    	if(incx == 1)
    	{
    		for(int j=0; j<n; j++)
    		{
    			if(x[j] != Zero)
    			{
    				Real Temp = Alpha * x[j];
    				BlasAxpy(j+1,Temp,x,1,&A[kk],1);
                 }
                 kk += j + 1;
             }           
         }
         else
         {
        	int jx = kx;
            for(int j=0; j<n; j++)
            {
            	if(x[jx] != Zero)
            	{
            		Real Temp = Alpha * x[jx];
            		BlasAxpy(j+1,Temp,x,incx,&A[kk],1);
  				}
  				jx += incx;
  				kk += j + 1;  			
  			}
		}
    }
}

#endif
