
#ifndef _BlasSyrk
#define _BlasSyrk

	/* Perform one of the symmetric rank k operations,
	C := Alpha * A * A^T + Beta * C, or C := Alpha * A^T * A + Beta * C */
	void BlasSyrk(bool Upper, bool Trans, int n, int k, double Alpha, double *A, int lda,
		double Beta, double *C, int ldc);

/*

BlasSyrk performs one of the symmetric rank k operations

     C := Alpha * A * A^T + Beta * C,

  or

     C := Alpha * A^T * A + Beta * C,

where  Alpha and Beta are scalars, C is an n by n symmetric matrix
and A is an n by k matrix in the first case and a k by n matrix
in the second case.

Parameters
----------

Upper (boolean)	-	Specifies where the matrix is upper or lower
					triangular.
					
Trans (boolean)	-	Specifies where to solve the normal or transposed
					equation.
					
n (integer)		-	Specifies the dimension of C and x (which must
					be at least zero).

k (integer)		-	Specifies the other dimension of A (which must
					be at least zero).

Alpha (real)	-	Specifies the scalar Alpha.

A (array) 	    -	An array of dimension lda x ka where ka is k or n.

lda (integer)	-	On entry, lda specifies the leading dimension of A.
					When Trans = 0, then lda must be at least Max(1,n),
					otherwise  lda must be at least Max(1,k).
        
Beta (real)		- 	Specifies the scalar Beta.

C (array)		-	A array represting a matrix of dimension ldc x n.
					Before entry  with  UPLO = 'U' or 'u',  the leading  n by n
         			upper triangular part of the array C must contain the upper
         			triangular part  of the  symmetric matrix  and the strictly
         			lower triangular part of C is not referenced.  On exit, the
         			upper triangular part of the array  C is overwritten by the
         			upper triangular part of the updated matrix.
         			Before entry  with  UPLO = 'L' or 'l',  the leading  n by n
         			lower triangular part of the array C must contain the lower
         			triangular part  of the  symmetric matrix  and the strictly
         			upper triangular part of C is not referenced.  On exit, the
        			lower triangular part of the array  C is overwritten by the
         			lower triangular part of the updated matrix.

ldc (integer)	-	The leading dimension of the matrix C (must be at
					least Max(1,n))


This routine is adapted from the Level 3 Fortran Blas subroutine,

	DSYRK ( UPLO, TRANS, N, K, ALPHA, A, LDA, BETA, C, LDC )
	
	Written on 8-February-1989.
    Jack Dongarra, Argonne National Laboratory.
    Iain Duff, AERE Harwell.
    Jeremy Du Croz, Numerical Algorithms Group Ltd.
    Sven Hammarling, Numerical Algorithms Group Ltd.

*/

void BlasSyrk(bool Upper, bool Trans, int n, int k, double Alpha, double *A, int lda,
		double Beta, double *C, int ldc)
{	
		
	/* Test the input parameters. */
	if(n < 0)
	{
		Print("Error in BlasSyrk");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(k < 0)
	{
		Print("Error in BlasSyrk");
		Print("k < 0");
		Print("k",k);
		HaltExecution();
	}
	else if(!Trans && lda < Max(1,k))
	{
		Print("Error in BlasSyrk");
		Print("lda < Max(1,k)");
		Print("lda",lda);
		Print("k",k);
		HaltExecution();
	}
	else if(Trans && lda < Max(1,n))
	{
		Print("Error in BlasSyrk");
		Print("lda < Max(1,n)");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	
	/* Quick return if possible. */
	if((n == 0) || (((Alpha == Zero) || (k == 0)) && (Beta == One))) return;

	if(Upper)
	{
		     if(Beta == Zero) for(int i=0; i<n; i++) BlasSet(n-i,Zero,&C[ldc*i+i],1);
		else if(Beta != One)  for(int i=0; i<n; i++) BlasScal(n-i,Beta,&C[ldc*i+i],1);
	}
	else
	{
		     if(Beta == Zero) for(int i=0; i<n; i++) BlasSet(i+1,Zero,&C[lda*i],1);
		else if(Beta != One)  for(int i=0; i<n; i++) BlasScal(i+1,Beta,&C[lda*i],1);
	}

	/* And when Alpha is zero. */
	if(Alpha == Zero) return;
	
	/* Start the operations. */
	if(!Trans)
	{
		/* Form  C := Alpha * A * A^T + Beta * C. */
		if(Upper)
		{
			for(int i=0; i<n; i++) for(int j=i; j<n; j++)
				C[lda*i+j] += Alpha * BlasDot(k,&A[lda*i],1,&A[lda*j],1);
		}
		else
		{	
			for(int i=0; i<n; i++) for(int j=0; j<=i; j++)
				C[lda*i+j] += Alpha * BlasDot(k,&A[lda*i],1,&A[lda*j],1);
		}
	}
	else
	{
		/* Form  C := Alpha * A^T * A + Beta * C. */
		if(Upper)
		{
			for(int i=0; i<n; i++) for(int l=0; l<k; l++)
			{
				double Temp = Alpha * A[lda*l+i];
				if(Temp != Zero) BlasAxpy(n-i,Temp,&A[lda*l+i],1,&C[lda*i+i],1);
			}
		}
		else
		{
			for(int i=0; i<n; i++) for(int l=0; l<k; l++)
			{
				double Temp = Alpha * A[lda*l+i];
				if(Temp != Zero) BlasAxpy(i+1,Temp,&A[lda*l],1,&C[lda*i],1);
			}
		}
	}
}

#endif
