
#ifndef _BlasTpmv
#define _BlasTpmv

	
	/* Perform one of the matrix-vector operations, x := A * x,   or   x := A^T * x,
	where A is a upper or lower triangular matrix, supplied in packed form */
	void BlasTpmv(bool Upper, bool Trans, bool Diag, int n, const double *A, double *x, int incx);

/*

Perform one of the matrix-vector operations

     x := A * x,   or   x := A^T * x,

where x is an n element vector and A is an nxn unit, or non-unit,
upper or lower triangular matrix, supplied in packed form.


Parameters
----------


Upper (boolean)	-	Specifies whether the matrix is an upper or
					lower triangular matrix as follows:
					
Trans (boolean)	-	Specifies the operation to be performed as follows:

					Trans = 0		x := A * x.
					Trans = 1		x := A^T * x.				
	
Diag (boolean)	-	Specifies whether or not A is unit triangular

n (integer)		-	Specifies the dimension of the matrix A. n must be at
					least zero.
					
A (array)		-	Array of dimension at least (n * (n + 1)) / 2.
        			Before entry with  UPLO = 'U' or 'u', the array AP must
      				contain the upper triangular matrix packed sequentially,
       				column by column, so that AP( 1 ) contains a( 1, 1 ),
  					AP( 2 ) and AP( 3 ) contain a( 1, 2 ) and a( 2, 2 )
     				respectively, and so on.
      				Before entry with UPLO = 'L' or 'l', the array AP must
    				contain the lower triangular matrix packed sequentially,
   					column by column, so that AP( 1 ) contains a( 1, 1 ),
    				AP( 2 ) and AP( 3 ) contain a( 2, 1 ) and a( 3, 1 )
      				respectively, and so on.
     				Note that when  DIAG = 'U' or 'u', the diagonal elements of
         			A are not referenced, but are assumed to be unity.
      				Unchanged on exit.

x (array)		-	Array of dimension at least   ( 1 + ( n - 1 )*abs( INCX ) ).
         			Before entry, the incremented array X must contain the n
        			element vector x. On exit, X is overwritten with the
       				tranformed vector x.

incx (integer)	-	Specifies the increment for the elements of
					X. INCX must not be zero.


Adapted from the Level 2 Blas routine,

	DTPMV ( UPLO, TRANS, DIAG, N, AP, X, INCX )

	Level 2 Blas routine.
	
	Written on 22-October-1986.
	Jack Dongarra, Argonne National Lab.
	Jeremy Du Croz, Nag Central Office.
	Sven Hammarling, Nag Central Office.
	Richard Hanson, Sandia National Labs.

*/

void BlasTpmv(bool Upper, bool Trans, bool Diag, int n, const double *A, double *x, int incx)
{
	
	double Temp;
	int i,ix,j,jx,k,kk,kx;
	
	/* Test the input parameters. */
	if(n < 0)
	{
		Print("Error in BlasTpmv");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(incx == 0)
	{
		Print("Error in BlasTpmv");
		Print("incx = 0");
		Print("incx",incx);
		HaltExecution();
	}
    
    /* Quick return if possible. */
    if(n == 0) return;

	/*
	
	Set up the start point in x if the increment is not unity. This
	will be  (n - 1) * incx too small for descending loops.
	
	*/
	
	if(incx < 0) kx = -(n - 1) * incx;
	else 	  kx = 0;

	/*
	
	Start the operations. In this version the elements of AP are
	accessed sequentially with one pass through AP.
	
	*/
	
	if(!Trans)
	{
		/* Form  x := A^T * x. */
		if(Upper)
		{
			kk = 0;
			if(incx == 1)
			{
				for(j=0; j<n; j++)
				{
					Temp = x[j];
					if(Diag) Temp *= A[kk];
					k = kk + 1;
					for(i=j+1; i<n; i++) Temp += A[k++] * x[i];
					x[j] = Temp;
					kk += n - j;
				}
			}
			else
			{
				jx = kx;
				for(j=0; j<n; j++)
				{
					Temp = x[jx];
					ix = jx;
					if(Diag) Temp *= A[kk];
					for(k=kk+1; k<=kk+n-j; k++)
					{
						ix += incx;
						Temp += A[k] * x[ix];
					}
					x[jx] = Temp;
					jx += incx;
					kk += n - j;
				}
			}
		}
		else
		{
			kk = n * (n + 1) / 2 - 1;
			if(incx == 1)
			{
				for(j=n-1; j>=0; j--)
				{
					Temp = x[j];
					if(Diag) Temp *= A[kk];
					k = kk - 1;
					for(i=j-1; i>=0; i--) Temp += A[k--] * x[i];
					x[j] = Temp;
					kk -= j + 1;
				}
  			}
  			else
  			{
  				jx = kx + (n - 1) * incx;
  				for(j=n-1; j>=0; j--)
  				{
  					Temp = x[jx];
  					ix = jx;
  					if(Diag) Temp *= A[kk];
					for(k=kk-1; k>=kk-j; k--)
					{
						ix -= incx;
						Temp += A[k] * x[ix];
					}
					x[jx] = Temp;
					jx -= incx;
					kk -= j + 1;
				}
			}
		}
	}
	else
	{
		/* Form  x:= A * x. */
		if(Upper)
		{
			kk = n * (n + 1) / 2 - 1;
			if(incx == 1)
			{
				for(j=n-1; j>=0; j--)
				{
					if(x[j] != Zero)
					{
						Temp = x[j];
						k = kk;
						for(i=n-1; i>j; i--) x[i] += Temp * A[k--];
						if(Diag) x[j] *= A[kk - n  + j + 1];
					}
					kk -= n - j;
				}
			}
			else
			{
				kx += (n - 1) * incx;
				jx = kx;
				for(j=n-1; j>=0; j--)
				{
					if(x[jx] != Zero)
					{
						Temp = x[jx];
						ix = kx;
						for(k=kk; k>=kk-n+j+2; k--)
						{
							x[ix] += Temp * A[k];
							ix -= incx;
						}
						if(Diag) x[jx] *= A[kk - n + j + 1];
					}
					jx -= incx;
					kk -= n - j;
				}
			}
		}
		else
		{
			kk = 0;
			if(incx == 1)
			{
				for(j=0; j<n; j++)
				{
					if(x[j] != Zero)
					{
						Temp = x[j];
						k = kk;
						for(i=0; i<j; i++) x[i] += Temp * A[k++];
						if(Diag) x[i] *= A[kk + j];
					}
                  	kk += j + 1;
                }
			}
			else
			{
				jx = kx;
				for(j=0; j<n; j++)
				{
					if(x[jx] != Zero)
					{
						Temp = x[jx];
						ix = kx;
						for(k=kk; k<kk+j; k++)
						{
							x[ix] += Temp * A[k];
							ix += incx;
						}
						if(Diag) x[jx] *= A[kk + j];
					}
					jx += incx;
					kk += j + 1;
				}
			}
		}
	}
}

#endif
