
#ifndef _BlasTpsv
#define _BlasTpsv

	/*
	
	Solve A * x = b or A^T * x = b where A is a lower or upper-triangular
	matrix stored in packed form
	
	*/
	
	template <class Real>
	void BlasTpsv(bool Upper, bool Trans, bool Diag, int n, const Real *A, Real *x, int incx);

/*

BlasTpsv solves one of the systems of equations,

	A * x = b,	or	A^T * x = b
	
where b and x are n element vectors and A is an n by n unit, or
non-unit, upper or lower triangular matrix, supplied in packed form.

No test for singularity or near-singularity is included in this
routine. Such tests must be performed before calling this routine.


Parameters
----------

Upper (boolean)	-	Specifies where the matrix is upper or lower
					triangular.
					
Trans (boolean)	-	Specifies where to solve the normal or transposed
					equation.

Diag (boolean)	-	Specifies whether the matrix has a unit diagonal.

n (integer)		-	Specifies the dimensions of A and x (which must
					be equal).

A (array) 	    -	An array of dimension n * (n + 1) / 2, which
					represents a lower or upper-triangular matrix
					in packed form.
					 
x (array)		-	An incremented array of dimension n which contains
					the right-hand side vector b. On exit, x is overwritten
					with the solution vector x.

incx (integer)	-	Specifies the increment for the elements of x. incx must
					not be zero.


This routine is adapted from the Level 2 Fortran Blas subroutine,

	DTPSV(UPLO, TRANS, DIAG, N, AP, X, INCX)
	
	Written on 22-October-1986.
    Jack Dongarra, Argonne National Lab.
    Jeremy Du Croz, Nag Central Office.
    Sven Hammarling, Nag Central Office.
    Richard Hanson, Sandia National Labs.

Adapted by MP.

Warning: I have not yet verified that negative increments are
working properly!

*/

template <class Real>
void BlasTpsv(bool Upper, bool Trans, bool Diag, int n, const Real *A, Real *x, int incx)
{

	//int i,ix,j,jx,k,kk,kx;
	
	/*
	
	Test the input parameters.
	
	(In the original program, Info = 1 if UpLo is not 'U' or 'L',
	Info = 2 if Trans is not 'N', T', or 'C', and Info = 3 if
	Diag is not 'U' or 'N'.
	
	*/
	
	if(n < 0)
	{
		// Dimension of the matrix must be positive
		Print("Error in BlasTpsv");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(incx == 0)
	{
		// Increment must not be zero
		Print("Error in BlasTpsv");
		Print("incx == 0");
		Print("incx",incx);
		HaltExecution();
    }
    
    /* Quick return if possible. */
    if(n == 0) return;
    
    /*
    
    Set up the start point in x if the increment is not unity. This
    will be  (n - 1) * incx  too small for descending loops.
    
    */
    
    int kx;
    if(incx <= 0) kx = -(n - 1) * incx;
    else 		  kx = 0;
    
    /*
	
	Start the operations. In this version the elements of AP are
    accessed sequentially with one pass through AP.

	*/

	if(!Trans)
	{
		/* Form  x := inv(A) * x. */
		if(Upper)
		{
			/* Upper triagular */
			int kk = (n * (n + 1)) / 2 - 1;
           	if(incx == 1)
           	{
           		/* Increment is 1 */
           		//Print("Tspv - Not Transpose, Upper, Increments is 1");
           		for(int j=n-1; j>=0; j--)
           		{
           			Real Temp = x[j];
               		int k = kk;
               		for(int i=n-1; i>j; i--)
               		{
               			Temp -= A[k] * x[i];
                   		k--;
					}
					if(Diag) Temp /= A[kk - n + j + 1];
					x[j] = Temp;
					kk -= n - j;
				}
			}
			else
			{
				/* Increment is not 1 */
				//Print("Tspv - Not Transpose, Upper, Increments is not 1");
				kx += (n - 1) * incx;
				int jx = kx;
				for(int j=n-1; j>=0; j--)
				{	
					Real Temp = x[jx];
               		int ix = kx;
               		for(int k=kk; k>kk-(n-j-1); k--)
               		{
               			Temp -= A[k] * x[ix];
                   		ix -= incx;
                   	}
					if(Diag) Temp /= A[kk - n + j + 1];
					x[jx] = Temp;
               		jx -= incx;
               		kk -= n - j;
               	}
            }
        }
        else
		{
			/* Lower triangular */
			int kk = 0;
			if(incx == 1)
			{
				/* The increment is 1 */
				for(int j=0; j<n; j++)
				{
				  	Real Temp = x[j] - BlasDot(j,&A[kk],1,&x[0],1);
                   	if(Diag) Temp /= A[kk + j];
               		x[j] = Temp;
               		kk += j + 1;
				}
			}
			else
			{
				/* The increment is not 1 */
				int jx = kx;
				for(int j=0; j<n; j++)
				{
					Real Temp = x[jx];
               		int ix = kx;
               		for(int k=kk; k<kk+j; k++)
               		{
               			Temp -= A[k] * x[ix];
                   		ix += incx;
                   	}
					if(Diag) Temp /= A[kk + j];
					x[jx] = Temp;
               		jx += incx;
               		kk += j + 1;
				}
			}
		}
    }
    else	
    {
    	/* Form  x := inv(A^T) * x. */
    	if(Upper)
    	{
    		/* Upper triangular */
			int kk = 0;
           	if(incx == 1)
           	{
           		/* The increment is 1 */
           		int k = 0;
           		for(int j=0; j<n; j++)
           		{
           			if(x[j] != Zero)
           			{
          				if(Diag) x[j] /= A[k];
           				Real Temp = x[j];
           				k++;
           				for(int i=j+1; i<n; i++)
           				{
           					x[i] -= Temp * A[k];
           					k++;
           				}
           			}
           		}
           	}
           	else
           	{
           		/* The increment is not 1 */
           		//Print("Tspv - Transpose, Upper, Increments is not 1");
           		int jx = kx;
           		for(int j=0; j<n; j++)
           		{
           			if(x[j] != Zero)
           			{
           				if(Diag) x[jx] /= A[kk];
           				Real Temp = x[jx];
           				int ix = jx;
           				for(int k=kk+1; k<kk+n-j; k++)
           				{
           					ix += incx;
           					x[ix] -= Temp * A[k];
           				}
           			}
					jx += incx;
					kk += n - j;
				}
			}
		}
		else
    	{
    		/* Lower triangular */
    		int kk = n * (n + 1) / 2 - 1;
            if(incx == 1)
            {
            	/* The increment is 1 */
            	for(int j=n-1; j>=0; j--)
            	{
            		if(x[j] != Zero)
            		{
            			if(Diag) x[j] /= A[kk];
            			Real Temp = x[j];
            			int k = kk - 1;
            			for(int i=j-1; i>=0; i--)
            			{
            				x[i] -= Temp * A[k];
                        	k--;
						}
					}
					kk -= j + 1;
				}
			}
			else
			{
				/* The increment is not 1 */
				int jx = kx + (n - 1) * incx;
				for(int j=n-1; j>=0; j--)
				{
					if(x[jx] != Zero)
					{
						if(Diag) x[jx] /= A[kk];
						Real Temp = x[jx];
						int ix = jx;
						for(int k=kk-1; k>=kk-j; k--)
						{
							ix -= incx;
							x[ix] -= Temp * A[k];
						}
					}
					x -= incx;
					kk -= j + 1;
				}
			}
		}		
	}	
}

#endif
