
#ifndef _BlasTrsm
#define _BlasTrsm

	void BlasTrsm(bool Left, bool Upper, bool TransA, bool Diag, int m, int n,
					double Alpha, double *A, int lda, double *B, int ldb);

/*

Solve one of the matrix equations

	op(A) * X = Alpha * B,   or   X * op(A) = Alpha * B,

where Alpha is a scalar, X and B are m by n matrices, A is a unit, or
non-unit,  upper or lower triangular matrix  and  op( A )  is one  of

     op(A) = A   or   op(A) = A^T.

The matrix X is overwritten on B.


Parameters
----------

Left (boolean)		-	Specifies whether op(A) appears on the left
						or right of X as follows:
						
						Left = true		op(A) * X = Alpha * B.
						Left = false	X * op(A) = Alpha * B.

Upper (boolean)		-	Specifies whether the matrix A is an upper or
						lower triangular matrix as follows:
						
TransA (boolean)	-	Specifies the form of op(A) to be used in
						the matrix multiplication as follows:
						
Diag (boolean)		-	Specifies whether or not A is unit triangular.
						
m (integer)			-	Specifies the number of rows of B. m must be at
						least zero.
						
n (integer)			-	Specifies the number of columns of B. n must be
						at least zero.

Alpha (Real)		-	Specifies the scalar Alpha. When Alpha is
						zero, then A is not referenced and B need not be set
						before entry.
						
A (array)			-	Array representing matrix of dimension lda x k, where k is m
						when Left = true  and is  n  when Left = false.
						Before entry  with  UPLO = 'U' or 'u',  the  leading  k by k
						upper triangular part of the array  A must contain the upper
						triangular matrix  and the strictly lower triangular part of
						A is not referenced.
						Before entry  with  UPLO = 'L' or 'l',  the  leading  k by k
						lower triangular part of the array  A must contain the lower
						triangular matrix  and the strictly upper triangular part of
						A is not referenced.
						Note that when  DIAG = 'U' or 'u',  the diagonal elements of
						A  are not referenced either,  but are assumed to be  unity.
						Unchanged on exit.
						
lda (integer)		-	On entry, LDA specifies the first dimension of A as declared
						in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
						LDA  must be at least  max( 1, m ),  when  SIDE = 'R' or 'r'
						then LDA must be at least max( 1, n ).
						Unchanged on exit.
						
B (array)			-	Array of DIMENSION ( LDB, n ).
						Before entry,  the leading  m by n part of the array  B must
						contain  the  right-hand  side  matrix  B,  and  on exit  is
						overwritten by the solution matrix  X.
						
ldb (integer)		-	On entry, LDB specifies the first dimension of B as declared
						in  the  calling  (sub)  program.   LDB  must  be  at  least
						max( 1, m ).
						Unchanged on exit.
						

Adapted from the Lapack subroutine,

	DTRSM ( SIDE, UPLO, TRANSA, DIAG, M, N, ALPHA, A, LDA, B, LDB )
	
	Level 3 Blas routine.
	-- Written on 8-February-1989.
	Jack Dongarra, Argonne National Laboratory.
	ain Duff, AERE Harwell.
	Jeremy Du Croz, Numerical Algorithms Group Ltd.
	Sven Hammarling, Numerical Algorithms Group Ltd.


Note: this routine probably has to be reworked to access adjacent elements
on the matrices. -MP

*/

void BlasTrsm(bool Left, bool Upper, bool TransA, bool Diag, int m, int n,
	double Alpha, double *A, int lda, double *B, int ldb)
{
	int nRowA;
	double Temp;
	
	/* Test the input parameters. */
	
	if(Left) nRowA = m;
	else     nRowA = n;
     
    int Info = 0;
    
    if(m < 0)
    {
    	Print("Error in BlasTrsm");
    	Print("m < 0");
    	Print("m",m);
		HaltExecution();
	}
    else if(n < 0)
    {
    	Print("Error in BlasTrsm");
    	Print("n < 0");
    	Print("n",n);
    	HaltExecution();
    }
    else if(lda < Max(1,nRowA))
    {
    	Print("Error in BlasTrsm");
    	Print("lda < Max(1,nRowA");
    	Print("lda",lda);
    	Print("nRowA",nRowA);
    	HaltExecution();
    }
    /*
    else if(ldb < Max(1,n))
    {
    	Print("Erro in BlasTrsm");
    	Print("ldb < Max(1,n)");
    	Print("ldb",ldb);
    	Print("n",n);
    	HaltExecution();
    }
    */
    
	/* Quick return if possible. */
	if(n == 0) return;

	/* And when  alpha.eq.zero. */
	if(Alpha == Zero)
	{
		for(int i=0; i<m; i++) for(int j=0; j<n; j++) B[ldb*i+j] = Zero;
		return;
	}
	
	/* Start the operations. */
	if(Left)
	{
		
		if(!TransA)
		{
			
			/* Form  B := Alpha * A^-1 * B. */
			if(Upper)
			{
			
				for(int j=0; j<n; j++)
				{
					/* For each rhs vector */
					if(Alpha != One) for(int i=0; i<m; i++) B[ldb*i+j] *= Alpha;
					for(int k=m-1; k>=0; k--)
					{
						if(B[ldb*k+j] != Zero)
						{
							if(Diag) B[ldb*k+j] /= A[lda*k+k];
							for(int i=0; i<k; i++) B[ldb*i+j] -= B[ldb*k+j] * A[lda*i+k];	// No Efficient Here
                        }
                    }
                }
			}
			else
			{
			
				/*
				
				The original code for this section was,
				
				for(j=0; j<n; j++)
				{
					if(Alpha != One) for(i=0; i<m; i++) B[lda*i+j] *= Alpha;
					for(k=0; k<m; k++)
					{
						if(B[ldb*k+j] != Zero)
						{
							if(Diag) B[ldb*k+j] /= A[lda*k+k];
							for(i=k+1; i<m; i++) B[ldb*i+j] -= B[ldb*k+j] * A[lda*i+k];
						}
					}
				}
				
				*/
				
				/*
				
				This code had to be reworked to access memory sequentially. To this end,
				we replaced it with the algorithm for Upper-Transpose and switched the
				(i,j) access elements. Also, we use BlasDot to make use of loop unrolling.
				The same this should be done for the other 7 cases in this file.
				
				*/
				
				for(int i=0; i<m; i++) BlasScal(n,Alpha,&B[ldb*i],1);
				for(int k=0; k<m; k++)
				{
					if(Diag) BlasScal(n,One/A[lda*k+k],&B[ldb*k],1);
					for(int i=k+1; i<m; i++)
					{
						Temp = -A[lda*i+k];
						BlasAxpy(n,Temp,&B[ldb*k],1,&B[ldb*i],1);
					}
				}			
			}
		}
		else
		{
		
			/* Form  B := Alpha * inv(A^T) * B. */
			if(Upper)
			{
			
				/* This code has to be made efficient in order to make
				the block-cholesky algorithm fast */
		
				/*
				for(j=0; j<n; j++)
				{
					for(i=0; i<m; i++)
					{
						Temp = Alpha * B[ldb*i+j];
						for(k=0; k<i; k++)
						{
							Temp -= A[lda*k+i] * B[ldb*k+j];
						}
						if(Diag) Temp /= A[lda*i+i];
						B[ldb*i+j] = Temp;
					}
				}
				*/
				
				for(int i=0; i<m; i++)
				{
					for(int k=0; k<i; k++)
					{
						Temp = -Alpha * A[lda*k+i];
						BlasAxpy(n,Temp,&B[ldb*k],1,&B[ldb*i],1);
					}
					if(Diag) BlasScal(n,One/A[lda*i+i],&B[ldb*i],1);
				}
			}
			else
			{
				for(int j=0; j<n; j++)
				{
					for(int i=m-1; i>=0; i--)
					{
						Temp = Alpha * B[ldb*i+j];
						for(int k=i+1; k<m; k++) Temp -= A[lda*k+i] * B[ldb*k+j];	// No Efficient Here
                    	if(Diag) Temp /= A[lda*i+i];
                   		B[ldb*i+j] = Temp; 	
                  	}
                }
            }
        }
    }
    else
    {
    
    	if(!TransA)
    	{
    	
    		/* Form  B := Alpha * B * A^-1. */
    		if(Upper)
    		{
    			for(int j=0; j<n; j++)
    			{
    				if(Alpha != One) for(int i=0; i<m; i++) B[ldb*i+j] *= Alpha;
    				for(int k=0; k<j; k++)
    				{
    					if(A[lda*k+j] != Zero) for(int i=0; i<m; i++) B[ldb*i+j] -= A[lda*k+j] * B[ldb*i+k];	// No Efficient Here
 					}
 					if(Diag)
 					{
 						Temp = One / A[lda*j+j];
 						for(int i=0; i<m; i++) B[ldb*i+j] *= Temp;
					}
				}
			}
			else
			{
				for(int j=n-1; j>=0; j--)
				{
					if(Alpha != One) for(int i=0; i<m; i++) B[ldb*i+j] *= Alpha;
					for(int k=j+1; k<n; k++)
					{
						if(A[lda*k+j] != Zero) for(int i=0; i<m; i++) B[ldb*i+j] -= A[lda*k+j] * B[ldb*i+k];	// No Efficient Here
					}
					if(Diag)
					{
						Temp = One / A[lda*j+j];
						for(int i=0; i<m; i++) B[lda*i+j] *= Temp;
					}
				}
			}
		}
		else
		{
		
			/* Form  B := Alpha * B * (A^-1)^T. */
			if(Upper)
			{
				for(int k=n-1; k>=0; k--)
				{
					if(Diag)
					{
						Temp = One / A[lda*k+k];
						for(int i=0; i<m; i++) B[ldb*i+k] *= Temp;	// No Efficient Here
					}
					for(int j=0; j<k; j++)
					{
						if(A[lda*j+k] != Zero)
						{
							Temp = A[lda*j+k];
							for(int i=0; i<m; i++) B[ldb*i+j] -= Temp * B[ldb*i+k];		// No Efficient Here	
						}
					}
					if(Alpha != One) for(int i=0; i<m; i++) B[ldb*i+k] *= Alpha;
				}
			}
			else
			{
				for(int k=0; k<n; k++)
				{
					if(Diag)
					{
						Temp = One / A[lda*k+k];
						for(int i=0; i<m; i++) B[ldb*i+k] *= Temp;		// No Efficient Here
					}
					for(int j=k+1; j<n; j++)
					{
						if(A[lda*j+k] != Zero)
						{
							Temp = A[lda*j+k];
							for(int i=0; i<m; i++) B[ldb*i+j] -= Temp * B[ldb*i+k];		// No Efficient Here
						}
					}
					if(Alpha != One) for(int i=0; i<m; i++) B[ldb*i+k] *= Alpha;
				}
			}
		}
	}
}

#endif
