
#ifndef _GenMatUtil_CholeskyDecompObj
#define _GenMatUtil_CholeskyDecompObj

/*

Compute the Cholesky factorization of a symmetric positive-definite
matrix A, i.e. compute a lower triangular matrix L such that
A = L * L^T. If the matrix is not symmetric or positive-definite, the
function computes only a partial decomposition. This can be tested with
the IsSpd() flag.

*/

template <class Real>
class CholeskyDecompObj
{

	private:

	int n;						// The dimension of the matrix
	int Info;					// ...
	int Spd;					// 1 if Matrix to be factored was SPD
	int Type;
	LowerMatrixUN<Real> L;
	//LowerMatrix<Real> L;

	public:

	CholeskyDecompObj();
	CholeskyDecompObj(const GeneralMatrix<Real> &A);
		
	LowerMatrix<Real> GetL() const;
	
	Vector<Real> Solve(const Vector<Real> &b);
	Matrix<Real> Solve(const Matrix<Real> &B);
	
	Vector<Real> SolveL(const Vector<Real> &b);
	Matrix<Real> SolveL(const Matrix<Real> &B);
	
	Vector<Real> SolveLTrans(const Vector<Real> &b);
	Matrix<Real> SolveLTrans(const Matrix<Real> &B);
	
	int IsSpd() const;
	int GetInfo() const;
	SymmetricMatrixLU<Real> Inverse();
	//SymmetricMatrix<Real> Inverse();
	Real Determinant();
};

/* Default Constructor */
template <class Real>
CholeskyDecompObj<Real>::CholeskyDecompObj()
{
	n = 0;
	Info = 0;
	Spd = 0;
	L = LowerMatrixUN<Real>();
	//L = LowerMatrix<Real>();
}

template <class Real>
CholeskyDecompObj<Real>::CholeskyDecompObj(const GeneralMatrix<Real> &A)
{
	/* Check if the Matrix is square */
	n = A.Rows();
	if(n != A.Cols())
	{
		Print("Error in CholeskyDecompObj");
		Print("Matrix is Not Square");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		HaltExecution();
	}
	
	if(!IsSymmetric(A))
	{
		Print("Error in CholeskyDecompObj");
		Print("Matrix is Not Symmetric");
		Print("A.Name",A.Name);
		Print("A.Size()",A.Size());
		HaltExecution();
	}
	
	L = LowerMatrixUN<Real>(n);
	//L = LowerMatrix<Real>(n);
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) L(i,j) = A(i,j);
	
	/* Preform decomposition */
	Info = LapackPotrf(0,n,L.Begin(),n);
	//Info = LapackPptrf(0,n,L.Begin());
    Spd = (Info == 0);	
}


/*

Return 1 if the original matrix to be factored was symmetric
and positive-definite

*/
 
template <class Real>
int CholeskyDecompObj<Real>::IsSpd() const
{
	return Spd;
}

template <class Real>
int CholeskyDecompObj<Real>::GetInfo() const
{
	return Info;
}

/* Return the lower triangular factor, L, such that L * L^T = A. */
template <class Real>
LowerMatrix<Real> CholeskyDecompObj<Real>::GetL() const
{
	if(!Spd)
	{
		Throw(ToString("Error in CholeskyDecompObj::GetL\n")
			  + "L: " + ToString(L) + "\n"
		      + "Not Positive Definite");
	}
	
	LowerMatrix<double> L2(n);
	Copy(L2,L);
	return L2;
}

/*

Constructs a lower triangular matrix L, such that L * L^T = A.
If A is not symmetric positive-definite (SPD), only a
partial factorization is performed.  If is_spd()
evalutate true (1) then the factorizaiton was successful.

*/

/*

Constructs a lower triangular matrix L, such that L*L'= A.
If A is not symmetric positive-definite (SPD), only a
partial factorization is performed.  If is_spd()
evalutate true (1) then the factorizaiton was successful.

*/

/*
template <class Real>
CholeskyDecompObj<Real>::CholeskyDecompObj(const SymmetricMatrix<Real> &A)
{
  	n = A.Dim();
	L_P = LowerMatrix<Real>(n);
	BlasCopy(A.Length(),A.Begin(),1,L_P.Begin(),1);
	Type = 1;
	
	/* Preform decomposition */
/*	Info = LapackPptrf(0,n,L_P.Begin());
    Spd = (Info == 0);
}
*/

/*

Solve a linear system A * x = b, using the previously computed
cholesky factorization of A: L * L^T.

@param  B   A Matrix with as many rows as A and any number of columns.
@return     x so that L*L'*x = b.  If b is nonconformat, or if A
			   was not symmetric posidtive definite, a null (0x0)
			   array is returned.
			   
*/

template <class Real>
Vector<Real> CholeskyDecompObj<Real>::Solve(const Vector<Real> &b)
{
	/* Check for Incompadible Dimensions */
	if(b.Size() != n)
	{
		Print("Error in CholeskyDecompObj::Solve(Vector)");
		Print("Incompadible Dimensions");
		Print("n",n);
		Print("b.Size()",b.Size());
		HaltExecution();
	}
	
	/* Solve L * L^T * x = b */
	Vector<Real> x = b.Copy();
	
	LapackPotrs(0,n,1,L.Begin(),n,x.Begin(),1);
	//LapackPptrs(0,n,1,L.Begin(),x.Begin(),1);
	
	return x;
}


/*

Solve a linear system A * X = B, using the previously computed
cholesky factorization of A: L * L^T.

@param  B   A Matrix with as many rows as A and any number of columns.
@return     X so that L*L'*X = B.  If B is nonconformat, or if A
 				was not symmetric posidtive definite, a null (0x0)
   						array is returned.

*/

template <class Real>
Matrix<Real> CholeskyDecompObj<Real>::Solve(const Matrix<Real> &B)
{
	int nRhs = B.Cols();

	/* Check for Incompadible Dimensions */
	if(B.Rows() != n)
	{
		Print("Error in CholeskyDecompObj::Solve(Matrix)");
		Print("Incombadible Dimensions");
		Print("n",n);
		Print("B.Name",B.Name);
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Solve L * L^T * X = B */
	Matrix<Real> X = B.Copy();
		
	LapackPotrs(0,n,nRhs,L.Begin(),n,X.Begin(),nRhs);
	//LapackPptrs(0,n,nRhs,L.Begin(),X.Begin(),nRhs);
	
    return X;
}

template <class Real>
Vector<Real> CholeskyDecompObj<Real>::SolveL(const Vector<Real> &b)
{
	
	/* Check for Incompadible Dimensions */
	if(b.Size() != n)
	{
		Print("Error in CholeskyDecompObj::SolveL(Vector)");
		Print("Incombadible Dimensions");
		Print("n",n);
		Print("b.Name",b.Name);
		Print("b.Size()",b.Size());
		HaltExecution();
	}

	/* Solve L * x = b */
	Vector<Real> x = b.Copy();
	BlasTrsv(0,0,1,n,L.Begin(),n,&x[0],1);
	//BlasTpsv(0,0,1,n,L.Begin(),n,&x[0],1);
	return x;
}

template <class Real>
Matrix<Real> CholeskyDecompObj<Real>::SolveL(const Matrix<Real> &B)
{
	int nRhs = B.Cols();

	/* Check for Incompadible Dimensions */
	if(B.Rows() != n)
	{
		Print("Error in CholeskyDecompObj::Solve(Matrix)");
		Print("Incombadible Dimensions");
		Print("n",n);
		Print("B.Name",B.Name);
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/* Solve L * X = B */
	Matrix<Real> X = B.Copy();
		
	/*if(Type == 1)
	{
		for(int i=0; i<nRhs; i++) BlasTpsv(0,0,1,n,A,&B[i],ldb);
	}
	else
	{*/
		Print("Error yyy in CholeskyDecompObj");
		Print("This part not done yet");
		HaltExecution();
	/*}*/

    return X;
}

template <class Real>
Vector<Real> CholeskyDecompObj<Real>::SolveLTrans(const Vector<Real> &b)
{
	
	/* Check for Incompadible Dimensions */
	if(b.Size() != n)
	{
		Print("Error in CholeskyDecompObj::SolveLTrans(Vector)");
		Print("Incombadible Dimensions");
		Print("n",n);
		Print("b.Name",b.Name);
		Print("b.Size()",b.Size());
		HaltExecution();
	}

	/* Solve L^T * x = b */
	Vector<Real> x = b.Copy();
	BlasTrsv(0,1,1,n,L.Begin(),n,&x[0],1);
	//BlasTpsv(0,1,1,n,L.Begin(),n,&x[0],1);
	return x;
}


/* Invert a symetric positive-definite Matrix */
template <class Real>
SymmetricMatrixLU<Real> CholeskyDecompObj<Real>::Inverse()
//SymmetricMatrix<Real> CholeskyDecompObj<Real>::Inverse()
{
	/* Compute the inverse of A */
	SymmetricMatrixLU<Real> AInv(n);
	//SymmetricMatrix<Real> AInv(n);
	for(int i=0; i<n; i++) for(int j=0; j<=i; j++) AInv(i,j) = L(i,j);
	int Info = LapackPotri(0,n,AInv.Begin(),n);
	//int Info = LapackPptri(0,n,AInv.Begin(),n);
	if(Info > 0)
	{
		Print("Error in CholeskyDecompObj::Inverse()");
		Print("Matrix is Not Invertable");
		Print("Info",Info);
		HaltExecution();
	}
	return AInv;
}

template <class Real>
Real CholeskyDecompObj<Real>::Determinant()
{
	Real d = One;
	for(int i=0; i<n; i++) d *= L(i,i);
	return Sqr(d);
}

#endif
