
#ifndef _ComputeDerivStep
#define _ComputeDerivStep

/*

Compute step sizes for numerical derivatives

Input:

	x		-	Current value of x
	Step	-	Size of unnormalized step
	Type	-	Type >= 1 and <= 3 for absolute step, otherwise use a relative step
	TypX    -   Vector constaining typical values of x
	Z		-	Projection matrix
	
Output:

	hZ		-	Normalized steps


*/

	/* Derivative step */
	void ComputeDerivStep(Vector<double> &hZ, const Vector<double> &x, double Step, int Type, const Vector<double> &TypX);

	/* Projected derivative step */
	void ComputeDerivStep(Vector<double> &hZ, const Vector<double> &x, const Matrix<double> &Z, double Step,
		int Type, const Vector<double> &TypX);


/* Derivative step */
void ComputeDerivStep(Vector<double> &hZ, const Vector<double> &x, double Step, int Type, const Vector<double> &TypX)
{
	int n = x.Size();

	if(Type >= 1 && Type <= 3)
	{
		// Relative step
		for(int i=0; i<n; i++)
		{
			hZ[i] = abs(x[i]) * Step;
			if(hZ[i] < TypX[i]) hZ[i] = TypX[i] * Step;
		}
	}
	else
	{
		// Absolute step
		for(int i=0; i<n; i++) hZ[i] = Step;		
	}
}
	
/* Projected derivative step */
void ComputeDerivStep(Vector<double> &hZ, const Vector<double> &x, const Matrix<double> &Z, double Step,
	int Type, const Vector<double> &TypX)
{
	int n  = x.Size();
	int m1 = n - Z.Cols(); 

	Vector<double> h(n);	h.Name = "ComputeDerivStep=::h";
	if(Type >= 1 && Type <= 3)
	{
		// Relative step
		for(int i=0; i<n; i++)
		{
			h[i] = abs(x[i]) * Step;
			if(h[i] < TypX[i]) h[i] = TypX[i] * Step;
		}
	}
	else
	{
		// Absolute step
		for(int i=0; i<n; i++) h[i] = Step;	
	}
	
	for(int i=0; i<n-m1; i++)
	{
		int MxIdx = -1;
		double MxVal = -Inf;
		for(int j=0; j<n; j++)
		{
			double CurrVal = abs(Z(j,i)) * h[j];
			if(CurrVal > MxVal)
			{
				MxIdx = j;
				MxVal = CurrVal;
			}
		}
		hZ[i] = h[MxIdx];			
	}
}	

/* Projected derivative step */
#ifdef _GenMat_SparseMatrixCC
	void ComputeDerivStep(Vector<double> &hZ, const Vector<double> &x, const SparseMatrixCC<double> &ZS, double Step,
		int Type, const Vector<double> &TypX)
	{
		int n  = x.Size();
		int m1 = n - ZS.Cols(); 

		Vector<double> h(n);	h.Name = "ComputeDerivStep=::h";
		if(Type >= 1 && Type <= 3)
		{
			// Relative step
			for(int i=0; i<n; i++)
			{
				h[i] = abs(x[i]) * Step;
				if(h[i] < TypX[i]) h[i] = TypX[i] * Step;
			}
		}
		else
		{
			// Absolute step
			for(int i=0; i<n; i++) h[i] = Step;	
		}
		
		for(int i=0; i<n-m1; i++)
		{
			int MxIdx = -1;
			double MxVal = -Inf;
			for(int k=ZS.c[i]; k<ZS.c[i+1]; k++)
			{
				int j = ZS.r[k];
				double CurrVal = abs(ZS.v[k]) * h[j];
				if(CurrVal > MxVal)
				{
					MxIdx = j;
					MxVal = CurrVal;
				}
			}
			hZ[i] = h[MxIdx];			
		}
	}	
#endif

#endif