
#ifndef _GenMat_DiagonalMatrix
#define _GenMat_DiagonalMatrix

/*

Templated two-dimensional numerical array which
looks like a conventional C multiarray.
Elements are accessed via A(i,j) notation and
diagonal elements can be accessed using A(i)
notation

Array assignment is by reference (i.e. shallow assignment).
That is, B = A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.Copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.

This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.

Taken from Template Numerical Toolkit (TNT)

*/

template <class T>
class DiagonalMatrix : public GeneralMatrix<T>
{
	//private:
	public:

    T* v_;				// Pointer to first diagonal element
	int n_;				// Size of DiagonalMatrix
    int len_;			// Size of underlying array (= n)
    int *RefCount_;		// Check for uneeded DiagonalMatrix objects
	T* ZeroPointer;		// Pointer to the value 0.0, used for (i,j) accessing of elements outside the blocks


	void Initialize_(int n);
    void Copy_(T* p, const T*  q, int len) const;
    void Set_(const T& val);
    void Destroy_();
	inline const T* Begin_() const;
	inline T* Begin_();

	public:

  	string Name;		// Name of the DiagonalMatrix

    typedef T value_type;

	       DiagonalMatrix();
	       DiagonalMatrix(int n);
	       DiagonalMatrix(int n,  T *a);
	       DiagonalMatrix(int n, const T &a);
	inline DiagonalMatrix(const DiagonalMatrix &A);
	inline DiagonalMatrix & operator=(const T &a);
	inline DiagonalMatrix & operator=(const DiagonalMatrix &A);
	inline DiagonalMatrix & Ref(const DiagonalMatrix &A);
	       DiagonalMatrix Copy() const;
	       DiagonalMatrix Conform() const;
		   DiagonalMatrix & Inject(const DiagonalMatrix & A);
	inline T& operator[](int i);
	inline const T& operator[](int i) const;
	inline T& operator()(int i);
	inline const T& operator()(int i) const;

	/*

	The code used to be this:

	inline T operator()(int i, int j);
	inline const T operator()(int i, int j) const;

	Hope that this doesn't cause problems elsewhere

	*/
	inline T& operator()(int i, int j);
	inline const T& operator()(int i, int j) const;

	inline const int Type() const;
	inline const int Dim() const;
	inline const int Rows() const;
	inline const int Cols() const;
	inline const int Length() const;
	inline const Vector<int> Size() const;
	inline const int RefCount() const;
	inline T*  Begin();
	inline T*  End();
               ~DiagonalMatrix();
};

/* Public Funcitons */

/*

Copy constructor

Array data is NOT copied, but shared.
Thus, in Matrix B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
Matrix B(A.copy()), or B = A.Copy(), instead.

*/

template <class T>
DiagonalMatrix<T>::DiagonalMatrix(const DiagonalMatrix<T> &A)
	: v_(A.v_), n_(A.n_), RefCount_(A.RefCount_), ZeroPointer(A.ZeroPointer), Name(A.Name)
{
	len_ = n_;
	(*RefCount_)++;
	//ZeroPointer = new T;
	//*ZeroPointer = 0;
}


/*

Constructor 1:

Create a new (n x n) DiagonlaMatrix, WITHOUT initializing array elements.
This version avoids the O(n) initialization overhead and
is used just before manual assignment.

@param n:	The dimension of the new DiagonalMatrix.

*/

template <class T>
DiagonalMatrix<T>::DiagonalMatrix(int n) : v_(0), n_(n), RefCount_(0), ZeroPointer(0)
{
	if(n > 0) Initialize_(n);
	Name = "NoName";
	len_ = n_;
	RefCount_ = new int;
	*RefCount_ = 1;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/*

Constructor 2:

Create a new (n x n) DiagonlaMatrix, initializing array elements to
constant specified by argument.

@param n:	The dimension of the new DiagonalMatrix.
@param val:	The constant value to set all elements of the new DiagonalMatrix to.

*/

template <class T>
DiagonalMatrix<T>::DiagonalMatrix(int n, const T &val) : v_(0), n_(n), RefCount_(0), ZeroPointer(0)
{
	if(n > 0) Initialize_(n);
	Set_(val);
	Name = "NoName";
	len_ = n_;
	RefCount_ = new int;
	*RefCount_ = 1;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/*

Constructor 3:

Create a new (n x n) DiagonlaMatrix, as a view of an existing one-dimensional
array stored. Note that the storage for this pre-existing array will
never be garbage collected by the DiagonalMatrix class.

@param n the second (column) dimension of the new Matrix.
@param a the one dimensional C array to use as data storage for the array.

*/

template <class T>
DiagonalMatrix<T>::DiagonalMatrix(int n, T *a) : v_(0), n_(n), RefCount_(0), ZeroPointer(0)
{
	T* p = a;
	Name = "NoName";
	len_ = n_;
	RefCount_ = new int;
	*RefCount_ = 2;		// This avoid destorying original data
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/* ... */
template <class T>
inline T& DiagonalMatrix<T>::operator[](int i)
{
	return v_[i];
}

/* ... */
template <class T>
inline const T& DiagonalMatrix<T>::operator[](int i) const
{
	return v_[i];
}

/* Access elements of the Matrix. Use _CheckBounds for bounds-checking */
template <class T>
inline T& DiagonalMatrix<T>::operator()(int i)
{
	#ifdef _CheckBounds
		if(i < 0 && i >= n_)
		{
			Print("Error in DiagonalMatrix");
			Print("(i) Out of Bounds");
			Print("Current Index",i );
			Print("High Index",n_-1);
			HaltExecution();
		}
	#endif

	return v_[i];
}

/* Access elements of the Matrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline const T& DiagonalMatrix<T>::operator()(int i) const
{
	#ifdef _CheckBounds
		if(i < 0 && i >= n_)
		{
			Print("Error in DiagonalMatrix");
			Print("(i) Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("n",n_);
			HaltExecution();
		}
	#endif

	return v_[i];
}

/* Access elements of the Matrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline T& DiagonalMatrix<T>::operator()(int i, int j)
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n_)
		{
			Print("Error in DiagonalMatrix");
			Print("Index Out of Bounds");
			Print("i",i);
			Print("n",n_);
			HaltExecution();
		}
	#endif

	if(i == j) return v_[i];
	else
	{
		*ZeroPointer = 0;
		return *ZeroPointer;
	}

}

/* Access elements of the Matrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline const T& DiagonalMatrix<T>::operator()(int i, int j) const
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n_)
		{
			Print("Error in DiagonalMatrix");
			Print("Index Out of Bounds");
			Print("i",i);
			Print("n",n_);
			HaltExecution();
		}
	#endif

	if(i == j)
	{
		return v_[i];
	}
	else
	{
		*ZeroPointer = 0;
		return *ZeroPointer;
	}
}

/* Assigns all elements of A to a constant scalar */
template <class T>
DiagonalMatrix<T> & DiagonalMatrix<T>::operator=(const T &a)
{
	Set_(a);
	return *this;
}

/*

Create a new copy of existing DiagonalMatrix.
Used in B = A.Copy() or in the construction
of B, e.g. DiagonalMatrix B(A.copy()), to create a
new DiagonalMatrix that does not share data.

*/
template <class T>
DiagonalMatrix<T> DiagonalMatrix<T>::Copy() const
{
	DiagonalMatrix A(n_);
	Copy_(A.Begin_(),Begin_(),len_);
	return A;
}

/*

Create a new DiagonalMatrix with same dimensions
as existing DiagonalMatrix.

*/

template <class T>
DiagonalMatrix<T> DiagonalMatrix<T>::Conform() const
{
	DiagonalMatrix A(n_);
	return A;
}


/*

Copy the elements to from one DiagonalMatrix to another,
in place. That is B.Inject(A), both A and B must conform
(i.e. have identical row and column dimensions).

This differs from B = A.Copy() in that references to B
before this assignment are also affected.  That is, if
we have

<pre>
DiagonalMatrix A(m,n);
DiagonalMatrix C(m,n);
DiagonalMatrix B(C);	// elements of B and C are shared.

<pre>
then B.Inject(A) affects both and C, while B = A.Copy() creates
a new array B which shares no data with C or A.

@param A the DiagonalMatrix from elements will be copied
@return an instance of the modifed array. That is, in B.inject(A),
it returns B.  If A and B are not conformat, no modifications to
B are made.

*/
template <class T>
DiagonalMatrix<T> & DiagonalMatrix<T>::Inject(const DiagonalMatrix &A)
{
	if(A.n_ == n_)
	Copy_(Begin_(),A.Begin_(),len_);
	return *this;
}

/*

Create a reference (shallow assignment) to another existing
DiagonalMatrix. In B.Ref(A), B and A shared the same data and
subsequent changes to the array elements of one will be
reflected in the other.

<p>
This is what operator= calls, and B = A and B.Ref(A) are equivalent
operations.

@return The new referenced array: in B.Ref(A), it returns B.

*/
template <class T>
DiagonalMatrix<T> & DiagonalMatrix<T>::Ref(const DiagonalMatrix<T> &A)
{
	if(this != &A)
	{
		(*RefCount_)--;
		if(*RefCount_ < 1) Destroy_();
		n_ = A.n_;
		len_ = A.len_;
		v_ = A.v_;
		Name = A.Name;
		RefCount_ = A.RefCount_;
		ZeroPointer = A.ZeroPointer;
		(*RefCount_)++ ;
	}
	return *this;
}

/* B = A is shorthand notation for B.Ref(A) */
template <class T>
DiagonalMatrix<T> & DiagonalMatrix<T>::operator=(const DiagonalMatrix<T> &A)
{
	return Ref(A);
}

/* Return the Type of the DiagonalMatrix ( = 2 ) */
template <class T>
inline const int DiagonalMatrix<T>::Type() const
{
	return MatrixType::DiagonalMatrix;
}

/* @return the number of rows in the Matrix */
template <class T>
inline const int DiagonalMatrix<T>::Dim() const { return n_; }

/* @return the number of rows in the Matrix */
template <class T>
inline const int DiagonalMatrix<T>::Rows() const { return n_; }

/* @return the number of columns in the Matrix */
template <class T>
inline const int DiagonalMatrix<T>::Cols() const { return n_; }

/* Return the Size of a Matrix in a 2-element Vector */
template <class T>
const inline Vector<int> DiagonalMatrix<T>::Size() const
{
	Vector<int> sz(2);
	sz[0] = n_;
	sz[1] = n_;
	return sz;
}

/* Return the length of the underlying array */
template <class T>
inline const int DiagonalMatrix<T>::Length() const { return len_; }

/*

@return the number of arrays that share the same storage area
as this one.  (Must be at least one.)

*/

template <class T>
inline int DiagonalMatrix<T>::RefCount() const
{
	return *RefCount_;
}

/*

Pointer to begining of array

*/

template <class T>
inline T* DiagonalMatrix<T>::Begin()
{
	return &v_[0];
}

/*

Pointer to end of array
(i.e., the first element that is not in the array)

*/

template <class T>
inline T* DiagonalMatrix<T>::End()
{
	return &v_[0] + len_;
}


/* Destructor */
template <class T>
DiagonalMatrix<T>::~DiagonalMatrix()
{
	(*RefCount_)--;
	if(*RefCount_ < 1) Destroy_();
}

/* Private internal functions */

/* Allocate memory for the Matrix */
template <class T>
void DiagonalMatrix<T>::Initialize_(int n)
{
	try
	{
		v_ = new T[n];
	}
	catch(bad_alloc ba)
	{
		Print("Error in DiagonalMatrix: Error Allocating Memory");
		Print("Size",n);
		Print("Memory per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",len_*sizeof(T)/1e6);
		Print("Halting Execution");
		exit(0);
	}
	n_ = n;
}

/* Set all the elements of a Matrix equal to a specified constant */
template <class T>
void DiagonalMatrix<T>::Set_(const T& a)
{
	T *begin = &v_[0];
	T *end = begin + n_;
	for(T* p=begin; p<end; p++) *p = a;
}

/* Copy the elements of one Matrix to another Matrix */
template <class T>
void DiagonalMatrix<T>::Copy_(T* p, const T* q, int len) const
{
	T *end = p + len;
	while(p < end) *p++ = *q++;
}

/* Delete the Matrix from memory */
template <class T>
void DiagonalMatrix<T>::Destroy_()
{
	if(v_ != 0) delete[] (v_);
	if(RefCount_ != 0) delete RefCount_;
	if(ZeroPointer != 0) delete ZeroPointer;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
const T* DiagonalMatrix<T>::Begin_() const { return &(v_[0]); }

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* DiagonalMatrix<T>::Begin_() { return &(v_[0]); }

/* Create a null (0x0) array */
template <class T>
DiagonalMatrix<T>::DiagonalMatrix() : v_(0), n_(0)
{
	len_ = 0;
	RefCount_ = new int;
	*RefCount_ = 1;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

#endif

