
#ifndef _Prob_DistBernoulli
#define _Prob_DistBernoulli



/* Function Prototypes */

	double BernoulliCdf     (double x,   double p                  );
	int    BernoulliCdfInv  (double cdf, double p                  );
	bool   BernoulliCheck   (            double p                  );
	double BernoulliMean    (            double p                  );
	int    BernoulliSample  (            double p, long &RandomSeed);
	double BernoulliVariance(            double p                  );


/*

Purpose:

	BERNOULLI_CDF evaluates the Bernoulli CDF.
	
Modified:

	22 May 2004

Author:

	John Burkardt
	
Parameters:

	Input, int X, the number of successes on a single trial.
	X = 0 or 1.
	
	Input, double A, the probability of success on one trial.
	0.0 <= A <= 1.0.
	
	Output, double BERNOULLI_CDF, the value of the CDF.
	
Modified by MP

*/

double BernoulliCdf(double x, double p)
{
	double cdf;
	if(x < Zero)     cdf = Zero;
	else if(x < One) cdf = One - p;
	else             cdf = One;
	return cdf;
}


/*

Purpose:

	BERNOULLI_CDF_INV inverts the Bernoulli CDF.
	
Modified:

	22 May 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double CDF, the value of the CDF.
	0.0 <= CDF <= 1.0.
	
	Input, double A, the parameter of the PDF.
	0.0 <= A <= 1.0.
	
	Output, int BERNOULLI_CDF_INV, the corresponding argument.

*/

int BernoulliCdfInv(double cdf, double p)
{

	if(cdf < Zero || cdf > One)
	{
		Print("BERNOULLI_CDF_INV - Fatal error!");
		Print(" CDF < 0 or 1 < CDF.");
		HaltExecution();
	}

	int x;	
	if(cdf <= One - p) x = 0;
	else               x = 1;
	return x;
}


/*

Purpose:

	BERNOULLI_CHECK checks the parameter of the Bernoulli CDF.
	
Modified:

	22 May 2004
	
Author:

	John Burkardt

Parameters:

	Input, double A, the parameter of the PDF.
	0.0 <= A <= 1.0.
	
	Output, bool BERNOULLI_CHECK, is TRUE if the data is acceptable.
	
*/

bool BernoulliCheck(double p)
{
	if(p < Zero || p > One) return false;
	else                    return true;
}


/*

Purpose:

	BERNOULLI_MEAN returns the mean of the Bernoulli PDF.
	
Modified:

	22 May 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, the probability of success.
	0.0 <= A <= 1.0.
	
	Output, double BERNOULLI_MEAN, the mean of the PDF.

*/

double BernoulliMean(double p)
{
	return p;
}


/*

Purpose:

	BERNOULLI_PDF evaluates the Bernoulli PDF.
	
Formula:

	PDF(X)(A) = A**X * ( 1.0 - A )**( X - 1 )
	
	X = 0 or 1.
	
Discussion:

	The Bernoulli PDF describes the simple case in which a single trial
	is carried out, with two possible outcomes, called "success" and
	"failure"; the probability of success is A.
	
Modified:

	22 May 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, int X, the number of successes on a single trial.
	X = 0 or 1.
	
	Input, double A, the probability of success on one trial.
	0.0 <= A <= 1.0.
	
	Output, double BERNOULLI_PDF, the value of the PDF.

*/

/*
double BernoulliPdf(int x, double a)
{

	double pdf;
	
	if(x < 0)
	{
		pdf = 0.0;
	}
	else if(x == 0)
	{
		pdf = 1.0 - a;
	}
	else if(x == 1)
	{
		pdf = a;
	}
	else
	{
		pdf = 0.0;
	}
	
	return pdf;
}
*/

/*

Purpose:

	BERNOULLI_SAMPLE samples the Bernoulli PDF.
	
Modified:

	22 May 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, the probability of success on one trial.
	0.0 <= A <= 1.0.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, int BERNOULLI_SAMPLE, a sample of the PDF.
	
*/

int BernoulliSample(double p, long &RandomSeed)
{
	return BernoulliCdfInv(UniformSample(RandomSeed),p);
}


/*

Purpose:

	BERNOULLI_VARIANCE returns the variance of the Bernoulli PDF.
	
Modified:

	22 May 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, the probability of success on one trial.
	0.0 <= A <= 1.0.
	
	Output, double BERNOULLI_VARIANCE, the variance of the PDF.
	
*/

double BernoulliVariance(double p)
{
	return p * (One - p);
}


#endif