
#ifndef _Prob_DistBeta
#define _Prob_DistBeta


	double BetaFunc(double x, double y);
	double BetaInc(double a, double b, double x);
	
	double BetaCdf(double x, double a, double b);
	double BetaCdfInv(double cdf, double a, double b);
	bool BetaCheck(double a, double b);
	void BetaCdfValues(int *n_data, double *a, double *b, double *x, double *fx);
	void BetaIncValues(int *n_data, double *a, double *b, double *x, double *fx);
	double BetaMean(double a, double b);
	double BetaPdf(double x, double a, double b);
	double BetaSample(double a, double b, long &RandomSeed);
	double BetaVariance(double a, double b);

/*

Purpose:

	BETA returns the value of the Beta function.
	
Formula:

	BETA(X,Y) = ( GAMMA(X) * GAMMA(Y) ) / GAMMA(X+Y)
	
Properties:

	BETA(X,Y) = BETA(Y,X).
	BETA(X,Y) = Integral ( 0 <= T <= 1 ) T**(X-1) (1-T)**(Y-1) dT.
	BETA(X,Y) = GAMMA(X) * GAMMA(Y) / GAMMA(X+Y)
	
Restrictions:

	Both X and Y must be greater than 0.
	
Modified:

	09 May 2003
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, Y, the two parameters that define the Beta function.
	X and Y must be greater than 0.
	
	Output, double BETA, the value of the Beta function.
	
*/

double BetaFunc(double x, double y)
{
	if(x <= Zero || y <= Zero)
	{
		Print("BETA - Fatal error!");
		Print("Both X and Y must be greater than 0.");
		HaltExecution();
	}
	
	return (exp(gamma_log(x) + gamma_log(y) - gamma_log(x + y)));
}


/*

Purpose:
	
	BETA_INC returns the value of the incomplete Beta function.
	
Discussion:

	This calculation requires an iteration.  In some cases, the iteration
	may not converge rapidly, or may become inaccurate.
	
Formula:

	BETA_INC(A,B,X)
	
		=   Integral ( 0 <= T <= X ) T**(A-1) (1-T)**(B-1) dT
          / Integral ( 0 <= T <= 1 ) T**(A-1) (1-T)**(B-1) dT

		=   Integral ( 0 <= T <= X ) T**(A-1) (1-T)**(B-1) dT
          / BETA(A,B)
          
Modified:

	10 October 2004
	
Reference:

	Majumder and Bhattacharjee,
	Algorithm AS63,
	Applied Statistics,
	1973, volume 22, number 3.
	
Parameters:

	Input, double A, B, the parameters of the function.
	0.0 < A,
	0.0 < B.
	
	Input, double X, the argument of the function.
	Normally, 0.0 <= X <= 1.0.
	
	Output, double BETA_INC, the value of the function.
	
*/

double BetaInc(double a, double b, double x)
{
	double cx;
	int i;
	int it;
	int it_max = 1000;
	bool indx;
	int ns;
	double pp;
	double psq;
	double qq;
	double rx;
	double temp;
	double term;
	double tol = 1.0E-07;
	double value;
	double xx;
	
	if(a <= Zero)
	{
		Print("BETA_INC - Fatal error");
		Print("A <= 0.");
		HaltExecution();
	}
	
	if(b <= Zero)
	{
		Print("BETA_INC - Fatal error!");
		Print("B <= 0.");
		HaltExecution();
	}
	
	if(x <= Zero)
	{
		value = Zero;
		return value;
	}
	else if(One <= x)
	{
		value = One;
		return value;
	}
	
	// Change tail if necessary and determine S.
	
	psq = a + b;
	
	if(a < (a + b) * x)
	{
		xx = One - x;
		cx = x;
		pp = b;
		qq = a;
		indx = true;
	}
	else
	{
		xx = x;
		cx = One - x;
		pp = a;
		qq = b;
		indx = false;
	}
	
	term = One;
	i = 1;
	value = One;
	
	ns = (int)(qq + cx * (a + b));

	// Use Soper's reduction formulas.
	
	rx = xx / cx;
	
	temp = qq - (double)i;
	
	if(ns == 0)
	{
		rx = xx;
	}
	
	it = 0;
	
	for(;;)
	{
		it = it + 1;
		
		if(it_max < it)
		{
			Print("BETA_INC - Fatal error!");
			Print("Maximum number of iterations exceeded!");
			Print(" IT_MAX = " + ToString(it_max));
			Print("a",a);
			Print("b",b);
			Print("x",x);
			HaltExecution();
		}

		term = term * temp * rx / (pp + (double)(i));
		value = value + term;
		temp = fabs(term);
		
		if(temp <= tol && temp <= tol * value)
		{
			break;
		}
		
		i = i + 1;
		ns = ns - 1;
		
		if(0 <= ns)
		{
			temp = qq - (double)i;
			if(ns == 0)
			{
				rx = xx;
			}
		}
		else
		{
			temp = psq;
			psq = psq + One;
		}
	}
	
	/* Finish calculation. */
	
	value = value * exp(pp * log(xx) + (qq - One) * log(cx)) / (BetaFunc(a,b) * pp);
	if(indx) value = One - value;	
	return value;
}


/*

Purpose:

	BETA_CDF evaluates the Beta CDF.
	
Modified:

	10 October 2004

Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the CDF.
	
	Input, double A, B, the parameters of the PDF.
	0.0 < A,
	0.0 < B.
	
	Output, double BETA_CDF, the value of the CDF.
	
*/

double BetaCdf(double x, double a, double b)
{
	double cdf;
	    if(x <= Zero) cdf = Zero;
	else if(x <= One) cdf = BetaInc(a,b,x);
	else              cdf = One;
	return cdf;
}


/*

Purpose:

	BETA_CDF_INV inverts the Beta CDF.
	
Modified:

	22 October 2004
	
Reference:

	Abernathy and Smith,
	Algorithm 724,
	ACM Transactions on Mathematical Software,
	Volume 19, Number 4, December 1993, pages 481-483.
	
Parameters:

	Input, double CDF, the value of the CDF.
	0.0 <= CDF <= 1.0.
	
	Input, double A, B, the parameters of the PDF.
	0.0 < A,
	0.0 < B.
	
	Output, double BETA_CDF_INV, the argument of the CDF.

*/

double BetaCdfInv(double cdf, double a, double b)
{

	#define MAXK 20

  double bcoeff;
  double cdf_x;
  double d[MAXK * (MAXK-1)];
  double error = 0.0001;
  double errapp = 0.01;
  int i;
  int j;
  int k;
  int loopct;
  double pdf_x;
  double q;
  double s1;
  double s2;
  double sum2;
  double t;
  double tail;
  double x;
  double xold;

  if(cdf < 0.0 || 1.0 < cdf)
  {
    Print("BETA_CDF_INV - Fatal error!");
    Print("CDF < 0 or 1 < CDF.");
    HaltExecution();
  }
  
	/* Estimate the solution. */
	
	x = a / ( a + b );
	
	xold = 0.0;
	loopct = 2;
	
	while(errapp <= fabs((x - xold) / x) && loopct != 0)
	{
		xold = x;
		loopct = loopct - 1;
		
		/*
		CDF_X = PROB { BETA(A,B) <= X }.
		Q = ( CDF - CDF_X ) / PDF_X.
		*/
		
		cdf_x = BetaCdf ( x, a, b );
		
		pdf_x = BetaPdf ( x, a, b );
		
		q = ( cdf - cdf_x ) / pdf_x;
		
		/* D(N,K) = C(N,K) * Q**(N+K-1) / (N-1)! */
		
		t = 1.0 - x;
		s1 = q * ( b - 1.0 ) / t;
		s2 = q * ( 1.0 - a ) / x;
		d[2-1+0*MAXK] = s1 + s2;
		tail = d[2-1+0*MAXK] * q / 2.0;
		x = x + q + tail;
		
		k = 3;
		
		while(error < fabs(tail / x) && k <= MAXK)
		{
			
			/* Find D(2,K-2). */
			
			s1 = q * ( ( double ) ( k ) - 2.0 ) * s1 / t;
			s2 = q * ( 2.0 - ( double ) ( k ) ) * s2 / x;
			d[2-1+(k-2)*MAXK] = s1 + s2;
			
			/* Find D(3,K-3), D(4,K-4), D(5,K-5), ... , D(K-1,1). */
			
			for(i=3; i<=k-1; i++)
			{
				sum2 = d[2-1+0*MAXK] * d[i-2+(k-i)*MAXK];
				bcoeff = 1.0;
				
				for(j=1; j<=k-i; j++)
				{
					bcoeff = (bcoeff * (double)(k - i - j + 1)) / (double)(j);
  					sum2 = sum2 + bcoeff * d[2-1+j*MAXK] * d[i-2+(k-i-j)*MAXK];
				}
				d[i-1+(k-i)*MAXK] = sum2 + d[i-2+(k-i+1)*MAXK] / (double)(i - 1);
			}
			
			/* Compute D(K,0) and use it to expand the series. */
			
			d[k-1+0*MAXK] = d[2-1+0*MAXK] * d[k-2+0*MAXK] + d[k-2+1*MAXK] / (double)(k - 1);
 			tail = d[k-1+0*MAXK] * q / (double)(k);
 			x = x + tail;
 			
 			/* Check for divergence. */
 			
 			if(x <= 0.0 || 1.0 <= x)
 			{
 				Print("BETA_CDF_INV - Fatal error!");
 				Print("The series has diverged.");
 				x = -1.0;
 				return x;
			}
			k = k + 1;
		}
	}
	return x;
	#undef MAXK
}


/*

Purpose:
	
	BETA_CHECK checks the parameters of the Beta PDF.
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < A,
	0.0 < B.
	
	Output, bool BETA_CHECK, is true if the parameters are legal.
	
*/

bool BetaCheck(double a, double b)
{

	if(a <= Zero)
	{
		Print("BETA_CHECK - Fatal error!");
		Print("  A <= 0.");
		return false;
	}
	
	if(b <= Zero)
	{
		Print("BETA_CHECK - Fatal error!");
		Print("  B <= 0.");
		return false;
	}
	
	return true;
}


/*

Purpose:

	BETA_CDF_VALUES returns some values of the Beta CDF.
	
Discussion:

	In Mathematica, the function can be evaluated by:
	Needs["Statistics`ContinuousDistributions`"]
	dist = BetaDistribution [ a, b ]
	CDF [ dist, x ]
	
Modified:

	02 September 2004
	
Author:

	John Burkardt
	
Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Karl Pearson,
	Tables of the Incomplete Beta Function,
	Cambridge University Press, 1968.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.
	
Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, double *A, B, the parameters of the function.
	
	Output, double *X, the argument of the function.
	
	Output, double *FX, the value of the function.
	
*/

void BetaCdfValues(int *n_data, double *a, double *b, double *x, double *fx)
{
	#define N_MAX 12

  double a_vec[N_MAX] = {
      0.10E+01,
      0.10E+01,
      0.10E+01,
      0.10E+01,
      0.10E+01,
      0.10E+01,
      0.10E+01,
      0.10E+01,
      0.20E+01,
      0.30E+01,
      0.40E+01,
      0.50E+01 };

  double b_vec[N_MAX] = {
      0.50E+00,
      0.50E+00,
      0.50E+00,
      0.50E+00,
      0.20E+01,
      0.30E+01,
      0.40E+01,
      0.50E+01,
      0.20E+01,
      0.20E+01,
      0.20E+01,
      0.20E+01 };

  double fx_vec[N_MAX] = {
     0.5131670194948620E-01,
     0.1055728090000841E+00,
     0.1633399734659245E+00,
     0.2254033307585166E+00,
     0.3600000000000000E+00,
     0.4880000000000000E+00,
     0.5904000000000000E+00,
     0.6723200000000000E+00,
     0.2160000000000000E+00,
     0.8370000000000000E-01,
     0.3078000000000000E-01,
     0.1093500000000000E-01 };

  double x_vec[N_MAX] = {
     0.10E+00,
     0.20E+00,
     0.30E+00,
     0.40E+00,
     0.20E+00,
     0.20E+00,
     0.20E+00,
     0.20E+00,
     0.30E+00,
     0.30E+00,
     0.30E+00,
     0.30E+00 };

  if ( *n_data < 0 )
  {
    *n_data = 0;
  }

  *n_data = *n_data + 1;

  if ( N_MAX < *n_data )
  {
    *n_data = 0;
    *a = 0.0;
    *b = 0.0;
    *x = 0.0;
    *fx = 0.0;
  }
  else
  {
    *a = a_vec[*n_data-1];
    *b = b_vec[*n_data-1];
    *x = x_vec[*n_data-1];
    *fx = fx_vec[*n_data-1];
  }

	return;
	#undef N_MAX
}


/*

Purpose:

	BETA_INC_VALUES returns some values of the incomplete Beta function.
	
Discussion:

	The incomplete Beta function may be written
	
		BETA_INC(A,B,X) = Integral (0 to X) T**(A-1) * (1-T)**(B-1) dT
		                / Integral (0 to 1) T**(A-1) * (1-T)**(B-1) dT
		                
	Thus,
	
		BETA_INC(A,B,0.0) = 0.0;
		BETA_INC(A,B,1.0) = 1.0
		
	In Mathematica, the function can be evaluated by:
	
		BETA[X,A,B] / BETA[A,B]
		
Modified:

	09 June 2004
	
Author:

	John Burkardt
	
Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Karl Pearson,
	Tables of the Incomplete Beta Function,
	Cambridge University Press, 1968.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.
	
Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, double *A, B, the parameters of the function.
	
	Output, double *X, the argument of the function.
	
	Output, double *FX, the value of the function.
	
*/

void BetaIncValues(int *n_data, double *a, double *b, double *x, double *fx)
{
	#define N_MAX 30

  double a_vec[N_MAX] = {
      0.5E+00,
      0.5E+00,
      0.5E+00,
      1.0E+00,
      1.0E+00,
      1.0E+00,
      1.0E+00,
      1.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      5.5E+00,
     10.0E+00,
     10.0E+00,
     10.0E+00,
     10.0E+00,
     20.0E+00,
     20.0E+00,
     20.0E+00,
     20.0E+00,
     20.0E+00,
     30.0E+00,
     30.0E+00,
     40.0E+00 };

  double b_vec[N_MAX] = {
      0.5E+00,
      0.5E+00,
      0.5E+00,
      0.5E+00,
      0.5E+00,
      0.5E+00,
      0.5E+00,
      1.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      2.0E+00,
      5.0E+00,
      0.5E+00,
      5.0E+00,
      5.0E+00,
     10.0E+00,
      5.0E+00,
     10.0E+00,
     10.0E+00,
     20.0E+00,
     20.0E+00,
     10.0E+00,
     10.0E+00,
     20.0E+00 };

  double fx_vec[N_MAX] = {
     0.6376856085851985E-01,
     0.2048327646991335E+00,
     0.1000000000000000E+01,
     0.0000000000000000E+00,
     0.5012562893380045E-02,
     0.5131670194948620E-01,
     0.2928932188134525E+00,
     0.5000000000000000E+00,
     0.2800000000000000E-01,
     0.1040000000000000E+00,
     0.2160000000000000E+00,
     0.3520000000000000E+00,
     0.5000000000000000E+00,
     0.6480000000000000E+00,
     0.7840000000000000E+00,
     0.8960000000000000E+00,
     0.9720000000000000E+00,
     0.4361908850559777E+00,
     0.1516409096347099E+00,
     0.8978271484375000E-01,
     0.1000000000000000E+01,
     0.5000000000000000E+00,
     0.4598773297575791E+00,
     0.2146816102371739E+00,
     0.9507364826957875E+00,
     0.5000000000000000E+00,
     0.8979413687105918E+00,
     0.2241297491808366E+00,
     0.7586405487192086E+00,
     0.7001783247477069E+00 };

  double x_vec[N_MAX] = {
     0.01E+00,
     0.10E+00,
     1.00E+00,
     0.00E+00,
     0.01E+00,
     0.10E+00,
     0.50E+00,
     0.50E+00,
     0.10E+00,
     0.20E+00,
     0.30E+00,
     0.40E+00,
     0.50E+00,
     0.60E+00,
     0.70E+00,
     0.80E+00,
     0.90E+00,
     0.50E+00,
     0.90E+00,
     0.50E+00,
     1.00E+00,
     0.50E+00,
     0.80E+00,
     0.60E+00,
     0.80E+00,
     0.50E+00,
     0.60E+00,
     0.70E+00,
     0.80E+00,
     0.70E+00 };

  if ( *n_data < 0 )
  {
    *n_data = 0;
  }

  *n_data = *n_data + 1;

  if ( N_MAX < *n_data )
  {
    *n_data = 0;
    *a = 0.0;
    *b = 0.0;
    *x = 0.0;
    *fx = 0.0;
  }
  else
  {
    *a = a_vec[*n_data-1];
    *b = b_vec[*n_data-1];
    *x = x_vec[*n_data-1];
    *fx = fx_vec[*n_data-1];
  }

  	return;
	#undef N_MAX
}


/*

Purpose:

	BETA_MEAN returns the mean of the Beta PDF.
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < A,
	0.0 < B.
	
	Output, double BETA_MEAN, the mean of the PDF.
	
*/

double BetaMean(double a, double b) 
{
	double mean;
	mean = a / ( a + b );
	return mean;
}


/*

Purpose:

	BetaPdf evaluates the Beta PDF.
	
Formula:

	PDF(A,B;X) = X**(A-1) * (1-X)**(B-1) / BETA(A,B).
	
Discussion:

	A = B = 1 yields the Uniform distribution on [0,1].
	A = B = 1/2 yields the Arcsin distribution.
	    B = 1 yields the power function distribution.
	A = B -> Infinity tends to the Normal distribution.
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	0.0 <= X <= 1.0.
	
	Input, double A, B, the parameters of the PDF.
	0.0 < A,
	0.0 < B.
	
	Output, double BetaPdf, the value of the PDF.
	
*/

double BetaPdf(double x, double Alpha, double Beta)
{
	if(x < Zero || One < x) return Zero;
	else                    return pow(x,(Alpha - One)) * pow((One - x),(Beta - One)) / BetaFunc(Alpha,Beta);
}


/*

Purpose:

	BETA_SAMPLE samples the Beta PDF.
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Reference:

	William Kennedy and James Gentle,
	Algorithm BN,
	Statistical Computing,
	Dekker, 1980.

Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < A,
	0.0 < B.
	
	Input/output, int SEED, a seed for the random number generator.
	
	Output, double BETA_SAMPLE, a sample of the PDF.
	
*/

double BetaSample(double a, double b, long &RandomSeed)
{

	double mu;
	double stdev;
	double test;
	double u;
	double x;
	double y;
	
	mu = (a - One) / (a + b - Two);
	stdev = Half / sqrt (a + b - Two);

	for(;;)
	{
	
		y = NormalSample(RandomSeed);
		
		x = mu + stdev * y;

    	if(x < Zero || x > One) continue;

		u = UniformSample(RandomSeed);
		
		test =     (a - 1.0)     * log(       x  / (a - 1.0))
                 + (b - 1.0)     * log((1.0 - x) / (b - 1.0))
                 + (a + b - 2.0) * log(a + b - 2.0) + 0.5 * y * y;

		if(log(u) <= test) break;
    
    }
    
    return x;
}


/*

Purpose:

	BETA_VARIANCE returns the variance of the Beta PDF.
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < A,
	0.0 < B.
	
	Output, double BETA_VARIANCE, the variance of the PDF.
	
*/

double BetaVariance(double a, double b)
{
	double variance;
	variance = (a * b) / ((a + b) * (a + b) * (One + a + b));
	return variance;
}

#endif
