
#ifndef _Prob_DistBinomial
#define _Prob_DistBinomial


	/* Function Prototypes */
	
	double BinomialCdf(double x, int a, double b);
	void BinomialCdfValues(int *n_data, int *a, double *b, int *x, double *fx);
	int BinomialCdfInv(double cdf, int a, double b);
	bool BinomialCheck(int a, double b);
	double BinomialMean(int a, double b);
	double BinomialPdf(int x, int a, double b);
	int BinomialSample(int a, double b, long &RandomSeed);
	double BinomialVariance(int a, double b);
	
	//int BinomialCoefInt(int n, int k);
	double BinomialCoef(int n, int k);
	double BinomialCoefLog(int n, int k);
	

/*

Purpose:

	BinomialCdf evaluates the Binomial CDF.
	
Definition:

	CDF(X)(A,B) is the probability of at most X successes in A trials,
	given that the probability of success on a single trial is B.
	
Discussion:

	A sequence of trials with fixed probability of success on
	any trial is known as a sequence of Bernoulli trials.
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, int X, the desired number of successes.
	0 <= X <= N.
	
	Input, int N, the number of trials.
	N >= 1.
	
	Input, double p, the probability of success on one trial.
	0.0 <= p <= 1.0.
	
	Output, double CDF, the value of the CDF.
	
*/

double BinomialCdf(double x, int N, double p)
{
			
	     if(x < Zero)   return Zero;
	else if(N <= x)     return One;
	else if(p == Zero) return One;
	else if(p == One)  return Zero;
	else
	{
		double Cdf = Zero;
		for(int j=0; j<=x; j++)
		{
			int cnk = BinomialCoef(N,j);
			double pr = (double)(cnk) * pow(p,j) * pow((One - p),(N - j));
			Cdf += pr;
		}
		return Cdf;
	}

}


/*

Purpose:

	BinomialCdfValues returns some values of the binomial CDF.
	
Discussion:

	CDF(X)(A,B) is the probability of at most X successes in A trials,
	given that the probability of success on a single trial is B.
	
	In Mathematica, the function can be evaluated by:
	Needs["Statistics`DiscreteDistributions`]
	dist = BinomialDistribution [ n, p ]
	CDF [ dist, x ]
	
Modified:

	15 August 2004

Author:

	John Burkardt
	
Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.
	
	Daniel Zwillinger,
	CRC Standard Mathematical Tables and Formulae,
	30th Edition, CRC Press, 1996, pages 651-652.
	
Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, int *A, a parameter of the function.
	
	Output, double *B, a parameter of the function.
	
	Output, int *X, the argument of the function.
	
	Output, double *FX, the value of the function.
	
*/

void BinomialCdfValues(int *n_data, int *a, double *b, int *x, double *fx)
{
	#define N_MAX 17

	int a_vec[N_MAX] = { 2, 2, 2, 2, 2, 4, 4, 4, 4, 10, 10, 10, 10, 10, 10, 10, 10 };

	double b_vec[N_MAX] = { 0.05E+00, 0.05E+00, 0.05E+00, 0.50E+00, 0.50E+00, 0.25E+00, 0.25E+00,
	                        0.25E+00, 0.25E+00, 0.05E+00, 0.10E+00, 0.15E+00, 0.20E+00, 0.25E+00,
	                        0.30E+00, 0.40E+00, 0.50E+00  };

	double fx_vec[N_MAX] = { 0.9025000000000000E+00, 0.9975000000000000E+00, 0.1000000000000000E+01,
	                         0.2500000000000000E+00, 0.7500000000000000E+00, 0.3164062500000000E+00,
	                         0.7382812500000000E+00, 0.9492187500000000E+00, 0.9960937500000000E+00,
	                         0.9999363101685547E+00, 0.9983650626000000E+00, 0.9901259090013672E+00,
	                         0.9672065024000000E+00, 0.9218730926513672E+00, 0.8497316674000000E+00,
	                         0.6331032576000000E+00, 0.3769531250000000E+00 };

	int x_vec[N_MAX] = { 0, 1, 2, 0, 1, 0, 1, 2, 3, 4, 4, 4, 4, 4, 4, 4, 4 };

	if(*n_data < 0) *n_data = 0;

	*n_data = *n_data + 1;
	
	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*a = 0;
		*b = Zero;
		*x = 0;
		*fx = Zero;
	}
	else
	{
		*a = a_vec[*n_data-1];
		*b = b_vec[*n_data-1];
		*x = x_vec[*n_data-1];
		*fx = fx_vec[*n_data-1];
	}
	
	return;
	#undef N_MAX
}


/*

Purpose:

	BinomialCdfInv inverts the Binomial CDF.

Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double Cdf, the value of the CDF.
	0.0 <= Cdf <= 1.0.
	
	Input, int N, the number of trials.
	N >= 1.
	
	Input, double p, the probability of success on one trial.
	0.0 <= p <= 1.0.
		
	Output, int BinomialCdfInv, the corresponding argument.
	
*/

int BinomialCdfInv(double Cdf, int N, double p)
{
	
	if(Cdf < Zero || Cdf > One)
	{
    	Print("Error in BinomialCdfInv");
    	Print("CDF < 0 or CDF > 1");
		HaltExecution();
	}

	double Cdf2 = Zero;
	for(int x=0; x<=N; x++)
	{
		double Pdf = BinomialPdf(x,N,p);
		Cdf2 += Pdf;
		if(Cdf <= Cdf2) return x;
	}

	return 0;
}

/*

Purpose:

	BinomialCheck checks the parameter of the Binomial PDF.
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, int N, the number of trials.
	N >= 1.
	
	Input, double p, the probability of success on one trial.
	0.0 <= p <= 1.0.
	
	Output, bool BinomialCheck, is true if the parameters are legal.
	
*/

bool BinomialCheck(int N, double p)
{
	if(N < 1) return false;
	if(p < Zero || p > One) return false;
	return true;
}


/*

Purpose:

	BinomialCoef computes the Binomial coefficient C(N,K).
	
Discussion:

	The value is calculated in such a way as to avoid overflow and
	roundoff.  The calculation is done in integer arithmetic.
	
Formula:

	CNK = C(N,K) = N! / ( K! * (N-K)! )
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Reference:

	M L Wolfson and H V Wright,
	Combinatorial of M Things Taken N at a Time,
	ACM Algorithm 160,
	Communications of the ACM,
	April, 1963.
	
Parameters:

	Input, int N, K, are the values of N and K.
	
	Output, int CNK, the number of combinations of N
	things taken K at a time.
	
*/

double BinomialCoef(int n, int k)
{

	int mn = Min(k,n-k);
	
	if(mn < 0)
	{
		return 0;
	}
	else if(mn == 0)
	{
		return 1;
	}
	else
	{
		int mx = Max(k,n-k);
		double cnk = mx + 1;
		for(int i=2; i<=mn; i++) cnk = (cnk * (mx + i)) / i;
		return cnk;
	}
}

/*

Burkardt's code does not seem to work.
Here is my own code

e.g. it computes bc(100,8) as a negative number

*/

/*
double BinomialCoef(int n, int k)
{
	double logN = Zero;
	int j = 1;
	for(int i=1; i<=k; i++)
	{
		logN += log(j)-log(i);
		j++;
	}
	for(int i=1; i<=n-k; i++)
	{
		logN += log(j)-log(i);
		j++;
	}
	return exp(logN);
}
*/

/*

Purpose:
	
	BinomialCoefLog computes the logarithm of the Binomial coefficient.
	
Formula:

	CNK_LOG = LOG ( C(N,K) ) = LOG ( N! / ( K! * (N-K)! ) ).
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, int N, K, are the values of N and K.
	
	Output, double CNK_LOG, the logarithm of C(N,K).
	
*/

double BinomialCoefLog(int n, int k)
{
	double cnk_log;
	cnk_log = factorial_log(n) - factorial_log(k) - factorial_log(n-k);
	return cnk_log;
}


/*

Purpose:

	BinomialMean returns the mean of the Binomial PDF.
	
Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, int N, the number of trials.
	N >= 1.
	
	Input, double p, the probability of success on one trial.
	0.0 <= p <= 1.0.
	
	Output, double BinomialMean, the expected value of the number of
	successes in A trials.

*/

double BinomialMean(int N, double p)
{
	return (double)(N) * p;
}


/*

Purpose:

	BinomialPdf evaluates the Binomial PDF.
	
Definition:

	PDF(A,B;X) is the probability of exactly X successes in A trials,
	given that the probability of success on a single trial is B.
	
Formula:

	PDF(A,B;X) = C(N,X) * B**X * ( 1.0 - B )**( A - X )
	
Discussion:

	BinomialPdf(1,B;X) = Bernoulli_PDF(B;X).
	
Modified:

	10 October 2004

Author:

	John Burkardt
	
Parameters:

	Input, int X, the desired number of successes.
	0 <= X <= N.
	
	Input, int N, the number of trials.
	N >= 1.
	
	Input, double p, the probability of success on one trial.
	0.0 <= p <= 1.0.
	
	Output, double BinomialPdf, the value of the PDF.
	
*/

double BinomialPdf(int x, int N, double p)
{

	if(N < 1)
	{
		return Zero;
	}
	else if(x < 0 || x > N)
	{
		return Zero;
	}
	else if(p == Zero)
	{
		if(x == 0) return One;
		else       return Zero;
	}
	else if(p == One)
	{
		if(x == N) return One;
		else       return Zero;
	}
	else
	{
		double cnk = BinomialCoef(N,x);
		return (double)(cnk) * pow(p,x) * pow((One - p),(N - x));
 	}

}


/*

Purpose:

	BinomialSample samples the Binomial PDF.

Modified:

	10 October 2004
	
Author:

	John Burkardt
	
Reference:

	William Kennedy and James Gentle,
	Algorithm BU,
	Statistical Computing,
	Dekker, 1980.
	
Parameters:

	Input, int N, the number of trials.
	N >= 1.
	
	Input, double p, the probability of success on one trial.
	0.0 <= p <= 1.0.
	
	Input/output, long RandomSeed, a seed for the random number generator.
	
	Output, int BinomialSample, a sample of the PDF.
	
*/

int BinomialSample(int N, double p, long &RandomSeed)
{
	int x = 0;
	for(int i=1; i<=N; i++)
	{
		double u = UniformSample(RandomSeed);
		if(u <= p) x++;
	}
	
	return x;
}


/*
 *  Mathlib : A C Library of Special Functions
 *  Copyright (C) 1998 Ross Ihaka
 *  Copyright (C) 2000-2002 The R Development Core Team
 *  Copyright (C) 2007 The R Foundation
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, a copy is available at
 *  http://www.r-project.org/Licenses/
 *
 *  SYNOPSIS
 *
 *	#include <Rmath.h>
 *	double rbinom(double nin, double pp)
 *
 *  DESCRIPTION
 *
 *	Random variates from the binomial distribution.
 *
 *  REFERENCE
 *
 *	Kachitvichyanukul, V. and Schmeiser, B. W. (1988).
 *	Binomial random variate generation.
 *	Communications of the ACM 31, 216-222.
 *	(Algorithm BTPEC).
 */


int BinomialSample2(int N, double p, long &RandomSeed)
{
	return (int)rbinom(N,p,RandomSeed);
}


/*

Purpose:
	
	BinomialVariance returns the variance of the Binomial PDF.
	
Modified:

	28 January 1999
	
Author:

	John Burkardt
	
Parameters:

	Input, int N, the number of trials.
	N >= 1.
	
	Input, double p, the probability of success on one trial.
	0.0 <= p <= 1.0.
	
	Output, double BinomialVariance, the variance of the PDF.

*/

double BinomialVariance(int N, double p)
{
	return (double)(N) * p * (One - p);
}


#endif
