
#ifndef _DistExponential
#define _DistExponential


	double ExponentialCdf     (double x);
	double ExponentialCdfInv  (double cdf);
	double ExponentialMean    ();
	double ExponentialPdf     (double x);
	double ExponentialSample  (long &RandomSeed);
	double ExponentialVariance();
	
	double ExponentialCdf      (double x,   double a, double b);
	double ExponentialCdfInv   (double cdf, double a, double b);
	void   ExponentialCdfValues(int *n_data, double *lambda, double *x, double *fx);
	//bool   ExponentialCheck    (            double a, double b);
	double ExponentialMean     (            double a, double b);
	double ExponentialPdf      (double x,   double a, double b);
	double ExponentialSample   (            double a, double b, long &RandomSeed);
	//double ExponentialVariance (            double a, double b);


/*

Purpose:

	EXPONENTIAL_01_CDF evaluates the Exponential 01 CDF.
	
Modified:

	15 October 2004

Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	
	Output, double EXPONENTIAL_01_CDF, the value of the CDF.

*/

double ExponentialCdf(double x)
{
	double cdf;
	if(x <= Zero) cdf = Zero;
	else		  cdf = One - exp(-x);
	return cdf;
}


/*

Purpose:

	EXPONENTIAL_01_CDF_INV inverts the Exponential 01 CDF.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double CDF, the value of the CDF.
	0.0 <= CDF <= 1.0.
	
	Output, double EXPONENTIAL_CDF_INV, the corresponding argument.
	
*/

double ExponentialCdfInv(double cdf)
{
	double x;
	
	if(cdf < Zero || One < cdf)
	{
		Print("Error in ExponentialCdfInv(double cdf)");
		Print("Cdf < Zero || Cdf > One");
		HaltExecution();
	}
	
	x = -log(One - cdf);
	
	return x;
}


/*

Purpose:

	EXPONENTIAL_01_MEAN returns the mean of the Exponential 01 PDF.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Output, double EXPONENTIAL_MEAN, the mean of the PDF.
	
*/

double ExponentialMean()
{
	return One;
}


/*

Purpose:

	EXPONENTIAL_01_PDF evaluates the Exponential 01 PDF.
	
Formula:

	PDF(X) = EXP ( - X )
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	0.0 <= X
	
	Output, double PDF, the value of the PDF.
	
*/

double ExponentialPdf(double x)
{
	double pdf;
	if(x < Zero) pdf = Zero;
	else         pdf = exp(-x);
	return pdf;
}


/*

Purpose:

	EXPONENTIAL_01_SAMPLE samples the Exponential PDF with parameter 1.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double X, a sample of the PDF.
	
*/

double ExponentialSample(long &RandomSeed)
{
	double cdf;
	double x;
	cdf = UniformSample(RandomSeed);
	x = -log(One - cdf);
	return x;
}


/*

Purpose:

	EXPONENTIAL_01_VARIANCE returns the variance of the Exponential 01 PDF.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Output, double VARIANCE, the variance of the PDF.
	
*/

double ExponentialVariance()
{
	double variance;
	variance = One;
	return variance;
}


/*

Purpose:

	EXPONENTIAL_CDF evaluates the Exponential CDF.
	
Modified:

	15 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	
	Input, double A, B, the parameter of the PDF.
	0.0 < B.
	
	Output, double EXPONENTIAL_CDF, the value of the CDF.
	
*/

double ExponentialCdf(double x, double a, double b)
{
	double cdf;
	if(x <= a) cdf = Zero;
	else       cdf = One - exp((a - x) / b);
	return cdf;
}


/*

Purpose:

	EXPONENTIAL_CDF_INV inverts the Exponential CDF.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double CDF, the value of the CDF.
	0.0 <= CDF <= 1.0.
	
	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, double EXPONENTIAL_CDF_INV, the corresponding argument.
	
*/

double ExponentialCdfInv(double cdf, double a, double b)
{
	double x;
	
	if(cdf < Zero || One < cdf)
	{
		Print("Error in ExponentialCdfInv");
		Print("Cdf < 0 || Cdf > 1");
		HaltExecution();
	}
	
	x = a - b * log(One - cdf);
	
	return x;
}


/*

Purpose:

	EXPONENTIAL_CDF_VALUES returns some values of the Exponential CDF.
	
Discussion:

	In Mathematica, the function can be evaluated by:
	
		Needs["Statistics`ContinuousDistributions`"]
		dist = ExponentialDistribution [ lambda ]
		CDF [ dist, x ]
		
Modified:

	29 August 2004
	
Author:

	John Burkardt
	
Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.
	
Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, double *LAMBDA, the parameter of the distribution.
	
	Output, double *X, the argument of the function.
	
	Output, double *FX, the value of the function.
	
*/

void ExponentialCdfValues(int *n_data, double *lambda, double *x, double *fx)
{

	#define N_MAX 9
	
	double fx_vec[N_MAX] = {
		0.3934693402873666E+00,
		0.6321205588285577E+00,
		0.7768698398515702E+00,
		0.8646647167633873E+00,
		0.8646647167633873E+00,
		0.9816843611112658E+00,
		0.9975212478233336E+00,
		0.9996645373720975E+00,
		0.9999546000702375E+00 };
		
	double lambda_vec[N_MAX] = {
		0.5000000000000000E+00,
		0.5000000000000000E+00,
		0.5000000000000000E+00,
		0.5000000000000000E+00,
		0.1000000000000000E+01,
		0.2000000000000000E+01,
		0.3000000000000000E+01,
		0.4000000000000000E+01,
		0.5000000000000000E+01 };
		
	double x_vec[N_MAX] = {
		0.1000000000000000E+01,
		0.2000000000000000E+01,
		0.3000000000000000E+01,
		0.4000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01,
		0.2000000000000000E+01 };
		
	if(*n_data < 0)
	{
		*n_data = 0;
	}
	
	*n_data = *n_data + 1;
	
	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*lambda = 0.0;
		*x = 0.0;
		*fx = 0.0;
	}
	else
	{
		*lambda = lambda_vec[*n_data-1];
		*x = x_vec[*n_data-1];
		*fx = fx_vec[*n_data-1];
	}
	
	return;
	#undef N_MAX
}


/*

Purpose:

	EXPONENTIAL_CHECK checks the parameters of the Exponential CDF.
	
Modified:

	15 October 2004

Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameter of the PDF.
	0.0 < B.
	
Output, bool EXPONENTIAL_CHECK, is true if the parameters are legal.

*/

/*
bool ExponentialCheck(double a, double b)
{
	if(b <= 0.0)
	{
		Print("");
		Print("Error in ExponentialCheck");
		Print("B <= 0.0");
		return false;
	}
	return true;
}
*/


/*

Purpose:

	EXPONENTIAL_MEAN returns the mean of the Exponential PDF.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, double EXPONENTIAL_MEAN, the mean of the PDF.
	
*/

double ExponentialMean(double a, double b)
{
	return a + b;
}


/*

Purpose:

	EXPONENTIAL_PDF evaluates the Exponential PDF.
	
Formula:

	PDF(A,B;X) = ( 1 / B ) * EXP ( ( A - X ) / B )
	
Discussion:

	The time interval between two Poisson events is a random
	variable with the Exponential PDF.  The parameter B is the
	average interval between events.
	
	In another context, the Exponential PDF is related to
	the Boltzmann distribution, which describes the relative
	probability of finding a system, which is in thermal equilibrium
	at absolute temperature T, in a given state having energy E.
	The relative probability is
	
		Boltzmann_Relative_Probability(E,T) = exp ( - E / ( k * T ) ),
		
	where k is the Boltzmann constant,
	
		k = 1.38 * 10**(-23) joules / degree Kelvin
		
	and normalization requires a determination of the possible
	energy states of the system.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	A <= X
	
	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, double EXPONENTIAL_PDF, the value of the PDF.
	
*/

double ExponentialPdf(double x, double a, double b)
{
	double pdf;
	if(x < a) pdf = Zero;
	else      pdf = (One / b) * exp((a - x) / b);
	return pdf;
}


/*

Purpose:

	EXPONENTIAL_SAMPLE samples the Exponential PDF.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double EXPONENTIAL_SAMPLE, a sample of the PDF.
	
*/

double ExponentialSample(double a, double b, long &RandomSeed)
{
	double cdf;
	double x;
	cdf = UniformSample(RandomSeed);
	x = ExponentialCdfInv(cdf,a,b);
	return x;
}


/*

Purpose:

	EXPONENTIAL_VARIANCE returns the variance of the Exponential PDF.
	
Modified:

	16 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, the parameters of the PDF.
	0.0 < B.
	
	Output, double EXPONENTIAL_VARIANCE, the variance of the PDF.
	
*/

/*
double ExponentialVariance(double a, double b)
{
	double variance;
	variance = b * b;
	return variance;
}
*/

#endif
