

#ifndef _Prob_DistGamma
#define _Prob_DistGamma


	/* Two Parameter Gamma Distribution */
	double GammaMean(double Alpha, double Beta);
	double GammaSample(double Alpha, double Beta, long &RandomSeed);
	double GammaVariance(double Alpha, double Beta);
	
	/* Three Parameter Gamma Distribution */
	double GammaCdf(double x, double a, double b, double c);
	void GammaCdfValues(int *n_data, double *mu, double *sigma, double *x, double *fx);
	bool GammaCheck(double a, double b, double c);
	double GammaMean(double a, double b, double c);
	double GammaPdf(double x, double a, double b, double c);
	double GammaSample(double a, double b, double c, long &RandomSeed);
	//double GammaVariance(double a, double b, double c);
	
	/* Gamma Functions */
	double gamma(double x);
	double gamma_inc(double p, double x);
	void gamma_inc_values(int *n_data, double *a, double *x, double *fx);
	double gamma_log(double x);
	double gamma_log_int(int n);


/*

Purpose:

	GAMMA_CDF evaluates the Gamma CDF.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	A <= X
	
	Input, double A, B, C, the parameters of the PDF.
	0.0 < B,
	0.0 < C.
	
	Output, double GAMMA_CDF, the value of the CDF.
	
*/

double GammaCdf(double x, double a, double b, double c)
{
	double cdf;
	double p2;
	double x2;

	x2 = (x - a) / b;
	p2 = c;

	cdf = gamma_inc(p2, x2);

	return cdf;
}


/*

Purpose:

	GAMMA_CDF_VALUES returns some values of the Gamma CDF.
	
Discussion:

	In Mathematica, the function can be evaluated by:
	
		Needs["Statistics`ContinuousDistributions`"]
		dist = GammaDistribution [ mu, sigma ]
		CDF [ dist, x ]
		
Modified:

	05 September 2004
	
Author:

	John Burkardt
	
Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.
	
Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, double *MU, the mean of the distribution.
	
	Output, double *SIGMA, the variance of the distribution.
	
	Output, double *X, the argument of the function.
	
	Output, double *FX, the value of the function.
	
*/

void GammaCdfValues(int *n_data, double *mu, double *sigma, double *x, double *fx)
{
	#define N_MAX 12

	double fx_vec[N_MAX] = {
     0.8646647167633873E+00,
     0.9816843611112658E+00,
     0.9975212478233336E+00,
     0.9996645373720975E+00,
     0.6321205588285577E+00,
     0.4865828809674080E+00,
     0.3934693402873666E+00,
     0.3296799539643607E+00,
     0.4421745996289254E+00,
     0.1911531694619419E+00,
     0.6564245437845009E-01,
     0.1857593622214067E-01 };

	double mu_vec[N_MAX] = {
     0.1000000000000000E+01,
     0.1000000000000000E+01,
     0.1000000000000000E+01,
     0.1000000000000000E+01,
     0.1000000000000000E+01,
     0.1000000000000000E+01,
     0.1000000000000000E+01,
     0.1000000000000000E+01,
     0.2000000000000000E+01,
     0.3000000000000000E+01,
     0.4000000000000000E+01,
     0.5000000000000000E+01 };

	double sigma_vec[N_MAX] = {
     0.5000000000000000E+00,
     0.5000000000000000E+00,
     0.5000000000000000E+00,
     0.5000000000000000E+00,
     0.2000000000000000E+01,
     0.3000000000000000E+01,
     0.4000000000000000E+01,
     0.5000000000000000E+01,
     0.2000000000000000E+01,
     0.2000000000000000E+01,
     0.2000000000000000E+01,
     0.2000000000000000E+01 };

	double x_vec[N_MAX] = {
     0.1000000000000000E+01,
     0.2000000000000000E+01,
     0.3000000000000000E+01,
     0.4000000000000000E+01,
     0.2000000000000000E+01,
     0.2000000000000000E+01,
     0.2000000000000000E+01,
     0.2000000000000000E+01,
     0.3000000000000000E+01,
     0.3000000000000000E+01,
     0.3000000000000000E+01,
     0.3000000000000000E+01 };

	if(*n_data < 0) *n_data = 0;
	
	*n_data = *n_data + 1;
	
	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*mu = 0.0;
		*sigma = 0.0;
		*x = 0.0;
		*fx = 0.0;
	}
	else
	{
		*mu = mu_vec[*n_data-1];
		*sigma = sigma_vec[*n_data-1];
		*x = x_vec[*n_data-1];
		*fx = fx_vec[*n_data-1];
	}
	
	return;
	#undef N_MAX
}


/*

Purpose:

	GAMMA_CHECK checks the parameters of the Gamma PDF.

Modified:

	16 October 2004

Author:

	John Burkardt

Parameters:

	Input, double A, B, C, the parameters of the PDF.
	0.0 < B,
	0.0 < C.

	Output, bool GAMMA_CHECK, is true if the parameters are legal.

*/

/*
bool GammaCheck(double a, double b, double c)
{
	if(b <= 0.0)
	{
		Print("Error in GammaCheck");
		Print("B <= 0");
		Print("B",B);
		return false;
	}

	if(c <= 0.0)
	{
		Print("Error in GammaCheck");
		Print("C <= 0");
		Print("C",C);
		return false;
	}

	return true;
}
*/

/*

Purpose:

	GAMMA_MEAN returns the mean of the Gamma PDF.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double A, B, C, the parameters of the PDF.
	0.0 < B,
	0.0 < C.
	
	Output, double GAMMA_MEAN, the mean of the PDF.
	
*/

double GammaMean(double a, double b, double c)
{
	return (a + b * c);
}

double GammaMean(double Alpha, double Beta)
{
	return (Alpha / Beta);
}


/*

Purpose:

	GAMMA_PDF evaluates the Gamma PDF.
	
Formula:

	PDF(A,B,C;X) = EXP ( - ( X - A ) / B ) * ( ( X - A ) / B )**(C-1)
				 / ( B * GAMMA ( C ) )
				 
Discussion:

	GAMMA_PDF(A,B,C;X), where C is an integer, is the Erlang PDF.
	GAMMA_PDF(A,B,1;X) is the Exponential PDF.
	GAMMA_PDF(0,2,C/2;X) is the Chi Squared PDF with C degrees of freedom.
	
Modified:

	17 October 2004
	
Author:

	John Burkardt
	
Parameters:

	Input, double X, the argument of the PDF.
	A <= X
	
	Input, double A, B, C, the parameters of the PDF.
	0.0 < B,
	0.0 < C.
	
	Output, double GAMMA_PDF, the value of the PDF.
	
*/

double GammaPdf(double x, double a, double b, double c)
{
	double pdf;
	double y;
	
	if(x <= a)
	{
		pdf = Zero;
	}
	else
	{
		y = (x - a) / b;
		pdf = pow(y, c - One) / (b * gamma(c) * exp(y));
	}
	
	return pdf;
}


/*

Purpose:

	GAMMA_SAMPLE samples the Gamma PDF.
	
Modified:

	17 October 2004
	
Reference:

	J H Ahrens and U Dieter,
	Generating Gamma Variates by a Modified Rejection Technique,
	Communications of the ACM,
	Volume 25, Number 1, January 1982, pages 47 - 54.
	
	J H Ahrens and U Dieter,
	Computer Methods for Sampling from Gamma, Beta, Poisson and
	Binomial Distributions.
	Computing, Volume 12, 1974, pages 223 - 246.
	
	J H Ahrens, K D Kohrt, and U Dieter,
	Algorithm 599,
	ACM Transactions on Mathematical Software,
	Volume 9, Number 2, June 1983, pages 255-257.

Parameters:

	Input, double A, B, C, the parameters of the PDF.
	0.0 < B,
	0.0 < C.
	
	Input/output, int *SEED, a seed for the random number generator.
	
	Output, double X, a sample of the PDF.
	
*/

double GammaSample(double a, double b, double c, long &RandomSeed)
{
	double a1 =   0.3333333;
	double a2 = - 0.2500030;
	double a3 =   0.2000062;
	double a4 = - 0.1662921;
	double a5 =   0.1423657;
	double a6 = - 0.1367177;
	double a7 =   0.1233795;
	double bcoef;
	double co;
	double d;
	double e;
	double e1 = 1.0;
	double e2 = 0.4999897;
	double e3 = 0.1668290;
	double e4 = 0.0407753;
	double e5 = 0.0102930;
	double euler = 2.71828182845904;
	double p;
	double q;
	double q0;
	double q1 =  0.04166669;
	double q2 =  0.02083148;
	double q3 =  0.00801191;
	double q4 =  0.00144121;
	double q5 = -0.00007388;
	double q6 =  0.00024511;
	double q7 =  0.00024240;
	double r;
	double s;
	double si;
	double s2;
	double t;
	double u;
	double v;
	double w;
	double x;

	/* Allow C = 0. */

	if(c == Zero)
	{
		x = a;
		return x;
	}

	if(c < One)
	{
		/* C < 1. */

		for(;;)
		{
			//u = d_uniform_01 ( seed );
			u = UniformSample(RandomSeed);
			t = 1.0 + c / euler;
			p = u * t;

			s = ExponentialSample(RandomSeed);

			if(p < One)
			{
				x = exp(log(p) / c);
				if(x <= s)
				{
					break;
				}
			}
			else
			{
				x = - log((t - p) / c);
				if((One - c) * log(x) <= s)
				{
					break;
				}
			}
		}

		x = a + b * x;
		return x;
	}
	else
	{
		/* 1 <= C. */

		s2 = c - 0.5;
		s = sqrt(c - 0.5);
		d = sqrt(32.0) - 12.0 * sqrt(c - 0.5);

		t = NormalSample(RandomSeed);
		x = pow((sqrt(c - 0.5) + 0.5 * t),2);

		if(0.0 <= t)
		{
			x = a + b * x;
			return x;
		}

		u = UniformSample(RandomSeed);

		if(d * u <= t * t * t)
		{
			x = a + b * x;
			return x;
		}

		r = One / c;

		q0 = ( ( ( ( ( (
			  q7   * r
			+ q6 ) * r
			+ q5 ) * r
			+ q4 ) * r
			+ q3 ) * r
			+ q2 ) * r
			+ q1 ) * r;

		if(c <= 3.686)
		{
			bcoef = 0.463 + s - 0.178 * s2;
			si = 1.235;
			co = 0.195 / s - 0.079 + 0.016 * s;
		}
		else if(c <= 13.022)
		{
			bcoef = 1.654 + 0.0076 * s2;
			si = 1.68 / s + 0.275;
			co = 0.062 / s + 0.024;
		}
		else
		{
			bcoef = 1.77;
			si = 0.75;
			co = 0.1515 / s;
		}

		if(0.0 < sqrt(c - 0.5) + 0.5 * t)
		{
			v = 0.5 * t / s;

			if(0.25 < fabs(v))
			{
				q = q0 - s * t + 0.25 * t * t + 2.0 * s2 * log(1.0 + v);
			}
			else
			{
				q = q0 + 0.5 * t * t * ( ( ( ( ( (
					  a7   * v
					+ a6 ) * v
					+ a5 ) * v
					+ a4 ) * v
					+ a3 ) * v
					+ a2 ) * v
					+ a1 ) * v;
			}

			if(log(1.0 - u) <= q)
			{
				x = a + b * x;
				return x;
			}
		}

		for(;;)
		{
			e = ExponentialSample(RandomSeed);

			u = UniformSample(RandomSeed);

			u = 2.0 * u - 1.0;
			t = bcoef + fabs(si * e) * d_sign(u);

			if(-0.7187449 <= t)
			{
				v = 0.5 * t / s;

				if(0.25 < fabs(v))
				{
					q = q0 - s * t + 0.25 * t * t + 2.0 * s2 * log(1.0 + v);
				}
    			else
    			{
    				q = q0 + 0.5 * t * t * ( ( ( ( ( (
    					  a7   * v
						+ a6 ) * v
						+ a5 ) * v
						+ a4 ) * v
						+ a3 ) * v
						+ a2 ) * v
						+ a1 ) * v;
				}

				if(0.0 < q)
				{
					if(0.5 < q)
					{
						w = exp(q) - 1.0;
					}
					else
					{
						w = ( ( ( (
							  e5   * q
							+ e4 ) * q
							+ e3 ) * q
							+ e2 ) * q
							+ e1 ) * q;
					}

					if(co * fabs(u) <= w * exp (e - 0.5 * t * t))
					{
						x = a + b * pow(s + 0.5 * t,2);
						return x;
					}
				}
			}
		}
	}
}


double GammaSample(double Alpha, double Beta, long &RandomSeed)
{
	double a = Zero;
	double b = One / Beta;
	double c = Alpha;
	return GammaSample(a,b,c,RandomSeed);
}

/*

Purpose:

	GAMMA_VARIANCE returns the variance of the Gamma PDF.

Modified:

	17 October 2004

Author:

	John Burkardt

Parameters:

	Input, double A, B, C, the parameters of the PDF.
	0.0 < B,
	0.0 < C.

	Output, double VARIANCE, the variance of the PDF.

*/

/*
double GammaVariance(double a, double b, double c)
{
	double variance;
	variance = b * b * c;
	return variance;
}
*/

double GammaVariance(double Alpha, double Beta)
{
	return Alpha / Sqr(Beta);
}


/*

Purpose:

	GAMMA calculates the Gamma function for a real argument X.
	
Definition:

	GAMMA(X) = Integral ( 0 <= T <= Infinity ) T**(X-1) EXP(-T) DT
	
Recursion:

	GAMMA(X+1) = X * GAMMA(X)
	
Special values:

	GAMMA(0.5) = SQRT(PI)
	If N is a positive integer, GAMMA(N+1) = N!, the standard factorial.
	
Discussion:

	Computation is based on an algorithm outlined in reference 1.
	The program uses rational functions that approximate the GAMMA
	function to at least 20 significant decimal digits.  Coefficients
	for the approximation over the interval (1,2) are unpublished.
	Those for the approximation for X .GE. 12 are from reference 2.
	The accuracy achieved depends on the arithmetic system, the
	compiler, the intrinsic functions, and proper selection of the
	machine-dependent constants.
	
Machine-dependent constants:

	BETA: radix for the floating-point representation.
	MAXEXP: the smallest positive power of BETA that overflows.
	XBIG: the largest argument for which GAMMA(X) is representable
	      in the machine, i.e., the solution to the equation
	      GAMMA(XBIG) = BETA**MAXEXP.
	XMININ: the smallest positive floating-point number such that
	        1/XMININ is machine representable.

Approximate values for some important machines are:

                              BETA       MAXEXP        XBIG

    CRAY-1         (S.P.)        2         8191        966.961
    Cyber 180/855
      under NOS    (S.P.)        2         1070        177.803
    IEEE (IBM/XT,
      SUN, etc.)   (S.P.)        2          128        35.040
    IEEE (IBM/XT,
      SUN, etc.)   (D.P.)        2         1024        171.624
    IBM 3033       (D.P.)       16           63        57.574
    VAX D-Format   (D.P.)        2          127        34.844
    VAX G-Format   (D.P.)        2         1023        171.489

                              XMININ

    CRAY-1         (S.P.)   1.84E-2466
    Cyber 180/855
      under NOS    (S.P.)   3.14E-294
    IEEE (IBM/XT,
      SUN, etc.)   (S.P.)   1.18E-38
    IEEE (IBM/XT,
      SUN, etc.)   (D.P.)   2.23D-308
    IBM 3033       (D.P.)   1.39D-76
    VAX D-Format   (D.P.)   5.88D-39
    VAX G-Format   (D.P.)   1.12D-308

Author:

	W. J. Cody and L. Stoltz,
	Applied Mathematics Division,
	Argonne National Laboratory,
	Argonne, Illinois, 60439.
	
Reference:

	W J Cody,
	"An Overview of Software Development for Special Functions",
	Lecture Notes in Mathematics, 506,
	Numerical Analysis Dundee, 1975,
	G. A. Watson (ed.),
	Springer Verlag, Berlin, 1976.
	
	Hart et al,
	Computer Approximations,
	Wiley and sons, New York, 1968.
	
Parameters:

	Input, double X, the argument of the function.
	
	Output, double GAMMA, the value of the function.
	The computation is believed to be free of underflow and overflow.
	
*/

double gamma(double x)
{
	double c[7] = {
	    -1.910444077728E-03,
	     8.4171387781295E-04,
	    -5.952379913043012E-04,
	     7.93650793500350248E-04,
	    -2.777777777777681622553E-03,
	     8.333333333333333331554247E-02,
	     5.7083835261E-03 };
	double fact;
	int i;
	int n;
	double p[8] = {
	    -1.71618513886549492533811E+00,
	     2.47656508055759199108314E+01,
	    -3.79804256470945635097577E+02,
	     6.29331155312818442661052E+02,
	     8.66966202790413211295064E+02,
	    -3.14512729688483675254357E+04,
	    -3.61444134186911729807069E+04,
	     6.64561438202405440627855E+04 };
	bool parity;
	double pi =
    	3.141592653589793E+00;
	double q[8] = {
	    -3.08402300119738975254353E+01,
	     3.15350626979604161529144E+02,
	    -1.01515636749021914166146E+03,
	     -3.10777167157231109440444E+03,
	     2.25381184209801510330112E+04,
	     4.75584627752788110767815E+03,
	    -1.34659959864969306392456E+05,
	    -1.15132259675553483497211E+05 };
	double sqrtpi = 0.9189385332046727417803297E+00;
	double sum2;
	double value;
	double xbig = 35.040E+00;
	double xden;
	double xminin = 1.18E-38;
	double xnum;
	double y;
	double y1;
	double ysq;
	double z;
	
	parity = false;
	fact = 1.0;
	n = 0;
	y = x;

	/* Argument is negative. */
	if(y <= 0.0)
	{
		y = -x;
		y1 = (double)((int)(y));
		value = y - y1;
		
		if(value != 0.0)
		{
			if(y1 != (double)((int)(y1 * 0.5)) * 2.0)
			{
				parity = true;
			}
			
			fact = -pi / sin(pi * value);
			y = y + 1.0;
		}
		else
		{
			value = d_huge();
			return value;
		}
	}
	
	/* Argument < EPS */
	if(y < d_epsilon())
	{
		if(xminin <= y)
		{
			value = 1.0 / y;
		}
		else
		{
			value = d_huge();
		    return value;
		}
	}
	else if(y < 12.0)
	{
		y1 = y;
		
		if(y < 1.0)
		{
			/* 0.0 < argument < 1.0 */
			z = y;
			y = y + 1.0;
		}
		else
		{
			/* 1.0 < argument < 12.0, reduce argument if necessary. */
			n = int ( y ) - 1;
			y = y - ( double ) ( n );
			z = y - 1.0;
		}
		
		/* Evaluate approximation for 1.0 < argument < 2.0. */
		xnum = 0.0;
		xden = 1.0;
		for(i=0; i<8; i++)
		{
			xnum = (xnum + p[i]) * z;
			xden = xden * z + q[i];
		}
		
		value = xnum / xden + 1.0;
		
		if(y1 < y)
		{
			/* Adjust result for case  0.0 < argument < 1.0. */
			value = value / y1;
		}
		else if(y < y1)
		{
			/* Adjust result for case  2.0 < argument < 12.0. */
			for(i=1; i<=n; i++)
			{
				value = value * y;
				y = y + 1.0;
			}
		}
	}
 	else
  	{
  		/* Evaluate for 12 <= argument. */
  		if(y <= xbig)
  		{
  			ysq = y * y;
  			sum2 = c[6];
  			for(i=0; i<6; i++)
  			{
  				sum2 = sum2 / ysq + c[i];
  			}
  			sum2 = sum2 / y - y + sqrtpi;
  			sum2 = sum2 + (y - 0.5) * log(y);
  			value = exp(sum2);
  		}
  		else
  		{
  			value = d_huge();
  			return value;
  		}
  	}
  	
  	/* Final adjustments and return. */
  	if(parity) value = -value;
  	if(fact != 1.0) value = fact / value;
  	
  	return value;
}


/*

Purpose:

	GAMMA_INC computes the incomplete Gamma function.

Definition:

	GAMMA_INC(P,X) = Integral ( 0 <= T <= X ) T**(P-1) EXP(-T) DT / GAMMA(P).

Discussion:

	GAMMA_INC(P,       0) = 0,
	GAMMA_INC(P,Infinity) = 1.

Modified:

	16 October 2004

Reference:

	B L Shea,
	Chi-squared and Incomplete Gamma Integral,
	Algorithm AS239,
	Applied Statistics,
	Volume 37, Number 3, 1988, pages 466-473.

Parameters:

	Input, double P, the exponent parameter.
	0.0 < P.

	Input, double X, the integral limit parameter.
	If X is less than or equal to 0, GAMMA_INC is returned as 0.

	Output, double GAMMA_INC, the value of the function.

*/

double gamma_inc(double p, double x)
{
	double a;
	double arg;
	double b;
	double c;
	//double cdf;	// M: Is this an error in the original code?
	double exp_arg_min = -88.0E+00;
	double overflow = 1.0E+37;
	double plimit = 1000.0E+00;
	double pn1;
	double pn2;
	double pn3;
	double pn4;
	double pn5;
	double pn6;
	double rn;
	double value;
	double tol = 1.0E-07;
	double xbig = 1.0E+08;

	value = Zero;

	if(p <= Zero)
	{
		Print("Error in GammaInc");
		Print("Parameter P <= 0");
		HaltExecution();
	}

    if(x <= Zero)
    {
    	value = Zero;
    	return value;
    }

    /* Use a normal approximation if PLIMIT < P. */

	if(plimit < p)
	{
		pn1 = 3.0 * sqrt(p) * (pow(x / p,1.0 / 3.0) + 1.0 / (9.0 * p) - 1.0);
    	value = NormalCdf(pn1);
    	return value;
    }

    /* Is X extremely large compared to P? */
    if(xbig < x)
    {
    	value = 1.0;
    	return value;
    }

   	if(x <= 1.0 || x < p)
    {

		/*

  		Use Pearson's series expansion.
		(P is not large enough to force overflow in the log of Gamma.

		*/

    	arg = p * log(x) - x - gamma_log(p + 1.0);
		c = 1.0;
		value = 1.0;
		a = p;

		for(;;)
		{
			a = a + 1.0;
			c = c * x / a;
			value = value + c;

			if(c <= tol)
			{
				break;
			}
		}

		arg = arg + log(value);

		if(exp_arg_min <= arg)
		{
			value = exp(arg);
		}
		else
		{
			value = 0.0;
		}
	}
	else
	{

		/* Use a continued fraction expansion. ) */

		arg = p * log(x) - x - gamma_log(p);
		a = 1.0 - p;
		b = a + x + 1.0;
		c = 0.0;
		pn1 = 1.0;
		pn2 = x;
		pn3 = x + 1.0;
		pn4 = x * b;
		value = pn3 / pn4;

		for(;;)
		{
			a = a + 1.0;
			b = b + 2.0;
			c = c + 1.0;
			pn5 = b * pn3 - a * c * pn1;
			pn6 = b * pn4 - a * c * pn2;

			if(0.0 < fabs(pn6))
			{
				rn = pn5 / pn6;

				if(fabs(value - rn) <= d_min(tol,tol * rn))
				{
					arg = arg + log(value);

					if(exp_arg_min <= arg)
					{
						value = 1.0 - exp(arg);
					}
					else
					{
						value = 1.0;
					}

					return value;
				}

				value = rn;
			}

			pn1 = pn3;
			pn2 = pn4;
			pn3 = pn5;
			pn4 = pn6;

			/* Rescale terms in continued fraction if terms are large. */

			if(overflow <= fabs(pn5))
			{
				pn1 = pn1 / overflow;
				pn2 = pn2 / overflow;
				pn3 = pn3 / overflow;
				pn4 = pn4 / overflow;
			}
		}
	}

	return value;
}


/*

Purpose:

	GAMMA_INC_VALUES returns some values of the incomplete Gamma function.
	
Discussion:

	The (normalized) incomplete Gamma function P(A,X) is defined as:
	
		PN(A,X) = 1/Gamma(A) * Integral ( 0 <= T <= X ) T**(A-1) * exp(-T) dT.
		
	With this definition, for all A and X,
	
		0 <= PN(A,X) <= 1
		
	and
	
		PN(A,INFINITY) = 1.0
		
	In Mathematica, the function can be evaluated by:
	
		1 - GammaRegularized[A,X]
		
Modified:

	20 November 2004
	
Author:

	John Burkardt

Reference:

	Milton Abramowitz and Irene Stegun,
	Handbook of Mathematical Functions,
	US Department of Commerce, 1964.
	
	Stephen Wolfram,
	The Mathematica Book,
	Fourth Edition,
	Wolfram Media / Cambridge University Press, 1999.
	
Parameters:

	Input/output, int *N_DATA.  The user sets N_DATA to 0 before the
	first call.  On each call, the routine increments N_DATA by 1, and
	returns the corresponding data; when there is no more data, the
	output value of N_DATA will be 0 again.
	
	Output, double *A, the parameter of the function.
	
	Output, double *X, the argument of the function.
	
	Output, double *FX, the value of the function.

*/

void gamma_inc_values(int *n_data, double *a, double *x, double *fx)
{
	#define N_MAX 20

	double a_vec[N_MAX] = {
     0.10E+00,
     0.10E+00,
     0.10E+00,
     0.50E+00,
     0.50E+00,
     0.50E+00,
     0.10E+01,
     0.10E+01,
     0.10E+01,
     0.11E+01,
     0.11E+01,
     0.11E+01,
     0.20E+01,
     0.20E+01,
     0.20E+01,
     0.60E+01,
     0.60E+01,
     0.11E+02,
     0.26E+02,
     0.41E+02  };

	double fx_vec[N_MAX] = {
     0.7382350532339351E+00,
     0.9083579897300343E+00,
     0.9886559833621947E+00,
     0.3014646416966613E+00,
     0.7793286380801532E+00,
     0.9918490284064973E+00,
     0.9516258196404043E-01,
     0.6321205588285577E+00,
     0.9932620530009145E+00,
     0.7205974576054322E-01,
     0.5891809618706485E+00,
     0.9915368159845525E+00,
     0.1018582711118352E-01,
     0.4421745996289254E+00,
     0.9927049442755639E+00,
     0.4202103819530612E-01,
     0.9796589705830716E+00,
     0.9226039842296429E+00,
     0.4470785799755852E+00,
     0.7444549220718699E+00 };

	double x_vec[N_MAX] = {
     0.30E-01,
     0.30E+00,
     0.15E+01,
     0.75E-01,
     0.75E+00,
     0.35E+01,
     0.10E+00,
     0.10E+01,
     0.50E+01,
     0.10E+00,
     0.10E+01,
     0.50E+01,
     0.15E+00,
     0.15E+01,
     0.70E+01,
     0.25E+01,
     0.12E+02,
     0.16E+02,
     0.25E+02,
     0.45E+02 };

	if(*n_data < 0) *n_data = 0;

	*n_data = *n_data + 1;
	
	if(N_MAX < *n_data)
	{
		*n_data = 0;
		*a = 0.0;
		*x = 0.0;
		*fx = 0.0;
	}
	else
	{
		*a = a_vec[*n_data-1];
		*x = x_vec[*n_data-1];
		*fx = fx_vec[*n_data-1];
	}
	
	return;
	#undef N_MAX
}

#endif
