
#ifndef _Prob_DistUtil
#define _Prob_DistUtil


	double d_epsilon();
	double d_huge();
	double d_min(double x, double y);
	double d_sign(double x);
	double d_uniform_01(int *seed);
	template <class Real>
	Real poly_value(int n, double a[], const Real &x);
	double *dvec_uniform_01(int n, int *seed);

	int d_nint(double x);

	double i_factorial(int n);
	int i_max(int i1, int i2);
	int i_min(int i1, int i2);

	double factorial_log(int n);
	double factorial_stirling(int n);


/*

Purpose:

	D_EPSILON returns the round off unit for floating arithmetic.

Discussion:

	D_EPSILON is a number R which is a power of 2 with the property that,
	to the precision of the computer's arithmetic,
		1 < 1 + R
	but
		1 = ( 1 + R / 2 )

Modified:

	06 May 2003

Author:

	John Burkardt

Parameters:

	Output, double D_EPSILON, the floating point round-off unit.

*/

double d_epsilon()
{
	double r = 1.0;
	while(1.0 < (double) (1.0 + r)) r = r / 2.0;
	return 2.0 * r;
}


/*

Purpose:

	D_HUGE returns a "huge" real value, usually the largest legal real.

Discussion:

	HUGE_VAL is the largest representable legal real number, and is usually
	defined in math.h, or sometimes in stdlib.h.

Modified:

	08 May 2003

Author:

	John Burkardt

Parameters:

	Output, double D_HUGE, a "huge" real value.

*/

double d_huge()
{
	return HUGE_VAL;
}


/*

Purpose:

	D_MIN returns the minimum of two double precision values.

Modified:

	31 August 2004

Author:

	John Burkardt

Parameters:

	Input, double X, Y, the quantities to compare.

	Output, double D_MIN, the minimum of X and Y.

*/

double d_min(double x, double y)
{
	if(y < x) return y;
	else      return x;
}


/*

Purpose:

	D_SIGN returns the sign of a double precision number.

Modified:

	18 October 2004

Author:

	John Burkardt

Parameters:

	Input, double X, the number whose sign is desired.

	Output, double D_SIGN, the sign of X.

*/


double d_sign(double x)
{
	if(x < 0.0E+00) return -1.0E+00;
	else            return  1.0E+00;
}


/*

Purpose:

	D_UNIFORM_01 is a portable pseudorandom number generator.

Discussion:

	This routine implements the recursion

	seed = 16807 * seed mod ( 2**31 - 1 )
	unif = seed / ( 2**31 - 1 )

	The integer arithmetic never requires more than 32 bits,
	including a sign bit.

Modified:

	11 August 2004

Reference:

	Paul Bratley, Bennett Fox, L E Schrage,
	A Guide to Simulation,
	Springer Verlag, pages 201-202, 1983.

	Bennett Fox,
	Algorithm 647:
	Implementation and Relative Efficiency of Quasirandom
	Sequence Generators,
	ACM Transactions on Mathematical Software,
	Volume 12, Number 4, pages 362-376, 1986.

Parameters:

	Input/output, int *SEED, the "seed" value.  Normally, this
	value should not be 0.  On output, SEED has been updated.

	Output, double D_UNIFORM_01, a new pseudorandom variate, strictly between
	0 and 1.

*/

double d_uniform_01(int *seed)
{
	int k;
	double r;
	k = *seed / 127773;
	*seed = 16807 * (*seed - k * 127773) - k * 2836;
	if(*seed < 0) *seed = *seed + 2147483647;

	/*

	Although SEED can be represented exactly as a 32 bit integer,
	it generally cannot be represented exactly as a 32 bit real number!

	*/

	r = (double)(*seed) * 4.656612875E-10;
	return r;
}


/*

Purpose:

	DPOLY_VALUE evaluates a double precision polynomial.

Discussion:

	For sanity's sake, the value of N indicates the NUMBER of
	coefficients, or more precisely, the ORDER of the polynomial,
	rather than the DEGREE of the polynomial.  The two quantities
	differ by 1, but cause a great deal of confusion.

	Given N and A, the form of the polynomial is:

		p(x) = a[0] + a[1] * x + ... + a[n-2] * x^(n-2) + a[n-1] * x^(n-1)

Modified:

	13 August 2004

Author:

	John Burkardt

Parameters:

	Input, int N, the order of the polynomial.

	Input, double A[N], the coefficients of the polynomial.
	A[0] is the constant term.

	Input, double X, the point at which the polynomial is to be evaluated.

	Output, double DPOLY_VALUE, the value of the polynomial at X.

*/

template <class Real>
Real dpoly_value(int n, double a[], const Real &x)
{
	Real value = Zero;;
	for(int i=n-1; 0<=i; i--) value = value * x + a[i];
	return value;
}


/*

Purpose:

	DVEC_UNIFORM_01 fills a double precision vector with pseudorandom values.

Discussion:

	This routine implements the recursion

	seed = 16807 * seed mod ( 2**31 - 1 )
	unif = seed / ( 2**31 - 1 )

	The integer arithmetic never requires more than 32 bits,
	including a sign bit.

Modified:

	19 August 2004

Reference:

	Paul Bratley, Bennett Fox, L E Schrage,
	A Guide to Simulation,
	Springer Verlag, pages 201-202, 1983.

	Bennett Fox,
	Algorithm 647:
	Implementation and Relative Efficiency of Quasirandom
	Sequence Generators,
	ACM Transactions on Mathematical Software,
	Volume 12, Number 4, pages 362-376, 1986.

Parameters:

	Input, int N, the number of entries in the vector.

	Input/output, int *SEED, a seed for the random number generator.

	Output, double DVEC_UNIFORM_01[N], the vector of pseudorandom values.

*/

double *dvec_uniform_01(int n, int *seed)
{
	int i;
	int k;
	double *r;
	r = new double[n];
	for(i=0; i<n; i++)
	{
		k = *seed / 127773;
		*seed = 16807 * (*seed - k * 127773) - k * 2836;
		if(*seed < 0) *seed = *seed + 2147483647;
		r[i] = (double)(*seed) * 4.656612875E-10;
	}
	return r;
}


/*

Purpose:

	I_FACTORIAL returns N!.

Definition:

	N! = Product ( 1 <= I <= N ) I

Modified:

	02 February 1999

Author:

	John Burkardt

Parameters:

	Input, int N, the argument of the factorial function.
	0 <= N.

	Output, double I_FACTORIAL, the factorial of N.

*/

double i_factorial(int n)
{
	/* Check. */
	if(n < 0)
	{
		Print("Error in I_FACTORIAL");
		Print("n < 0");
		HaltExecution();
	}

	double fact = 1.0;
	for(int i=2; i<=n; i++) fact = fact * (double)i;
	return fact;
}


/*

Purpose:

	D_NINT returns the nearest integer to a double precision value.

Examples:

        X         D_NINT

      1.3         1
      1.4         1
      1.5         1 or 2
      1.6         2
      0.0         0
     -0.7        -1
     -1.1        -1
     -1.6        -2

Modified:

	26 August 2004

Author:

	John Burkardt

Parameters:

	Input, double X, the value.

	Output, int D_NINT, the nearest integer to X.

*/

int d_nint(double x)
{
	int s;
	if(x < 0.0) s = -1;
	else        s =  1;
	return (s*(int)(fabs(x) + 0.5));
}


/*

Purpose:

	I_MAX returns the maximum of two integers.

Modified:

	13 October 1998

Author:

	John Burkardt

Parameters:

	Input, int I1, I2, are two integers to be compared.

	Output, int I_MAX, the larger of I1 and I2.

*/

int i_max(int i1, int i2)
{
	if(i2 < i1) return i1;
	else		return i2;
}


/*

Purpose:

	I_MIN returns the smaller of two integers.

Modified:

	13 October 1998

Author:

	John Burkardt

Parameters:

	Input, int I1, I2, two integers to be compared.

	Output, int I_MIN, the smaller of I1 and I2.

*/

int i_min(int i1, int i2)
{
	if(i1 < i2) return i1;
	else		return i2;
}


/*

Purpose:

	FACTORIAL_LOG returns the logarithm of N!.

Definition:

	N! = Product ( 1 <= I <= N ) I

Method:

	N! = Gamma(N+1).

Modified:

	11 October 2004

Author:

	John Burkardt

Parameters:

	Input, int N, the argument of the function.
	0 <= N.

	Output, double FACTORIAL_LOG, the logarithm of N!.

*/

double factorial_log(int n)
{
	if(n < 0)
	{
		Print("FACTORIAL_LOG - Fatal error");
		Print("N < 0");
		HaltExecution();
	}
	
	double value = 0.0;
	for(int i=2; i<=n; i++) value = value + log((double)(i));
	return value;
}


/*

Purpose:

	FACTORIAL_STIRLING computes Stirling's approximation to N!.

Definition:

	N! = Product ( 1 <= I <= N ) I

	Stirling ( N ) = sqrt ( 2 * PI * N ) * ( N / E )**N * E**(1/(12*N) )

Discussion:

	This routine returns the raw approximation for all nonnegative
	values of N.  If N is less than 0, the value is returned as 0,
	and if N is 0, the value of 1 is returned.  In all other cases,
	Stirling's formula is used.

Modified:

	01 February 1999

Author:

	John Burkardt

Parameters:

	Input, int N, the argument of the function.

	Output, double FACTORIAL_STIRLING, an approximation to N!.

*/

double factorial_stirling(int n)
{
	#define PI 3.141592653589793
	double e_natural = 2.71828182845904523536028747135266249775724709369995;
	double value;
	     if(n < 0)  value = 0.0;
	else if(n == 0) value = 1.0;
	else            value = sqrt(2.0*PI*(double)(n)) * pow((double)(n)/e_natural,n) * exp(1.0/(double)(12*n));
	return value;
	#undef PI
}


#endif
