
#ifndef _GenMatUtil_EigenvaluesObj
#define _GenMatUtil_EigenvaluesObj

/*

Computes eigenvalues and eigenvectors of a real (non-complex)
matrix.
 
<P>
If A is symmetric, then A = V * D * V^T where the eigenvalue matrix D is
diagonal and the eigenvector matrix V is orthogonal. That is,
the diagonal values of D are the eigenvalues, and
V * V^T = I, where I is the identity matrix.  The columns of V 
represent the eigenvectors in the sense that A * V = V * D.
    
<P>
If A is not symmetric, then the eigenvalue matrix D is block diagonal
with the real eigenvalues in 1-by-1 blocks and any complex eigenvalues,
a + i*b, in 2-by-2 blocks, [a, b; -b, a].  That is, if the complex
eigenvalues look like
<pre>

          u + iv     .        .          .      .    .
            .      u - iv     .          .      .    .
            .        .      a + ib       .      .    .
            .        .        .        a - ib   .    .
            .        .        .          .      x    .
            .        .        .          .      .    y
</pre>
        then D looks like
<pre>

            u        v        .          .      .    .
           -v        u        .          .      .    . 
            .        .        a          b      .    .
            .        .       -b          a      .    .
            .        .        .          .      x    .
            .        .        .          .      .    y
</pre>
This keeps V a real matrix in both symmetric and non-symmetric
cases, and A * V = V * D.
   
<p>
The matrix V may be badly
conditioned, or even singular, so the validity of the equation
A = V * D * V^-1 depends upon the condition number of V.

<p>
(Adapted from JAMA, a Java Matrix Library, developed by jointly 
by the Mathworks and NIST; see  http://math.nist.gov/javanumerics/jama).

MP:

I modified the nonsymmetric component of this code to work faster.
The modified code should produce exactlt the same results.
However, I found that some changes that mathematically should not
have an effect changed the computed eigivecxtor dramatrixallly.
This is possibly beucase of badly condtioned matrices.

*/

template <class Real>
class EigenvaluesObj
{
	
	int n; 				// Row and column dimension (square matrix)
	int IsSymmetric;	// boolean

	/* Arrays for internal storage of eigenvalues */
	
	Vector<Real> d; // real part
	Vector<Real> e;	// img part
	
	/* Array for internal storage of eigenvectors */
	
    Matrix<Real> V;
    
    /*
    
    Array for internal storage of nonsymmetric Hessenberg form.
    @serial internal storage of nonsymmetric Hessenberg form.
    
    */
    
   	Matrix<Real> H;
   
	/*
	
	Working storage for nonsymmetric algorithm.
	@serial working storage for nonsymmetric algorithm.
	
	*/
	
   	Vector<Real> ort;

	/* Symmetric Householder reduction to tridiagonal form. */
	void tred2()
	{
		/*
		
		This is derived from the Algol procedures tred2 by
		Bowdler, Martin, Reinsch, and Wilkinson, Handbook for
		Auto. Comp., Vol.ii-Linear Algebra, and the corresponding
		Fortran subroutine in EISPACK.
		
		*/

		for(int j=0; j<n; j++) d[j] = V(n-1,j);
      	
      	/* Householder reduction to tridiagonal form. */
   		for(int i=n-1; i>0; i--)
   		{
   			/* Scale to avoid under/overflow. */
   			Real scale = 0.0;
         	Real h = 0.0;
 			for(int k=0; k<i; k++) scale = scale + abs(d[k]);
         	if(scale == 0.0)
         	{
            	e[i] = d[i-1];
            	for(int j=0; j<i; j++)
            	{
               		d[j] = V(i-1,j);
               		V(i,j) = 0.0;
               		V(j,i) = 0.0;
            	}
            }
            else
            {
   				/* Generate Householder vector. */
   				for(int k=0; k<i; k++)
   				{
   					d[k] /= scale;
   					h += d[k] * d[k];
   				}
   				Real f = d[i-1];
   				Real g = sqrt(h);
   				if(f > 0) g = -g;
            	e[i] = scale * g;
            	h = h - f * g;
            	d[i-1] = f - g;
            	for(int j=0; j<i; j++) e[j] = 0.0;
           
           		/* Apply similarity transformation to remaining columns. */
   				for(int j=0; j<i; j++)
   				{
   					f = d[j];
   					V(j,i) = f;
   					g = e[j] + V(j,j) * f;
   					
   					/* Mp: Innermost loop */
   					
   					for(int k=j+1; k<=i-1; k++)
   					{
   						g += V(k,j) * d[k];
   						e[k] += V(k,j) * f;
   					}
   					
   					e[j] = g;
            	}
            	f = 0.0;
            	for(int j=0; j<i; j++)
            	{
            		e[j] /= h;
            		f += e[j] * d[j];
            	}
            	Real hh = f / (h + h);
            	for(int j=0; j<i; j++) e[j] -= hh * d[j];
            	
            	for(int j=0; j<i; j++)
            	{
               		f = d[j];
   					g = e[j];
   					
   					/* Mp: Innermost loop */
               		for(int k=j; k<=i-1; k++) V(k,j) -= (f * e[k] + g * d[k]);
               		
               		d[j] = V(i-1,j);
               		V(i,j) = 0.0;
               	}
            }
         	d[i] = h;
      	}
   
   		/* Accumulate transformations. */
  		for(int i=0; i<n-1; i++)
  		{
        	V(n-1,i) = V(i,i);
        	V(i,i) = 1.0;
        	Real h = d[i+1];
        	if(h != 0.0)
        	{
            	for(int k=0; k<=i; k++) d[k] = V(k,i+1) / h;
           		for(int j=0; j<=i; j++)
           		{
           			Real g = 0.0;
           			
           			/* Mp: Innermost loop */
           			for(int k=0; k<=i; k++) g += V(k,i+1) * V(k,j);
					for(int k=0; k<=i; k++) V(k,j) -= g * d[k];
             	}
             }
             for(int k=0; k<=i; k++) V(k,i+1) = 0.0;
        }
        
        for(int j=0; j<n; j++)
        {
    		d[j] = V(n-1,j);
			V(n-1,j) = 0.0;
    	}
     	V(n-1,n-1) = 1.0;
   		e[0] = 0.0;
   	} 

	/* Symmetric tridiagonal QL algorithm. */
	void tql2()
	{
		/*
		
		This is derived from the Algol procedures tql2, by
		Bowdler, Martin, Reinsch, and Wilkinson, Handbook for
		Auto. Comp., Vol.ii-Linear Algebra, and the corresponding
		Fortran subroutine in EISPACK.
		
		*/
		
		for(int i=1; i<n; i++) e[i-1] = e[i];
      	e[n-1] = 0.0;
   
      	Real f = 0.0;
		Real tst1 = 0.0;
		Real eps = pow(2.0,-52.0);
    	for(int l=0; l<n; l++)
    	{
			/* Find small subdiagonal element */
			tst1 = max(tst1,abs(d[l]) + abs(e[l]));
  			int m = l;

			/* Original while-loop from Java code */
			while(m < n)
			{
            	if(abs(e[m]) <= eps * tst1) break;
            	m++;
            }

   			/*
   			
   			If m == l, d[l] is an eigenvalue,
   			otherwise, iterate.
   			
   			*/
   
   			if(m > l)
   			{
            	int iter = 0;
            	do
            	{
            		iter = iter + 1; // (Could check iteration count here.)
            		
            		/* Compute implicit shift */
   					Real g = d[l];
					Real p = (d[l+1] - g) / (2.0 * e[l]);
					Real r = hypot(p,1.0);
    				if(p < 0) r = -r;
              		d[l] = e[l] / (p + r);
					d[l+1] = e[l] * (p + r);
          			Real dl1 = d[l+1];
     				Real h = g - d[l];
     				for(int i=l+2; i<n; i++) d[i] -= h;
              		f = f + h;
   
             		/* Implicit QL transformation. */
					p = d[m];
              		Real c = 1.0;
       				Real c2 = c;
     				Real c3 = c;
     				Real el1 = e[l+1];
					Real s = 0.0;
   					Real s2 = 0.0;
					for(int i=m-1; i>=l; i--)
					{
						c3 = c2;
   						c2 = c;
						s2 = s;
						g = c * e[i];
						h = c * p;
						r = hypot(p,e[i]);
						e[i+1] = s * r;
 						s = e[i] / r;
   						c = p / r;
						p = c * d[i] - s * g;
                  		d[i+1] = h + s * (c * g + s * d[i]);
   
   						/* Accumulate transformation. */
   						
   						/* Mp: Innermost loop */
   						for(int k=0; k<n; k++)
   						{
							h = V(k,i+1);
							V(k,i+1) = s * V(k,i) + c * h;
      						V(k,i) = c * V(k,i) - s * h;
                  		}
                  	}
                  	p = -s * s2 * c3 * el1 * e[l] / dl1;
                  	e[l] = s * p;
               		d[l] = c * p;
   				}
   				while(abs(e[l]) > eps*tst1); // Check for convergence.
         	}
         	d[l] = d[l] + f;
         	e[l] = 0.0;
      	}
     
      	/* Sort eigenvalues and corresponding vectors. */
   		for(int i=0; i<n-1; i++)
   		{
        	int k = i;
			Real p = d[i];
			for(int j=i+1; j<n; j++)
			{
            	if(d[j] < p)
            	{
            		k = j;
            		p = d[j];
            	}
            }
         	if(k != i)
         	{
            	d[k] = d[i];
            	d[i] = p;
            	for(int j=0; j<n; j++)
            	{
               		p = V(j,i);
               		V(j,i) = V(j,k);
               		V(j,k) = p;
            	}
            }
      	}
   	}

 	/* Nonsymmetric reduction to Hessenberg form. */
	void orthes()
	{
   		/*
   		
   		This is derived from the Algol procedures orthes and ortran,
   		by Martin and Wilkinson, Handbook for Auto. Comp.,
   		Vol.ii-Linear Algebra, and the corresponding
   		Fortran subroutines in EISPACK.
   		
   		*/
   
   		int low  = 0;
      	int high = n - 1;
      	Vector<double> fVec(n);		// Temporary storage
   
      	for(int m=low+1; m<=high-1; m++)
      	{
          	/* Scale column. */
   			Real scale = 0.0;
         	for(int i=m; i<=high; i++) scale = scale + abs(H(i,m-1));
         	
         	if(scale != 0.0)
         	{
               	/* Compute Householder transformation. */
   				Real h = 0.0;
            	for(int i=high; i>=m; i--)
            	{
            		ort[i] = H(i,m-1) / scale;
               		h += ort[i] * ort[i];
            	}
            	Real g = sqrt(h);
            	if(ort[m] > 0) g = -g;
            	h = h - ort[m] * g;
            	ort[m] = ort[m] - g;
   
           		/*
           		
           		Apply Householder similarity transformation
           		H = (I - u * u' / h) * H * (I - u * u') / h)
           		
           		*/
           		
           		/*
           		for(int j=m; j<n; j++)
   				{
               		Real f = 0.0;
               		for(int i=high; i>=m; i--) f += ort[i] * H(i,j);
					f = f / h;
					for(int i=m; i<=high; i++) H(i,j) -= f * ort[i];
              	}
              	*/
   
   				BlasSet(n-m,Zero,&fVec[m],1);
				for(int i=high; i>=m; i--) BlasAxpy(n-m,ort[i],&H(i,m),1,&fVec[m],1);
				for(int j=m; j<n; j++) fVec[j] /= h;
				for(int i=m; i<=high; i++) BlasAxpy(n-m,-ort[i],&fVec[m],1,&H(i,m),1);
              	
              	/*   				
   				for(int i=0; i<=high; i++)
   				{
   					Real f = 0.0;
   					for(int j=high; j>=m; j--) f += ort[j] * H(i,j);
   					f = f / h;
   					for(int j=m; j<=high; j++) H(i,j) -= f * ort[j];
              	}
              	*/
              	
             	for(int i=0; i<=high; i++) fVec[i] = BlasDot(high-m+1,&ort[m],-1,&H(i,m),-1) / h;
   				for(int i=0; i<=high; i++) BlasAxpy(high-m+1,-fVec[i],&ort[m],1,&H(i,m),1);
                          	
            	ort[m] = scale * ort[m];
            	H(m,m-1) = scale * g;
         	}
      	}
   
  		/* Accumulate transformations (Algol's ortran). */
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) V(i,j) = (i == j ? 1.0 : 0.0);
        
        for(int m=high-1; m>=low+1; m--)
        {
        	if(H(m,m-1) != 0.0)
        	{
            	for(int i=m+1; i<=high; i++) ort[i] = H(i,m-1);
            	for(int j=m; j<=high; j++)
            	{
               		//Real g = 0.0;
               		//for(int i=m; i<=high; i++) g += ort[i] * V(i,j);
               		Real g = BlasDot(high-m+1,&ort[m],1,&V(m,j),n);
               		
               		/* Double division avoids possible underflow */
               		g = (g / ort[m]) / H(m,m-1);
               		
               		//for(int i=m; i<=high; i++) V(i,j) += g * ort[i];
               		BlasAxpy(high-m+1,g,&ort[m],1,&V(m,j),n);
               	}
            }
        }
   	}

	/* Complex scalar division. */
	Real cdivr, cdivi;
   	void cdiv(Real xr, Real xi, Real yr, Real yi)
   	{
    	Real r,d;
      	if(abs(yr) > abs(yi))
      	{
        	r = yi / yr;
        	d = yr + r * yi;
         	cdivr = (xr + r * xi) / d;
         	cdivi = (xi - r * xr) / d;
      	}
      	else
      	{
        	r = yr / yi;
         	d = yi + r * yr;
         	cdivr = (r * xr + xi) / d;
         	cdivi = (r * xi - xr) / d;
      	}
   	}

	/* Nonsymmetric reduction from Hessenberg to real Schur form. */
	void hqr2()
	{
   		/*
   		
   		This is derived from the Algol procedure hqr2,
   		by Martin and Wilkinson, Handbook for Auto. Comp.,
   		Vol.ii-Linear Algebra, and the corresponding
   		Fortran subroutine in EISPACK.
   		
   		*/
   
   		/* Initialize */
   		int nn = this->n;
    	int n = nn-1;
    	int ld = nn; // MP: my code
    	Vector<Real> Work(nn); // Mp: my code
     	int low = 0;
    	int high = nn-1;
     	Real eps = pow(2.0,-52.0);
     	Real exshift = 0.0;
     	Real p=0,q=0,r=0,s=0,z=0,t,w,x,y;
   
  		/* Store roots isolated by balanc and compute matrix norm */
   		Real norm = 0.0;
      	for(int i=0; i<nn; i++)
      	{
      		if((i < low) || (i > high))
      		{
            	d[i] = H(i,i);
            	e[i] = 0.0;
         	}
         	for(int j=max(i-1,0); j<nn; j++) norm = norm + abs(H(i,j));
        }
   
      	/* Outer loop over eigenvalue index */
      	
   		int iter = 0;
   		while(n >= low)
   		{
   			/* Look for single small sub-diagonal element */
   			int l = n;
         	while (l > low)
         	{
            	s = abs(H(l-1,l-1)) + abs(H(l,l));
           		if(s == 0.0) s = norm;
            	if(abs(H(l,l-1)) < eps * s) break;
           		l--;
         	}
         	
         	/* Check for convergence */         	
   			if(l == n)
   			{
   				/* One root found */
            	H(n,n) = H(n,n) + exshift;
            	d[n] = H(n,n);
            	e[n] = 0.0;
            	n--;
            	iter = 0;
   			}
   			else if(l == n-1)
   			{
   				/* Two roots found */
            	w = H(n,n-1) * H(n-1,n);
            	p = (H(n-1,n-1) - H(n,n)) / 2.0;
            	q = p * p + w;
            	z = sqrt(abs(q));
            	H(n,n) = H(n,n) + exshift;
            	H(n-1,n-1) = H(n-1,n-1) + exshift;
            	x = H(n,n);
   
            	if(q >= 0)
   				{
   					/* Real pair */
   					
               		if(p >= 0) z = p + z;
               		else z = p - z;
              		d[n-1] = x + z;
               		d[n] = d[n-1];
               		if(z != 0.0) d[n] = x - w / z;
              		e[n-1] = 0.0;
               		e[n] = 0.0;
               		x = H(n,n-1);
               		s = abs(x) + abs(z);
               		p = x / s;
               		q = z / s;
               		r = sqrt(p * p+q * q);
               		p = p / r;
               		q = q / r;
   
   					/* Row modification */
   					for(int j=n-1; j<nn; j++)
   					{
                  		z = H(n-1,j);
                  		H(n-1,j) = q * z + p * H(n,j);
                  		H(n,j) = q * H(n,j) - p * z;
               		}
   
   					/* Column modification */
   					for(int i=0; i<=n; i++)
   					{
   						z = H(i,n-1);
                  		H(i,n-1) = q * z + p * H(i,n);
                  		H(i,n) = q * H(i,n) - p * z;
               		}
   
					/* Accumulate transformations */
					for(int i=low; i<=high; i++)
					{
                  		z = V(i,n-1);
                  		V(i,n-1) = q * z + p * V(i,n);
                  		V(i,n) = q * V(i,n) - p * z;
                  	}
   				}
   				else
   				{
   					/* Complex pair */
   				
               		d[n-1] = x + p;
               		d[n] = x + p;
               		e[n-1] = z;
               		e[n] = -z;
            	}
            	n = n - 2;
            	iter = 0;
   			}
   			else
   			{
   				/* No convergence yet */
   			
   				/* Form shift */
   				x = H(n,n);
            	y = 0.0;
            	w = 0.0;
            	if(l < n)
            	{
               		y = H(n-1,n-1);
               		w = H(n,n-1) * H(n-1,n);
            	}
   
            	/* Wilkinson's original ad hoc shift */
   				if(iter == 10)
   				{
               		exshift += x;
               		for(int i=low; i<=n; i++) H(i,i) -= x;
             		s = abs(H(n,n-1)) + abs(H(n-1,n-2));
               		x = y = 0.75 * s;
               		w = -0.4375 * s * s;
            	}

            	/* MATLAB's new ad hoc shift */
				if(iter == 30)
				{
                	s = (y - x) / 2.0;
                	s = s * s + w;
                	if(s > 0)
                	{
                    	s = sqrt(s);
                    	if (y < x) s = -s;
                    	s = x - w / ((y - x) / 2.0 + s);
                    	for (int i = low; i <= n; i++) H(i,i) -= s;
                    	exshift += s;
                    	x = y = w = 0.964;
                    }
                }
   
            	iter = iter + 1;   // (Could check iteration count here.)
   
   				/* Look for two consecutive small sub-diagonal elements */
  				int m = n - 2;
            	while(m >= l)
            	{
               		z = H(m,m);
               		r = x - z;
               		s = y - z;
               		p = (r * s - w) / H(m+1,m) + H(m,m+1);
               		q = H(m+1,m+1) - z - r - s;
               		r = H(m+2,m+1);
               		s = abs(p) + abs(q) + abs(r);
               		p = p / s;
               		q = q / s;
               		r = r / s;
               		if(m == l) break;
               		if (abs(H(m,m-1)) * (abs(q) + abs(r)) < eps * (abs(p) * (abs(H(m-1,m-1)) + abs(z)
                		+ abs(H(m+1,m+1))))) break;
               		m--;
            	}
            	
            	for(int i=m+2; i<=n; i++)
            	{
               		H(i,i-2) = 0.0;
               		if(i > m+2) H(i,i-3) = 0.0;
             	}
   
            	/* Double QR step involving rows l:n and columns m:n */
  				for(int k=m; k<=n-1; k++)
  				{
               		int notlast = (k != n-1);
    				if(k != m)
    				{
                  		p = H(k,k-1);
                  		q = H(k+1,k-1);
                  		r = (notlast ? H(k+2,k-1) : 0.0);
                  		x = abs(p) + abs(q) + abs(r);
                  		if(x != 0.0)
                  		{
                    		p = p / x;
                    		q = q / x;
                    		r = r / x;
                  		}
               		}
               		if(x == 0.0) break;
               
               		s = sqrt(p * p + q * q + r * r);
               		if(p < 0) s = -s;
               
              		if(s != 0)
              		{
              			if(k != m) H(k,k-1) = -s * x;
                  		else if(l != m) H(k,k-1) = -H(k,k-1);
                 		p = p + s;
                  		x = p / s;
                  		y = q / s;
                  		z = r / s;
                  		q = q / p;
                  		r = r / p;
   
   						/* This part takes the most time! */
   
                 		/* Row modification */
  						/*
  						for(int j=k; j<nn; j++)
  						{
  							p = H(k,j) + q * H(k+1,j);
  							if(notlast)
  							{
                        		p = p + r * H(k+2,j);
                       			H(k+2,j) = H(k+2,j) - p * z;
                     		}
                    		H(k,j) = H(k,j) - p * x;
                     		H(k+1,j) = H(k+1,j) - p * y;
                  		}
                  		*/
                  		
                  		BlasCopy(nn-k,&H(k,k),1,&Work[k],1);
						BlasAxpy(nn-k,q,&H(k+1,k),1,&Work[k],1);
                  		if(notlast)
                  		{
                  			BlasAxpy(nn-k,r,&H(k+2,k),1,&Work[k],1);
                  			BlasAxpy(nn-k,-z,&Work[k],1,&H(k+2,k),1);
                     	}
                     	BlasAxpy(nn-k,-x,&Work[k],1,&H(k,k),1);
                     	BlasAxpy(nn-k,-y,&Work[k],1,&H(k+1,k),1);
                    	   			
            			/* Column modification */
            			/*
   						for(int i=0; i<=min(n,k+3); i++)
   						{
                     		p = x * H(i,k) + y * H(i,k+1);
                     		if(notlast)
                     		{
                        		p = p + z * H(i,k+2);
                        		H(i,k+2) = H(i,k+2) - p * r;
                     		}
                     		H(i,k) = H(i,k) - p;
                     		H(i,k+1) = H(i,k+1) - p * q;
                  		}
                  		*/
                  		
	               		BlasSet(min(n,k+3)+1,Zero,&Work[0],1);
                  		BlasAxpy(min(n,k+3)+1,x,&H(0,k)  ,ld,&Work[0],1);
                  		BlasAxpy(min(n,k+3)+1,y,&H(0,k+1),ld,&Work[0],1);
                  		if(notlast)
                     	{
                        	BlasAxpy(min(n,k+3)+1,z,&H(0,k+2),ld,&Work[0],1);
                        	BlasAxpy(min(n,k+3)+1,-r,&Work[0],1,&H(0,k+2),ld);
                     	}
                     	BlasAxpy(min(n,k+3)+1,-One,&Work[0],1,&H(0,k)  ,ld);
                     	BlasAxpy(min(n,k+3)+1,-q  ,&Work[0],1,&H(0,k+1),ld);
                  		   
                 		/* Accumulate transformations */
                 		/*
  						for(int i=low; i<=high; i++)
  						{
                     		p = x * V(i,k) + y * V(i,k+1);
                     		if(notlast)
                     		{
                        		p = p + z * V(i,k+2);
                        		V(i,k+2) = V(i,k+2) - p * r;
                     		}
                     		V(i,k) = V(i,k) - p;
                     		V(i,k+1) = V(i,k+1) - p * q;
                  		}
                  		*/
                  		
                  		BlasSet(high-low+1,Zero,&Work[0],1);
                  		BlasAxpy(high-low+1,x,&V(low,k)  ,ld,&Work[0],1);
                  		BlasAxpy(high-low+1,y,&V(low,k+1),ld,&Work[0],1);
                  		if(notlast)
                     	{
                        	BlasAxpy(high-low+1,z,&V(low,k+2),ld,&Work[0],1);
                        	BlasAxpy(high-low+1,-r,&Work[0],1,&V(low,k+2),ld);
                     	}
                     	BlasAxpy(high-low+1,-One,&Work[0],1,&V(low,k)  ,ld);
                     	BlasAxpy(high-low+1,-q  ,&Work[0],1,&V(low,k+1),ld);
                  		
               		} // (s != 0)
            	} // k loop
         	} // check convergence
      	} // while (n >= low)
      
      	/* Backsubstitute to find vectors of upper triangular form */
		if(norm == 0.0) return;
      
   		for(n=nn-1; n>=0; n--)
   		{
        	p = d[n];
         	q = e[n];
   
         	if(q == 0)
   			{
   				/* Real vector */
   			
            	int l = n;
            	H(n,n) = 1.0;
            	for(int i=n-1; i>=0; i--)
            	{
               		w = H(i,i) - p;
               		r = 0.0;
               		
               		/* Innermost loop */
               		//for(int j=l; j<=n; j++) r = r + H(i,j) * H(j,n);
               		r = BlasDot(n-l+1,&H(i,l),1,&H(l,n),ld);
               		
              		if(e[i] < 0.0)
              		{
                 		z = w;
                  		s = r;
               		}
               		else
               		{
                  		l = i;
                  		if(e[i] == 0.0)
                  		{
                     		if(w != 0.0) H(i,n) = -r / w;
                     		else H(i,n) = -r / (eps * norm);
   						}
   						else
   						{
   							/* Solve real equations */
                     		x = H(i,i+1);
                     		y = H(i+1,i);
                    		q = (d[i] - p) * (d[i] - p) + e[i] * e[i];
                    		t = (x * s - z * r) / q;
                     		H(i,n) = t;
                     		if(abs(x) > abs(z)) H(i+1,n) = (-r - w * t) / x;
                     		else H(i+1,n) = (-s - y * t) / z;
                     	}
                  	
                  		/* Overflow control */
   						t = abs(H(i,n));
                  		
                  		/* Innermost loop */
                  		//if((eps * t) * t > 1) for(int j=i; j<=n; j++) H(j,n) = H(j,n) / t;
                  		if((eps * t) * t > 1) BlasScal(n-i+1,One / t,&H(i,n),ld);
                	}
            	}
   			}
   			else if(q < 0)
   			{
   				/* Complex vector */
   			
				int l = n-1;

   	    		/* Last vector component imaginary so matrix is triangular */
   				if(abs(H(n,n-1)) > abs(H(n-1,n)))
   				{
            		H(n-1,n-1) = q / H(n,n-1);
               		H(n-1,n) = -(H(n,n) - p) / H(n,n-1);
            	}
            	else
            	{
            	  	cdiv(0.0,-H(n-1,n),H(n-1,n-1)-p,q);
               		H(n-1,n-1) = cdivr;
            		H(n-1,n) = cdivi;
            	}
        		H(n,n-1) = 0.0;
            	H(n,n) = 1.0;
            	for(int i=n-2; i>=0; i--)
            	{
           	   		Real ra,sa,vr,vi;
            		
            		//ra = 0.0;
            		//sa = 0.0;
            		//for(int j=l; j<=n; j++)
            		//{
                	//	ra = ra + H(i,j) * H(j,n-1);
                	//	sa = sa + H(i,j) * H(j,n);
               		//}
               		
               		ra = BlasDot(n-l+1,&H(i,l),1,&H(l,n-1),ld);
               		sa = BlasDot(n-l+1,&H(i,l),1,&H(l,n)  ,ld);
               		
               		w = H(i,i) - p;
   					if(e[i] < 0.0)
               		{
               			z = w;
                		r = ra;
                		s = sa;
               		}
               		else
               		{
               			l = i;
                  		if(e[i] == 0)
                  		{
                    		cdiv(-ra,-sa,w,q);
                    		H(i,n-1) = cdivr;
                    		H(i,n) = cdivi;
               			}
                  		else
                  		{
   							/* Solve complex equations */
   							x = H(i,i+1);
                     		y = H(i+1,i);
                     		vr = (d[i] - p) * (d[i] - p) + e[i] * e[i] - q * q;
                     		vi = (d[i] - p) * 2.0 * q;
                     		if((vr == 0.0) && (vi == 0.0))
                     		{
                       			vr = eps * norm * (abs(w) + abs(q) +
                     	  	 	abs(x) + abs(y) + abs(z));
                     		}
                     		cdiv(x * r - z * ra + q * sa,x * s - z * sa - q * ra,vr,vi);
                     		H(i,n-1) = cdivr;
                     		H(i,n) = cdivi;
                     		if(abs(x) > (abs(z) + abs(q)))
                     		{
                       			H(i+1,n-1) = (-ra - w * H(i,n-1) + q * H(i,n)) / x;
                     	  	 	H(i+1,n)   = (-sa - w * H(i,n) - q * H(i,n-1)) / x;
                     		}
                     		else
                     		{
                       			cdiv(-r - y * H(i,n-1),-s - y * H(i,n),z,q);
                       	 		H(i+1,n-1) = cdivr;
                       		 	H(i+1,n)   = cdivi;
                     		}
                  		}
  			 
            	      	/* Overflow control */
						t = max(abs(H(i,n-1)),abs(H(i,n)));
               		   	if((eps * t) * t > 1)
               	   		{
               	   			//for(int j=i; j<=n; j++)
                    		//{
                       		// 	H(j,n-1) = H(j,n-1) / t;
                       	 	//	H(j,n) = H(j,n) / t;
                     		//}
                     		
                     		double Temp = One / t;
                     		BlasScal(n-i+1,Temp,&H(i,n-1),ld);
                     		BlasScal(n-i+1,Temp,&H(i,n)  ,ld);
                  		}
               		}
            	}
        	}
		}

		/* Vectors of isolated roots */
   		for(int i=0; i<nn; i++) if(i < low || i > high) for(int j=i; j<nn; j++) V(i,j) = H(i,j);
  	  
		/* Back transformation to get eigenvectors of original matrix */
		/*
   		for(int j=nn-1; j>=low; j--)
   		{
   			for (int i=low; i<=high; i++)
   		 	{
   		     	z = 0.0;
				for(int k=low; k<=min(j,high); k++) z = z + V(i,k) * H(k,j);
        		V(i,j) = z;
        	}
  		}
  		*/
  		
  		for(int i=low; i<=high; i++)
   		{
   			BlasSet(nn-low,Zero,&Work[low],-1);
   			for(int k=low; k<=high; k++)
   			{
   				int l = max(k,low);
   				BlasAxpy(nn-l,V(i,k),&H(k,l),1,&Work[l],1);
   			}
			BlasCopy(nn-low,&Work[low],-1,&V(i,low),-1);
       	}
	}

	public:

	/*
	
	Check for symmetry, then construct the eigenvalue decomposition
   	@param A    Square real (non-complex) matrix
   	
   	*/

 	EigenvaluesObj(const Matrix<Real> &A)
 	{
    	n = A.Rows();
    	
    	if(A.Cols() != n)
    	{
    		Print("Error in EigenvaluesObj::EigenvaluesObj(Matrix<Real> A)");
    		Print("A is not a Square Matrix");
    		Print("A.Name",A.Name);
    		Print("A.Size()",A.Size());
    		HaltExecution();
    	}
    	
    	V = Matrix<Real>(n,n);
    	d = Vector<Real>(n);
    	e = Vector<Real>(n);
    	
    	IsSymmetric = 1;
      	for(int j=0; (j<n) && IsSymmetric; j++) for(int i=0; (i<n) && IsSymmetric; i++)
      		IsSymmetric = (A(i,j) == A(j,i));
         	
        if(IsSymmetric)
        {
        	for(int i=0; i<n; i++) for(int j=0; j<n; j++) V(i,j) = A(i,j);
            	
          	/* Tridiagonalize. */
        	tred2();
   
   			/* Diagonalize. */
      		tql2();
		}
		else
		{
       		H = Matrix<Real>(n,n);
       		ort = Vector<Real>(n);
      		for(int j=0; j<n; j++) for(int i=0; i<n; i++) H(i,j) = A(i,j);
          	
          	Timer tm;
          	
           	/* Reduce to Hessenberg form. */
			tm.Name = "ReduceHess";
           	tm.Start();
       		orthes();
       		tm.Stop();
   
  			/* Reduce Hessenberg to real Schur form. */
  			tm.Name = "ReduceShur";
  			tm.Start();
       		hqr2();
       		tm.Stop();
   		}
  	}
  	
  	#ifdef _GenMat_BlasMatrix
  		EigenvaluesObj(const BlasMatrix<Real> &A)
	 	{
	    	n = A.Rows();
	    	
	    	if(A.Cols() != n)
	    	{
	    		Print("Error in EigenvaluesObj::EigenvaluesObj(Matrix<Real> A)");
	    		Print("A is not a Square Matrix");
	    		Print("A.Name",A.Name);
	    		Print("A.Size()",A.Size());
	    		HaltExecution();
	    	}
	    	
	    	V = Matrix<Real>(n,n);
	    	d = Vector<Real>(n);
	    	e = Vector<Real>(n);
	    	
	    	IsSymmetric = 1;
	      	for(int j=0; (j<n) && IsSymmetric; j++) for(int i=0; (i<n) && IsSymmetric; i++)
	      		IsSymmetric = (A(i,j) == A(j,i));
	         	
	        if(IsSymmetric)
	        {
	        	for(int i=0; i<n; i++) for(int j=0; j<n; j++) V(i,j) = A(i,j);
	            	
	          	/* Tridiagonalize. */
	        	tred2();
	   
	   			/* Diagonalize. */
	      		tql2();
			}
			else
			{
	       		H = Matrix<Real>(n,n);
	       		ort = Vector<Real>(n);
	      		for(int j=0; j<n; j++) for(int i=0; i<n; i++) H(i,j) = A(i,j);
	          	
	          	Timer tm;
	          	
	           	/* Reduce to Hessenberg form. */
				tm.Name = "ReduceHess";
	           	tm.Start();
	       		orthes();
	       		tm.Stop();
	   
	  			/* Reduce Hessenberg to real Schur form. */
	  			tm.Name = "ReduceShur";
	  			tm.Start();
	       		hqr2();
	       		tm.Stop();
	   		}
	  	}
  	#endif
  	
  	EigenvaluesObj(const SymmetricMatrix<Real> &A)
 	{
    	n = A.Dim();
    	V = Matrix<Real>(n,n);
    	d = Vector<Real>(n);
    	e = Vector<Real>(n);
    	
		IsSymmetric = 1;
      	
      	Copy(V,A);
            	
        /* Tridiagonalize. */
        tred2();
   
   		/* Diagonalize. */
      	tql2();
	}

	/* Return the eigenvector matrix V */
   	Matrix<Real> GetV()
   	{
   		return V;
   	}
   
	/* Return the real parts of the eigenvalues */
   	Vector<Real> GetRealEigenvalues()
   	{
   		return d;
   	}

	/* Return the imaginary parts of the eigenvalues */
 	Vector<Real> GetImagEigenvalues()
 	{
 		return e;
 	}

	/* Return the complex eigenvalues */
	Vector< Complex<Real> > GetEigenvalues()
	{
		Vector< Complex<Real> > de(n);
		for(int i=0; i<n; i++)
		{
			de[i].Real = d[i];
			de[i].Imag = e[i];
		}
		return de;
	}
	
 	/*
 	
 	Computes the block diagonal eigenvalue matrix.
    If the original matrix A is not symmetric, then the eigenvalue 
	matrix D is block diagonal with the real eigenvalues in 1-by-1 
	blocks and any complex eigenvalues,
    a + i*b, in 2-by-2 blocks, [a, b; -b, a].  That is, if the complex
    eigenvalues look like
	<pre>

          u + iv     .        .          .      .    .
            .      u - iv     .          .      .    .
            .        .      a + ib       .      .    .
            .        .        .        a - ib   .    .
            .        .        .          .      x    .
            .        .        .          .      .    y
	</pre>
        then D looks like
	<pre>

            u        v        .          .      .    .
           -v        u        .          .      .    . 
            .        .        a          b      .    .
            .        .       -b          a      .    .
            .        .        .          .      x    .
            .        .        .          .      .    y
	</pre>
    This keeps V a real matrix in both symmetric and non-symmetric
    cases, and A*V = V*D.

	@param D: upon return, the matrix is filled with the block diagonal 
	eigenvalue matrix.
	
	*/

	Matrix<double> GetD()
	{
    	Matrix<Real> D(n,n);
		for(int i=0; i<n; i++)
		{
        	for(int j=0; j<n; j++) D(i,j) = Zero;
         	D(i,i) = d[i];
         	if(e[i] > 0) D(i,i+1) = e[i];
    		else if(e[i] < 0) D(i,i-1) = e[i];
       	}
       	return D;
   	}
   	
	int Rank()
	{
		/*

		Use matlab tolerence,

		Tol = Max(Size(A)) * Max(Abs(Lambda)));

		*/

		double Temp;
		double Mx = Zero;
		for(int i=0; i<n; i++)
		{
			Temp = sqrt(Sqr(d[i]) + Sqr(e[i]));
			if(Temp > Mx) Mx = Temp;
		}

		double Tol = n * Mx * MachEps;

		int r = 0;
		for(int i=0; i<n; i++) if(sqrt(Sqr(d[i]) + Sqr(e[i])) > Tol) r++;
		return r;   	
   	}
   	
   	int Rank(double Tol)
	{
		/* User Specified Tolerance */
		int r = 0;
		for(int i=0; i<n; i++) if(sqrt(Sqr(d[i]) + Sqr(e[i])) > Tol) r++;
		return r;   	
   	}
   	
	int IsSpd()
	{
		/*

		A matrix is positive definite if it has strictly
		positive eigenvalues. As in Rank(), use matlab
		tolerence for detecting singularity,

		Tol = Max(Size(A)) * Max(Abs(Lambda)));

		*/

		double Temp;
		double Mx = Zero;
		for(int i=0; i<n; i++)
		{
			Temp = sqrt(Sqr(d[i]) + Sqr(e[i]));
			if(Temp > Mx) Mx = Temp;
		}
		double Tol = n * Mx * MachEps;

		for(int i=0; i<n; i++) if(sqrt(Sqr(d[i]) + Sqr(e[i])) < Tol || e[i] != Zero || d[i] < Zero) return 0;
		return 1;
	}
	
	int IsSpd(double Tol)
	{
		/*

		A matrix is positive definite if it has strictly
		positive eigenvalues. User Specified Tolerance

		*/

		for(int i=0; i<n; i++) if(sqrt(Sqr(d[i]) + Sqr(e[i])) < Tol || e[i] != Zero || d[i] < Zero) return 0;
		return 1;
	}
	
	double Cond()
	{
		return sqrt((Sqr(d[n-1]) + Sqr(e[n-1])) / (Sqr(d[0]) + Sqr(e[0])));
	}
	
	Matrix<double> Inverse()
   	{
		/* A^-1 = V * D^-1 * V^T */

		Matrix<double> AInv(n,n);

		if(IsSymmetric)
		{
		
			/* This part could probably be rewritten to avoid using a new matrix
			by allowing crease a routine to multiple such things */

			Matrix<double> DInvVT(n,n);
			for(int i=0; i<n; i++) for(int j=0; j<n; j++)
				DInvVT(i,j) = V(j,i) / d(i);
		
			ProdAB(AInv,V,DInvVT);
		}
		else
		{
			Print("Error in EigenvaluesObj::Inverse");
			Print("This part not done yet");
			HaltExecution();
		}
		
		return AInv;
	}
	
	Matrix<double> PDInverse(double Tol)
   	{
   	
   		/* Compute the postive definite matrix which is 'close' to being an inverse */
   
		/* A^-1 = V * D^-1 * V^T */
	
		Matrix<double> AInv(n,n);

		/* This part could probably be rewritten to avoid using a new matrix
		by allowing crease a routine to multiple such things */

		if(IsSymmetric)
		{
			Matrix<double> DInvVT(n,n);
			for(int i=0; i<n; i++) for(int j=0; j<n; j++)
			{
				double Temp;
				if(d(i) >= Tol) Temp = One / d(i);
				else            Temp = One / Tol;
				DInvVT(i,j) = Temp * V(j,i);
			}
			ProdAB(AInv,V,DInvVT);
		}
		else
		{
			Print("Error in EigenvaluesObj::PDInverse");
			Print("Not Symemtric");
			HaltExecution();
		}

		return AInv;
	}
};


#endif
