
#ifndef _Opt_Gradient3
#define _Opt_Gradient3

/*

Utility for computing the jacobian of a function.

Arguments

	Func:			A pointer to a function
	x:				Point at which to evaluate derivatives
	Z:				Projection matrix
	f:				Return function value
	df:				Return derivative value
	Step:			Step size for numerical derivatives
	Type:			Type of derivative approximation to use
					1	-	Forward  difference finite difference derivatives, absolute step
					2	-	Backward difference finite difference derivatives, absolute step
					3	-	Central  difference finite difference derivatives, absolute step
					4	-	Forward  difference finite difference derivatives, relative step
					5	-	Backward difference finite difference derivatives, relative step
					6	-	Central  difference finite difference derivatives, relative step
	PrintLevel:
	TypX:			Vector containing scales of x

*/

	/* Compute the gradient of a scalar-valued function */
	int Gradient(int (*Func)(const Vector<double> &, double &),
				const Vector<double> &x,
				double &f,
				Vector<double> &df,
				double Step,
				int Type,
				const Vector<double> &TypX);
				
	/* Compute the projected gradient of a scalar-valued function */
	int Gradient(int (*Func)(const Vector<double> &, double &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				double &f,
				Vector<double> &df,
				double Step,
				int Type,
				const Vector<double> &TypX);
				
	/* Compute the gradient of a scalar-valued and vector-valued function */
	int Gradient(int (*Func)(const Vector<double> &, double &, Vector<double> &),
				const Vector<double> &x,
				double &f,
				Vector<double> &df,
				Vector<double> &cf,
				Matrix<double> &dcf,
				double Step,
				int Type,
				const Vector<double> &TypX);

	/* Compute the projected gradient of a scalar-valued and vector-valued function */
	int Gradient(int (*Func)(const Vector<double> &, double &, Vector<double> &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				double &f,
				Vector<double> &df,
				Vector<double> &cf,
				Matrix<double> &dcf,
				double Step,
				int Type,
				const Vector<double> &TypX);

/* Compute the gradient of a scalar-valued function */
int Gradient(int (*Func)(const Vector<double> &, double &),
				const Vector<double> &x,
				double &f,
				Vector<double> &df,
				double Step,
				int Type,
				const Vector<double> &TypX)
{
	
	/* Check dimensions */
	int n = x.Size();		// Number of variables
	Vector<double> xCopy = x.Copy();
	
	/* Compute normalized step sizes */
	Vector<double> hZ(n);
	ComputeDerivStep(hZ,xCopy,Step,Type,TypX);
		
	/* Central function value */
	if(Func(xCopy,f)) return 1;
	if(IsNan(f)) return 2;
	
	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */		
		double fp;
		for(int i=0; i<n; i++)
		{
			/* Compute f(x + h * Z(j)) */
			xCopy[i] += hZ[i];
			
			if(Func(xCopy,fp)) return 5;
			if(IsNan(fp)) return 6;
			xCopy[i] -= hZ[i];
					
			/* Compute first derivatives */
			df[i]  = (fp  - f) / hZ[i];
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fm;
		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fm)) return 3;
			if(IsNan(fm)) return 4;
			xCopy[i] += hZ[i];
			
			/* Compute first derivatives */
			df[i]  = (f - fm) / hZ[i];
		}
	}
	else
	{
		/* Central difference derivatives */
		double fm,fp;
		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fm)) return 3;
			if(IsNan(fm)) return 4;
				
			/* Compute f(x + h * Z(j)) */
			xCopy[i] += 2.0 * hZ[i];
			if(Func(xCopy,fp)) return 5;
			if(IsNan(fp)) return 6;
			xCopy[i] -= hZ[i];
				
			/* Compute first derivatives */
			df[i] = (fp - fm) / (Two * hZ[i]);
		}
	}
	
	return 0;
}

/* Compute the projected gradient of a scalar-valued function */
int Gradient(int (*Func)(const Vector<double> &, double &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				double &f,
				Vector<double> &df,
				double Step,
				int Type,
				const Vector<double> &TypX)
{

	/* Check dimensions */
	int n = x.Size();		// Number of variables
	int m1 = n - Z.Cols();	// Number of linear constraints
	Vector<double> xCopy = x.Copy();

	/* Compute normalized step sizes */
	Vector<double> hZ(n-m1);
	ComputeDerivStep(hZ,xCopy,Z,Step,Type,TypX);
	
	/* Central function value */
	if(Func(xCopy,f)) return 1;
	if(IsNan(f)) return 2;
	
	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */		
		double fp;
		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			if(Func(xCopy,fp)) return 5;
			if(IsNan(fp)) return 6;
			for(int j=0; j<n; j++) xCopy[j] -= Z(j,i) * hZ[i];
					
			/* Compute first derivatives */
			df[i]  = (fp  - f) / hZ[i];
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fm;
		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] -= Z(j,i) * hZ[i];
			if(Func(xCopy,fm)) return 3;
			if(IsNan(fm)) return 4;
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			
			/* Compute first derivatives */
			df[i]  = (f - fm) / hZ[i];
		}
	}
	else
	{
		/* Central difference derivatives */
		double fm,fp;
		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] -= Z(j,i) * hZ[i];
			if(Func(xCopy,fm)) return 3;
			if(IsNan(fm)) return 4;
				
			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] += Two * Z(j,i) * hZ[i];
			if(Func(xCopy,fp)) return 5;
			if(IsNan(fp)) return 6;
			for(int j=0; j<n; j++) xCopy[j] -= Z(j,i) * hZ[i];
				
			/* Compute first derivatives */
			df[i] = (fp - fm) / (Two * hZ[i]);
		}
	}
		
	return 0;
}

/* Compute the projected gradient of a scalar-valued function */
#ifdef _GenMat_SparseMatrixCC
	int Gradient(int (*Func)(const Vector<double> &, double &),
					const Vector<double> &x,
					const SparseMatrixCC<double> &ZS,
					double &f,
					Vector<double> &df,
					double Step,
					int Type,
					const Vector<double> &TypX)
	{

		/* Check dimensions */
		int n = x.Size();		// Number of variables
		int m1 = n - ZS.Cols();	// Number of linear constraints
		Vector<double> xCopy = x.Copy();

		/* Compute normalized step sizes */
		Vector<double> hZ(n-m1);
		ComputeDerivStep(hZ,xCopy,ZS,Step,Type,TypX);
		
		/* Central function value */
		if(Func(xCopy,f)) return 1;
		if(IsNan(f)) return 2;
		
		if(Type == 1 || Type == 4)
		{
			/* Forward difference derivatives */		
			double fp;
			for(int i=0; i<n-m1; i++)
			{
				/* Compute f(x + h * Z(j)) */
				for(int k=ZS.c[i]; k<ZS.c[i+1]; k++)
				{
					int j = ZS.r[k];
					xCopy[j] += ZS.v[k] * hZ[i];
				}
				if(Func(xCopy,fp)) return 5;
				if(IsNan(fp)) return 6;
				for(int k=ZS.c[i]; k<ZS.c[i+1]; k++)
				{
					int j = ZS.r[k];
					xCopy[j] -= ZS.v[k] * hZ[i];
				}
									
				/* Compute first derivatives */
				df[i]  = (fp  - f) / hZ[i];
			}
		}
		else if(Type == 2 || Type == 5)
		{
			/* Backward difference derivatives */
			double fm;
			for(int i=0; i<n-m1; i++)
			{
				/* Compute f(x - h * Z(j)) */
				for(int k=ZS.c[i]; k<ZS.c[i+1]; k++)
				{
					int j = ZS.r[k];
					xCopy[j] -= ZS.v[k] * hZ[i];
				}
				if(Func(xCopy,fm)) return 3;
				if(IsNan(fm)) return 4;
				for(int k=ZS.c[i]; k<ZS.c[i+1]; k++)
				{
					int j = ZS.r[k];
					xCopy[j] += ZS.v[k] * hZ[i];
				}
				
				/* Compute first derivatives */
				df[i]  = (f - fm) / hZ[i];
			}
		}
		else
		{
			/* Central difference derivatives */
			double fm,fp;
			for(int i=0; i<n-m1; i++)
			{
				/* Compute f(x - h * Z(j)) */
				for(int k=ZS.c[i]; k<ZS.c[i+1]; k++)
				{
					int j = ZS.r[k];
					xCopy[j] -= ZS.v[k] * hZ[i];
				}
				if(Func(xCopy,fm)) return 3;
				if(IsNan(fm)) return 4;
					
				/* Compute f(x + h * Z(j)) */
				for(int k=ZS.c[i]; k<ZS.c[i+1]; k++)
				{
					int j = ZS.r[k];
					xCopy[j] += Two * ZS.v[k] * hZ[i];
				}
				if(Func(xCopy,fp)) return 5;
				if(IsNan(fp)) return 6;
				for(int k=ZS.c[i]; k<ZS.c[i+1]; k++)
				{
					int j = ZS.r[k];
					xCopy[j] -= ZS.v[k] * hZ[i];
				}
					
				/* Compute first derivatives */
				df[i] = (fp - fm) / (Two * hZ[i]);
			}
		}
			
		return 0;
	}
#endif

/* Compute the gradient of a scalar-valued and vector-valued function */
int Gradient(int (*Func)(const Vector<double> &, double &, Vector<double> &),
				const Vector<double> &x,
				double &f,
				Vector<double> &df,
				Vector<double> &cf,
				Matrix<double> &dcf,
				double Step,
				int Type,
				const Vector<double> &TypX)
{
	
	/* Check dimensions */
	int n = x.Size();		// Number of variables
	int m2 = cf.Size();
	Vector<double> xCopy = x.Copy();
	
	/* Compute normalized step sizes */
	Vector<double> hZ(n);
	ComputeDerivStep(hZ,xCopy,Step,Type,TypX);
		 	 					
	/* Central function value */
	if(Func(xCopy,f,cf)) return 1;
	if(IsNan(f)) return 2;
	if(IsNan(cf)) return 7;
		
	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */		
		double fp;
		Vector<double> cfp(m2);
		
		for(int i=0; i<n; i++)
		{
			/* Compute f(x + h * Z(j)) */
			xCopy[i] += hZ[i];
			if(Func(xCopy,fp,cfp)) return 5;
			if(IsNan(fp)) return 6;
			if(IsNan(cfp)) return 9;
			xCopy[i] -= hZ[i];
					
			/* Compute first derivatives */
			df[i]  = (fp  - f) / hZ[i];
			for(int j=0; j<m2; j++)
				dcf(j,i) = (cfp[j] - cf[j]) / hZ[i];
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fm;
		Vector<double> cfm(m2);
			
		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fm,cfm)) return 3;
			if(IsNan(fm)) return 4;
			if(IsNan(cfm)) return 8;
			xCopy[i] += hZ[i];
			
			/* Compute first derivatives */
			df[i]  = (f - fm) / hZ[i];
			for(int j=0; j<m2; j++)
				dcf(j,i) = (cf[j] - cfm[j]) / hZ[i];
		}
	}
	else
	{
		/* Central difference derivatives */
		double fm,fp;
		Vector<double> cfm(m2);
		Vector<double> cfp(m2);
					
		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fm,cfm)) return 3;
			if(IsNan(fm)) return 4;
			if(IsNan(cfm)) return 8;
				
			/* Compute f(x + h * Z(j)) */
			xCopy[i] += Two * hZ[i];
			if(Func(xCopy,fp,cfp)) return 5;
			if(IsNan(fp)) return 6;
			if(IsNan(cfp)) return 9;
			xCopy[i] -= hZ[i];
				
			/* Compute first derivatives */
			df[i] = (fp - fm) / (Two * hZ[i]);
			for(int j=0; j<m2; j++)
				dcf(j,i) = (cfp[j] - cfm[j]) / (Two * hZ[i]);
		}
	}
		
	return 0;
}

/* Compute the projected gradient of a scalar-valued and vector-valued function */
int Gradient(int (*Func)(const Vector<double> &, double &, Vector<double> &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				double &f,
				Vector<double> &df,
				Vector<double> &cf,
				Matrix<double> &dcf,
				double Step,
				int Type,
				const Vector<double> &TypX)
{
	/* Check dimensions */
	int n = x.Size();		// Number of variables
	int m1 = n - Z.Cols();	// Number of linear constraints
	int m2 = cf.Size();
	Vector<double> xCopy = x.Copy();
	
	/* Compute normalized step sizes */
	Vector<double> hZ(n-m1);
	ComputeDerivStep(hZ,xCopy,Z,Step,Type,TypX);
					
	/* Central function value */
	if(Func(xCopy,f,cf)) return 1;
	if(IsNan(f)) return 2;
	if(IsNan(cf)) return 7;
			
	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */		
		double fp;
		Vector<double> cfp(m2);
		
		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			if(Func(xCopy,fp,cfp)) return 5;
			if(IsNan(fp)) return 6;
			if(IsNan(cfp)) return 9;
			for(int j=0; j<n; j++) xCopy[j] -= Z(j,i) * hZ[i];
					
			/* Compute first derivatives */
			df[i]  = (fp  - f) / hZ[i];
			for(int j=0; j<m2; j++)
				dcf(j,i) = (cfp[j] - cf[j]) / hZ[i];
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fm;
		Vector<double> cfm(m2);
			
		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] -= Z(j,i) * hZ[i];
			if(Func(xCopy,fm,cfm)) return 3;
			if(IsNan(fm)) return 4;
			if(IsNan(cfm)) return 8;
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			
			/* Compute first derivatives */
			df[i]  = (f - fm) / hZ[i];
			for(int j=0; j<m2; j++)
				dcf(j,i) = (cf[j] - cfm[j]) / hZ[i];
		}
	}
	else
	{
		/* Central difference derivatives */
		double fm,fp;
		Vector<double> cfm(m2);
		Vector<double> cfp(m2);
					
		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] -= Z(j,i) * hZ[i];
			if(Func(xCopy,fm,cfm)) return 3;
			if(IsNan(fm)) return 4;
			if(IsNan(cfm)) return 8;
							
			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] += Two * Z(j,i) * hZ[i];
			if(Func(xCopy,fp,cfp)) return 5;
			if(IsNan(fp)) return 6;
			if(IsNan(cfp)) return 9;
			for(int j=0; j<n; j++) xCopy[j] -= Z(j,i) * hZ[i];
				
			/* Compute first derivatives */
			df[i] = (fp - fm) / (Two * hZ[i]);
			for(int j=0; j<m2; j++)
				dcf(j,i) = (cfp[j] - cfm[j]) / (Two * hZ[i]);
		}
	}
		
	return 0;
}

#endif