
#ifndef _Opt_Hessian3
#define _Opt_Hessian3

/*

Utility for computing the jacobian of a function.

Arguments

	Func:			A pointer to a function
	x:				Point at which to evaluate derivatives
	Z:				Projection matrix
	f:				Return function value
	df:				Return derivative value
	Step:			Step size for numerical derivatives
	Type:			Type of derivative approximation to use
					1	-	Forward  difference finite difference derivatives, absolute step
					2	-	Backward difference finite difference derivatives, absolute step
					3	-	Central  difference finite difference derivatives, absolute step
					4	-	Forward  difference finite difference derivatives, relative step
					5	-	Backward difference finite difference derivatives, relative step
					6	-	Central  difference finite difference derivatives, relative step
	PrintLevel:
	TypX:			Vector containing scales of x

*/

	/* Compute the hessian of a scalar-valued function */
	int Hessian(int (*Func)(const Vector<double> &, double &),
				const Vector<double> &x,
				double &f,
				Vector<double> &df,
				SymmetricMatrix<double> &ddf,
				double Step,
				int Type,
				const Vector<double> &TypX);

	/* Compute the projected hessian of a scalar-valued function */
	int Hessian(int (*Func)(const Vector<double> &, double &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				double &f,
				Vector<double> &df,
				SymmetricMatrix<double> &ddf,
				double Step,
				int Type,
				const Vector<double> &TypX);

	/* Compute the hessian of a vector-valued function */
	int Hessian(int (*Func)(const Vector<double> &, Vector<double> &),
				const Vector<double> &x,
				Vector<double> &f,
				Vector< Vector<double> > &df,
				Vector< SymmetricMatrix<double> > &ddf,
				double Step,
				int Type,
				const Vector<double> &TypX);

	/* Compute the projected hessian of a vector-valued function */
	int Hessian(int (*Func)(const Vector<double> &, Vector<double> &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				Vector<double> &f,
				Vector< Vector<double> > &df,
				Vector< SymmetricMatrix<double> > &ddf,
				double Step,
				int Type,
				const Vector<double> &TypX);

	/* Compute the hessian of a scalar-valued and vector-valued function */
	int Hessian(int (*Func)(const Vector<double> &, double &, Vector<double> &),
				const Vector<double> &x,
				double &f,
				Vector<double> &df,
				SymmetricMatrix<double> &ddf,
				Vector<double> &cf,
				Matrix<double> &dcf,
				Vector< SymmetricMatrix<double> > &ddcf,
				double Step,
				int Type,
				const Vector<double> &TypX);

	/* Compute the projected hessian of a scalar-valued and vector-valued function */
	int Hessian(int (*Func)(const Vector<double> &, double &, Vector<double> &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				double &f,
				Vector<double> &df,
				SymmetricMatrix<double> &ddf,
				Vector<double> &cf,
				Matrix<double> &dcf,
				Vector< SymmetricMatrix<double> > &ddcf,
				double Step,
				int Type,
				const Vector<double> &TypX);

	/* Simple Hessian */
	SymmetricMatrix<double> Hessian(double (*Func)(const Vector<double> &), Vector<double> &x, double h, int Type);



int Hessian(int (*Func)(const Vector<double> &, double &),
				const Vector<double> &x,
				double &f,
				Vector<double> &df,
				SymmetricMatrix<double> &ddf,
				double Step,
				int Type,
				const Vector<double> &TypX)
{
	/* Check dimensions */
	int n = x.Size();		// Number of variables
	Vector<double> xCopy = x.Copy();
	if(df.Size() != n || ddf.Dim() != n)
	{
		Print("Error in Hessian");
		Print("Incompadible Dimensions");
		Print("x.Size()",x.Size());
		Print("df.Size()",df.Size());
		Print("ddf.Size()",ddf.Size());
		Print("TypX.Size()",TypX.Size());
		HaltExecution();
	}

	/* Compute normalized step sizes */
	Vector<double> hZ(n);	hZ.Name = "Hessian::hZ";
	ComputeDerivStep(hZ,xCopy,Step,Type,TypX);

	/* Central function value */
	if(Func(xCopy,f)) return 1;

	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */
		double fpp;
		Vector<double> fp(n);			fp.Name   = "Hessian::fp";

		for(int i=0; i<n; i++)
		{
			/* Compute f(x + h * Z(j)) */
			xCopy[i] += hZ[i];
			if(Func(xCopy,fp[i])) return 1;

			/* Compute f(x + 2 * h * Z(j) */
			xCopy[i] += hZ[i];
			if(Func(xCopy,fpp)) return 1;
			xCopy[i] -= 2.0 * hZ[i];

			/* Compute first derivatives */
			df[i]  = (fp[i]  - f) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - 2.0 * fp[i] + f) / Sqr(hZ[i]);
		}

		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x + h * Z(i) + h * Z(j)) */
				xCopy[i] += hZ[i];
				xCopy[j] += hZ[j];
				if(Func(xCopy,fpp)) return 1;
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fp[i] - fp[j] + f) / (hZ[i] * hZ[j]);
			}
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fmm;
		Vector<double> fm(n);
		fm.Name = "Hessian::fm";

		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fm[i])) return 1;

			/* Compute f(x - 2 * h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fmm)) return 1;
			xCopy[i] += 2.0 * hZ[i];

			/* Compute first derivatives */
			df[i]  = (f - fm[i]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (f - fm[i] + fmm) / Sqr(hZ[i]);
		}

		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];
				if(Func(xCopy,fmm)) return 1;
				xCopy[i] += hZ[i];
				xCopy[j] += hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (f - fm[i] - fm[j] + fmm) / (hZ[i] * hZ[j]);
			}
		}
	}
	else
	{
		/* Central difference derivatives */
		double fmm,fmp,fpm,fpp;

		for(int i=0; i<n; i++)
		{
			/* Compute f(x - 2 * h * Z(j)) */
			xCopy[i] -= 2.0 * hZ[i];
			if(Func(xCopy,fmm)) return 1;

			/* Compute f(x + 2 * h * Z(j)) */
			xCopy[i] += 4.0 * hZ[i];
			if(Func(xCopy,fpp)) return 1;
			xCopy[i] -= 2.0 * hZ[i];

			/* Compute first derivatives */
			df[i] = (fpp - fmm) / (4.0 * hZ[i]);

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - Two * f + fmm) / (Four * hZ[i] * hZ[i]);
		}

		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];
				if(Func(xCopy,fmm)) return 1;

				/* Compute f(x - h * Z(i) + h * Z(j)) */
				xCopy[j] += Two * hZ[j];
				if(Func(xCopy,fmp)) return 1;

				/* Compute f(x + h * Z(i) + h * Z(j)) */
				xCopy[i] += Two * hZ[i];
				if(Func(xCopy,fpp)) return 1;

				/* Compute f(x + h * Z(i) - h * Z(j)) */
				xCopy[j] -= Two * hZ[j];
				if(Func(xCopy,fpm)) return 1;
				xCopy[i] -= hZ[i];
				xCopy[j] += hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fpm - fmp + fmm) / (Four * hZ[i] * hZ[j]);
			}
		}
	}

	return 0;
}


int Hessian(int (*Func)(const Vector<double> &, double &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				double &f,
				Vector<double> &df,
				SymmetricMatrix<double> &ddf,
				double Step,
				int Type,
				const Vector<double> &TypX)
{
	/* Check dimensions */
	int n = x.Size();		// Number of variables
	int m1 = n - Z.Cols();	// Number of linear constraints
	Vector<double> xCopy = x.Copy();

	/*
	if(Z.Cols() != n || df.Size() != m || df.Cols() != n || TypX.Size() != n)
	{
		Print("Error in Hessian3");
		Print("Incompadible dimensions");
		Print("n",n);
		Print("m",m);
		Print("r",r);
		Print("Z.Size()",Z.Size());
		Print("df.Size()",df.Size());
		Print("TypX.Size()",TypX.Size());
		Print("Halting execution");
		exit(0);
	}
	*/

	/* Compute normalized step sizes */
	Vector<double> hZ(n-m1);	hZ.Name = "Hessian::hZ";
	ComputeDerivStep(hZ,xCopy,Z,Step,Type,TypX);

	/* Central function value */
	if(Func(x,f)) return 1;

	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */
		double fpp;
		Vector<double> fp(n-m1);			fp.Name   = "Hessian::fp";

		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			if(Func(xCopy,fp[i])) return 1;

			/* Compute f(x + 2 * h * Z(j) */
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			if(Func(xCopy,fpp)) return 1;
			for(int j=0; j<n; j++) xCopy[j] -= Two * Z(j,i) * hZ[i];

			/* Compute first derivatives */
			df[i]  = (fp[i]  - f) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - Two * fp[i] + f) / Sqr(hZ[i]);
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x + h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] += Z(k,j) * hZ[j];
				if(Func(xCopy,fpp)) return 1;
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fp[i] - fp[j] + f) / (hZ[i] * hZ[j]);
			}
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fmm;
		Vector<double> fm(n-m1);
		fm.Name = "Hessian::fm";

		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
			if(Func(xCopy,fm[i])) return 1;

			/* Compute f(x - 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
			if(Func(xCopy,fmm)) return 1;
			for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,i) * hZ[i];

			/* Compute first derivatives */
			df[i]  = (f - fm[i]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (f - fm[i] + fmm) / Sqr(hZ[i]);
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];
				if(Func(xCopy,fmm)) return 1;
				for(int k=0; k<n; k++) xCopy[k] += Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] += Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (f - fm[i] - fm[j] + fmm) / (hZ[i] * hZ[j]);
			}
		}
	}
	else
	{
		/* Central difference derivatives */

		double fmm,fmp,fpm,fpp;

		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,i) * hZ[i];
			if(Func(xCopy,fmm)) return 1;

			/* Compute f(x + 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] += Four * Z(k,i) * hZ[i];
			if(Func(xCopy,fpp)) return 1;
			for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,i) * hZ[i];

			/* Compute first derivatives */
			df[i] = (fpp - fmm) / (Four * hZ[i]);

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - Two * f + fmm) / (Four * hZ[i] * hZ[i]);
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];
				if(Func(xCopy,fmm)) return 1;

				/* Compute f(x - h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,j) * hZ[j];
				if(Func(xCopy,fmp)) return 1;

				/* Compute f(x + h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,i) * hZ[i];
				if(Func(xCopy,fpp)) return 1;

				/* Compute f(x + h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,j) * hZ[j];
				if(Func(xCopy,fpm)) return 1;
				for(int k=0; k<n; k++) xCopy[k] -=  Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] +=  Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fpm - fmp + fmm) / (Four * hZ[i] * hZ[j]);
			}
		}
	}

	return 0;
}

/* Compute the hessian of a vector-valued function */
int Hessian(int (*Func)(const Vector<double> &, Vector<double> &),
			const Vector<double> &x,
			Vector<double> &f,
			Vector< Vector<double> > &df,
			Vector< SymmetricMatrix<double> > &ddf,
			double Step,
			int Type,
			const Vector<double> &TypX)
{
	/* Check dimensions */
	int n  = x.Size();	// Number of variables
	int m2 = f.Size();
	Vector<double> xCopy = x.Copy();

	/* Compute normalized step sizes */
	Vector<double> hZ(n);
	hZ.Name = "Hessian::hZ";
	ComputeDerivStep(hZ,xCopy,Step,Type,TypX);

	/* Central function value */
	if(Func(xCopy,f)) return 1;

	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */
		Vector<double> fpp(m2);			fpp.Name = "Hessian::fpp";
		Vector< Vector<double> > fp(n);	fp.Name  = "Hessian::fp";
		for(int i=0; i<n; i++)
		{
			fp(i) = Vector<double>(m2);
			fp.Name = "Hessian::fp_" + ToString(i);
		}
		for(int i=0; i<n; i++)
		{
			/* Compute f(x + h * Z(j)) */
			xCopy(i) += hZ(i);
			if(Func(xCopy,fp(i))) return 1;

			/* Compute f(x + 2 * h * Z(j) */
			xCopy(i) += hZ(i);
			if(Func(xCopy,fpp)) return 1;
			xCopy(i) -= 2.0 * hZ(i);

			/* Compute first derivatives */
			for(int k=0; k<m2; k++) df(k)(i) = (fp(i)(k) - f(k)) / hZ(i);

			/* Compute diagonal portion of second derivatives */
			for(int k=0; k<m2; k++) ddf(k)(i,i) = (fpp(k) - 2.0 * fp(i)(k) + f(k)) / Sqr(hZ(i));
		}
		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x + h * Z(i) + h * Z(j)) */
				xCopy(i) += hZ(i);
				xCopy(j) += hZ(j);
				if(Func(xCopy,fpp)) return 1;
				xCopy(i) -= hZ(i);
				xCopy(j) -= hZ(j);

				/* Compute off-diagonal portion of second derivatives */
				for(int k=0; k<m2; k++) ddf(k)(i,j) = (fpp(k) - fp(i)(k) - fp(j)(k) + f(k)) / (hZ(i) * hZ(j));
			}
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		Vector<double> fmm(m2);			fmm.Name = "Hessian::fmm";
		Vector< Vector<double> > fm(n);	fm.Name  = "Hessian::fm";
		for(int i=0; i<n; i++)
		{
			fm(i)   = Vector<double>(m2);
			fm.Name = "Hessian::fm_" + ToString(i);
		}

		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			xCopy(i) -= hZ(i);
			if(Func(xCopy,fm(i))) return 1;

			/* Compute f(x - 2 * h * Z(j)) */
			xCopy(i) -= hZ(i);
			if(Func(xCopy,fmm)) return 1;
			xCopy(i) += 2.0 * hZ(i);

			/* Compute first derivatives */
			for(int k=0; k<m2; k++) df(k)(i) = (f(k) - fm(i)(k)) / hZ(i);

			/* Compute diagonal portion of second derivatives */
			for(int k=0; k<m2; k++) ddf[k](i,i) = (f(k) - fm(i)(k) + fmm(k)) / Sqr(hZ(i));
		}

		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				xCopy(i) -= hZ(i);
				xCopy(j) -= hZ(j);
				if(Func(xCopy,fmm)) return 1;
				xCopy(i) += hZ(i);
				xCopy(j) += hZ(j);

				/* Compute off-diagonal portion of second derivatives */
				for(int k=0; k<m2; k++) ddf(k)(i,j) = (f(k) - fm(i)(k) - fm(j)(k) + fmm(k)) / (hZ(i) * hZ(j));
			}
		}
	}
	else
	{
		/* Central difference derivatives */
		Vector<double> fmm(m2);
		Vector<double> fmp(m2);
		Vector<double> fpm(m2);
		Vector<double> fpp(m2);

		for(int i=0; i<n; i++)
		{
			/* Compute f(x - 2 * h * Z(j)) */
			xCopy[i] -= Two * hZ[i];
			if(Func(xCopy,fmm)) return 1;

			/* Compute f(x + 2 * h * Z(j)) */
			xCopy[i] += Four * hZ[i];
			if(Func(xCopy,fpp)) return 1;
			xCopy[i] -= Two * hZ[i];

			/* Compute first derivatives */
			for(int k=0; k<m2; k++) df[k][i] = (fpp[k] - fmm[k]) / (Four * hZ[i]);

			/* Compute diagonal portion of second derivatives */
			for(int k=0; k<m2; k++) ddf[k](i,i) = (fpp[k] - Two * f[k] + fmm[k]) / (Four * Sqr(hZ[i]));
		}

		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];
				if(Func(xCopy,fmm)) return 1;

				/* Compute f(x - h * Z(i) + h * Z(j)) */
				xCopy[j] += Two * hZ[j];
				if(Func(xCopy,fmp)) return 1;

				/* Compute f(x + h * Z(i) + h * Z(j)) */
				xCopy[i] += Two * hZ[i];
				if(Func(xCopy,fpp)) return 1;

				/* Compute f(x + h * Z(i) - h * Z(j)) */
				xCopy[j] -= Two * hZ[j];
				if(Func(x,fpm)) return 1;
				xCopy[i] -= hZ[i];
				xCopy[j] += hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				for(int k=0; k<m2; k++) ddf[k](i,j) = (fpp[k] - fpm[k] - fmp[k] + fmm[k]) / (Four * hZ[i] * hZ[j]);
			}
		}
	}

	return 0;
}

int Hessian(int (*Func)(const Vector<double> &, Vector<double> &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				Vector<double> &f,
				Vector< Vector<double> > &df,
				Vector< SymmetricMatrix<double> > &ddf,
				double Step,
				int Type,
				const Vector<double> &TypX)
{
	/* Check dimensions */
	int n  = x.Size();		// Number of variables
	int m1 = n - Z.Cols();	// Number of linear constraints
	int m2 = f.Size();
	Vector<double> xCopy = x.Copy();

	/* Compute normalized step sizes */
	Vector<double> hZ(n-m1);	hZ.Name = "Hessian2::hZ";
	ComputeDerivStep(hZ,xCopy,Z,Step,Type,TypX);

	/* Central function value */
	if(Func(xCopy,f)) return 1;

	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */
		Vector<double> fpp(m2);				fpp.Name = "Hessian::fpp";
		Vector< Vector<double> > fp(n-m1);	fp.Name  = "Hessian::fp";
		for(int i=0; i<n-m1; i++)
		{
			fp[i] = Vector<double>(m2);
			fp.Name = "Hessian::fp_" + ToString(i);
		}
		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			if(Func(xCopy,fp[i])) return 1;

			/* Compute f(x + 2 * h * Z(j) */
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			if(Func(xCopy,fpp)) return 1;
			for(int j=0; j<n; j++) xCopy[j] -= Two * Z(j,i) * hZ[i];

			/* Compute first derivatives */
			for(int k=0; k<m2; k++) df[k][i] = (fp[i][k] - f[k]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			for(int k=0; k<m2; k++) ddf[k](i,i) = (fpp[k] - Two * fp[i][k] + f[k]) / Sqr(hZ[i]);
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x + h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] += Z(k,j) * hZ[j];
				if(Func(xCopy,fpp)) return 1;
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				for(int k=0; k<m2; k++) ddf[k](i,j) = (fpp[k] - fp[i][k] - fp[j][k] + f[k]) / (hZ[i] * hZ[j]);
			}
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		Vector<double> fmm(m2);				fmm.Name = "Hessian::fmm";
		Vector< Vector<double> > fm(n-m1);	fm.Name  = "Hessian::fm";
		for(int i=0; i<n-m1; i++)
		{
			fm[i] = Vector<double>(m2);
			fm.Name = "Hessian::dm_" + ToString(i);
		}

		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
			if(Func(xCopy,fm[i])) return 1;

			/* Compute f(x - 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
			if(Func(xCopy,fmm)) return 1;
			for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,i) * hZ[i];

			/* Compute first derivatives */
			for(int k=0; k<m2; k++) df[k][i] = (f[k] - fm[i][k]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			for(int k=0; k<m2; k++) ddf[k](i,i) = (f[k] - fm[i][k] + fmm[k]) / Sqr(hZ[i]);
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];
				if(Func(xCopy,fmm)) return 1;
				for(int k=0; k<n; k++) xCopy[k] += Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] += Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				for(int k=0; k<m2; k++) ddf[k](i,j) = (f[k] - fm[i][k] - fm[j][k] + fmm[k]) / (hZ[i] * hZ[j]);
			}
		}
	}
	else
	{
		/* Central difference derivatives */

		Vector<double> fmm(m2);
		Vector<double> fmp(m2);
		Vector<double> fpm(m2);
		Vector<double> fpp(m2);

		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,i) * hZ[i];
			if(Func(xCopy,fmm)) return 1;

			/* Compute f(x + 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] += Four * Z(k,i) * hZ[i];
			if(Func(xCopy,fpp)) return 1;
			for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,i) * hZ[i];

			/* Compute first derivatives */
			for(int k=0; k<m2; k++) df[k][i] = (fpp[k] - fmm[k]) / (Four * hZ[i]);

			/* Compute diagonal portion of second derivatives */
			for(int k=0; k<m2; k++) ddf[k](i,i) = (fpp[k] - Two * f[k] + fmm[k]) / (Four * Sqr(hZ[i]));
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];
				if(Func(xCopy,fmm)) return 1;

				/* Compute f(x - h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,j) * hZ[j];
				if(Func(xCopy,fmp)) return 1;

				/* Compute f(x + h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,i) * hZ[i];
				if(Func(xCopy,fpp)) return 1;

				/* Compute f(x + h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,j) * hZ[j];
				if(Func(xCopy,fpm)) return 1;
				for(int k=0; k<n; k++) xCopy[k] -=  Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] +=  Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				for(int k=0; k<m2; k++) ddf[k](i,j) = (fpp[k] - fpm[k] - fmp[k] + fmm[k]) / (Four * hZ[i] * hZ[j]);
			}
		}
	}

	return 0;
}

/* Compute the hessian of a scalar-valued and vector-valued function */
int Hessian(int (*Func)(const Vector<double> &, double &, Vector<double> &),
			const Vector<double> &x,
			double &f,
			Vector<double> &df,
			SymmetricMatrix<double> &ddf,
			Vector<double> &cf,
			Matrix<double> &dcf,
			Vector< SymmetricMatrix<double> > &ddcf,
			double Step,
			int Type,
			const Vector<double> &TypX)
{
	/* Check dimensions */
	int n  = x.Size();		// Number of variables
	int m2 = cf.Size();
	Vector<double> xCopy = x.Copy();

	/* Compute normalized step sizes */
	Vector<double> hZ(n);	hZ.Name = "Hessian::hZ";
	ComputeDerivStep(hZ,xCopy,Step,Type,TypX);

	/* Central function value */
	if(Func(xCopy,f,cf)) return 1;

	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */
		double fpp;
		Vector<double> fp(n);				fp.Name   = "Hessian::fp";
		Vector<double> cfpp(m2);			cfpp.Name = "Hessian::cfpp";
		Vector< Vector<double> > cfp(n);	cfp.Name  = "Hessian::cfp";
		for(int i=0; i<n; i++)
		{
			cfp[i] = Vector<double>(m2);
			cfp.Name = "Hessian::cdp_" + ToString(i);
		}
		for(int i=0; i<n; i++)
		{
			/* Compute f(x + h * Z(j)) */
			xCopy[i] += hZ[i];
			if(Func(xCopy,fp[i],cfp[i])) return 1;

			/* Compute f(x + 2 * h * Z(j) */
			xCopy[i] += hZ[i];
			if(Func(xCopy,fpp,cfpp)) return 1;
			xCopy[i] -= Two * hZ[i];

			/* Compute first derivatives */
			df[i]  = (fp[i]  - f) / hZ[i];
			for(int k=0; k<m2; k++)
				dcf(k,i) = (cfp[i][k] - cf[k]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - Two * fp[i] + f) / Sqr(hZ[i]);
			for(int k=0; k<m2; k++) ddcf[k](i,i) = (cfpp[k] - Two * cfp[i][k] + cf[k]) / Sqr(hZ[i]);
		}
		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x + h * Z(i) + h * Z(j)) */
				xCopy[i] += hZ[i];
				xCopy[j] += hZ[j];
				if(Func(xCopy,fpp,cfpp)) return 1;
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fp[i] - fp[j] + f) / (hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddcf[k](i,j) = (cfpp[k] - cfp[i][k] - cfp[j][k] + cf[k]) / (hZ[i] * hZ[j]);
			}
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fmm;
		Vector<double> fm(n);				fm.Name   = "Hessian::fm";
		Vector<double> cfmm(m2);			cfmm.Name = "Hessian::cfmm";
		Vector< Vector<double> > cfm(n);	cfm.Name  = "Hessian::cfm";
		for(int i=0; i<n; i++)
		{
			cfm[i] = Vector<double>(m2);
			cfm.Name = "Hessian::cdm_" + ToString(i);
		}

		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fm[i],cfm[i])) return 1;

			/* Compute f(x - 2 * h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fmm,cfmm)) return 1;
			xCopy[i] += Two * hZ[i];

			/* Compute first derivatives */
			df[i]  = (f - fm[i]) / hZ[i];
			for(int k=0; k<m2; k++) dcf(k,i) = (cf[k] - cfm[i][k]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (f - fm[i] + fmm) / Sqr(hZ[i]);
			for(int k=0; k<m2; k++) ddcf[k](i,i) = (cf[k] - cfm[i][k] + cfmm[k]) / Sqr(hZ[i]);
		}

		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];
				if(Func(xCopy,fmm,cfmm)) return 1;
				xCopy[i] += hZ[i];
				xCopy[j] += hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (f - fm[i] - fm[j] + fmm) / (hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddcf[k](i,j) = (cf[k] - cfm[i][k] - cfm[j][k] + cfmm[k]) / (hZ[i] * hZ[j]);
			}
		}
	}
	else
	{
		/* Central difference derivatives */
		double fmm,fmp,fpm,fpp;
		Vector<double> cfmm(m2);
		Vector<double> cfmp(m2);
		Vector<double> cfpm(m2);
		Vector<double> cfpp(m2);

		for(int i=0; i<n; i++)
		{
			/* Compute f(x - 2 * h * Z(j)) */
			xCopy[i] -= Two * hZ[i];
			if(Func(xCopy,fmm,cfmm)) return 1;

			/* Compute f(x + 2 * h * Z(j)) */
			xCopy[i] += Four * hZ[i];
			if(Func(xCopy,fpp,cfpp)) return 1;
			xCopy[i] -= Two * hZ[i];

			/* Compute first derivatives */
			df[i] = (fpp - fmm) / (Four * hZ[i]);
			for(int k=0; k<m2; k++) dcf(k,i) = (cfpp[k] - cfmm[k]) / (Four * hZ[i]);

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - Two * f + fmm) / (Four * hZ[i] * hZ[i]);
			for(int k=0; k<m2; k++) ddcf[k](i,i) = (cfpp[k] - Two * cf[k] + cfmm[k]) / (Four * Sqr(hZ[i]));
		}

		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];
				if(Func(xCopy,fmm,cfmm)) return 1;

				/* Compute f(x - h * Z(i) + h * Z(j)) */
				xCopy[j] += Two * hZ[j];
				if(Func(xCopy,fmp,cfmp)) return 1;

				/* Compute f(x + h * Z(i) + h * Z(j)) */
				xCopy[i] += Two * hZ[i];
				if(Func(xCopy,fpp,cfpp)) return 1;

				/* Compute f(x + h * Z(i) - h * Z(j)) */
				xCopy[j] -= Two * hZ[j];
				if(Func(xCopy,fpm,cfpm)) return 1;
				xCopy[i] -= hZ[i];
				xCopy[j] += hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fpm - fmp + fmm) / (Four * hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddcf[k](i,j) = (cfpp[k] - cfpm[k] - cfmp[k] + cfmm[k]) / (Four * hZ[i] * hZ[j]);
			}
		}
	}

	return 0;
}

int Hessian(int (*Func)(const Vector<double> &, double &, Vector<double> &),
				const Vector<double> &x,
				const Matrix<double> &Z,
				double &f,
				Vector<double> &df,
				SymmetricMatrix<double> &ddf,
				Vector<double> &cf,
				Matrix<double> &dcf,
				Vector< SymmetricMatrix<double> > &ddcf,
				double Step,
				int Type,
				const Vector<double> &TypX)
{
	/* Check dimensions */
	int n  = x.Size();		// Number of variables
	int m1 = n - Z.Cols();	// Number of linear constraints
	int m2 = cf.Size();
	Vector<double> xCopy = x.Copy();

	/* Compute normalized step sizes */
	Vector<double> hZ(n-m1);	hZ.Name = "Hessian::hZ";
	ComputeDerivStep(hZ,xCopy,Z,Step,Type,TypX);

	/* Central function value */
	if(Func(x,f,cf)) return 1;

	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */
		double fpp;
		Vector<double> fp(n-m1);			fp.Name   = "Hessian::fp";
		Vector<double> cfpp(m2);			cfpp.Name = "Hessian::cfpp";
		Vector< Vector<double> > cfp(n-m1);	cfp.Name  = "Hessian::cfp";
		for(int i=0; i<n-m1; i++)
		{
			cfp[i] = Vector<double>(m2);
			cfp.Name = "Hessian2::cdp_" + ToString(i);
		}
		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			if(Func(xCopy,fp[i],cfp[i])) return 1;

			/* Compute f(x + 2 * h * Z(j) */
			for(int j=0; j<n; j++) xCopy[j] += Z(j,i) * hZ[i];
			if(Func(xCopy,fpp,cfpp)) return 1;
			for(int j=0; j<n; j++) xCopy[j] -= Two * Z(j,i) * hZ[i];

			/* Compute first derivatives */
			df[i]  = (fp[i]  - f) / hZ[i];
			for(int k=0; k<m2; k++) dcf(k,i) = (cfp[i][k] - cf[k]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - 2.0 * fp[i] + f) / Sqr(hZ[i]);
			for(int k=0; k<m2; k++) ddcf[k](i,i) = (cfpp[k] - Two * cfp[i][k] + cf[k]) / Sqr(hZ[i]);
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x + h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] += Z(k,j) * hZ[j];
				if(Func(xCopy,fpp,cfpp)) return 1;
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fp[i] - fp[j] + f) / (hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddcf[k](i,j) = (cfpp[k] - cfp[i][k] - cfp[j][k] + cf[k]) / (hZ[i] * hZ[j]);
			}
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fmm;
		Vector<double> fm(n-m1);			fm.Name   = "Hessian::fm";
		Vector<double> cfmm(m2);			cfmm.Name = "Hessian::cfmm";
		Vector< Vector<double> > cfm(n-m1);	cfm.Name  = "Hessian::cfm";
		for(int i=0; i<n-m1; i++)
		{
			cfm[i] = Vector<double>(m2);
			cfm.Name = "Hessian::cdm_" + ToString(i);
		}

		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
			if(Func(xCopy,fm[i],cfm[i])) return 1;

			/* Compute f(x - 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
			if(Func(xCopy,fmm,cfmm)) return 1;
			for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,i) * hZ[i];

			/* Compute first derivatives */
			df[i]  = (f - fm[i]) / hZ[i];
			for(int k=0; k<m2; k++) dcf(k,i) = (cf[k] - cfm[i][k]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (f - fm[i] + fmm) / Sqr(hZ[i]);
			for(int k=0; k<m2; k++) ddcf[k](i,i) = (cf[k] - cfm[i][k] + cfmm[k]) / Sqr(hZ[i]);
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];
				if(Func(xCopy,fmm,cfmm)) return 1;
				for(int k=0; k<n; k++) xCopy[k] += Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] += Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (f - fm[i] - fm[j] + fmm) / (hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddcf[k](i,j) = (cf[k] - cfm[i][k] - cfm[j][k] + cfmm[k]) / (hZ[i] * hZ[j]);
			}
		}
	}
	else
	{
		/* Central difference derivatives */

		double fmm,fmp,fpm,fpp;
		Vector<double> cfmm(m2);
		Vector<double> cfmp(m2);
		Vector<double> cfpm(m2);
		Vector<double> cfpp(m2);

		for(int i=0; i<n-m1; i++)
		{
			/* Compute f(x - 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,i) * hZ[i];
			if(Func(xCopy,fmm,cfmm)) return 1;

			/* Compute f(x + 2 * h * Z(j)) */
			for(int k=0; k<n; k++) xCopy[k] += Four * Z(k,i) * hZ[i];
			if(Func(xCopy,fpp,cfpp)) return 1;
			for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,i) * hZ[i];

			/* Compute first derivatives */
			df[i] = (fpp - fmm) / (Four * hZ[i]);
			for(int k=0; k<m2; k++) dcf(k,i) = (cfpp[k] - cfmm[k]) / (Four * hZ[i]);

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - Two * f + fmm) / (Four * hZ[i] * hZ[i]);
			for(int k=0; k<m2; k++) ddcf[k](i,i) = (cfpp[k] - Two * cf[k] + cfmm[k]) / (Four * Sqr(hZ[i]));
		}

		for(int i=0; i<n-m1; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] -= Z(k,j) * hZ[j];
				if(Func(xCopy,fmm,cfmm)) return 1;

				/* Compute f(x - h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,j) * hZ[j];
				if(Func(xCopy,fmp,cfmp)) return 1;

				/* Compute f(x + h * Z(i) + h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] += Two * Z(k,i) * hZ[i];
				if(Func(xCopy,fpp,cfpp)) return 1;

				/* Compute f(x + h * Z(i) - h * Z(j)) */
				for(int k=0; k<n; k++) xCopy[k] -= Two * Z(k,j) * hZ[j];
				if(Func(xCopy,fpm,cfpm)) return 1;
				for(int k=0; k<n; k++) xCopy[k] -=  Z(k,i) * hZ[i];
				for(int k=0; k<n; k++) xCopy[k] +=  Z(k,j) * hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fpm - fmp + fmm) / (Four * hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddcf[k](i,j) = (cfpp[k] - cfpm[k] - cfmp[k] + cfmm[k]) / (Four * hZ[i] * hZ[j]);
			}
		}
	}

	return 0;
}

/* Simple Hessian */

SymmetricMatrix<double> Hessian(double (*Func)(const Vector<double> &), Vector<double> &x, double h, int Type)
{

	int n = x.Size();
	double f = Func(x);
	Vector<double> df(n);
	SymmetricMatrix<double> ddf(n);
	Vector<double> vh(n);
	if(Type == 4 || Type == 5 || Type == 6) for(int i=0; i<n; i++) vh[i] = h;
	else for(int i=0; i<n; i++)
	{
		vh[i] = h * fabs(x[i]);
		if(vh[i] == Zero) vh[i] = h;
	}

	if(Type == 1 || Type == 4)		// Forward Difference
	{
		double fpp;
		Vector<double> fp(n);
		for(int i=0; i<n; i++)
		{
			x[i] += vh[i];
			fp[i] = Func(x);
			x[i] -= vh[i];
			df[i] = (fp[i] - f) / vh[i];
			x[i] += Two * vh[i];
			fpp = Func(x);
			x[i] -= Two * vh[i];
			ddf(i,i) = (fpp - Two * fp[i] + f) / (vh[i] * vh[i]);
		}
		for(int i=0; i<n-1; i++)
		{
			for(int j=i+1; j<n; j++)
			{
				x[i] += vh[i];
				x[j] += vh[j];
				fpp = Func(x);
				x[i] -= vh[i];
				x[j] -= vh[j];
				ddf(i,j) = (fpp - fp[i] - fp[j] + f) / (vh[i] * vh[j]);
				ddf(j,i) = (fpp - fp[i] - fp[j] + f) / (vh[i] * vh[j]);
			}
		}
	}
	if(Type == 2 || Type == 5)		// Backward Difference
	{
		double fmm;
		Vector<double> fm(n);
		for(int i=0; i<n; i++)
		{
			x[i] -= vh[i];
			fm[i] = Func(x);
			x[i] += vh[i];
			df[i] = (f - fm[i]) / vh[i];
			x[i] -= Two * vh[i];
			fmm = Func(x);
			x[i] += Two * vh[i];
			ddf(i,i) = (f - Two * fm[i] + fmm) / (vh[i] * vh[i]);
		}
		for(int i=1; i<n-1; i++)
		{
			for(int j=i+1; j<n; j++)
			{
				x[i] -= vh[i];
				x[j] -= vh[j];
				fmm = Func(x);
				x[i] += vh[i];
				x[j] += vh[j];
				ddf(i,j) = (f - fm[i] - fm[j] + fmm) / (vh[i] * vh[j]);
				ddf(j,i) = (f - fm[i] - fm[j] + fmm) / (vh[i] * vh[j]);
			}
		}
	}
	else if(Type == 3 || Type == 6)		// Central Difference
	{
		Vector<double> fpp(n);
		Vector<double> fmm(n);
		for(int i=0; i<n; i++)
		{
			x[i] -= Two * vh[i];
			fmm[i] = Func(x);
			x[i] += Four * vh[i];
			fpp[i] = Func(x);
			x[i] -= Two * vh[i];
			df[i] = (fpp[i] - fmm[i]) / (Four * vh[i]);							// Compute First Derivatives
			ddf(i,i) = (fpp[i] - Two * f + fmm[i]) / (Four * vh[i] * vh[i]);	// Compute Second Derivatives
		}
		// Compute Cross Derivatives:
		double pp,pm,mp,mm;
		for(int i=0; i<n-1; i++)
		{
			for(int j=i+1; j<n; j++)
			{
				x[i] -= vh[i];
				x[j] -= vh[j];
				mm = Func(x);
				x[j] += Two * vh[j];
				mp = Func(x);
				x[i] += Two * vh[i];
				pp = Func(x);
				x[j] -= Two * vh[j];
				pm = Func(x);
				x[i] -= vh[i];
				x[j] += vh[j];
				ddf(i,j) = (pp - pm - mp + mm) / (Four * vh[i] * vh[j]);
				ddf(j,i) = (pp - pm - mp + mm) / (Four * vh[i] * vh[j]);
			}
		}
	}

	return ddf;
}

/* Compute the hessian of the lagrangian */
int Hessian(int (*Func)(const Vector<double> &, double &, Vector<double> &),
			const Vector<double> &x,
			const Vector<double> &lambda,
			double &f,
			Vector<double> &df,
			SymmetricMatrix<double> &ddf,
			Vector<double> &cf,
			Matrix<double> &dcf,
			double Step,
			int Type,
			const Vector<double> &TypX)
{
	/* Check dimensions */
	int n  = x.Size();		// Number of variables
	int m2 = cf.Size();
	Vector<double> xCopy = x.Copy();

	/* Compute normalized step sizes */
	Vector<double> hZ(n);	hZ.Name = "Hessian::hZ";
	ComputeDerivStep(hZ,xCopy,Step,Type,TypX);

	/* Central function value */
	if(Func(xCopy,f,cf)) return 1;

	if(Type == 1 || Type == 4)
	{
		/* Forward difference derivatives */
		double fpp;
		Vector<double> fp(n);				fp.Name   = "Hessian::fp";
		Vector<double> cfpp(m2);			cfpp.Name = "Hessian::cfpp";
		Vector< Vector<double> > cfp(n);	cfp.Name  = "Hessian::cfp";
		for(int i=0; i<n; i++)
		{
			cfp[i] = Vector<double>(m2);
			cfp.Name = "Hessian::cdp_" + ToString(i);
		}
		for(int i=0; i<n; i++)
		{
			/* Compute f(x + h * Z(j)) */
			xCopy[i] += hZ[i];
			if(Func(xCopy,fp[i],cfp[i])) return 1;

			/* Compute f(x + 2 * h * Z(j) */
			xCopy[i] += hZ[i];
			if(Func(xCopy,fpp,cfpp)) return 1;
			xCopy[i] -= Two * hZ[i];

			/* Compute first derivatives */
			df[i]  = (fp[i]  - f) / hZ[i];
			for(int k=0; k<m2; k++)
				dcf(k,i) = (cfp[i][k] - cf[k]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - Two * fp[i] + f) / Sqr(hZ[i]);
			for(int k=0; k<m2; k++) ddf(i,i) += lambda[k] * (cfpp[k] - Two * cfp[i][k] + cf[k]) / Sqr(hZ[i]);
		}
		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x + h * Z(i) + h * Z(j)) */
				xCopy[i] += hZ[i];
				xCopy[j] += hZ[j];
				if(Func(xCopy,fpp,cfpp)) return 1;
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fp[i] - fp[j] + f) / (hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddf(i,j) += lambda[k] * (cfpp[k] - cfp[i][k] - cfp[j][k] + cf[k]) / (hZ[i] * hZ[j]);
			}
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward difference derivatives */
		double fmm;
		Vector<double> fm(n);				fm.Name   = "Hessian::fm";
		Vector<double> cfmm(m2);			cfmm.Name = "Hessian::cfmm";
		Vector< Vector<double> > cfm(n);	cfm.Name  = "Hessian::cfm";
		for(int i=0; i<n; i++)
		{
			cfm[i] = Vector<double>(m2);
			cfm.Name = "Hessian::cdm_" + ToString(i);
		}

		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fm[i],cfm[i])) return 1;

			/* Compute f(x - 2 * h * Z(j)) */
			xCopy[i] -= hZ[i];
			if(Func(xCopy,fmm,cfmm)) return 1;
			xCopy[i] += Two * hZ[i];

			/* Compute first derivatives */
			df[i]  = (f - fm[i]) / hZ[i];
			for(int k=0; k<m2; k++) dcf(k,i) = (cf[k] - cfm[i][k]) / hZ[i];

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (f - fm[i] + fmm) / Sqr(hZ[i]);
			for(int k=0; k<m2; k++) ddf(i,i) += lambda[k] * (cf[k] - cfm[i][k] + cfmm[k]) / Sqr(hZ[i]);
		}

		for(int i=0; i<n; i++)
		{
			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];
				if(Func(xCopy,fmm,cfmm)) return 1;
				xCopy[i] += hZ[i];
				xCopy[j] += hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (f - fm[i] - fm[j] + fmm) / (hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddf(i,j) += lambda[k] * (cf[k] - cfm[i][k] - cfm[j][k] + cfmm[k]) / (hZ[i] * hZ[j]);
			}
		}
	}
	else
	{
		/* Central difference derivatives */
		double fmm,fmp,fpm,fpp;
		Vector<double> cfmm(m2);
		Vector<double> cfmp(m2);
		Vector<double> cfpm(m2);
		Vector<double> cfpp(m2);

		for(int i=0; i<n; i++)
		{
			Print("i diag",i);

			/* Compute f(x - 2 * h * Z(j)) */
			xCopy[i] -= Two * hZ[i];
			if(Func(xCopy,fmm,cfmm)) return 1;

			/* Compute f(x + 2 * h * Z(j)) */
			xCopy[i] += Four * hZ[i];
			if(Func(xCopy,fpp,cfpp)) return 1;
			xCopy[i] -= Two * hZ[i];

			/* Compute first derivatives */
			df[i] = (fpp - fmm) / (Four * hZ[i]);
			for(int k=0; k<m2; k++) dcf(k,i) = (cfpp[k] - cfmm[k]) / (Four * hZ[i]);

			/* Compute diagonal portion of second derivatives */
			ddf(i,i) = (fpp - Two * f + fmm) / (Four * hZ[i] * hZ[i]);
			for(int k=0; k<m2; k++) ddf(i,i) += lambda[k] * (cfpp[k] - Two * cf[k] + cfmm[k]) / (Four * Sqr(hZ[i]));
		}

		for(int i=0; i<n; i++)
		{
			Print("i off",i);

			for(int j=0; j<i; j++)
			{
				/* Compute f(x - h * Z(i) - h * Z(j)) */
				xCopy[i] -= hZ[i];
				xCopy[j] -= hZ[j];
				if(Func(xCopy,fmm,cfmm)) return 1;

				/* Compute f(x - h * Z(i) + h * Z(j)) */
				xCopy[j] += Two * hZ[j];
				if(Func(xCopy,fmp,cfmp)) return 1;

				/* Compute f(x + h * Z(i) + h * Z(j)) */
				xCopy[i] += Two * hZ[i];
				if(Func(xCopy,fpp,cfpp)) return 1;

				/* Compute f(x + h * Z(i) - h * Z(j)) */
				xCopy[j] -= Two * hZ[j];
				if(Func(xCopy,fpm,cfpm)) return 1;
				xCopy[i] -= hZ[i];
				xCopy[j] += hZ[j];

				/* Compute off-diagonal portion of second derivatives */
				ddf(i,j) = (fpp - fpm - fmp + fmm) / (Four * hZ[i] * hZ[j]);
				for(int k=0; k<m2; k++) ddf(i,j) += lambda[k] * (cfpp[k] - cfpm[k] - cfmp[k] + cfmm[k]) / (Four * hZ[i] * hZ[j]);
			}
		}
	}

	return 0;
}

#endif
