
#ifndef _Opt_Jacobian3
#define _Opt_Jacobian3


/*

Utility for computing the jacobian of a function.

Arguments

	Func:			A pointer to a function
	x:				Point at which to evaluate derivatives
	Z:				Projection matrix
	f:				Return function value
	df:				Return derivative value
	Step:			Step size for numerical derivatives
	Type:			Type of derivative approximation to use
					1	-	Forward  difference finite difference derivatives, relative step
					2	-	Backward difference finite difference derivatives, relative step
					3	-	Central  difference finite difference derivatives, relative step
					4	-	Forward  difference finite difference derivatives, absolute step
					5	-	Backward difference finite difference derivatives, absolute step
					6	-	Central  difference finite difference derivatives, absolute step
	PrintLevel:
	TypX:			Vector containing scales of x

*/

	/* Compute the jacobian of a vector-valued function */
	int Jacobian(int (*Func)(const Vector<double> &, Vector<double> &),
				Vector<double> &x,
		 		Vector<double> &f,
		  		Matrix<double> &df,
		  		double Step,
		  		int Type,
		  		const Vector<double> &TypX);

	/* Compute the projected jacobian of a vector-valued function */
	int Jacobian(int (*Func)(const Vector<double> &, Vector<double> &),
				Vector<double> &x,
		 		Matrix<double> &Z,
		  		Vector<double> &f,
		  		Matrix<double> &df,
		  		double Step,
		  		int Type,
		  		const Vector<double> &TypX);


/* Compute the Jacobian of a Vector-valued Function */
int Jacobian(int (*Func)(const Vector<double> &, Vector<double> &),
				Vector<double> &x,
		 		Vector<double> &f,
		  		Matrix<double> &df,
		  		double Step,
		  		int Type,
		  		const Vector<double> &TypX)
{

	/* Check Dimensions */
	int n = x.Size();		// Number of variables
	int m = f.Size();		// Number of components in function
	if(df.Rows() != m || df.Cols() != n || TypX.Size() != n)
	{
		Print("Error in Jacobian3");
		Print("Incompadible dimensions");
		Print("n",n);
		Print("m",m);
		Print("df.Size()",df.Size());
		Print("TypX.Size()",TypX.Size());
		HaltExecution();
	}

	/* Compute Normalized Step Sizes */
	Vector<double> hZ(n);		hZ.Name = "Jacobian3::hZ";
	ComputeDerivStep(hZ,x,Step,Type,TypX);

	/* Central Function Value */
	if(Func(x,f)) return 1;

	if(Type == 1 || Type == 4)
	{
		/* Forward Difference Derivatives */
		Vector<double> fp(m);	fp.Name  = "Jacobian3::fp";
		for(int i=0; i<n; i++)
		{
			/* Compute f(x + h * Z(j)) */
			x(i) += hZ(i);
			if(Func(x,fp)) return 1;
			x(i) -= hZ(i);

			/* Compute First Derivatives */
			for(int j=0; j<m; j++) df(j,i) = (fp(j) - f(j)) / hZ(i);
		}
	}
	else if(Type == 2 || Type == 5)
	{
		/* Backward Difference Derivatives */
		Vector<double> fm(m);	fm.Name  = "Jacobian3::fm";
		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			x(i) -= hZ(i);
			if(Func(x,fm)) return 1;
			x(i) += hZ(i);

			/* Compute First Derivatives */
			for(int j=0; j<m; j++) df(j,i) = (f(j) - fm(j)) / hZ(i);
		}
	}
	else
	{
		/* Central Difference Derivatives */
		Vector<double> fm(m);	fm.Name = "Jacobian3::fm";
		Vector<double> fp(m);	fp.Name = "Jacobian3::fp";
		for(int i=0; i<n; i++)
		{
			/* Compute f(x - h * Z(j)) */
			x(i) -= hZ(i);
			if(Func(x,fm)) return 1;

			/* Compute f(x + h * Z(j)) */
			x(i) += Two * hZ(i);
			if(Func(x,fp)) return 1;
			x(i) -= hZ(i);

			/* Compute First Derivatives */
			for(int j=0; j<m; j++)
				df(j,i) = (fp(j) - fm(j)) / (Two * hZ(i));
		}
	}

	return 0;
}

/* Compute the Projected Jacobian of a Vector-valued Function */
int Jacobian(int (*Func)(const Vector<double> &, Vector<double> &),
				Vector<double> &x,
		 		Matrix<double> &Z,
		  		Vector<double> &f,
		  		Matrix<double> &df,
		  		double Step,
		  		int Type,
		  		const Vector<double> &TypX)
{

	/* Check Dimensions */
	int n = x.Size();		// Number of Variables
	int m = f.Size();		// Number of Components in Function
	int r = n - Z.Cols();	// Number of Linear Constraints
	if(Z.Rows() != n || df.Rows() != m || df.Cols() != n - r || TypX.Size() != n)
	{
		Print("Error in Jacobian3");
		Print("Incompadible Dimensions");
		Print("n",n);
		Print("m",m);
		Print("r",r);
		Print("Z.Size()",Z.Size());
		Print("df.Size()",df.Size());
		Print("TypX.Size()",TypX.Size());
		HaltExecution();
	}

	/* Compute Normalized Step Sizes */
	Vector<double> hZ(n-r);		hZ.Name = "Jacobian3::hZ";
	ComputeDerivStep(hZ,x,Z,Step,Type,TypX);

	/* Central Function Value */
	if(Func(x,f)) return 1;

	if(Type == 1)
	{
		/* Forward Difference Derivatives */
		Vector<double> fp(m);		fp.Name  = "Jacobian3::fp";
		for(int i=0; i<n-r; i++)
		{
			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) x(j) += Z(j,i) * hZ(i);
			if(Func(x,fp)) return 1;
			for(int j=0; j<n; j++) x(j) -= Z(j,i) * hZ(i);

			/* Compute first derivatives */
			for(int j=0; j<m; j++) df(j,i) = (fp(j) - f(j)) / hZ(i);
		}
	}
	else if(Type == 2)
	{
		/* Backward Difference Derivatives */
		Vector<double> fm(m);		fm.Name  = "Jacobian3::fm";
		for(int i=0; i<n-r; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int j=0; j<n; j++) x(j) -= Z(j,i) * hZ(i);
			if(Func(x,fm)) return 1;
			for(int j=0; j<n; j++) x(j) += Z(j,i) * hZ(i);

			/* Compute First Derivatives */
			for(int j=0; j<m; j++) df(j,i) = (f(j) - fm(j)) / hZ(i);
		}
	}
	else
	{
		/* Central Difference Derivatives */
		Vector<double> fm(m);		fm.Name = "Jacobian3::fm";
		Vector<double> fp(m);		fp.Name = "Jacobian3::fp";
		for(int i=0; i<n-r; i++)
		{
			/* Compute f(x - h * Z(j)) */
			for(int j=0; j<n; j++) x(j) -= Z(j,i) * hZ(i);
			if(Func(x,fm)) return 1;

			/* Compute f(x + h * Z(j)) */
			for(int j=0; j<n; j++) x(j) += Two * Z(j,i) * hZ(i);
			if(Func(x,fp)) return 1;
			for(int j=0; j<n; j++) x(j) -= Z(j,i) * hZ(i);

			/* Compute First Derivatives */
			for(int j=0; j<m; j++)
				df(j,i) = (fp(j) - Two * f(j) + fm(j)) / (Two * hZ(i));
		}
	}

	return 0;
}

#endif
