
#ifndef _LDLUtil
#define _LDLUtil

/*

Utilities involving the LDL factorization of a Matrix

Taken from Practial Optimization, by Gill, Murry and Wright

*/

	/*
	Compute the LDL factorization of A + v * v^T where A = L * d * L^T
	in O(n^2) operations
	*/

	void LDLUpdatePos(Matrix<double> &L, Vector<double> &d, Vector<double> &v);
	void LDLUpdatePos(LowerMatrix<double> &L, DiagonalMatrix<double> &D, Vector<double> &v);

	/*
	Compute the LDL factorization of A + v * v^T where A = L * d * L^T
	in O(n^2) operations
	Note: This may cuase problems when the matrix A + v * v^T is not
	positive definite
	*/

	void LDLUpdateNeg(Matrix<double> &L, Vector<double> &d, Vector<double> &v);
	void LDLUpdateNeg(LowerMatrix<double> &L, DiagonalMatrix<double> &D, Vector<double> &v);


void LDLUpdatePos(Matrix<double> &L, Vector<double> &d, Vector<double> &v)
{
	int j,r;
	int n = v.Size();
	double beta;
	Vector<double> p(n);
	Vector<double> t(n+1);

	t[0] = 1.0;
	for(j=0; j<n; j++)
	{
		p[j] = v[j];
		t[j+1] = t[j] + Sqr(p[j]) / d[j];
		d[j] *= t[j+1] / t[j];
		beta = p[j] / (d[j] * t[j]);
		for(r=j+1; r<n; r++)
		{
			v[r] -= p[j] * L(r,j);
			L(r,j) += beta * v[r];
		}
	}
}

void LDLUpdateNeg(Matrix<double> &L, Vector<double> &d, Vector<double> &v)
{
	int j,r;
	int n = v.Size();
	double beta;
	Vector<double> p(n);
	Vector<double> t(n+1);

	// Solve L * p = v
	for(j=0; j<n; j++)
	{
    	p[j] = v[j];
    	for(r=0; r<j; r++) p[j] -= p[r] * L(j,r);
		p[j] /= L(j,j);
	}

	// Ensure Positive Definiteness
	t[n] = 1.0;
	for(j=0; j<n; j++) t[n] -= Sqr(p[j]) / d[j];
	if(t[n] <= MachEps) t[n] = MachEps;

	// Main Loop
	for(j=n-1; j>=0; j--)
	{
		t[j] = t[j+1] + Sqr(p[j]) / d[j];
		d[j] *= t[j+1] / t[j];
		beta = -p[j] / (d[j] * t[j]);
		v[j] = p[j];
		for(r=j+1; r<n; r++)
		{
			L(r,j) += beta * v[r];
			v[r] *= (1.0 - Sqr(beta));
			v[r] += beta * L(r,j);
		}
	}
}

void LDLUpdatePos(LowerMatrix<double> &L, DiagonalMatrix<double> &D, Vector<double> &v)
{
	int j,r;
	int n = v.Size();
	double beta;
	Vector<double> p(n);
	Vector<double> t(n+1);

	t[0] = 1.0;
	for(j=0; j<n; j++)
	{
		p[j] = v[j];
		t[j+1] = t[j] + Sqr(p[j]) / D(j);
		D(j) *= t[j+1] / t[j];
		beta = p[j] / (D(j) * t[j]);
		for(r=j+1; r<n; r++)
		{
			v[r] -= p[j] * L(r,j);
			L(r,j) += beta * v[r];
		}
	}
}

void LDLUpdateNeg(LowerMatrix<double> &L, DiagonalMatrix<double> &D, Vector<double> &v)
{
	int j,r;
	int n = v.Size();
	double beta;
	Vector<double> p(n);
	Vector<double> t(n+1);

	// Solve L * p = v
	for(j=0; j<n; j++)
	{
    	p[j] = v[j];
    	for(r=0; r<j; r++) p[j] -= p[r] * L(j,r);
		p[j] /= L(j,j);
	}

	// Ensure Positive Definiteness
	t[n] = 1.0;
	for(j=0; j<n; j++) t[n] -= Sqr(p[j]) / D(j);
	if(t[n] <= MachEps) t[n] = MachEps;

	// Main Loop
	for(j=n-1; j>=0; j--)
	{
		t[j] = t[j+1] + Sqr(p[j]) / D(j);
		D(j) *= t[j+1] / t[j];
		beta = -p[j] / (D(j) * t[j]);
		v[j] = p[j];
		for(r=j+1; r<n; r++)
		{
			L(r,j) += beta * v[r];
			v[r] *= (1.0 - Sqr(beta));
			v[r] += beta * L(r,j);
		}
	}
}

#endif


