
#ifndef _GenMatUtil_LUDecompObj
#define _GenMatUtil_LUDecompObj

/*

LU Decomposition.
<P>
For an m-by-n matrix A with m >= n, the LU decomposition is an m-by-n
unit lower triangular matrix L, an n-by-n upper triangular matrix U,
and a permutation vector piv of length m so that A(piv,:) = L*U.
If m < n, then L is m-by-m and U is m-by-n.
<P>
The LU decompostion with pivoting always exists, even if the matrix is
singular, so the constructor will never fail.  The primary use of the
LU decomposition is in the solution of square systems of simultaneous
linear equations.  This will fail if IsNonsingular() returns false.

*/

template <class Real>
class LUDecompObj
{

	private:

	int m;
	int n;
	int min_mn;
	int PivSign;
	int Info; 
	Matrix<Real> LU;
	Vector<int> Piv;

	public :

	/* Blank Constructor */
	
	LUDecompObj() { }

   	/* Standard Constructor */
   	
 	LUDecompObj(const Matrix<Real> &A)
	{
		m = A.Rows();
		n = A.Cols();
		min_mn = Min(m,n);
		LU = A.Copy();
		Piv = Vector<int>(min_mn);
		
		/* Preform Decomposition */
		
		Info = LapackGetrf(m,n,LU.Begin(),n,Piv.Begin());
	}
	
	/* Constructor for Symmetric Matrices */
 	
 	LUDecompObj(const SymmetricMatrix<Real> &A)
	{
		m = A.Dim();
		n = m;
		min_mn = m;
		LU = Matrix<double>(m,n);
		Copy(LU,A);
		Piv = Vector<int>(min_mn);
	
		/* Preform Decomposition */
		
		Info = LapackGetrf(m,n,LU.Begin(),n,Piv.Begin());
	}
	
	/*
	
	Is the matrix singular? Return 1 (true) if upper
	triangular factor U (and hence A) is singular, 0 otherwise.
	
	*/
	
 	bool IsSingular()
	{
 		return (Info != 0);
  	}
	
	/*
	
	Is the matrix nonsingular? Return 1 (true) if upper
	triangular factor U (and hence A) is nonsingular, 0 otherwise.
	
	*/
	
 	bool IsNonsingular()
	{
 		return (Info == 0);
  	}

	/* Access elements of the LU factorization */

	double operator()(int i, int j)
	{
		return LU(i,j);
	}

	/* Return the lower triangular factor */
		
	Matrix<Real> GetL()
	{
		Matrix<Real> L(m,min_mn);
		for(int i=0; i<m; i++) for(int j=0; j<min_mn; j++)
        {
         	if(i > j)       L(i,j) = LU(i,j);
           	else if(i == j) L(i,j) = One;
           	else            L(i,j) = Zero;
        }
      	return L;
   	}
   	
   	/* Return the lower triangular factor */
   		
	LowerMatrix<Real> GetLLower()
	{
		
		/* Check if the matrix is a square */

		if(m != n)
		{
			Print("Error in LUDecompObj::GetLLower");
			HaltExecution();
		}
		
		LowerMatrix<Real> L(n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++)
        {
         	if(i > j)       L(i,j) = LU(i,j);
           	else if(i == j) L(i,j) = One;
        }
      	return L;
   	}

   	/* Return the upper triangular factor */
   	
	Matrix<Real> GetU()
	{
		Matrix<Real> U(min_mn,n);
      	for(int i=0; i<min_mn; i++) for(int j=0; j<n; j++)
      	{
        	if(i <= j && i < m) U(i,j) = LU(i,j);
            else                U(i,j) = Zero;
		}           
		return U;
	}
	
	/* Return the upper triangular factor */
	
	UpperMatrix<Real> GetUUpper()
	{
		if(m != n)
		{
			Print("Error in LUDecompObj::GetUUpper");
			HaltExecution();
		}
		
   		UpperMatrix<Real> U(n);
      	for(int i=0; i<n; i++) for(int j=i; j<n; j++) U(i,j) = LU(i,j);
		return U;
	}

	/* Return the Pivot Permutation Vector */
	
	Vector<int> GetPiv()
	{
		return Piv.Copy();
	}

	/*
	
	Compute determinant using LU factors.
	Return the determinant of A
	
	*/
	
   	Real Determinant()
   	{
    	
    	if(m != n)
    	{
    		Print("Error in LUDecompObj::Determinant");
    		Print("Matrix is not Square");
    		Print("m",m);
    		Print("n",n);
    		HaltExecution();
    	}
    	
    	Real d = Real(PivSign);
      	for(int i=0; i<n; i++) d *= LU(i,i);
     	return d;
   	}

	/*
	
	Solve A * x = b, where x and b are vectors of length equal
	to the number of rows in A.
	
	*/
	
	Vector<Real> Solve(const Vector<Real> &b) 
   	{
	
		/* Check if Matrix is Square */
		
		if(m != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Matrix is not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}		
	
		/* Check for Incompadible Dimensions */
		
		if(b.Size() != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Incompadible Dimensions");
			Print("n",n);
			Print("b.Size()",b.Size());
		}
	
		/* Check for Singularity */
		 
		if(Info)
     	{
     		Print("Error in LUDecompObj::Solve(Vector<double>)");
     		Print("Singular");
     		Print("Info",Info);
     		HaltExecution();
     	}
     	
     	/* Solve A * x = b for x */
     	
     	Vector<Real> x = b.Copy();
     	LapackGetrs(0,n,1,LU.Begin(),n,Piv.Begin(),x.Begin(),1);
     	return x;
   	}
   	
   	Vector<Real> SolveTrans(const Vector<Real> &b) 
   	{
	
		/* Check if Matrix is Square */
		
		if(m != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Matrix is not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}
	
		/* Check for Singularity */
		  	
     	if(Info)
     	{
     		Print("Error in LUDecompObj::SolveTrans(Vector<double>)");
     		Print("Singular");
     		Print("Info",Info);
     		HaltExecution();
     	}
     	
     	/* Solve A * x = b for x */
     	
     	Vector<Real> x = b.Copy();
     	LapackGetrs(1,n,1,LU.Begin(),n,Piv.Begin(),x.Begin(),1);
      	return x;
   	}
   	
   	/*
	
	Solve A * x = b, where x and b are vectors of length equal
	to the number of rows in A.
	
	*/
	
	int Solve(Vector<Real> &x, const Vector<Real> &b) 
   	{
   	
   		/* Check if Matrix is Square */
		
		if(m != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Matrix is not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}
   	
   		/* Solve A * x = b for x */
   		      	
      	Copy(x,b);
      	if(Info) return Info;
		LapackGetrs(0,n,1,LU.Begin(),n,Piv.Begin(),x.Begin(),1);
      	return 0;
   	}
   	
   	/*
	
	Solve A^T * x = b, where x and b are vectors of length equal
	to the number of rows in A.
	
	*/
	
	int SolveTrans(Vector<Real> &x, const Vector<Real> &b) 
   	{
   	
   		/* Check if Matrix is Square */
		
		if(m != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Matrix is not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}
   	
   		/* Solve A^T * x = b for x */
   		      	
      	Copy(x,b);
      	if(Info) return Info;
		LapackGetrs(1,n,1,LU.Begin(),n,Piv.Begin(),x.Begin(),1);
      	return 0;
   	}
   	
   	/* Solve A * X = B */
   	
	Matrix<Real> Solve(const Matrix<Real> &B) 
	{
		 
		 /* Check if Matrix is Square */
		
		if(m != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Matrix is not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}
		      	
      	/* Check for Singularity */
		  	
     	if(Info)
     	{
     		Print("Error in LUDecompObj::Solve(Matrix<double>)");
     		Print("Singular");
     		Print("Info",Info);
     		HaltExecution();
     	}
      	
      	/* Solve A * x = b for x */
     	
     	int nRhs = B.Cols();
     	Matrix<Real> X = B.Copy();
      	LapackGetrs(0,n,nRhs,LU.Begin(),n,Piv.Begin(),X.Begin(),nRhs);
      	return X;
   	}
   	
   	/* Solve A^T * X = B */
   	
	Matrix<Real> SolveTrans(const Matrix<Real> &B) 
	{
		   
		/* Check if Matrix is Square */
		
		if(m != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Matrix is not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}	
		   	
      	/* Check for Singularity */
		  	
     	if(Info)
     	{
     		Print("Error in LUDecompObj::SolveTrans(Matrix<double>)");
     		Print("Singular");
     		Print("Info",Info);
     		HaltExecution();
     	}
      	
      	/* Solve A^T * X = B for X */
     	
     	int nRhs = B.Cols();
     	Matrix<double> X = B.Copy();
      	LapackGetrs(1,n,nRhs,LU.Begin(),n,Piv.Begin(),X.Begin(),nRhs);
      	return X;
   	}

	/* Solve A * X = B */
   	
	int Solve(Matrix<Real> &X, const Matrix<Real> &B) 
	{
		
		/* Check if Matrix is Square */
		
		if(m != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Matrix is not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}
		
		int nRhs = B.Cols();
		Copy(X,B);
      	if(Info) return Info;
		LapackGetrs(0,n,nRhs,LU.Begin(),n,Piv.Begin(),X.Begin(),nRhs);
      	return 0;
   	}
   	
   	/* Solve A^T * X = B */
   	
	int SolveTrans(Matrix<Real> &X, const Matrix<Real> &B) 
	{
		
		/* Check if Matrix is Square */
		
		if(m != n)
		{
			Print("Error in LUDecompObj::Solve(Vector<Real>)");
			Print("Matrix is not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}
		
		int nRhs = B.Cols();
		Copy(X,B);
      	if(Info) return Info;
		LapackGetrs(1,n,nRhs,LU.Begin(),n,Piv.Begin(),X.Begin(),nRhs);
      	return 0;
   	}
   	
	Matrix<Real> Inverse()
   	{
   		
   		if(m != n)
   		{
   			Print("Error in LUDecompObj::Inverse");
   			Print("Matrix is not Square");
   			Print("m",m);
   			Print("n",n);
   			HaltExecution();
   		}
   		
   		if(Info)
   		{
   			Print("Error in LUDecompObj::Inverse");
   			Print("Matrix is Singular");
   			HaltExecution();
   		}
   		
   		Vector<double> Work(10000000);
   		
   		Matrix<double> AInv = LU.Copy();
   		int Info2 = LapackGetri(n,AInv.Begin(),n,Piv.Begin(),Work.Begin(),Work.Size());
   		if(Info2)
   		{
   			Print("Error in LUDecompObj::Inverse");
   			Print("Matrix is Singular");
   			HaltExecution();
   		}
   		
   		return AInv;	
   	}
   	
   	int Inverse(Matrix<Real> &AInv)
   	{
   		
   		if(m != n)
   		{
   			Print("Error in LUDecompObj::Inverse(Matrix<Real>)");
   			Print("Matrix is not Square");
   			Print("m",m);
   			Print("n",n);
   			HaltExecution();
   		}
   		
   		/* Check for Incompadible Dimensions */
   		
   		if(AInv.Rows() != n && AInv.Cols() != n)
   		{
   			Print("Error in LUDecompObj::Inverse(Matrix<Real>)");
   			Print("Incompadible Dimensions");
   			Print("m",m);
   			Print("n",n);
   			Print("AInv.Size",AInv.Size());
   			HaltExecution();
   		}   
   		
   		if(Info) return Info;
   		   		
   		Vector<double> Work(10000000);
   		
   		int Info2 = LapackGetri(n,AInv.Begin(),n,Piv.Begin(),Work.Begin(),Work.Size());
		return Info2;   		
   	}
   	
   	int GetInfo()
   	{
   		return Info;
   	}
   	
   	/*
   	Vector<Real> LowerSolve(const Vector<Real> &b)
   	{
   	
   		/* Solve L1 * y = b where L1 is the upper mxm block of L */
   	/*	
   		Vector<Real> y = PermuteCopy(b,Piv);
      	for(int j=0; j<m; j++) for(int i=j+1; i<m; i++) y[i] -= y[j] * LU(i,j);
    	return y;
    }
    
    Vector<Real> UpperSolve(const Vector<Real> &b)
   	{

   		/* Solve U1 * x = b where U1 is the upper mxm block of U */
	/*
   		Vector<Real> x = PermuteCopy(b,piv);
      	for(int j=m-1; j>=0; j--)
      	{
      		x[j] /= LU(j,j);
      		for(int i=0; i<j; i++) x[i] -= x[j] * LU(i,j);
      	}
      	return x;
	}
	*/
	
	Matrix<double> Test()
	{
		/* Form Product */
		Matrix<double> A2 = GetL() * GetU();
		
		/* Permute the Rows */
		for(int i=m-1; i>=0; i--)
		{
			int ip = Piv(i);
			for(int j=0; j<n; j++)
			{
				double Temp = A2(i,j);
				A2(i,j) = A2(ip,j);
				A2(ip,j) = Temp;
			}
		}
		
		return A2;
	}
};

#endif
