
#ifndef _LapackGetf2
#define _LapackGetf2

//#define _LapackGetf2CrossOver 100

	int LapackGetf2(int m, int n, double *A, int lda, int *Piv);

/*

Compute an LU factorization of a general m-by-n matrix A
using partial pivoting with row interchanges.

The factorization has the form

     A = P * L * U

where P is a permutation matrix, L is lower triangular with unit
diagonal elements (lower trapezoidal if m > n), and U is upper
triangular (upper trapezoidal if m < n).

This is the right-looking Level 2 BLAS version of the algorithm.

Arguments
---------

m (integer) 	-	The number of rows of the matrix A. m >= 0.

n (integer)		-	The number of columns of the matrix A. n >= 0.

A (array)		-	Array, dimension (lda,N)
					On entry, the m by n matrix to be factored.
					On exit, the factors L and U from the factorization
					A = P * L * U; the unit diagonal elements of L are not stored.

lda (integer)	-	The leading dimension of the array A. lda >= max(1,m).

Piv (array)		-	Array, dimension (Min(m,n))
					The pivot indices; for 1 <= i <= Min(m,n), row i of the
					matrix was interchanged with row Piv(i).
					
Info (integer)	-	= 0: successful exit
					> 0: if Info = k, U(k,k) is exactly zero. The factorization
					has been completed, but the factor U is exactly
					singular, and division by zero will occur if it is used
					to solve a system of equations.
					

Adapted from the Lapack subroutine,

	DGETF2( M, N, A, LDA, IPIV, INFO )

	-- LAPACK routine (version 3.0) --
	Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
	Courant Institute, Argonne National Lab, and Rice University
	June 30, 1992
	
*/

int LapackGetf2(int m, int n, double *A, int lda, int *Piv)
{

	/* Test the input parameters. */
	if(m < 0)
	{
		Print("Error in LapackGetf2");
		Print("m < 0");
		Print("m",m);
		HaltExecution();
	}
	else if(n < 0)
	{
		Print("Error in LapackGetf2");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(lda < Max(1,n))
	{
		Print("Error in LapackGetf2");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	
	/* Quick return if possible */
	if(m == 0 || n == 0) return 0;
	
	/*
	
	Additional code for sensible pivots:
	
		for(i=0; i<m; i++) Piv[i] = i;
	
	*/
	
	/*if(Min(m,n) <= _LapackGetf2CrossOver)
	{
		/*
				
		Use the Lapack right-looking algorithm,
		which is more efficient when Min(m,n) is
		small. Localizing references does not
		improve speed in this algorithm.
		
		Oriignally, I found that this code was faster for matrices of size <= 100.
		But this code doesn't seem to work, and I find the same problem with the CLAPACK
		code
		
		*/
	
		/*
		for(int j=0; j<Min(m,n); j++)
		{
					
			/* Find pivot and test for singularity. */
		/*	int jp = j + BlasIdaMax(m-j,&A[lda*j+j],lda);
			
			Piv[j] = jp;
		
			if(A[lda * jp + j] != Zero)
			{
				/* Apply the interchange to columns 1:N. */
		/*		if(jp != j) BlasSwap(n,&A[lda*j],1,&A[lda*jp],1);
				
				/* Compute elements J+1:M of J-th column. */				
		/*		if(j < m - 1) BlasScal(m-j-1,One / A[lda*j+j],&A[lda*(j+1)+j],lda);
			}
			else
			{							
				/* Singularity found */
				/*		return j + 1;
			}
		
			if(j < Min(m,n) - 1)
			{
				/* Update trailing submatrix. */
		/*		BlasGer(m-j-1,n-j-1,-One,&A[lda*(j+1)+j],lda,&A[lda*j+j+1],1,&A[lda*(j+1)+j+1],lda);
			}
		
			/* Copy back the j-th column. */
		/*	
		}
		
		return 0;
	}
	else
	{
	*/
		/*
		
		Use the TNT left-looking dot-product Crout/Doolittle
		algorithm, which is more efficient when Min(m,n) is
		large. This version can be made substantially faster
		by making a copy of the jth column of the matrix
		to localize references
		
		*/
		
		Vector<double> LUcolj(m);
		LUcolj.Name = "LapackGetf2::LUcolj";
		
		for(int j=0; j<Min(m,n); j++)
		{
			/* Make a copy of the j-th column to localize references. */		
			BlasCopy(m,&A[j],lda,LUcolj.Begin(),1);
					
		    /* Apply previous transformations. */
			for(int i=0; i<m; i++)
			{
		   	 	/* Most of the time is spent in the following dot product. */
				A[lda*i+j] = LUcolj[i] -= BlasDot(Min(i,j),&A[lda*i],1,&LUcolj[0],1);
			}
			
			/* Find pivot */
			int jp = j + BlasIdaMax(m-j,&LUcolj[j],1);

			/*
	       	
	       	This code is for traditional pivots. To implement more sensible pivots,
	       	using the code below and initialize Piv = (0,1,2,...,Min(m,n)-1):
	       	
	       		 int k = Piv[jp]; Piv[jp] = Piv[j]; Piv[j] = k; Piv[j] = p;
	       	
	       	*/
	       	
	       	Piv[j] = jp;
	       	
	       	/* Exchange rows j and jp */
	    	if(jp != j) BlasSwap(n,&A[lda*j],1,&A[lda*jp],1);
					
	        /* Compute multipliers. */
			if(j < m - 1 && A[lda*j+j] != Zero) BlasScal(m-j-1,One/A[lda*j+j],&A[lda*(j+1)+j],lda);
		}
		
		return 0;
	/*}*/
}

#endif

