
#ifndef _LapackGetrf
#define _LapackGetrf


	int LapackGetrf(int m, int n, double *A, int lda, int *Piv);

/*

Compute an LU factorization of a general m-by-n matrix A
using partial pivoting with row interchanges.

The factorization has the form

     A = P * L * U

where P is a permutation matrix, L is lower triangular with unit
diagonal elements (lower trapezoidal if m > n), and U is upper
triangular (upper trapezoidal if m < n).

This is the right-looking Level 3 BLAS version of the algorithm.


Arguments
---------

m (integer)		-	The number of rows of the matrix A. m >= 0.

n (integer)		-	The number of columns of the matrix A. n >= 0.

A (array)		-	Array represent an m by n matrix with a leading
					dimension of lda.
					On entry, this should contain the matrix to be
					factored.
					On exit, contains the factors L and U from the
					factorization A = P * L * U; the unit diagonal
					elements of L are not stored.

lda (integer)	-	The leading dimension of the matrix represneted by A.
					lda >= Max(1,n).

Piv (array)		-	Array representing a vector of dimension Min(m,n).
					The pivot indices; for 1 <= i <= Min(m,n), row i of the
					matrix was interchanged with row Piv(i).
					
Info (integer)	-	= 0:  successful exit
					> 0:  if Info = i, U(i-1,i-1) is exactly zero. The
					factorization has been completed, but the factor U
					is exactly singular, and division by zero will occur
					if it is used to solve a system of equations.
					

Adapted from the Lapack subroutine,

	DGETRF( M, N, A, LDA, IPIV, INFO )
	
	-- LAPACK routine (version 3.0) --
	Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
	Courant Institute, Argonne National Lab, and Rice University
	March 31, 1993

Note:

While the blocked LU factorization works, it did not yeild any speedup
on any of my machines. Thus, nb is set to 1.

*/

int LapackGetrf(int m, int n, double *A, int lda, int *Piv)
{
	
	int i,iInfo,j,jb,nb;
	
	/* Quick return if possible */
	if(m == 0 || n == 0) return 0;
	
	/* Test the input parameters. */
	int Info = 0;
	if(m < 0)
	{
		Print("Error in LapackGetrf");
		Print("m < 0");
		Print("m",m);
		HaltExecution();
	}
	else if(n < 0)
	{
		Print("Error in LapackGetrf");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(lda < Max(1,n))
	{
		Print("Error in LacpackGetrf");
		Print("lda < Max(1,n)");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	
	/*
	
	Determine the block size for this environment.
	Although the block-code works properly,
	the fastest preformance was found using
	no blocking.
	
	*/
	
	nb = 8;
	
	if(nb <= 1 || nb >= Min(m,n))
	{
		/* Use unblocked code. */
		Info = LapackGetf2(m,n,A,lda,Piv);
	}
	else
	{
		/* Use blocked code. */
		for(j=0; j<Min(m,n); j+=nb)
		{
			jb = Min(Min(m,n)-j,nb);
			
			/* Factor diagonal and subdiagonal blocks and test for exact
			singularity. */
			iInfo = LapackGetf2(m-j,jb,&A[lda*j+j],lda,&Piv[j]);
			
			/* Adjust Info and the pivot indices. */
			if(Info == 0 && iInfo > 0) Info = iInfo + j;
			for(i=j; i<Min(m,j+jb); i++) Piv[i] = j + Piv[i];
			
			/* Apply interchanges to columns 1:j-1. */
			LapackLaswp(j,A,lda,j,j+jb-1,Piv,1);

			if(j + jb < n)
			{
				/* Apply interchanges to columns j+jb:N. */
				LapackLaswp(n-j-jb,&A[j+jb],lda,j,j+jb-1,Piv,1);				
				
				/* Compute block row of U. */
				BlasTrsm(1,0,0,0,jb,n-j-jb,One,&A[lda*j+j],lda,&A[lda*j+j+jb],lda);
				
				if(j + jb < m)
				{
					/* Update trailing submatrix. */
					BlasGemm(0,0,m-j-jb,n-j-jb,jb,-One,&A[lda*(j+jb)+j],lda,&A[lda*j+j+jb],lda,One,&A[lda*(j+jb)+j+jb],lda);
				}
			}
		}
	}
	
	return Info;
}

#endif
