
#ifndef _LapackGetrs
#define _LapackGetrs


	void LapackGetrs(bool Trans, int n, int nRhs, double *A, int lda, int *Piv, double *B, int ldb);

/*

Solve a system of linear equations

	A * X = B  or  A^T * X = B

with a general n-by-n matrix A using the LU factorization computed
by LapackGetrf.


Arguments
---------

Trans (boolean)		-	Specifies the form of the system of equations:
						= 'N':  A * X = B  (No transpose)
						= 'T':  A'* X = B  (Transpose)
						= 'C':  A'* X = B  (Conjugate transpose = Transpose)
						
n (integer)			-	The order of the matrix A.  N >= 0.

nRhs (integer)		-	The number of right hand sides, i.e., the number of columns
						of the matrix B.  NRHS >= 0.
						
A (array)			-	Array, dimension (LDA,N)
						The factors L and U from the factorization A = P*L*U
						as computed by DGETRF.

lda (integer)		-	The leading dimension of the array A.  LDA >= max(1,N).

Piv (array)			-	Array, dimension (N)
						The pivot indices from DGETRF; for 1<=i<=N, row i of the
						matrix was interchanged with row IPIV(i).

B (array)			-	Array, dimension (LDB,NRHS)
						On entry, the right hand side matrix B.
						On exit, the solution matrix X.

ldb (integer)		-	The leading dimension of the array B.  LDB >= max(1,N).

						
						
Adapted from the Lapack subroutine,

	DGETRS( TRANS, N, NRHS, A, LDA, IPIV, B, LDB, INFO )
	
	-- LAPACK routine (version 3.0) --
	Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
	Courant Institute, Argonne National Lab, and Rice University
	March 31, 1993

Note: I tested this only for nRhs = 1 and n < 32. We need to check n >= 32
since one of the called sbroutines use block-code. -MP
	
*/

void LapackGetrs(bool Trans, int n, int nRhs, double *A, int lda, int *Piv, double *B, int ldb)
{

	/* Test the Input Parameters */
	
	if(n < 0)
	{
		Print("Error in LapackGetrs");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(nRhs < 0)
	{
		Print("Error in LapackGetrs");
		Print("nRhs < 0");
		Print("nRhs",nRhs);
		HaltExecution();
	}
	else if(lda < Max(1,n))
	{
		Print("Error in LapackGetrs");
		Print("lda < Max(1,n)");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	else if(ldb < 1)
	{
		Print("Error in LapackGetrs");
		Print("ldb < 1");
		Print("ldb",ldb);
		HaltExecution();
	}
	
	/* Quick return if possible */
	if(n == 0 || nRhs == 0) return;
	
	if(!Trans)
	{
	
		/* Solve A * X = B. */
		
		/* Apply row interchanges to the right hand sides. */
		LapackLaswp(nRhs,B,ldb,0,n-1,Piv,1);
		
		/* Solve L * X = B, overwriting B with X. */
		BlasTrsm(1,0,0,0,n,nRhs,One,A,lda,B,ldb);
		
		/* Solve U * X = B, overwriting B with X. */
		BlasTrsm(1,1,0,1,n,nRhs,One,A,lda,B,ldb);

	}
	else
	{
		/* Solve A^T * X = B. */
		
		/* Solve U^T * X = B, overwriting B with X. */
		BlasTrsm(1,1,1,1,n,nRhs,One,A,lda,B,ldb);
		
		/* Solve L^T * X = B, overwriting B with X. */
		BlasTrsm(1,0,1,0,n,nRhs,One,A,lda,B,ldb);
		
		/* Apply row interchanges to the solution vectors. */
		LapackLaswp(nRhs,B,ldb,0,n-1,Piv,-1);
	}
}

#endif
