
#ifndef _LapackPotf2
#define _LapackPotf2


	int LapackPotf2(bool Upper, int n, double *A, int lda);
	

/*

Compute the Cholesky factorization of a real symmetric
positive definite matrix A.

The factorization has the form

   A = U^T * U,		or		A = L  * L^T,

where U is an upper triangular and L is lower triangular.

This is the unblocked version of the algorithm, calling Level 2 BLAS.


Arguments
---------

Upper (boolean)		-	Specifies whether the upper or lower triangular part of the
						symmetric matrix A is stored.

n (integer)			-	The dimension of the matrix A. n >= 0.

A (array)			-	Array of dimension (lda,n). On entry, the symmetric matrix A.
						If UPLO = 'U', the leading n by n upper triangular part of A
						contains the upper triangular part of the matrix A, and the
						strictly lower triangular part of A is not referenced.
						If UPLO = 'L', the leading n by n lower triangular part of A
						contains the lower triangular part of the matrix A, and the
						strictly upper triangular part of A is not referenced.

						On exit, if INFO = 0, the factor U or L from the Cholesky
						factorization A = U'*U  or A = L*L'.
						
lda (integer)		-	The leading dimension of the array A.  lda >= max(1,n).

Info (integer)		-	= 0: successful exit
						< 0: if INFO = -k, the k-th argument had an illegal value
						> 0: if INFO = k, the leading minor of order k is not
              			positive definite, and the factorization could not be
              			completed.


Adapted from the Lapack subroutine,

	DPOTF2( UPLO, N, A, LDA, INFO )

	LAPACK routine (version 3.0) --
    Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
    Courant Institute, Argonne National Lab, and Rice University
    February 29, 1992

*/

int LapackPotf2(bool Upper, int n, double *A, int lda)
{
	
	/*  Test the input parameters.*/
	if(n < 0)
	{
		Print("Error in LapackPotrf2");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(lda < Max(1,n))
	{
		Print("Error in LapackPotrf2");
		Print("lda < Max(1,n)");
		Print("lda",lda);
		Print("n",n);
		HaltExecution();
	}
	
	/* Quick return if possible */
    if(n == 0) return 0;
    
    if(Upper)
    {
    	/* Compute the Cholesky factorization A = U^T * U */
    	for(int j=0; j<n; j++)
		{
			/* Compute L(j,j) and test for non-positive-definiteness. */
			double Ajj = A[(lda+1)*j] - BlasDot(j,&A[j],lda,&A[j],lda);
			    		
    		if(Ajj <= Zero)
            {
				A[(lda+1)*j] = Ajj;
            	return j + 1;
            }
    		
    		Ajj = sqrt(Ajj);
			A[(lda+1)*j] = Ajj;

			/* Compute elements j+1:n of column j. */
			if(j < n - 1)
			{
				BlasGemv(1,j,n-j-1,-One,&A[j+1],lda,&A[j],lda,One,&A[lda*j+j+1],1);
				BlasScal(n-j-1,One/Ajj,&A[lda*j+j+1],1);
			}
		}        
    }
    else
    {
    	/* Compute the Cholesky factorization A = L * L^T. */
      	for(int j=0; j<n; j++)
      	{
      		/* Compute L(j,j) and test for non-positive-definiteness. */
			double Ajj = A[(lda+1)*j] - BlasDot(j,&A[lda*j],1,&A[lda*j],1);
			
      		if(Ajj <= Zero)
			{
				A[(lda+1)*j] = Ajj;
				return j + 1;
			}
			
        	Ajj = sqrt(Ajj);
			A[(lda+1)*j] = Ajj;
            
            /* Compute elements j+1:n of column j. */
       		if(j < n - 1)
        	{
        		BlasGemv(0,n-j-1,j,-One,&A[lda*(j+1)],lda,&A[lda*j],1,One,&A[lda*(j+1)+j],lda);
				BlasScal(n-j-1,One/Ajj,&A[lda*(j+1)+j],lda);
    		}
        }
    }
    
    return 0;	// Succesful execution
}

#endif