
#ifndef _LapackPotrf
#define _LapackPotrf

	
	int LapackPotrf(bool Upper, int n, double *A, int lda);

/*

Compute the Cholesky factorization of a real symmetric
positive definite matrix A.

The factorization has the form

	A = U^T * U,	or		A = L* L^T

where U is an upper triangular and L is lower triangular.

This is the block version of the algorithm, calling Level 3 BLAS.


Arguments
---------

Upper (boolean)		-	

n (integer)			-		The dimension of the matrix A.  n >= 0.

A (array)			-		Array, dimension (LDA,N)
							On entry, the symmetric matrix A.  If UPLO = 'U', the leading
					        N-by-N upper triangular part of A contains the upper
							triangular part of the matrix A, and the strictly lower
							triangular part of A is not referenced.  If UPLO = 'L', the
							leading N-by-N lower triangular part of A contains the lower
							triangular part of the matrix A, and the strictly upper
							triangular part of A is not referenced.

							On exit, if INFO = 0, the factor U or L from the Cholesky
							factorization A = U^T * U or A = L * L^T.

lda (integer)		-		The leading dimension of the array A.  lda >= max(1,n).

Info (integer)		-		= 0:  successful exit
							< 0:  if Info = -i, the i-th argument had an illegal value
							> 0:  if Info = i, the leading minor of order i is not
							positive definite, and the factorization could not be
							completed.


Adapted from the Lapack subroutine,

	SUBROUTINE DPOTRF( UPLO, N, A, LDA, INFO )

  	-- LAPACK routine (version 3.0) --
    Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
    Courant Institute, Argonne National Lab, and Rice University
    March 31, 1993
  
*/

int LapackPotrf(bool Upper, int n, double *A, int lda)
{
	int Info;
	int j,jb,nb;
	
	/* Test the input parameters. */
	if(n < 0)
	{
		Print("Error in LapackPotrf");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	else if(lda < Max(1,n))
	{
		Print("Error in LapackPotrf");
		Print("lda < Max(1,n)");
		Print("lda",lda);
		HaltExecution();
	}
	
    /* Quick return if possible */
    if(n == 0) return 0;

	/*
	
	Determine the block size for this environment.
	
	The best preformance was found with lower unpacked storage
	and a block size of 2. I don't quite understand why this is
	happening. Perhaps register blocking?
	
	On a second check, I found an optimal block size of 8.
	
	Actually, optimal block size increases with n.
	
	*/
 	
	nb = 1;
 	
 	if(nb <= 1 || nb >= n)
 	{
 		/* Use unblocked code. */
		return LapackPotf2(Upper,n,A,lda);
	}
	else
	{
		/* Use blocked code. */
		
		if(Upper)
		{
			/* Compute the Cholesky factorization A = U^T * U. */
			for(j=0; j<n; j+=nb)
			{
				/* 
				
				Update and factorize the current diagonal block and test
				for non-positive-definiteness.
				
				*/
				
				jb = Min(nb,n-j);
				BlasSyrk(1,1,jb,j,-One,&A[j],lda,One,&A[lda*j+j],lda);
				Info = LapackPotf2(1,jb,&A[lda*j+j],lda);
				
				if(Info != 0) return j + 1;
				
            	if(j + jb < n)
            	{
            		/* Compute the current block row. */
            		BlasGemm(1,0,jb,n-j-jb,j,-One,&A[j],lda,&A[j+jb],lda,One,&A[lda*j+j+jb],lda);
            		BlasTrsm(1,1,1,1,jb,n-j-jb,One,&A[lda*j+j],lda,&A[lda*j+j+jb],lda);
            	}
            }
        }
        else
        {
           	/* Compute the Cholesky factorization A = L * L^T. */
        	for(j=0; j<n; j+=nb)
        	{
        		/* 
        		
        		Update and factorize the current diagonal block and test
				for non-positive-definiteness.
				
				*/
				
				jb = Min(nb,n-j);
				BlasSyrk(0,0,jb,j,-One,&A[lda*j],lda,One,&A[lda*j+j],lda);
				Info = LapackPotf2(0,jb,&A[lda*j+j],lda);
				
				if(Info != 0) return j + 1;
				
				if(j + jb < n)
				{	
					/* Compute the current block column. */
					BlasGemm(0,1,n-j-jb,jb,j,-One,&A[lda*(j+jb)],lda,&A[lda*j],lda,One,&A[lda*(j+jb)+j],lda);
                    BlasTrsm(0,0,1,1,n-j-jb,jb,One,&A[lda*j+j],lda,&A[lda*(j+jb)+j],lda);
				}
			}
        }
    }
    
    return 0;
}

#endif
