
#ifndef _LapackPptrf
#define _LapackPptrf


	/* Compute the Cholesky factorization of a symmetrix positive definite
	matrix stored in packed format */
	template <class Real> int LapackPptrf(bool Upper, int n, Real *A);

/*

Compute the Cholesky factorization of a symmetric positive
definite matrix A stored in packed format.

The factorization has the form,

	A = U^T * U, or A = L * L^T
	
where U is an upper-triangular and L is lower-triangular.


Arguments
---------

Upper (boolean)	-	Specifies where the matrix A is stored as upper
					triangular or lower triangular.
					
n (integer)		-	Specifies the dimension of the matrix A. Must
					be at least zero.
					
A (array) 	    -	An array of dimension n * (n + 1) / 2, which
					represents a lower-triangular  or upper-triangular
					matrix in packed form.
					
return			-	=0	:	Successful exit
					>0	:	If Info = i, the leading minor of order
							i+1 is not positive definite, and the
							factorization could not be completed.

This routine is adapted from the Lapack subroutine,
 
 	DPPTRF(UPLO, N, AP, INFO)
 	
 	-- LAPACK routine (version 3.0) --
	Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,
	Courant Institute, Argonne National Lab, and Rice University
	March 31, 1993

*/

template <class Real>
int LapackPptrf(bool Upper, int n, Real *A)
{

	/*
	
	Test the input parameters.
	In the original routine, Info = -1 if UpLo is not 'U' or 'L'
	
	*/
		
	if(n < 0)
	{
		Print("Error in LapackPptrf");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	
	/* Quick return if possible */
	if(n == 0) return 0;
	
	if(Upper)
	{
		/* Compute the Cholesky factorization A = U^T * U. */
		int jj = 0;
		for(int j=0; j<n; j++)
       	{
       		/* Compute U(j,j) and test for non-positive-definiteness. */
       		Real Ajj = A[jj];
           	if(Ajj <= Zero)
           	{
           		A[jj] = Ajj;
           		return j + 1;
           	}
           	Ajj = sqrt(Ajj);
           	A[jj] = Ajj;
           	
           	/* Compute elements j+1:n of column j and update the trailing submatrix. */
           	if(j < n - 1)
           	{
           		BlasScal(n-j-1,One/Ajj,&A[jj+1],1);
           		BlasSpr(1,n-j-1,-Real(One),&A[jj+1],1,&A[jj+n-j]);
           		jj += n - j;
           	}
		}
 	}
 	else
	{
		/* Compute the Cholesky factorization A = L * L^T. */
		int jj = -1;		
        for(int j=0; j<n; j++)
        {
           	int jc = jj + 1;
            jj += j + 1;
                      
			/* Compute elements 1:j-1 of column j. */
			if(j > 0) BlasTpsv(0,0,1,j,A,&A[jc],1);
			
			/* Compute L(j,j) and test for non-positive-definiteness. */
			Real Ajj = A[jj] - BlasDot(j,&A[jc],1,&A[jc],1);
				
        	if(Ajj <= Zero)
            {
            	A[jj] = Ajj;
            	return j + 1;	// Matrix is not spd.
			}
			A[jj] = sqrt(Ajj);
		}
	}
	
	return 0; // Successful execution
}
 
#endif
