
#ifndef _GenMatUtil_LinSolve
#define _GenMatUtil_LinSolve



/* Solve A * x = b where A is a rectangular matrix using the LU decomposition */	

int LinSolve(Vector<double> &x, const Matrix<double> &A, const Vector<double> &b)
{
	LUDecompObj<double> lu(A);
	int Info = lu.Solve(x,b);
	return Info;
}

/* Solve A * x = b where A is a lower-triangular matrix */

/* Old Code:

for(int j=0; j<n; j++) x[j] = b[j];
for(int j=0; j<n; j++) 
{
	for(int i=0; i<j; i++) x[j] -= x[i] * A(j,i);
	x[j] /= A(j,j);
}
	
*/

int LinSolve(Vector<double> &x, const LowerMatrix<double> &A, const Vector<double> &b)
{
	int n = A.Dim();
	
	/* Check Dimensions */
	if(x.Dim() != n || b.Dim() != n)
	{
		Print("Error in LinSolve(Vector,LowerMatrix,Vector)");
		Print("Solve A * x = b for x");
		Print("Incompadible Dimensions");
		Print("x.Size()",x.Size());
		Print("A.Size()",A.Size());
		Print("b.Size()",b.Size());
		HaltExecution();
	}
	
	/* Solve A * x = b */
	for(int i=0; i<n; i++) if(A(i,i) == Zero) return i + 1;
	Copy(x,b);
	BlasTpsv(0,0,1,n,A.Begin(),x.Begin(),1);
	return 0;
}

#ifdef _GenMat_SparseMatrixCC
	int LinSolve(Vector<double> &x, const SparseMatrixCC<double> &A, const Vector<double> &b)
	{
		/* Check Dimensions */
		int n = A.Rows();
		if(A.Cols() != n || x.Dim() != n || b.Dim() != n)
		{
			Print("Error in LinSolve(Vector,SparseMatrixCC(Lower),Vector)");
			Print("Solve A * x = b for x");
			Print("Incompadible Dimensions");
			Print("x.Size()",x.Size());
			Print("A.Size()",A.Size());
			Print("b.Size()",b.Size());
			HaltExecution();
		}
		
		/* Solve A * x = b */
		Copy(x,b);
		BlasTssv(0,0,1,n,A.r.Begin(),A.c.Begin(),A.v.Begin(),x.Begin(),1);
		return 0;
	}
#endif

/* Solve A * x = b where A is a upper-triangular matrix */
#ifdef _GenMat_UpperMatrix
	int LinSolve(Vector<double> &x, const UpperMatrix<double> &A, const Vector<double> &b)
	{
		int n = A.Dim();
		
		for(int i=0; i<n; i++) x[i] = b[i];

		/* Solve A * x = b */
	    for(int j=n-1; j>=0; j--) 
		{
	    	for(int i=j+1; i<n; i++) x[j] -= x[i] * A(j,i);
			x[j] /= A(j,j);
	    }    

		if(IsNan(x)) return 2;
	    
	    return 0;
	}
#endif

/*
int LinSolveLUBand(Vector<double> &x, const BandMatrix<double> &A, const Vector<double> &b)
{
	int n = A.Dim();
	int lb = A.Lower();
	int ub = A.Upper();
	BandMatrix<double> L(n,lb,0);		L.Name = "LinSolveLUBand::L";
	BandMatrix<double> U(n,0,lb+ub+1);	U.Name = "LinSolveLUBand::U";
	Vector<int> Piv(n);					Piv.Name = "LinSolveLUBand::Piv";
	
	int Chk = LUDecompBand(A,U,L,Piv);
	
	if(Chk) return 1;
	
	/*
	int i,k,l;
	int m1 = lb;
	int m2 = ub;
	int mm;
	double dum;

	Set(x,b);

	mm = m1 + m2 + 1;
	l = m1;
	for(k=1; k<=n; k++)
	{
		i = Piv[k-1];
		if(i != k) SWAP(x[k-1],x[i-1])
		if(l < n) l++;
		for(i=k+1; i<=l; i++) x[i-1] -= L(k-1,i-k-1) * x[k-1];
	}
	l = 1;
	for(i=n; i>=1; i--)
	{
		dum = x[i];
		for(k=2;k<=l;k++) dum -= U(i-1,k-1) * x[k+i-1-1];
		x[i-1] = dum / U(i-1,1-1);
		if(l < mm) l++;
	}
	*/
	/*
	return 0;
}
*/

/* Solve A * x = b using the Cholesky decomposition */
/*
int LinSolveCholesky(Vector<double> &x, const SymmetricMatrixLU<double> &A, const Vector<double> &b)
{
	int n = A.Dim();
	
	
	
	LowerMatrixUN<double> L(n);
	
	int Info = CholeskyDecomp(
	
	LowerMatrix<double> L(n);
	Vector<int> Piv(n);
	int Chk = CholeskyDecomp(A,L);
	if(Chk) return 1;
	
	for(i=0; i<n; i++) x[i] = b[i];

	// Solve L * y = b;
	for(j=0; j<n; j++) 
	{
		for(i=0; i<j; i++) x[j] -= x[i] * L(j,i);
		x[j] /= L(j,j);
	}

	// Solve L^T * x = y;
    for(j=n-1; j>=0; j--)
	{
    	for(i=j+1; i<n; i++) x[j] -= x[i] * L(i,j);
		x[j] /= L(j,j);
    }
    
    if(IsNan(x)) return 2;
    return 0;
}
*/

#ifdef _GenMat_LowerBandMatrix
	int LinSolve(Vector<double> &x, const LowerBandMatrix<double> &A, const Vector<double> &b)
	{
		int n = A.Rows();
		int k = A.LowerBands();
		
		if(A.Cols() != n || x.Dim() != n || b.Dim() != n)
		{
			Print("Error in LinSolve(Vector,LowerBandMatrix,Vector)");
			Print("Solve A * x = b for x");
			Print("Incompadible Dimensions");
			Print("x.Size()",x.Size());
			Print("A.Size()",A.Size());
			Print("b.Size()",b.Size());
			HaltExecution();
		}
		
		/* Solve A * x = b; */
		for(int i=0; i<n; i++) if(A(i,i) == Zero) return i + 1;
		Copy(x,b);
		BlasTbsv(0,0,1,n,k,A.Begin(),k,x.Begin(),1);
		return 0;
	}
#endif

#ifdef _GenMat_LowerBandMatrix
	int LinSolveTrans(Vector<double> &x, const LowerBandMatrix<double> &A, const Vector<double> &b)
	{
		int n = A.Rows();
		int k = A.LowerBands();
		
		if(A.Cols() != n || x.Dim() != n || b.Dim() != n)
		{
			Print("Error in LinSolveTrans(Vector,LowerBandMatrix,Vector)");
			Print("Solve A * x = b for x");
			Print("Incompadible Dimensions");
			Print("x.Size()",x.Size());
			Print("A.Size()",A.Size());
			Print("b.Size()",b.Size());
			HaltExecution();
		}
		
		/* Solve A^T * x = b; */
		for(int i=0; i<n; i++) if(A(i,i) == Zero) return i + 1;
		Copy(x,b);
		BlasTbsv(0,1,1,n,k,A.Begin(),k,x.Begin(),1);
		return 0;
	}
#endif

#ifdef _GenMat_LowerBandMatrix
	int LinSolve(Matrix<double> &X, const LowerBandMatrix<double> &A, const Matrix<double> &B)
	{
		int n = A.Rows();
		int k = A.LowerBands();
		int nRhs = B.Cols();
		
		if(A.Cols() != n || X.Rows() != n || X.Cols() != nRhs || B.Rows() != n)
		{
			Print("Error in LinSolve(Matrix,LowerBandMatrix,Matrix)");
			Print("Solve A * X = B for X");
			Print("Incompadible Dimensions");
			Print("X.Size()",X.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		/* Solve A * x = b; */
		for(int i=0; i<n; i++) if(A(i,i) == Zero) return i + 1;
		Copy(X,B);
		for(int i=0; i<nRhs; i++) BlasTbsv(0,0,1,n,k,A.Begin(),k,&X[i],nRhs);
	    return 0;
	}
#endif

#ifdef _GenMat_LowerBandMatrix
	int LinSolveTrans(Matrix<double> &X, const LowerBandMatrix<double> &A, const Matrix<double> &B)
	{
		int n = A.Rows();
		int k = A.LowerBands();
		int nRhs = B.Cols();
		
		if(A.Cols() != n || X.Rows() != n || X.Cols() != nRhs || B.Rows() != n)
		{
			Print("Error in LinSolve(Matrix,LowerBandMatrix,Matrix)");
			Print("Solve A * X = B for X");
			Print("Incompadible Dimensions");
			Print("X.Size()",X.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}
		
		/* Solve A * x = b; */
		for(int i=0; i<n; i++) if(A(i,i) == Zero) return i + 1;
		Copy(X,B);
		for(int i=0; i<nRhs; i++) BlasTbsv(0,1,1,n,k,A.Begin(),k,&X[i],nRhs);
	    return 0;
	}
#endif

#ifdef _SparseMatrixCC
	int LinSolveTrans(Vector<double> &x, const SparseMatrixCC<double> &A, const Vector<double> &b)
	{
		/* Check Dimensions */
		int n = A.Rows();
		if(A.Cols() != n || x.Dim() != n || b.Dim() != n)
		{
			Print("Error in LinSolveTrans(Vector,SparseMatrixCC(Lower),Vector)");
			Print("Solve A * x = b for x");
			Print("Incompadible Dimensions");
			Print("x.Size()",x.Size());
			Print("A.Size()",A.Size());
			Print("b.Size()",b.Size());
			HaltExecution();
		}
		
		/* Solve A * x = b */
		Copy(x,b);
		BlasTssv(0,1,1,n,A.r.Begin(),A.c.Begin(),A.v.Begin(),x.Begin(),1);
		return 0;
	}
#endif

#endif
