
#ifndef _GenMat_LowerMatrix
#define _GenMat_LowerMatrix

/*

Templated numerical array resenting a lower-triangular
matrix. Storage corresponds to C (row-major) ordering.
Elements are accessed via A(i,j) notation.

Array assignment is by reference (i.e. shallow assignment).
That is, B = A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.Copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.

This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.

Adapted from the Template Numerical Toolkit (TNT)

*/

template <class T>
class LowerMatrix : public GeneralMatrix<T>
{
	private:

	T*   p;
	int  n;				// Dimension of LowerMatrix
	int  len;			// Dimension of underlying array
	int* RefCount;		// Check for uneeded arrays
	T*   ZeroPointer;	// Pointer to the value 0

	void Initialize();
    void Destroy();

	public:
	string Name;		// Name of the LowerMatrix

	       LowerMatrix();
	       LowerMatrix(int n);
	       LowerMatrix(int n, T *a);
	       LowerMatrix(int n, const T &a);
	inline LowerMatrix(const LowerMatrix &A);
	inline LowerMatrix & operator=(const T &a);
	inline LowerMatrix & operator=(const LowerMatrix &A);
	inline LowerMatrix & Ref(const LowerMatrix &A);
	       LowerMatrix Copy() const;
	       LowerMatrix Conform() const;
		   LowerMatrix & Inject(const LowerMatrix & A);
	inline T& operator[](int i);
	inline const T& operator[](int i) const;
	inline T& operator()(int i, int j);
	inline const T& operator()(int i, int j) const;
	inline int Type() const;
	inline int Dim()  const;
	inline int Rows() const;
	inline int Cols() const;
	inline int Length() const;
	inline const Vector<int> Size() const;
	inline int GetRefCount() const;
	inline T* Begin();
	inline const T* Begin() const;
	inline T* End();
	inline const T* End() const;
	       ~LowerMatrix();
};

/* Public Funcitons */

/*

Copy constructor

Array data is NOT copied, but shared.
Thus, in LowerMatrix B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
LowerMatrix B(A.Copy()), or B = A.Copy(), instead.

*/

template <class T>
LowerMatrix<T>::LowerMatrix(const LowerMatrix<T> &A) : p(A.p), n(A.n), RefCount(A.RefCount), ZeroPointer(A.ZeroPointer), Name(A.Name)
{
	len = n * (n + 1) / 2;
	(*RefCount)++;
}

/* Create a null (0x0) array */
template <class T>
LowerMatrix<T>::LowerMatrix() : p(0), n(0), Name("Name")
{
	RefCount = new int;
	*RefCount = 1;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/*

Constructor 1:

Create a new (m x n) array, WITHOUT initializing array elements.
This version avoids the O(m*n) initialization overhead and
is used just before manual assignment.

@param n:	The dimension of the new Matrix.

*/

template <class T>
LowerMatrix<T>::LowerMatrix(int n) : p(0), n(n), RefCount(0), ZeroPointer(0)
{
	Name = "NoName";
	len = n * (n + 1) / 2;
	Initialize();
	RefCount = new int;
	*RefCount = 1;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/*

Constructor 2:

Create a new (n x n) array, initializing array elements to
constant specified by argument.

@param n:	The second (column) dimension of the new Matrix.
@param a:	The constant value to set all elements of the new array to.

*/

template <class T>
LowerMatrix<T>::LowerMatrix(int n, const T &a) : p(0), n(n), RefCount(0), ZeroPointer(0)
{
	Name = "NoName";
	len = n * (n + 1) / 2;
	Initialize();
	BlasSet(len,a,p,1);
	RefCount = new int;
	*RefCount = 1;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/*

Constructor 3:

Create a new (m x n) array,  as a view of an existing one-dimensional
array stored in <b>C order</b>, i.e. right-most dimension varying fastest.
(Often referred to as "row-major" ordering.)
Note that the storage for this pre-existing array will
never be garbage collected by the Array2D class.

@param n the dimension of the new Matrix.
@param a the one dimensional C array to use as data storage for the array.

*/

template <class T>
LowerMatrix<T>::LowerMatrix(int n, T *a) : n(n), RefCount(0), ZeroPointer(0)
{
	Name = "NoName";
	len = n * (n + 1) / 2;
	p = a;
	RefCount = new int;
	*RefCount = 2;		/* this avoid destorying original data. */
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/* */
template <class T>
inline T& LowerMatrix<T>::operator[](int i)
{
	return p[i];
}

/* */
template <class T>
inline const T& LowerMatrix<T>::operator[](int i) const
{
	return p[i];
}

/* Return pointer to a Row of a Matrix */
/*
template <class T>
inline T* LowerMatrix<T>::operator()(int i)
{
	#ifdef _CheckBounds
		if(i >= 0 && i < n_) return v_[i];
		else
		{
			Print("Error in LowerMatrix: Get Row out of bounds");
			Print("Current Index",i );
			Print("High Index",n_-1);
			Print("Halting Execution");
			exit(0);
			return v_[i];
		}
	#else
		return v_[i];
	#endif
}
*/

/* Access elements of the Matrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline T& LowerMatrix<T>::operator()(int i, int j)
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n || j < 0 || j >= n)
		{
			Print("Error in LowerMatrix: Index Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("n",n);
			HaltExecution();
		}
	#endif

	if(j <= i) return p[i*(i+1)/2+j];
	else
	{
		*ZeroPointer = 0;
		return *ZeroPointer;
	}
}

/* Access elements of the Matrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline const T& LowerMatrix<T>::operator()(int i, int j) const
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n || j < 0 || j >= n)
		{
			Print("Error in LowerMatrix: Index Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("n",n);
			HaltExecution();
		}
	#endif

	if(j <= i) return p[i*(i+1)/2+j];
	else
	{
		*ZeroPointer = 0;
		return *ZeroPointer;
	}
}

/* Assigns all elemnts of A to a constant scalar */
template <class T>
LowerMatrix<T> & LowerMatrix<T>::operator=(const T &a)
{
	BlasSet(len,a,1,val);
	return *this;
}

/*

Create a new copy of existing LowerMatrix.
Used in B = A.Copy() or in the construction
of B, e.g. LowerMatrix B(A.Copy()), to create a
new array that does not share data.

*/

template <class T>
LowerMatrix<T> LowerMatrix<T>::Copy() const
{
	LowerMatrix A(n);
	BlasCopy(len,Begin(),1,A.Begin(),1);
	return A;
}

/*

Create a new LowerMatrix with same dimensions
as existing LowerMatrix.

*/

template <class T>
LowerMatrix<T> LowerMatrix<T>::Conform() const
{
	LowerMatrix A(n);
	return A;
}


/*

Copy the elements to from one array to another, in place.
That is B.Inject(A), both A and B must conform (i.e. have
identical row and column dimensions).

This differs from B = A.Copy() in that references to B
before this assignment are also affected.  That is, if
we have

<pre>
Matrix A(m,n);
Matrix C(m,n);
Matrix B(C);        // elements of B and C are shared.

<pre>
then B.inject(A) affects both and C, while B = A.copy() creates
a new array B which shares no data with C or A.

@param A the Matrix from elements will be copied
@return an instance of the modifed array. That is, in B.inject(A),
it returns B.  If A and B are not conformat, no modifications to
B are made.

*/

template <class T>
LowerMatrix<T> & LowerMatrix<T>::Inject(const LowerMatrix<T> &A)
{
	if(A.n == n);
	BlasCopy(len,A.p,1,p,1);
	return *this;
}

/*

Create a reference (shallow assignment) to another existing array.
In B.ref(A), B and A shared the same data and subsequent changes
to the array elements of one will be reflected in the other.

<p>
This is what operator= calls, and B = A and B.ref(A) are equivalent
operations.

@return The new referenced array: in B.ref(A), it returns B.

*/

template <class T>
LowerMatrix<T> & LowerMatrix<T>::Ref(const LowerMatrix<T> &A)
{
	if(this != &A)
	{
		(*RefCount)--;
		if(*RefCount < 1) Destroy();
		n = A.n;
		len = A.len;
		p = A.p;
		Name = A.Name;
		RefCount = A.RefCount;
		(*RefCount)++;
		ZeroPointer = A.ZeroPointer;
	}
	return *this;
}

/* B = A is shorthand notation for B.Ref(A) */
template <class T>
LowerMatrix<T> & LowerMatrix<T>::operator=(const LowerMatrix<T> &A)
{
	return Ref(A);
}

/* Return the Type of the LowerMatrix ( = 4 ) */
template <class T>
inline int LowerMatrix<T>::Type() const
{
	return 4;
}

/* @return the number of rows in the Matrix */
template <class T>
inline int LowerMatrix<T>::Dim() const
{
	return n;
}

/* @return the number of rows in the Matrix */
template <class T>
inline int LowerMatrix<T>::Rows() const
{
	return n;
}

/* @return the number of columns in the Matrix */
template <class T>
inline int LowerMatrix<T>::Cols() const
{
	return n;
}

/* Return the length of the underlying array */
template <class T>
inline int LowerMatrix<T>::Length() const
{
	return len;
}

/*

Return the number of arrays that share the same storage area
as this one. (Must be at least one.)

*/

template <class T>
inline int LowerMatrix<T>::GetRefCount() const
{
	return *RefCount;
}

/* Return the Size of a Matrix in a 2-element Vector */
template <class T>
inline const Vector<int> LowerMatrix<T>::Size() const
{
	Vector<int> sz(2);
	sz[0] = n;
	sz[1] = n;
	return sz;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* LowerMatrix<T>::Begin()
{
	return &(p[0]);
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
const T* LowerMatrix<T>::Begin() const
{
	return &(p[0]);
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* LowerMatrix<T>::End()
{
	return &(p[0] + len);
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
const T* LowerMatrix<T>::End() const
{
	return &(p[0] + len);
}

/* Destructor */
template <class T>
LowerMatrix<T>::~LowerMatrix()
{
	(*RefCount)--;
	if(*RefCount < 1) Destroy();
}

/* Private internal functions */

/* Allocate memory for the Matrix */
template <class T>
void LowerMatrix<T>::Initialize()
{
	try
	{
		if(n > 0) p = new T[n*(n+1)/2];
	}
	catch(bad_alloc ba)
	{
		Print("Error in LowerMatrix");
		Print("Error Allocating Memory");
		Print("Size",Sqr(n));
		Print("Memory per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",Sqr(n)*sizeof(T)/1e6);
		HaltExecution();
	}
}

/* Set all the elements of a Matrix equal to a specified constant */
/*
template <class T>
void LowerMatrix<T>::Set_(const T& a)
{
	T *begin = &p[0];
	T *end = begin + len;
	for(T* p=begin; p<end; p++) *p = a;
}
*/

/* Copy the elements of one Matrix to another Matrix */
/*
template <class T>
void LowerMatrix<T>::Copy_(T* p, const T* q, int Len) const
{
	T *end = p + Len;
	while(p < end) *p++ = *q++;
}
*/

/* Delete the Matrix from memory */
template <class T>
void LowerMatrix<T>::Destroy()
{
	if(p != 0) delete[] p;
	if(RefCount != 0) delete RefCount;
	if(ZeroPointer != 0) delete ZeroPointer;
}

#endif
