
#ifndef _GenMat_Matrix
#define _GenMat_Matrix

/*

Templated two-dimensional numerical array which
looks like a conventional C multiarray.
Storage corresponds to C (row-major) ordering.
Elements are accessed via A(i,j) notation.

Array assignment is by reference (i.e. shallow assignment).
That is, B = A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.Copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.

This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.

Taken from Template Numerical Toolkit (TNT)

*/

template <class T>
class Matrix : public GeneralAssignableMatrix<T>
{
	//private:
	public:
    
    T*  p;
    T** v;				// Pointers to memory
	int m;				// Number of rows in the Matrix
    int n;				// Number of columns in the Matrix
    int len;			// Dimension of the underlying array
    int *RefCount;		// Check for uneeded arrays
	
	void Initialize(int m, int n);
    void Destroy();
	
	public:
   	
   	string Name;		// Name of the Matrix

	       Matrix();
	       Matrix(int m, int n);
	       Matrix(int m, int n, const T &a);
	       Matrix(int m, int n, T *a);
	       Matrix(int m, int n, const T *a);
	inline Matrix(const Matrix &A);
	//inline Matrix & operator=(const T &a);
	inline Matrix & operator=(const Matrix &A);
	inline Matrix & Ref(const Matrix &A);
	       Matrix Copy() const;
	       Matrix Conform() const;
		   Matrix & Inject(const Matrix &A);
	inline T& operator[](int i);
	inline const T& operator[](int i) const;
	inline T* operator()(int i); // Pointer Operator
	inline Vector<double> RowVector(int i);
	inline T& operator()(int i, int j);
	inline const T& operator()(int i, int j) const;
	inline int Type() const;
	inline int Rows() const;
	inline int Cols() const;
	inline int Length() const;
	inline const Vector<int> Size() const;
	inline double MemSize() const;
	inline T* Begin();
	inline const T* Begin() const;
	inline T* End();
	inline const T* End() const;
	inline T* Loc(int i);
	inline int GetRefCount() const;
           ~Matrix();
           
	inline void operator += (const T &c);
    inline void operator -= (const T &c);
    inline void operator *= (const T &c);
    inline void operator /= (const T &c);           
           
    inline void operator += (const Matrix<T> &A);
    inline void operator -= (const Matrix<T> &A);
    inline void operator *= (const Matrix<T> &A);
    inline void operator /= (const Matrix<T> &A);
};

/* Public Funcitons */

/*

Copy constructor

Array data is NOT copied, but shared.
Thus, in Matrix B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
Matrix B(A.Copy()), or B = A.Copy(), instead.

*/

template <class T>
Matrix<T>::Matrix(const Matrix<T> &A)
{
	p = A.p;
	v = A.v;
	m = A.m;
	n = A.n;
	RefCount = A.RefCount;
	Name = A.Name;
	len = m * n;
	(*RefCount)++;
}

/*

Constructor 1:

Create a new (m x n) array, WITHOUT initializing array elements.
This version avoids the O(m*n) initialization overhead and
is used just before manual assignment.

@param m:	The first (row) dimension of the new Matrix.
@param n:	The second (column) dimension of the new Matrix.

*/

template <class T>
Matrix<T>::Matrix(int m, int n)
{
	p = 0;
	v = 0;
	this->m = m;
	this->n = n;
	Name = "NoName";	
	len = m * n;
	
	if(len < 0)
	{
		Print("Error in Matrix::Matrix(int m, int n)");
		Print("m < 0 or n < 0");
		Print("m",m);
		Print("n",n);
		Print("len",len);
		HaltExecution();
	}
	
	if(len > 0) Initialize(m,n);
	RefCount = new int;
	*RefCount = 1;
}

/*

Constructor 2:

Create a new (m x n) array, initializing array elements to
constant specified by argument.

@param m:	The first (row) dimension of the new Matrix.
@param n:	The second (column) dimension of the new Matrix.
@param val:	The constant value to set all elements of the new array to.

*/

template <class T>
Matrix<T>::Matrix(int m, int n, const T &a)
{
	p = 0;
	v = 0;
	this->m = m;
	this->n = n;
	Name = "NoName";
	len = m * n;
	
	if(len < 0)
	{
		Print("Error in Matrix::Matrix(int m, int n, T val)");
		Print("m < 0 or n < 0");
		Print("m",m);
		Print("n",n);
		Print("len",len);
		HaltExecution();
	}
	
	if(len > 0) Initialize(m,n);
	//Set(val);
	BlasSet(m*n,a,p,1);
	
	RefCount = new int;
	*RefCount = 1;
}

/*

Constructor 3:

Create a new (m x n) array,  as a view of an existing one-dimensional
array stored in <b>C order</b>, i.e. right-most dimension varying fastest.
(Often referred to as "row-major" ordering.)
Note that the storage for this pre-existing array will
never be garbage collected by the Array2D class.

@param m the first (row) dimension of the new Matrix.
@param n the second (column) dimension of the new Matrix.
@param a the one dimensional C array to use as data storage for the array.

*/

template <class T>
Matrix<T>::Matrix(int m, int n, T *a)
{
	p = 0;
	v = 0;
	this->m = m;
	this->n = n;
	Name = "NoName";
	len = m * n;

	if(m > 0 && n > 0)
	{
		p = a;
		v = new T*[m];
		for(int i=0; i<m; i++)
		{
			v[i] = p;
			p += n;
		}
		p = v[0];
	}
	
	RefCount = new int;
	*RefCount = 2;		// Thiss avoid destroying original data.
	
}

/*

Constructor 4:

Create a new (m x n) array,  as a view of an existing one-dimensional
array stored in <b>C order</b>, i.e. right-most dimension varying fastest.
(Often referred to as "row-major" ordering.)
Note that the storage for this pre-existing array will
never be garbage collected by the Array2D class.

@param m the first (row) dimension of the new Matrix.
@param n the second (column) dimension of the new Matrix.
@param a the one dimensional C array to use as data storage for the array.

*/

template <class T>
Matrix<T>::Matrix(int m, int n, const T *a)
{
	p = 0;
	v = 0;
	this->m = m;
	this->n = n;
	Name = "NoName";
	len = m * n;
	if(m > 0 && n > 0)
	{
		p = a;
		v = new T*[m];
		for(int i=0; i<m; i++)
		{
			v[i] = p;
			p += n;
		}
		p = v[0];
	}
	
	RefCount = new int;
	*RefCount = 2;		// Thiss avoid destroying original data.
	
}


/* ... */
template <class T>
inline T& Matrix<T>::operator[](int i)
{
	return p[i];
}

/* ... */
template <class T>
inline const T& Matrix<T>::operator[](int i) const
{
	return p[i];
}

/* Return pointer to a Row of a Matrix */
template <class T>
inline T* Matrix<T>::operator()(int i)
{
	#ifdef _CheckBounds
		if(i < 0 && i >= m)
		{
			Print("Error in Matrix");
			Print("(i) Out of Bounds");
			Print("Current Index",i );
			Print("High Index",n-1);		
			HaltExecution();
		}
	#endif
	
	return v[i];
}

/* Return a Vector Containing the Row of a Matrix */
template <class T>
inline Vector<double> Matrix<T>::RowVector(int i)
{
	#ifdef _CheckBounds
		if(i < 0 && i >= m)
		{
			Print("Error in Matrix");
			Print("(i) Out of Bounds");
			Print("High Index",n-1);		
			HaltExecution();
		}
	#endif
	
	return Vector<double>(m,&v[i]);
}

/* Access elements of the Matrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline T& Matrix<T>::operator()(int i, int j)
{
	#ifdef _CheckBounds
		if(i < 0 || i >= m || j < 0 || j >= n)
		{
			Print("Error in Matrix");
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}
	#endif
	
	return v[i][j];
}

/* Access elements of the Matrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline const T& Matrix<T>::operator()(int i, int j) const
{
	#ifdef _CheckBounds
		if(i < 0 || i >= m || j < 0 || j >= n)
		{
			Print("Error in Matrix");
			Print("(i,j) Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}
	#endif

	return v[i][j];
}

/* Assigns all elemnts of A to a constant scalar */
/*
template <class T>
Matrix<T> & Matrix<T>::operator=(const T &a)
{
	//Set(a);
	BlasSet(len,a,Begin(),1);
	return *this;
}
*/

/*

Create a new copy of existing Matrix.
Used in B = A.copy() or in the construction
of B, e.g. Matrix B(A.copy()), to create a
new array that does not share data.

*/

template <class T>
Matrix<T> Matrix<T>::Copy() const
{
	Matrix A(m,n);
	BlasCopy(len,Begin(),1,A.Begin(),1);
	return A;
}

/*

Create a new Matrix with same dimensions
as existin matrix.

*/

template <class T>
Matrix<T> Matrix<T>::Conform() const
{
	Matrix A(m,n);
	return A;
}


/*

Copy the elements to from one array to another, in place.
That is B.Inject(A), both A and B must conform (i.e. have
identical row and column dimensions).

This differs from B = A.Copy() in that references to B
before this assignment are also affected.  That is, if
we have

<pre>
Matrix A(m,n);
Matrix C(m,n);
Matrix B(C);        // elements of B and C are shared.

<pre>
then B.inject(A) affects both and C, while B = A.copy() creates
a new array B which shares no data with C or A.

@param A the Matrix from elements will be copied
@return an instance of the modifed array. That is, in B.inject(A),
it returns B.  If A and B are not conformat, no modifications to
B are made.

*/

template <class T>
Matrix<T> & Matrix<T>::Inject(const Matrix<T> &A)
{
	if(A.m_ == m_ && A.n_ == n_)
	BlasCopy(len_,&A.v_[0],1,&v_[0],1);
	return *this;
}

/*

Create a reference (shallow assignment) to another existing array.
In B.ref(A), B and A shared the same data and subsequent changes
to the array elements of one will be reflected in the other.

<p>
This is what operator= calls, and B = A and B.ref(A) are equivalent
operations.

@return The new referenced array: in B.ref(A), it returns B.

*/

template <class T>
Matrix<T> & Matrix<T>::Ref(const Matrix<T> &A)
{
	if(this != &A)
	{
		(*RefCount)--;
		if(*RefCount < 1) Destroy();
		m = A.m;
		n = A.n;
		len = A.len;
		p = A.p;
		v = A.v;
		Name = A.Name;
		RefCount = A.RefCount;
		(*RefCount)++;
	}
	
	return *this;
}

/* B = A is shorthand notation for B.Ref(A) */
template <class T>
Matrix<T> & Matrix<T>::operator=(const Matrix<T> &A)
{
	return Ref(A);
}

/* Return the Type of the Matrix ( = 1 ) */
template <class T>
inline int Matrix<T>::Type() const
{
	return MatrixType::Matrix;
}

/* @return the number of rows in the Matrix */
template <class T>
inline int Matrix<T>::Rows() const
{
	return m;
}

/* @return the number of columns in the Matrix */
template <class T>
inline int Matrix<T>::Cols() const
{
	return n;
}

/* Return the length of the underlying array */
template <class T>
inline int Matrix<T>::Length() const
{
	return len;
}

/* Return the Size of a Matrix in a 2-element Vector */
template <class T>
inline const Vector<int> Matrix<T>::Size() const
{
	Vector<int> sz(2);
	sz[0] = m;
	sz[1] = n;
	return sz;
}

/* Return the Number of Megabytes Used to Store a Metrix */
template <class T>
inline double Matrix<T>::MemSize() const
{
	return len * sizeof(T) / 1e6;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
inline T* Matrix<T>::Begin()
{
	return &p[0];
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
inline const T* Matrix<T>::Begin() const
{
	return &p[0];
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
inline T* Matrix<T>::End()
{
	return &p[0] + len;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
inline const T* Matrix<T>::End() const
{
	return &p[0] + len;
}

template <class T> T* Matrix<T>::Loc(int i)
{
	/* bounds checking */
	if(i < 0 || i >= len)
	{
		Print("Error in Matrix::Loc()");
		Print("Index out of bounds");
		Print("i",i);
		Print("len",len);
		HaltExecution();
	}
	return &p[i];
}

/*

@return the number of arrays that share the same storage area
as this one.  (Must be at least one.)

*/

template <class T>
inline int Matrix<T>::GetRefCount() const
{
	return *RefCount;
}

/* Destructor */
template <class T>
Matrix<T>::~Matrix()
{
	(*RefCount)--;
	if(*RefCount < 1) Destroy();
}

/* Private internal functions */

/* Allocate memory for the Matrix */
template <class T>
void Matrix<T>::Initialize(int m, int n)
{
	try
	{
		p = new T[m*n];
		v = new T*[m];
		for(int i=0; i<m; i++)
		{
			v[i] = p;
			p += n;
		}
		p = v[0];
		this->m = m;
		this->n = n;
	}
	catch(bad_alloc ba)
	{
		Print("Error in Matrix: Error Allocating Memory");
		Print("m",m);
		Print("n",n);
		Print("Size",len);
		Print("Memory per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",len*sizeof(T)/1e6);
		HaltExecution();
	}	
}

/* Set all the elements of a Matrix equal to a specified constant */
/*
template <class T>
void Matrix<T>::Set(const T& Val)
{
	T *begin = &p[0];
	T *end = begin + m * n;
	for(T* q=begin; q<end; p++) *q = Val;
}

/* Copy the elements of one Matrix to another Matrix */
/*
template <class T>
void Matrix<T>::Copy(T* p, const T* q, int Len) const
{
	T *End = p + Len;
	while(p < End) *p++ = *q++;
}
*/

/* Delete the Matrix from memory */
template <class T>
void Matrix<T>::Destroy()
{
	if(v != 0)
	{
		delete[] (v[0]);
		delete[] (v);
	}
	if(RefCount != 0) delete RefCount;
}

/* Create a null (0x0) array */
template <class T>
Matrix<T>::Matrix() : v(0), m(0), n(0), len(0)
{
	RefCount = new int;
	*RefCount = 1;
}

template <class T>
inline void Matrix<T>::operator += (const T &c)
{
	for(int i=0; i<len; i++) p[i] += c;
}

template <class T>
inline void Matrix<T>::operator -= (const T &c)
{
	for(int i=0; i<len; i++) p[i] -= c;
}

template <class T>
inline void Matrix<T>::operator *= (const T &c)
{
	for(int i=0; i<len; i++) p[i] *= c;
}

template <class T>
inline void Matrix<T>::operator /= (const T &c)
{
	for(int i=0; i<len; i++) p[i] /= c;
}


template <class T>
inline void Matrix<T>::operator += (const Matrix<T> &A)
{
	for(int i=0; i<len; i++) p[i] += A.p[i];
}

template <class T>
inline void Matrix<T>::operator -= (const Matrix<T> &A)
{
	for(int i=0; i<len; i++) p[i] -= A.p[i];
}

template <class T>
inline void Matrix<T>::operator *= (const Matrix<T> &A)
{
	for(int i=0; i<len; i++) p[i] *= A.p[i];
}

template <class T>
inline void Matrix<T>::operator /= (const Matrix<T> &A)
{
	for(int i=0; i<len; i++) p[i] /= A.p[i];
}

#endif
