
#ifndef _Stat_Mean
#define _Stat_Mean

/* Function Prototypes */

	/* Compute the sample mean */
	double Mean(const GeneralVector<double> &X);
	Vector<double> Mean(const Matrix<double> &X);
	Vector<double> Mean(const Matrix<double> &X, const Matrix<bool> &XMiss);
	Vector< DataPoint<double> > Mean(const Matrix< DataPoint<double> > &X);
	Vector<double> Mean(const Vector< Vector<double> > &X);
	SymmetricMatrix<double> Mean(const Vector< SymmetricMatrix<double> > &X);
	
	/* Weighted Mean */
	double MeanWeighted(const GeneralVector<double> &X, const GeneralVector<double> &W);
	
	/* Mean of Sub-vector */
	double MeanSub(const GeneralVector<double> &X, int n0, int n1);

/* Compute the Sample Mean for Scalar-valued Data */
double Mean(const GeneralVector<double> &X)
{
	return Sum(X) / (double)(X.Size());
}

/* Compute the sample mean for scalar-valued data */
Vector<double> Mean(const Vector< Vector<double> > &X)
{
	if(X.Size() == 0)
	{
		Print("Error in Mean(Vector(Vector))");
		Print("X.Size()",X.Size());
		HaltExecution();
	}
	int K = X(0).Size();
	int N = X.Size();
	Vector<double> Mu(K,Zero);
	Mu.Name = "Mean::Mu";
	Mu.Name = "Mean(" + X.Name + ")";
	for(int n=0; n<N; n++) Mu += X(n);
	return Mu;
}

/* Compute the Weighted Sample Mean for Scalar-Valued Data */
double MeanWeighted(const GeneralVector<double> &X, const GeneralVector<double> &W)
{
	int N = X.Size();
	
	/* Check for Incompadible Dimensions */
	if(W.Size() != N)
	{
		Print("Error in MeanWeighted(GeneralVector<double> X,GeneralVector<double> W)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("W.Name",W.Name);		
		Print("X.Size()",X.Size());
		Print("W.Size()",W.Size());
		HaltExecution();
	}
	
	if(N == 0) return Nan;
	
	/* Compute Weighted Mean */
	double Num = Zero;
	double Den = Zero;
	for(int n=0; n<N; n++)
	{
		Num += W(n) * X(n);
		Den += W(n);
	}
	return Num / Den;
}

/* Compute the Weighted Sample Mean for Scalar-Valued Data */
Vector< DataPoint<double> > MeanWeighted(const Matrix<double> &X, const GeneralVector<double> &W)
{
	int N = X.Rows();
	int K = X.Cols();
	
	/* Check for Incompadible Dimensions */
	if(W.Size() != N)
	{
		Print("Error in MeanWeighted(Matrix<double> X,GeneralVector<double> W)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("W.Name",W.Name);		
		Print("X.Size()",X.Size());
		Print("W.Size()",W.Size());
		HaltExecution();
	}
	
	if(N == 0) return Vector< DataPoint<double> >(K);
	
	/* Compute Weighted Mean */
	Vector< DataPoint<double> > Ret(K);
	double Den = Zero;
	for(int n=0; n<N; n++) Den += W(n);
	for(int k=0; k<K; k++)
	{
		Ret(k) = Zero;
		for(int n=0; n<N; n++) Ret(k) += W(n) * X(n,k);
		Ret(k) /= Den;
	}
	return Ret;
}

/* Compute the Weighted Sample Mean for Scalar-Valued Data */
DataPoint<double> MeanWeighted(const GeneralVector< DataPoint<double> > &X, const GeneralVector<double> &W)
{
	int N = X.Size();
	
	/* Check for Incompadible Dimensions */
	if(W.Size() != N)
	{
		Print("Error in MeanWeighted(GeneralVector< DataPoint > X,GeneralVector<double> W)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("W.Name",W.Name);		
		Print("X.Size()",X.Size());
		Print("W.Size()",W.Size());
		HaltExecution();
	}
	
	if(N == 0) return DataPoint<double>();
	
	/* Compute Weighted Mean */
	double Num = Zero;
	double Den = Zero;
	for(int n=0; n<N; n++) if(!X(n).IsMissing())
	{
		Num += W(n) * X(n).GetData();
		Den += W(n);
	}
	if(Den > Zero) return Num / Den;
	else           return DataPoint<double>();
}

/* Compute the sample mean for vector-valued data */
Vector<double> Mean(const Matrix<double> &X)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector<double> Ret(K,Zero);
	for(int n=0; n<N; n++) for(int k=0; k<K; k++) Ret[k] += X(n,k);
	for(int k=0; k<K; k++) Ret[k] /= (double)N;
	return Ret;
}

/* Compute the sample mean for vector-valued data with missing values */
Vector<double> Mean(const Matrix<double> &X, const Matrix<bool> &XMiss)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector<int> Count(K,0);
	Vector<double> Ret(K,Zero);
	for(int n=0; n<N; n++)
	{
		for(int k=0; k<K; k++)
		{
			if(!XMiss(n,k))
			{
				Ret[k] += X(n,k);
				Count[k]++;
			}
		}
	}
	for(int k=0; k<K; k++) Ret[k] /= (double)Count[k];
	return Ret;
}

/* Compute the sample mean for vector-valued data with missing values */
Vector< DataPoint<double> > Mean(const Matrix< DataPoint<double> > &X)
{
	int N = X.Rows();
	int K = X.Cols();
	Vector<int> Count1(K,0);
	Vector<double> Sum1(K,Zero);
	for(int n=0; n<N; n++)
	{
		for(int k=0; k<K; k++)
		{
			if(!X(n,k).IsMissing())
			{
				Sum1(k) += X(n,k).GetData();
				Count1(k)++;
			}
		}
	}
	Vector< DataPoint<double> > Ret(K);
	Ret.Name = "Mean(" + X.Name + ")";
	for(int k=0; k<K; k++)
	{
		if(Count1(k) > 0) Ret(k) = Sum1(k) / (double)Count1(k);
		else              Ret(k).SetToMissing();
	}
	return Ret;
}

/*
template <class T>
Vector<double> MeanBy(const Vector<double> &x, const Vector<T> &y, const Vector<T> &List)
{
	int N = x.Size();
	int K = List.Size();
	
	if(y.Size() != N)
	{
		Print("Error in MedianBy(x,y,List)");
		Print("Incompadible dimensions");
		Print("x.Name",x.Name);
		Print("y.Name",y.Name);
		Print("x.Size()",x.Size());
		Print("y.Size()",y.Size());
	}	
	
	Vector< Stack<double> > x_By_y(K);
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in MeanBy");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		x_By_y[Curr].Push(x[n]);
	}
	Vector<double> Ret(K);
	for(int k=0; k<K; k++) Ret[k] = Mean(x_By_y[k]);
	return Ret;
}
*/

template <class T>
Vector<double> MeanBy(const GeneralVector<double> &x, const GeneralVector<T> &y)
{
	int N = x.Size();
	if(y.Size() != N)
	{
		Print("Error in MeanBy(x,y)");
		Print("Incompadible Dimensions");
		Print("x.Name",x.Name);
		Print("y.Name",y.Name);
		Print("x.Size()",x.Size());
		Print("y.Size()",y.Size());
	}	
	
	Vector<T> List = ListValues(y);
	int K = List.Size();	
	Vector< Stack<double> > x_By_y(K);
	
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in MeanBy(x,y)");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		x_By_y[Curr].Push(x[n]);
	}
	Vector<double> Ret(K);
	for(int k=0; k<K; k++) Ret[k] = Mean(x_By_y[k]);
	return Ret;
}

template <class T>
Matrix<double> MeanBy(const Matrix<double> &X, const GeneralVector<T> &Y)
{
	int N = X.Rows();
	int J = X.Cols();
	if(Y.Size() != N)
	{
		Print("Error in MeanBy(X,Y)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("Y.Name",Y.Name);
		Print("X.Size()",X.Size());
		Print("Y.Size()",Y.Size());
	}	
	
	Vector<T> List = ListValues(Y);
	int K = List.Size();	
	Matrix< Stack<double> > X_By_Y(K,J);
	
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,Y(n));
		if(Curr < 0)
		{
			Print("Error in MeanBy(X,Y)");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		for(int j=0; j<J; j++) X_By_Y(Curr,j).Push(X(n,j));
	}
	Matrix<double> Ret(K,J);
	for(int k=0; k<K; k++) for(int j=0; j<J; j++) Ret(k,j) = Mean(X_By_Y(k,j));
	return Ret;
}


template <class T>
Vector< DataPoint<double> > MeanBy(const GeneralVector<double> &x, const GeneralVector<T> &y, const GeneralVector<T> &List)
{
	int N = x.Size();
	if(y.Size() != N)
	{
		Print("Error in MeanBy(x,y)");
		Print("Incompadible Dimensions");
		Print("x.Name",x.Name);
		Print("y.Name",y.Name);
		Print("x.Size()",x.Size());
		Print("y.Size()",y.Size());
	}	
	
	int K = List.Size();	
	Vector< Stack<double> > x_By_y(K);
	
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in MeanBy(x,y,List)");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		x_By_y[Curr].Push(x[n]);
	}
	Vector< DataPoint<double> > Ret(K);
	for(int k=0; k<K; k++)
	{
		if(x_By_y[k].Size() > 0) Ret[k] = Mean(x_By_y[k]);
	}
	return Ret;
}

template <class T>
Vector<double> MeanBy(const GeneralVector<double> &X, const GeneralVector<bool> &XMiss, const GeneralVector<T> &Y)
{
	int N = X.Size();
	if(XMiss.Size() != N || Y.Size() != N)
	{
		Print("Error in MeanBy(X,XMiss,Y)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("XMiss.Name",XMiss.Name);
		Print("Y.Name",Y.Name);
		Print("X.Size()",X.Size());
		Print("XMiss.Size()",XMiss.Size());
		Print("Y.Size()",Y.Size());
	}	
	
	Vector<T> List = ListValues(Y);
	int K = List.Size();	
	Vector< Stack<double> > x_By_y(K);
	
	for(int n=0; n<N; n++)
	{
		if(!XMiss(n))
		{
			int Curr = BinarySearch(List,Y[n]);
			if(Curr < 0)
			{
				Print("Error in MeanBy(X,XMiss,Y)");
				Print("Curr < 0");
				Print("Curr",Curr);
				HaltExecution();
			}
			x_By_y[Curr].Push(X[n]);
		}
	}
	Vector<double> Ret(K);
	for(int k=0; k<K; k++) Ret[k] = Mean(x_By_y[k]);
	return Ret;
}

DataPoint<double> Mean(const GeneralVector< DataPoint<double> > &X)
{
	int N = X.Size();
	int N1 = 0;
	double Ret = Zero;
	for(int n=0; n<N; n++)
	{
		if(!X(n).IsMissing())
		{
			N1++;
			Ret += X(n).Data;
		}
	}
	if(N1 == 0) return DataPoint<double>();
	else        return Ret / (double)N1;
}
	

template <class T>
Vector< DataPoint<double> > MeanBy(const GeneralVector< DataPoint<double> > &X, const GeneralVector<T> &Y)
{
	int N = X.Size();
	if(Y.Size() != N)
	{
		Print("Error in MeanBy(X,Y)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("Y.Name",Y.Name);
		Print("X.Size()",X.Size());
		Print("Y.Size()",Y.Size());
	}	
	
	Vector<T> List = ListValues(Y);
	int K = List.Size();	
	Vector< Stack<double> > x_By_y(K);
	
	for(int n=0; n<N; n++)
	{
		if(!X(n).IsMissing())
		{
			int Curr = BinarySearch(List,Y(n));
			if(Curr < 0)
			{
				Print("Error in MeanBy(X,Y)");
				Print("Curr < 0");
				Print("Curr",Curr);
				HaltExecution();
			}
			x_By_y(Curr).Push(X(n).Data);
		}
	}
	Vector< DataPoint<double> > Ret(K);
	for(int k=0; k<K; k++)
	{
		if(x_By_y(k).Size() > 0) Ret(k) = Mean(x_By_y(k));
		else                     Ret(k).SetToMissing();
	}
	return Ret;
}

template <class T>
Vector< DataPoint<double> > MeanBy(const GeneralVector< DataPoint<double> > &X, const GeneralVector<T> &Y, const GeneralVector<T> &List)
{
	int N = X.Size();
	if(Y.Size() != N)
	{
		Print("Error in MeanBy(X,Y)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("Y.Name",Y.Name);
		Print("X.Size()",X.Size());
		Print("Y.Size()",Y.Size());
	}	
	
	int K = List.Size();	
	Vector< Stack<double> > x_By_y(K);
	
	for(int n=0; n<N; n++)
	{
		if(!X(n).IsMissing())
		{
			int Curr = BinarySearch(List,Y(n));
			if(Curr < 0)
			{
				Print("Error in MeanBy(X,Y)");
				Print("Curr < 0");
				Print("Curr",Curr);
				HaltExecution();
			}
			x_By_y(Curr).Push(X(n).Data);
		}
	}
	Vector< DataPoint<double> > Ret(K);
	for(int k=0; k<K; k++)
	{
		if(x_By_y(k).Size() > 0) Ret(k) = Mean(x_By_y(k));
		else                     Ret(k).SetToMissing();
	}
	return Ret;
}


template <class T>
Matrix< DataPoint<double> > MeanBy(const Matrix< DataPoint<double> > &X, const GeneralVector<T> &Y)
{
	int N = X.Rows();
	int J = X.Cols();
	if(Y.Size() != N)
	{
		Print("Error in MeanBy(X,Y)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("Y.Name",Y.Name);
		Print("X.Size()",X.Size());
		Print("Y.Size()",Y.Size());
	}	
	
	Vector<T> List = ListValues(Y);
	int K = List.Size();	
	Matrix< Stack<double> > x_By_y(K,J);
	
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,Y(n));
		if(Curr < 0)
		{
			Print("Error in MeanBy(X,Y)");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		for(int j=0; j<J; j++)
		{
			if(!X(n,j).IsMissing())
			{
				x_By_y(Curr,j).Push(X(n,j).GetData());
			}
		}
	}
	Matrix< DataPoint<double> > Ret(K,J);
	for(int k=0; k<K; k++) for(int j=0; j<J; j++)
	{
		if(x_By_y(k,j).Size() > 0) Ret(k,j) = Mean(x_By_y(k,j));
		else                       Ret(k,j).SetToMissing();
	}
	return Ret;
}


template <class T>
Vector<double> MeanWeightedBy(const Vector<double> &X, const Vector<double> &W, const Vector<T> &Y)
{
	Vector<T> List = ListValues(Y);
	return MeanWeightedBy(X,W,Y,List);

	/*
	int N = x.Size();
	if(w.Size() != N || y.Size() != N)
	{
		Print("Error in MeanWeightedBy(x,w,y)");
		Print("Incompadible Dimensions");
		Print("x.Name",x.Name);
		Print("w.Name",w.Name);
		Print("y.Name",y.Name);
		Print("x.Size()",x.Size());
		Print("w.Size()",w.Size());
		Print("y.Size()",y.Size());
	}	
	
	Vector<T> List = ListValues(y);
	int K = List.Size();	
	Vector< Stack<double> > x_By_y(K);
	Vector< Stack<double> > w_By_y(K);
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in MeanWeightedBy(x,w,y)");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		x_By_y[Curr].Push(x[n]);
		w_By_y[Curr].Push(w[n]);
	}
	Vector<double> Ret(K);
	for(int k=0; k<K; k++) Ret[k] = MeanWeighted(x_By_y[k],w_By_y[k]);
	return Ret;
	*/
}

template <class T>
Vector<double> MeanWeightedBy(const Vector<double> &X, const Vector<double> &W, const Vector<T> &Y, const Vector<T> &List)
{
	int N = X.Size();
	if(W.Size() != N || Y.Size() != N)
	{
		Print("Error in MeanWeightedBy(X,W,Y,List)");
		Print("Incompadible Dimensions");
		Print("X.Name",X.Name);
		Print("W.Name",W.Name);
		Print("Y.Name",Y.Name);
		Print("X.Size()",X.Size());
		Print("W.Size()",W.Size());
		Print("Y.Size()",Y.Size());
	}	
	
	int K = List.Size();	
	Vector< Stack<double> > X_By_Y(K);
	Vector< Stack<double> > W_By_Y(K);
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,Y[n]);
		if(Curr < 0)
		{
			Print("Error in MeanWeightedBy(X,W,Y,List)");
			Print("Curr < 0");
			Print("Curr",Curr);
			Print("Y(n)",Y(n));
			Print("List",List);
			HaltExecution();
		}
		X_By_Y[Curr].Push(X[n]);
		W_By_Y[Curr].Push(W[n]);
	}
	Vector<double> Ret(K);
	for(int k=0; k<K; k++) Ret[k] = MeanWeighted(X_By_Y[k],W_By_Y[k]);
	return Ret;
}


template <class T>
Matrix<double> MeanBy(const Matrix<double> &X, const Vector<T> &y, const Vector<T> &List)
{
	int N = X.Rows();
	int K = X.Cols();
	int R = List.Size();
	
	if(y.Size() != N)
	{
		Print("Error in MedianBy(X,y,List)");
		Print("Incompadible dimensions");
		Print("X.Name",X.Name);
		Print("y.Name",y.Name);
		Print("X.Size()",X.Size());
		Print("y.Size()",y.Size());
	}	
	
	Matrix< Stack< double > > X_By_y(R,K);
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in MeanBy");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		for(int k=0; k<K; k++) X_By_y(Curr,k).Push(X(n,k));
	}
	Matrix<double> Ret(R,K);
	for(int r=0; r<R; r++) for(int k=0; k<K; k++) Ret(r,k) = Mean(X_By_y(r,k));
	return Ret;
}

template <class T>
Matrix<double> MeanWeightedBy(const Matrix<double> &X, const Vector<double> &w, const Vector<T> &y, const Vector<T> &List)
{
	int N = X.Rows();
	int K = X.Cols();
	int R = List.Size();
	
	if(y.Size() != N || w.Size() != N)
	{
		Print("Error in MeanWeightedBy(X,w,y,List)");
		Print("Incompadible dimensions");
		Print("X.Name",X.Name);
		Print("y.Name",y.Name);
		Print("X.Size()",X.Size());
		Print("y.Size()",y.Size());
	}	
	
	Matrix< Stack<double> > X_By_y(R,K);
	Vector< Stack<double> > w_By_y(R); 
	for(int n=0; n<N; n++)
	{
		int Curr = BinarySearch(List,y[n]);
		if(Curr < 0)
		{
			Print("Error in MeanBy");
			Print("Curr < 0");
			Print("Curr",Curr);
			HaltExecution();
		}
		for(int k=0; k<K; k++) X_By_y(Curr,k).Push(X(n,k));
		w_By_y[Curr].Push(w[n]);
	}
	Matrix<double> Ret(R,K);
	for(int r=0; r<R; r++) for(int k=0; k<K; k++) Ret(r,k) = MeanWeighted(X_By_y(r,k),w_By_y[r]);
	return Ret;
}

template <class T>
double MeanWeightedIf(const GeneralVector<double> &X, const GeneralVector<double> &W, const GeneralVector<T> &Y, const T &a)
{
	int N = X.Size();
	if(W.Size() != N || Y.Size() != N)
	{
		Print("Error in MeanWeightedIf(X,W,Y,a)");
		Print("Incompadible Dimensions");
		Print("X.Name"  ,X.Name);
		Print("W.Name"  ,W.Name);
		Print("Y.Name"  ,Y.Name);
		Print("X.Size()",X.Size());
		Print("W.Size()",W.Size());
		Print("Y.Size()",Y.Size());
	}	
	
	double Num = Zero;
	double Den = Zero;
	for(int n=0; n<N; n++)
	{
		if(Y(n) == a)
		{
			Num += W(n) * X(n);
			Den += W(n);
		}
	}
	
	return Num / Den;
}

SymmetricMatrix<double> Mean(const Vector< SymmetricMatrix<double> > &X)
{
	int n = X.Size();
	int m = X(0).Dim();
	SymmetricMatrix<double> Sum1(m,Zero);
	Sum1.Name = "Mean(Vector<SymmetricMatrix>)::Sum1";
	for(int i=0; i<n; i++) Sum1 += X(i);
	Sum1 /= (double)n;
	return Sum1;
}

/* Compute the Sample Mean for Scalar-valued Data */
double MeanSub(const GeneralVector<double> &X, int n0, int n1)
{
	return SumSub(X,n0,n1) / (double)(n1 - n0 + 1);
}

template <class T>
Vector< DataPoint<double> > MeanBy(const GeneralVector< DataPoint<double> > &X, const GeneralVector<T> &Y, Vector<T> &List)
{
	
	/* Check for Incompadible Dimensions */
	int N = X.Size();
	if(Y.Size() != N)
	{
		Print("Error in MeanBy(x,y,List)");
		Print("Incompadible dimensions");
		Print("X.Name",X.Name);
		Print("Y.Name",Y.Name);
		Print("X.Size()",X.Size());
		Print("Y.Size()",Y.Size());
	}
	
	Stack<T> Y1;
	for(int n=0; n<N; n++) if(!X(n).IsMissing()) Y1.Push(Y(n));
	
	int K = List.Size();
		
	Vector< Stack<double> > X_By_Y(K);
	for(int n=0; n<N; n++)
	{
		if(!X(n).IsMissing())
		{
			int Curr = BinarySearch(List,Y(n));
			if(Curr < 0)
			{
				Print("Error in MeanBy");
				Print("Curr < 0");
				Print("Curr",Curr);
				HaltExecution();
			}
			X_By_Y(Curr).Push(X(n).Data);
		}
	}
	Vector< DataPoint<double> > Ret(K);
	for(int k=0; k<K; k++)
	{
		if(X_By_Y(k).Size() > 0) Ret(k) = Mean(X_By_Y(k));
	}
	return Ret;
}



#endif