
#ifndef _Opt_MinNewtonLin
#define _Opt_MinNewtonLin

/*

Minimize a nonlinear function subject to linear equality constriants

This method procedes by projecting x onto a space of feasible points
If x0 is a feasible, and feasible point can be expresed by

x = x0 + Z * w

where A = [A1 A2]	Z = [ A1^-1 * A2 ]
		  				[     I      ]
		  				
If the intial point is not feasible, then x0 is set to [0 , A1^-1 * b ]

Option:

PrintLevel = 0	-	Print nothing except fatal errors
			 1	-	Print only exit status
			 2	-	Print one line per iteration
			 3	-	Normal Mode
			 4	-	Print backtracking
			 5	-	Print matrices

Passing the function;

	int = Func1(x,f)			-	Given x, compute function value
	int = Func2(x,f,df)			-	Given x, compute function and derivative
	int = Func3(x,f,df,ddf)		-	Given x, compute function, derivative, and hessian
	int = Func4(x,f,df,p)		-	Given x, compute function, derivative, and step direction

	Using Forward Derivatives
	
	int = Func5(x,f,df)

*/

class MinNewtonLin
{
	
	public:
	
	// Parameters
	int MaxIter;			// Maximum Number of Iterations
	double TolX;			// Tolerance in X
	double TolD;			// Tolerance for Projected Gradient
	double MinDiff;			// Minimum Decrease Required in Objective Function
	
	// Numerical Derivatives
	double Deriv1Step;		// Step Length for First Derivatives
	int Deriv1Type;			// Type of First Derivative
	double Deriv2Step;	// Step Length for Second Derivatives
	int Deriv2Type;		// Type of Second Derivative
	
	// Line Search
	double RhoFac;			// Rho Reduction Factor
	double MinStep;			// Minimum Step Size
	
	// Scaling Variables
	double TypF;
	Vector<double> TypX;
	Vector<double> TypW;
	double Tol;
	int SkipUpdate;
	int BfgsUpdate;
	
	// Printing
	int PrintLevel;			// Print Level (General)
	
	int FuncEval;			// Number of Function Evaluations
	int GradEval;
	int HessEval;
	int StepEval;
	int TotEval;
	double CpuTime;			// Cpu Time Used	
	
	// Algorithm Options
	int Recompute;
	int InitializeHessian;
	int TestPosDef;
	Vector<double> t;
	
	// Tolerances
	double xMax;
	double dfMax;
	
	double f;
	Matrix<double> Z;
		
	Vector<double> x;							// Current point
	
	bool FeasCheck;								// Check feasibility of initial point ?
	
	private:
	
	int n;										// Number of variables
	int m;										// Number of constraints
	int r;										// Number of free variables
	Vector<double> w;							// Current projected point
	int (*Func1)(const Vector<double> &, double &);				// Objective Function
	int (*Func2)(const Vector<double> &, double &, Vector<double> &, int);
	int (*Func3)(const Vector<double> &, double &, Vector<double> &, SymmetricMatrix<double> &);
	int (*Func4)(const Vector<double> &, double &, Vector<double> &, Vector<double> &, int);
		
	Matrix<double> A;							// Constraint Matrix
	Vector<double> b;							// Constraint Vector
	
	int Info;
	
	public:
	int Iter;
	
	private:
	int Step;
	double RhoCurr;				
	
	double fDiff;
	Vector<double> 			dfProj;		// Projected Gradient
	SymmetricMatrix<double> ddfProj;	// Projected Hessian
	Vector<double> df;					// Gradient
	SymmetricMatrix<double> ddf;		// Hessian
	LowerMatrix<double>    ddfL;
	DiagonalMatrix<double> ddfD;
	double fPrev;
	Vector<double> xPrev;
	Vector<double> wPrev;
	Vector<double> dfPrev;
	Vector<double> wDiff;
	Vector<double> dfDiff;
	int GradOpt;
	int HessOpt;
	int Restart;
	int NoUpdate;
	Vector<double> x0;
	double c1,c2;
	Vector<double> p;
	Vector<double> pProj;
	Vector<double> v;
	Vector<double> u;
	double PosDefFac;
	double PosDefTest;
	int tst;
	Vector<double> pTemp;
	
	public:
	
	/* Default Constructor */
	MinNewtonLin() {};
	
	/* Constructor 1: No Analytical Derivatives, No Linear Equality Constriants */
	MinNewtonLin(int (*Func1)(const Vector<double> &, double &), int n)
	{
		this->Func1 = Func1;
		this->n = n;
		m = 0;
		r = n;
		AllocateArrays();
		DefaultParameters();
		Z.Name = "MinNewtonLin::Z";
		GradOpt = 1;
		HessOpt = 1;
	}
	
	/* Constructor 2: Analytical Derivatives, No Linear Equality Constraints */
	MinNewtonLin(int (*Func2)(const Vector<double> &, double &, Vector<double> &, int), int n)
	{
		this->Func2 = Func2;
		this->n = n;
		m = 0;
		r = n;
		AllocateArrays();
		DefaultParameters();
		Z.Name = "MinNewtonLin::Z";
		GradOpt = 2;
		HessOpt = 1;
	}
	
	/* Constructor 3: Analytical Hessian, No Linear Equality Constraints */
	MinNewtonLin(int (*Func3)(const Vector<double> &, double &, Vector<double> &, SymmetricMatrix<double> &), int n)
	{
		this->Func3 = Func3;
		this->n = n;
		m = 0;
		r = n;
		AllocateArrays();
		DefaultParameters();
		Z.Name = "MinNewtonLin::Z";
		df  = Vector<double>(n);				df.Name   = "MinNewtonLin::df";
		ddf = SymmetricMatrix<double>(n,n);		ddf.Name  = "MinNewtonLin::ddf";
		GradOpt = 3;
		HessOpt = 2;
	}
	
	/* Constructor 4: Search Direction Provided, No Linear Equality Constraints */
	MinNewtonLin(int (*Func4)(const Vector<double> &, double &, Vector<double> &, Vector<double> &, int), int n)
	{
		this->Func4 = Func4;
		this->n = n;
		m = 0;
		r = n;
		AllocateArrays();
		DefaultParameters();
		Z.Name = "MinNewtonLin::Z";
		df  = Vector<double>(n);				df.Name   = "MinNewtonLin::df";
		pTemp = Vector<double>(n);				pTemp.Name = "MinNewtonLin::pTemp";
		GradOpt = 4;
		HessOpt = 3;
	}
	
	/* Constructor 5 */
	MinNewtonLin(int (*Func1)(const Vector<double> &, double &),
		const Matrix<double> &A, const Vector<double> &b, int n, int m)
	{
		this->Func1 = Func1;
		this->A = A;
		this->b = b;
		this->n = n;
		this->m = m;
		if(n > 0)
		{
			if(m > 0)
			{
				Z = Projection(A);
				Z.Name = "MinNewtonLin::Z";
				r = Z.Cols();
			}
			else r = n;
			AllocateArrays();
			DefaultParameters();
			GradOpt = 1;
			HessOpt = 1;
		}
	}
	
	/* Constructor 6 */
	MinNewtonLin(int (*Func2)(const Vector<double> &, double &, Vector<double> &, int),
		const Matrix<double> &A, const Vector<double> &b, int n, int m)
	{
		this->Func2 = Func2;
		this->A = A;
		this->b = b;
		this->n = n;
		this->m = m;
		if(n > 0)
		{
			if(m > 0)
			{
				Z = Projection(A);
				Z.Name = "MinNewtonLin::Z";
				r = Z.Cols();
			}
			else r = n;
			AllocateArrays();
			DefaultParameters();
			df  = Vector<double>(n);				df.Name    = "MinNewtonLin::df1";
			GradOpt = 2;
			HessOpt = 1;
		}
	}
	
	/* Constructor 7 */
	MinNewtonLin(int (*Func3)(const Vector<double> &, double &, Vector<double> &, SymmetricMatrix<double> &),
		const Matrix<double> &A, const Vector<double> &b, int n, int m)
	{
		this->Func3 = Func3;
		this->A = A;
		this->b = b;
		this->n = n;
		this->m = m;
		if(n > 0)
		{
			if(m > 0)
			{
				Z = Projection(A);
				Z.Name = "MinNewtonLin::Z";
				r = Z.Cols();
			}
			else r = n;
			AllocateArrays();
			DefaultParameters();
			df = Vector<double>(n);					df.Name    = "MinNewtonLin::df";
			ddf = SymmetricMatrix<double>(n,n);		ddf.Name   = "MinNewtonLin::ddf";
			GradOpt = 3;
			HessOpt = 2;
		}
	}
	
	/* Constructor 8 */
	MinNewtonLin(int (*Func4)(const Vector<double> &, double &, Vector<double> &, Vector<double> &, int),
		const Matrix<double> &A, const Vector<double> &b, int n, int m)
	{
		this->Func4 = Func4;
		this->A = A;
		this->b = b;
		this->n = n;
		this->m = m;
		if(n > 0)
		{
			if(m > 0)
			{
				Z = Projection(A);
				Z.Name = "MinNewtonLin::Z";
				r = Z.Cols();
			}
			else r = n;
			AllocateArrays();
			DefaultParameters();
			df = Vector<double>(n);					df.Name    = "MinNewtonLin::df";
			pTemp = Vector<double>(n);				pTemp.Name = "MinNewtonLin::pTemp";
			GradOpt = 4;
			HessOpt = 3;
		}
	}
	
	/* Constructor 1: No Analytical Derivatives, No Linear Equality Constriants */
	#ifdef _AutoDeriv
	
		int (*Func5)(const Vector< BDeriv<double> > &, BDeriv<double> &);
		int (*Func6)(const Vector< FDeriv<double> > &, FDeriv<double> &);
		
		MinNewtonLin(int (*Func5)(const Vector< BDeriv<double> > &, BDeriv<double> &), int n)
		{
			this->n = n;
			this->Func5 = Func5;
			m = 0;
			r = n;
			AllocateArrays();
			DefaultParameters();
			Z.Name = "MinNewtonLin::Z";
			GradOpt = 5;
			HessOpt = 1;
		}
	
		/* Constructor 5 */
		MinNewtonLin(int (*Func5)(const Vector< BDeriv<double> > &, BDeriv<double> &),
						const Matrix<double> &A, const Vector<double> &b, int n, int m)
		{
			this->Func5 = Func5;
			this->A = A;
			this->b = b;
			this->n = n;
			this->m = m;
			if(n > 0)
			{
				if(m > 0)
				{
					Z = Projection(A);
					Z.Name = "MinNewtonLin::Z";
					r = Z.Cols();
				}
				else r = n;
				AllocateArrays();
				DefaultParameters();
				GradOpt = 5;
				HessOpt = 1;
			}
		}
		
		/* Constructor 1: No Analytical Derivatives, No Linear Equality Constriants */
		MinNewtonLin(int (*Func6)(const Vector< FDeriv<double> > &, FDeriv<double> &), int n)
		{
			this->n = n;
			this->Func6 = Func6;
			m = 0;
			r = n;
			AllocateArrays();
			DefaultParameters();
			Z.Name = "MinNewtonLin::Z";
			GradOpt = 6;
			HessOpt = 1;
		}
		
		/* Constructor 5 */
		MinNewtonLin(int (*Func6)(const Vector< FDeriv<double> > &, FDeriv<double> &),
						const Matrix<double> &A, const Vector<double> &b, int n, int m)
		{
			this->Func6 = Func6;
			this->A = A;
			this->b = b;
			this->n = n;
			this->m = m;
		
			if(n > 0)
			{
				if(m > 0)
				{
					Z = Projection(A);
					Z.Name = "MinNewtonLin::Z";
					r = Z.Cols();
				}
				else r = n;
				AllocateArrays();
				DefaultParameters();
				GradOpt = 6;
				HessOpt = 1;
			}
		}
		
	#endif
	
	//~MinNewtonLin() {}
	
	void AllocateArrays()
	{
		/* Name Objects */
		x       = Vector<double>(n,Zero);			x.Name       = "MinNewtonLin::x";
		w       = Vector<double>(r);				w.Name       = "MinNewtonLin::w";
		df      = Vector<double>(n);				df.Name      = "MinNewtonLin::df";
		ddf     = SymmetricMatrix<double>(n);		ddf.Name     = "MinNewtonLin::ddf";
		dfProj  = Vector<double>(r);				dfProj.Name  = "MinNewtonLin::dfProj";
		ddfProj = SymmetricMatrix<double>(n);		ddfProj.Name = "MinNewtonLin::ddfProj";
		ddfD    = DiagonalMatrix<double>(r);		ddfL.Name    = "MinNewtonLin::ddfD";
		ddfL    = LowerMatrix<double>(r);			ddfD.Name    = "MinNewtonLin::ddfL";
		xPrev   = Vector<double>(n);				xPrev.Name   = "MinNewtonLin::xPrev";
		wPrev   = Vector<double>(r);				wPrev.Name   = "MinNewtonLin::wPrev";
		dfPrev  = Vector<double>(r);				dfPrev.Name  = "MinNewtonLin::dfPrev";	
		wDiff   = Vector<double>(r);				wDiff.Name   = "MinNewtonLin::wDiff";
		dfDiff  = Vector<double>(r);				dfDiff.Name  = "MinNewtonLin::dfDiff";
		//x0      = Vector<double>(n);				x0.Name      = "MinNewtonLin::x0";
		p       = Vector<double>(n);				p.Name       = "MinNewtonLin::p";
		pProj   = Vector<double>(r);				pProj.Name   = "MinNewtonLin::pProj";
		v       = Vector<double>(r);				v.Name       = "MinNewtonLin::v";
		u       = Vector<double>(r);				u.Name       = "MinNewtonLin::u";
		t       = Vector<double>(r);				t.Name       = "MinNewtonLin::t";
		TypX    = Vector<double>(n,One);			TypX.Name    = "MinNewtonLin::TypX";
		TypW    = Vector<double>(r,One);			TypW.Name    = "MinNewtonLin::TypW";
	}
	
	void DefaultParameters()
	{
		MaxIter   = 1000;
		TolX      = 0.00000001;
		TolD      = 0.000001;
		Deriv1Step = MachEps2;
		Deriv1Type = 1;
		Deriv2Step = MachEps3;
		Deriv2Type = 1;
		RhoFac = 0.5;
		MinStep = 1.0e-16;
		PrintLevel = 3;
		NoUpdate = 0;
		MinDiff = 0.00000001;
		Recompute = 0;
		FuncEval = 0;
		GradEval = 0;
		HessEval = 0;
		StepEval = 0;
		TotEval = 0;
		TypF = 1.0;
		PosDefTest = 0.2;
		Restart = 1;
		Recompute = 0;
		InitializeHessian = 0;
		BfgsUpdate = 1;
		FeasCheck = 1;
		Iter = 0;
	}
	
	int EvaluateFunc()
	{
		if(GradOpt == 1)
		{
			int Info = Func1(x,f);
			FuncEval++;
			return Info;
		}
		else if(GradOpt == 2)
		{
			int Info = Func2(x,f,df,OPT_EVAL_F);
			FuncEval++;
			return Info;
		}
		else if(GradOpt == 3)
		{
			int Info = Func3(x,f,df,ddf);
			HessEval++;
			return Info;
		}
		else if(GradOpt == 4)
		{
			int Info = Func4(x,f,df,pTemp,OPT_EVAL_F);
			FuncEval++;
			return Info;
		}
		#ifdef _AutoDeriv
			else if(GradOpt == 5)
			{
				Vector< BDeriv<double> > xB(n);
				for(int i=0; i<n; i++) xB(i) = x(i);
				BDeriv<double> fB;
				int Info = Func5(xB,fB);
				FuncEval++;
				f = fB.val();
				return Info;
			}
			else
			{
				Vector< FDeriv<double> > xF(n);
				for(int i=0; i<n; i++) xF(i) = x(i);
				FDeriv<double> fF;
				int Info = Func6(xF,fF);
				FuncEval++;
				f = fF.val();
				return Info;
			}
		#endif
	}
	
	int FindStep()
	{
		
		if(GradOpt == 1)
		{
			/* Evaluate Gradient Using Finite Differences */
			if(m == 0)
			{
				int Info = Gradient(Func1,x,f,dfProj,Deriv1Step,Deriv1Type,TypX);
				GradEval++;
				if(Info) return Info;
			}					
			else
			{
				int Info = Gradient(Func1,x,Z,f,dfProj,Deriv1Step,Deriv1Type,TypX);
				GradEval++;
				if(Info) return Info;
			}
			
		}
		else if(GradOpt == 2)
		{
			/* Evaluate Gradient Analytically */
			if(m == 0)
			{
				int Info = Func2(x,f,dfProj,OPT_EVAL_DF);
				GradEval++;
				if(Info) return Info;
			}
			else
			{
				int Info = Func2(x,f,df,OPT_EVAL_DF);
				GradEval++;
				if(Info) return Info;
				ProdAtX(dfProj,Z,df);
			}					
		}
		else if(GradOpt == 3)
		{
			/* Evaluate Gradient and Hessian Analytically */
			if(m == 0)
			{
				int Info = Func3(x,f,dfProj,ddfProj);
				HessEval++;
				if(Info) return Info;
			}
			else
			{
				int Info = Func3(x,f,df,ddf);
				HessEval++;
				if(Info) return Info;
				ProdAtX (dfProj ,Z,df);
				ProdAtBA(ddfProj,Z,ddf);
			}
		}
		else if(GradOpt == 4)
		{
			/* Evaluate Gradient and Step Direction Analytically */
			if(m == 0)
			{
				int Info = Func4(x,f,dfProj,p,OPT_EVAL_DF);
				StepEval++;
				if(Info) return Info;
			}
			else
			{
				int Info = Func4(x,f,df,p,OPT_EVAL_DF);
				StepEval++;
				if(Info) return Info;
				if(m == 0)
				{
					Copy(dfProj,df);
					Copy(pProj,p);
				}
				else
				{
					ProdAtX(dfProj,Z,df);
					ProdAtX(pProj ,Z,p );
				}
			}
		}
		#ifdef _AutoDeriv
			else if(GradOpt == 5)
			{
				/* Evaluate Gradient using Automatic Analytical Diffentiation (Backword Method) */
				if(m == 0)
				{
					Vector< BDeriv<double> > xB(n);
					for(int i=0; i<n; i++) xB(i) = x(i);
					BDeriv<double> fB;
					int Info = Func5(xB,fB);
					FuncEval++;
					for(int i=0; i<n; i++) fB.diff(0,1);
					f = fB.x();
					for(int i=0; i<n; i++) dfProj(i) = xB(i).d(0);
					if(Info) return Info;
				}
				else
				{
					Vector< BDeriv<double> > xB(n);
					for(int i=0; i<n; i++) xB(i) = x(i);
					BDeriv<double> fB;
					int Info = Func5(xB,fB);
					FuncEval++;
					for(int i=0; i<n; i++) fB.diff(0,1);
					f = fB.x();
					for(int i=0; i<n; i++) df(i) = xB(i).d(0);
					if(Info) return Info;
					ProdAtX(dfProj,Z,df);
				}
			}
			else
			{
				/* Evaluate Gradient using Automatic Analytical Diffentiation (Forward Method) */
				if(m == 0)
				{
					Vector< FDeriv<double> > xF(n);
					for(int i=0; i<n; i++) xF(i) = x(i);
					FDeriv<double> fF;
					for(int i=0; i<n; i++) xF(i).diff(i,n);
					int Info = Func6(xF,fF);
					FuncEval++;
					f = fF.x();
					for(int i=0; i<n; i++) dfProj(i) = fF.d(i);
					if(Info) return Info;
				}
				else
				{
					Vector< FDeriv<double> > xF(n);
					for(int i=0; i<n; i++) xF(i) = x(i);
					FDeriv<double> fF;
					for(int i=0; i<n; i++) xF(i).diff(i,n);
					int Info = Func6(xF,fF);
					FuncEval++;
					f = fF.x();
					for(int i=0; i<n; i++) dfProj(i) = fF.d(i);
					if(Info) return Info;
					ProdAtX(dfProj,Z,df);
				}					
			}	
		#endif
		
		if(HessOpt == 1)
		{
			/* No Analytical Hessian Provided */
			if((Recompute || Restart) && !InitializeHessian)
			{
				/* Set hessian to the identity matrix */
				if(PrintLevel >= 3) Print("Setting Hessian to Identity");
				for(int i=0; i<r; i++) for(int j=0; j<=i; j++) ddfL(i,j) = One * (i == j);
				for(int i=0; i<r; i++) ddfD(i)  = One;
				if(m == 0) 
				{
					for(int i=0; i<n; i++) pProj[i] = -dfProj[i];
					for(int i=0; i<n; i++) p[i]     = -dfProj[i];
				}
				else
				{
					for(int i=0; i<r; i++) pProj[i] = -dfProj[i];
					ProdAX(p,Z,pProj);
				}
			}
			else if((Recompute || Restart) && InitializeHessian)
			{
				/* Compute Hessian Using Finite Differences */
				if(PrintLevel >= 3) Print("Computing Hessian Using Finite Differences");
				if(m == 0)
				{
					int Info = Hessian(Func1,x,f,df,ddf,Deriv2Step,Deriv2Type,TypX);
					HessEval++;
					if(Info) return Info;
				}
				else
				{
					int Info = Hessian(Func1,x,Z,f,df,ddf,Deriv2Step,Deriv2Type,TypX);
					HessEval++;
					if(Info) return Info;
				}
				
				/* Cholesky Factorize the Hessian */
				Print("This Part is Not Done Yet");
				HaltExecution();
			}
			else
			{
				if(BfgsUpdate)
				{
					/* Update hesssian using BFGS (factored form) */
					if(PrintLevel >= 3) Print("Updating Hessian Using BFGS");
					Subtract(wDiff ,w ,wPrev);
					Subtract(dfDiff,dfProj,dfPrev);
					c1 = ProdXtY(wDiff,dfDiff);
					if(c1 > sqrt(MachEps) * NormL2(wDiff) * NormL2(dfDiff))
					{
						/* Do Not Skip Update */
						for(int i=0; i<r; i++)
						{
							/* t = L^T * z */
							t[i] = Zero;
							for(int j=i; j<r; j++) t[i] += ddfL(j,i) * wDiff[j];
						}
						
						/* c2 = s^T * L * D * L^T * s */
						c2 = Zero;
						for(int i=0; i<r; i++) c2 += ddfD(i) * Sqr(t[i]);
					 
						Tol = sqrt(MachEps);	// Change this line for analytical gradient
						SkipUpdate = 1;
						for(int i=0; i<r; i++)
						{
							u[i] = Zero;
							for(int j=0; j<=i; j++) u[i] += ddfD(j) * ddfL(i,j) * t[j];
							if(Abs(dfDiff[i] - u[i]) >= Tol * Max(Abs(dfProj[i]),Abs(dfPrev[i]))) SkipUpdate = 0;
						}
						
						if(!SkipUpdate)
						{
							/* Do Not Skip Update */
							for(int i=0; i<r; i++) dfDiff[i] /= sqrt(c1);
							for(int i=0; i<r; i++) u[i] /= sqrt(c2);
							LDLUpdatePos(ddfL,ddfD,dfDiff);
							LDLUpdateNeg(ddfL,ddfD,u);
						}
						else if(PrintLevel >= 3) Print("Skipping Hessian Update");
					}
					else if(PrintLevel >= 3) Print("Skipping Hessian Update");
				}
				else
				{
					/* Update Hessian using DFP */
					if(PrintLevel >= 3) Print("Updating Hessian Using DFP");
					Print("Error in MinNewtonLin");
					Print("THIS PART IS NOT DONE YET");
					HaltExecution();
				}
			
				/* Solve ddf * p = -df for p */
				for(int i=0; i<r; i++)
				{
					u[i] = -dfProj[i];
					for(int j=0; j<i; j++) u[i] -= ddfL(i,j) * u[j];
					u[i] /= ddfL(i,i);
				}
				for(int i=0; i<n-m; i++) u[i] /= ddfD(i);
				for(int i=r-1; i>=0; i--)
				{
					pProj[i] = u[i];
					for(int j=i+1; j<r; j++) pProj[i] -= ddfL(j,i) * pProj[j];
					pProj[i] /= ddfL(i,i);
				}
				if(m == 0) Copy(p,pProj);
				else       ProdAX(p,Z,pProj);
			}
		}
		else if(HessOpt == 2)
		{
			/* Compute Hessian Analytically */
			if(PrintLevel >= 3) Print("Analytical Hessian Provided");
			
			/* Compute the LDL decomposition */
			//int LDLDecomp1(ddf,ddfL,ddfD);
						
			/* Solve ddf * p = -df */
			Print("This Part is Not Done Yet");
			HaltExecution();
		}
		else
		{
			if(Restart)
			{
				/* Set Hessian to the Identity Matrix */
				if(PrintLevel >= 3) Print("Setting Hessian to Identity");
				for(int i=0; i<r; i++) for(int j=0; j<=i; j++) ddfL(i,j) = One * (i == j);
				for(int i=0; i<r; i++) ddfD(i)  = One;
				if(m == 0) 
				{
					for(int i=0; i<n; i++) pProj[i] = -dfProj[i];
					for(int i=0; i<n; i++) p[i]     = -dfProj[i];
				}
				else
				{
					for(int i=0; i<r; i++) pProj[i] = -dfProj[i];
					ProdAX(p,Z,pProj);
				}
			}
			else
			{
				/* ... */
				if(PrintLevel >= 3) Print("Step Provided Directly");
				if(m == 0)
				{
					Copy(df,dfProj);
					Copy(pProj,p);					
				}
				else
				{
					ProdAtX(dfProj,Z,df);
					ProdAtX(pProj,Z,p);
					ProdAX(p,Z,pProj);
				}
			}
		}
		
		return 0;
	}
		
	/* Preform a Line Search */
	void LineSearch()
	{
		for(int i=0; i<n; i++) x[i] += p[i];
		Info = EvaluateFunc();
		Step = 1;
		RhoCurr = One;
		xMax = NormLInf(p);
		while((!(f < fPrev - MinDiff) || Info) && (RhoCurr * xMax >= MinStep))
		{			
			for(int i=0; i<n; i++) x[i] -= RhoCurr * (One - RhoFac) * p[i];
			RhoCurr *= RhoFac;	
			Info = EvaluateFunc();
			if(PrintLevel >= 4)
			{
				if(Info) Print("Step",Step,"Info",Info,"fCurr",Inf,"RhoCurr",RhoCurr);
				else     Print("Step",Step,"Info",Info,"fCurr",f  ,"RhoCurr",RhoCurr);
			}
			Step++;
		}
		for(int i=0; i<r; i++) w[i] += RhoCurr * pProj[i];
		if(PrintLevel >= 3) Print("Steps",Step);
		if(PrintLevel >= 3) Print("fNew",f);
	}
	
	void ComputeTotEval()
	{
		TotEval = FuncEval;
		if(GradOpt == 1)
		{
			if(Deriv1Type == 3 || Deriv1Type == 6) TotEval += GradEval * (2 * (n - m) + 1);
			else                                   TotEval += GradEval * (    (n - m) + 1);
			
			if(Deriv2Type == 3 || Deriv2Type == 6) TotEval += HessEval * (1 + 2 * (n - m) + 4 * (n - m) * (n - m - 1) / 2);
			else                                   TotEval += HessEval * (1 + 2 * (n - m) +     (n - m) * (n - m - 1) / 2);
		}
		else
		{
			TotEval += GradEval + HessEval + StepEval;			
		}	
	}
	
	int TestConverge()
	{
		fDiff = f - fPrev;
		xMax  = RhoCurr * NormLInf(p);
		dfMax = Zero;
		double dfCurr;
		for(int i=0; i<r; i++)
		{
			dfCurr = abs(dfProj[i]) * Max(abs(w[i]),One / TypW[i]) / Max(abs(f),TypF);
			if(dfCurr > dfMax) dfMax = dfCurr;
		}
								
		if(PrintLevel >= 3) Print("RhoCurr",RhoCurr);
		if(PrintLevel >= 3) Print("fDiff",fDiff);
		if(PrintLevel >= 3) Print("xMax",xMax);
		if(PrintLevel >= 3) Print("dfMax",dfMax);
		if(PrintLevel == 2) Print("Iter",Iter,"f",f,"xMax",xMax,"dfMax",dfMax);
		
		/* Check if New Step Was an Improvement */
		if(!(fDiff < Zero) || Info)
		{
			if(Restart == 1)
			{
				/* No Improvement Found, Go Back to Previous Step */
				if(PrintLevel >= 2) Print("Exiting MinNewtonLin: Improvement Was Not Found");
				if(PrintLevel >= 2) Print("Number of Iterations",Iter);
				ComputeTotEval();
				if(PrintLevel >= 2) Print("Number of Function Evaluations",TotEval);
				if(PrintLevel == 1) Print("Exiting: Insufficient Change in X - Iter",Iter,"xMax",xMax,"dfMax",dfMax);
				Copy(x,xPrev);
				Copy(w,wPrev);
				return 23;
			}			
			else
			{
				/* Try Resarting */
				if(PrintLevel >= 2) Print("Improvement Was Not Found: Recomputing Hessian");
				Restart = 1;
				Copy(x,xPrev);
				Copy(w,wPrev);
				return -1;
			}
		}		
		
		if(dfMax < TolD)
		{
			if(PrintLevel >= 2) Print("Succesfully Exiting MinNewtonLin: D Within Tolerance");
			if(PrintLevel >= 2) Print("Number of Iterations",Iter);
			ComputeTotEval();
			if(PrintLevel >= 2) Print("Number of Function Evaluations",TotEval);
			if(PrintLevel == 1) Print("Exiting: Succesful - Iter",Iter,"xMax",xMax,"dfMax",dfMax);
			if(fPrev < f) Copy(x,xPrev);
			return 0;
		}
		
		if(!(fDiff < -MinDiff) || Info)
		{
			if(Restart == 1)
			{
				if(PrintLevel >= 2) Print("Unsuccessfully Exiting MinNewtonLin: Insufficient Decrease");
				if(PrintLevel >= 2) Print("Number of Iterations",Iter);
				ComputeTotEval();
				if(PrintLevel >= 2) Print("Number of Function Evaluations",TotEval);
				if(PrintLevel == 1) Print("Exiting: Insufficient Decrease - Iter",Iter,"xMax",xMax,"dfMax",dfMax);
				return 1;
			}
			else
			{
				/* Try Resarting */
				if(PrintLevel >= 2) Print("Insufficient Decrease: Recomputing Hessian");
				Restart = 1;
				Copy(x,xPrev);
				Copy(w,wPrev);
			}				
		}
		else if(xMax < TolX)
		{
			if(Restart == 1)
			{
				if(PrintLevel >= 2) Print("Unsucessfully Exiting MinNewtonLin: Insufficient Change in X");
				if(PrintLevel >= 2) Print("Number of Iterations",Iter);
				ComputeTotEval();
				if(PrintLevel >= 2) Print("Number of Function Evaluations",TotEval);
				if(PrintLevel == 1) Print("Exiting: Insufficient Change in X - Iter",Iter,"xMax",xMax,"dfMax",dfMax);
				return 2;
			}
			else
			{
				/* Try Resarting */
				if(PrintLevel >= 2) Print("Insufficient Change in X: Recomputing Hessian");
				Restart = 1;
				Copy(x,xPrev);
				Copy(w,wPrev);
			}
		}
		else Restart = 0;
		
		return -1;
	}
	
	/*
	
	Check if a stationary point is a local minimum by checking is the
	projected hessian is positive definite
	
	*/
	
	void CheckMinimum()
	{
		/* Compute Hessian using Finite Differences */
		Print("Computing Hessian Using Finite Differences");
		int Info = Hessian(Func1,x,Z,f,df,ddf,Deriv2Step,Deriv2Type,TypX);
		HessEval++;
		
		/*
		Matrix<double> ddfCopy = ddf.Copy();
		chk = Cholesky(ddfCopy);
		if(chk)
		{
			Print("Did Not Converge to a Minimum: Projected Hessian is Not Positive Definite");
			Print("ddf",ddf);
			Print("Preforming Addiitonal Diagnostics");
			Eigenvalue<double> e(ddf);
			Print("Eigenvalues of ddf",e.GetEigenvalues());
			//Basis bas(ddf);
			//Matrix<double> Bas = bas.GetBasisMatrix();
			//Print("Basis for ddf",Bas);
			
			//for(i=0; i<n-m; i++)
			//{
			//	Matrix<double> A = SubMatrix(ddf,0,i,0,i);
			//	Print("i",i);
			//	Eigenvalue<double> eA(A);
			//	Print("Eigenvlaues of Submatrix",eA.GetEigenvalues());
			//}
			
			Print("Z",Z);
			
			//Print("Hlaintg Executpo (remeber to get rid of this)");
			//exit(0);			
		}
		else Print("Converged to a Minimum: Projected Hessian is Positive Definite");
		*/
		
		Print("Error: This Part is Not Done Yet");
		HaltExecution();
	}
					
	int Solve()
	{
		// Print Problem Information
		if(PrintLevel >= 3) Print("");
		if(PrintLevel >= 2) Print("Calling MinNewtonLin");
		if(PrintLevel >= 3) Print("--------------------");
		if(PrintLevel >= 3) Print("Number of Variables",n);
		if(PrintLevel >= 3) Print("Number of Linear Equality Constraints",m);
		if(PrintLevel >= 2)	Print("Effective Size of Problem",n-m);
		if(PrintLevel >= 3) Print("");
		
		/* Initialize */
		Set(w,Zero);
					
		/* Intialize Constriants */
		if(r < n && FeasCheck)
		{
			/* Check if x is feasible */
			
			/* Form y = b - A * x0 */
			Vector<double> y(m);
			for(int j=0; j<m; j++)
			{
				y[j] = b[j];
				for(int i=0; i<n; i++) y[j] -= A(j,i) * x[i];
			}
						
			if(NormL2(y) > 1.0e-12)
			{
				/* Reset Initial Point */
				if(PrintLevel >= 2)	Print("The Initial Point Was Not Feasible");
				if(PrintLevel >= 2)	Print("Resseting Initial Point");
				
				if(PrintLevel >= 2)	Print("A",A);
				if(PrintLevel >= 2)	Print("b",b);
				if(PrintLevel >= 2)	Print("x",x);
				if(PrintLevel >= 2)	Print("A * x - b",A * x - b);
				
				/*
				
				Choose the initial piont to solve,
				
				min 0.5 * (x0 - x)^T * (x - x0)  s.t.  A * x = b
				
				by solving the system,
				
				(   I  -A^T ) (   x0   ) = (  x )
				(  -A    0  ) ( lambda )   ( -b )
				
				C * x0_lambda = d
				
				*/
				
				/*
				SymmetricMatrix<double> C(n+m);			C.Name    = "MinNewtonLin::C";
				Vector<double>          d(n+m);			d.Name    = "MinNewtonLin::d";
				Vector<double>          xlam(n+m);		xlam.Name = "MinNewtonLin::xlam"; 
				
				// Form C and d
				Set(C,0.0);
				for(i=0; i<n; i++) for(j=0; j<=i; j++) C(i,j) = 2.0 * (i == j);
				for(i=n; i<n+m; i++) for(j=0; j<n; j++) C(i,j) = -A(i-n,j);
				for(i=0; i<n; i++) d[i] = 2.0 * x[i];
				for(i=n; i<n+m; i++) d[i] = -b[i-n];
				
				Chk = LinSolveLU(xlam,C,d);
				if(Chk)
				{
					Print("Error in MinNewtonLin");
					Print("Cannot solve system C * xlam = d");
					Print("C",C);
					Print("d",d);
					HaltExecution();
				}
								
				for(i=0; i<n; i++) x[i] = xlam[i];
				*/
								
				/*
				
				Resest initial point:
				
				Solve min 0.5 * (x - x0)^T * (x - x0) s.t. A * x = b
				
				where,
				
				x0 is the initial point
				x  is a feasible point close to x0
								
				The solution is given by,
				
				x = x0 + A^T * (A * A^T) ^ -1 * (b - A * x0)
				
				*/
				
				/* Form AAT = A * A^T */
				SymmetricMatrix<double> AAT(m,m);
				AAT.Name = "MinNewtonLin::AAT";
				ProdAAt(AAT,A);
							
				/* Solve (A * A^T) * z = y */
				Vector<double> z(m);
				int Info = LinSolveCholesky(z,AAT,y);
				if(Info)
				{
					Print("Error in MinNewtonLin");
					Print("Failed solving (A * A^T) * z = y");
					Print("z",z);
					Print("AAT",AAT);
					Print("y",y);
					HaltExecution();
				}
								
				/* x = A^T * z + x0 */
				SumProdAtX(x,A,z);

				if(PrintLevel >= 2)	Print("New x",x);
				
				/* Make Sure the New Point Works */
				
				if(NormL2(A * x - b) > 1.0e-12)
				{
					/* Reset Initial Point */
					Print("Error: The initial NEW point was not feasible");
					
					Print("A",A);
					Print("b",b);
					Print("x",x);
					Print("A * x - b",A * x - b);
					
					HaltExecution();					
				}
			}
		}
		if(r == 0)
		{
			if(PrintLevel >= 2) Print("Succesfully Exiting MinNewtonLin");
			if(PrintLevel >= 2) Print("Only One Solution to Constraints");
			if(PrintLevel >= 2) Print("No Need to Iterate");
			return 0;
		}
		
		if(HessOpt != 3) Restart = 1;
		else             Restart = 0;
		
		/* Iterate */
		EvaluateFunc();
		for(Iter=1; Iter<=MaxIter; Iter++)
		{
			/* Evaluate New Point */
			if(PrintLevel >= 3) Print("Iter",Iter);
			
			/* Quadratic Programming Subproblem */
			int Info = FindStep();
			if(Info)
			{
				if(PrintLevel >= 2) Print("Unsucessfully Exiting MinNewtonLin");
				if(PrintLevel >= 2) Print("Unable to Calculate Step");
				if(PrintLevel >= 2) Print("Info",Info);
				if(PrintLevel >= 2) Print("Number of Iterations",Iter);
				ComputeTotEval();
				if(PrintLevel >= 2) Print("Number of Function Evaluations",TotEval);
				if(PrintLevel >= 2) Print("");
				if(PrintLevel == 1) Print("Exiting","Unable to Calculate Step","Iter",Iter,"xMax",xMax,"dfMax",dfMax);
				return 12;
			}
		
			if(PrintLevel >= 3) Print("f",f);
			if(PrintLevel >= 3) Print("x",x);
			if(PrintLevel >= 3) Print("dfProj",dfProj);
			if(PrintLevel >= 5) Print("ddfL",ddfL);
			if(PrintLevel >= 5) Print("ddfD",ddfD);
			if(PrintLevel >= 3) Print("p",p);
			
			/* Save Information */
			fPrev = f;
			Copy(xPrev,x);
			Copy(wPrev,w);
			Copy(dfPrev,dfProj);
									
			/* Line search */
			LineSearch();
						
			/* Test for Convergence */
			tst = TestConverge();
			if(tst >= 0) return tst;
											
			if(PrintLevel >= 3) Print("");
		}
																
		if(PrintLevel >= 2) Print("Unsucessfully Exiting MinNewtonLin");
		if(PrintLevel >= 2) Print("Too Many Iterations");
		if(PrintLevel >= 2) Print("Number of Iterations",Iter);
		ComputeTotEval();
		if(PrintLevel >= 2) Print("Number of Function Evaluations",TotEval);
		if(PrintLevel >= 2) Print("");
		if(PrintLevel == 1) Print("Exiting","Too Many Iterations","Iter",Iter,"xMax",xMax,"dfMax",dfMax);
		return 3;
	}
	
	int GetIter()
	{
		return Iter;
	}
	
	int GetTotEval()
	{
		ComputeTotEval();
		return TotEval;
	}
};			
			
#endif

