
#ifndef _GenMatUtil_ProdAtBA
#define _GenMatUtil_ProdAtBA

/* Function Prototypes */

/*

Compute the product, C = A^T * B * A

where,

C is a Matrix
A is a Matrix
B is a Matrix

*/

template <class T>
void ProdAtBA(Matrix<T> &C, const Matrix<T> &A, const Matrix<T> &B)
{
	int m = A.Rows();
	int n = A.Cols();
		
	if(B.Rows() != m || B.Cols() != m || C.Rows() != n || C.Cols() != n)
	{
		Print("Error in ProdAtBA(Matrix,Matrix,Matrix)");
		Print("C = A^T * B * A");
		Print("Incompadible Dimensions");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/*
	
	Compute C = A^T * B * A efficiently using only an
	additional vector of storage (i.e. we do not form
	D = B * A). 
	
	*/
	
	/*
	
	Old Code:
	
	Matrix<T> D(m,n);
	D.Name = "ProdAtBA(Matrix,Matrix,Matrix)::D";
	ProdAB(D,B,A);
	ProdAtB(C,A,D);
	
	*/
	
	/* Temporary Storage */
	Vector<double> d(n);	// Temporary Storage
	
	/* C = 0.0 */
	BlasSet(Sqr(n),Zero,C.Begin(),1);
	
	for(int k=0; k<m; k++)
	{
		/* Fill d = [B * A][k] */
		BlasSet(n,0.0,d.Begin(),1);
		for(int l=0; l<m; l++)
		{
			double Temp = B(k,l);
			BlasAxpy(n,Temp,&A(l,0),1,d.Begin(),1);
		}
	
		/* C += A(k,i) * d */
		for(int i=0; i<n; i++)
		{
			double Temp = A(k,i);
			BlasAxpy(n,Temp,d.Begin(),1,&C(i,0),1);
		}
	}	
}

/*

Compute the product, C = A^T * B * A

where

C is a SymmetricMatrix
A is a Matrix
B is a SymmetricMatrix

*/

template <class T>
void ProdAtBA(SymmetricMatrix<T> &C, const Matrix<T> &A, const SymmetricMatrix<T> &B)
{
	int m = A.Rows();
	int n = A.Cols();
		
	if(B.Dim() != m || C.Dim() != n)
	{
		Print("Error in ProdAtBA(SymmetricMatrix,Matrix,SymmetricMatrix)");
		Print("Incompadible Dimensions");
		Print("C = A^T * B * A");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	/*
	
	Compute C = A^T * B * A efficiently using only an
	additional vector of storage (i.e. we do not form
	D = B * A). 
	
	*/

	/*
	
	Old Code:
	
	Matrix<T> D(m,n);
	D.Name = " ProdAtBA(SymmetricMatrix,Matrix,SymmetricMatrix)::D";
	ProdAB(D,B,A);
	ProdAtB_(C,A,D);
	
	*/
	
	/* Temporary Storage */
	Vector<double> d(n);
	
	/* C = 0.0 */
	BlasSet(n*(n+1)/2,Zero,C.Begin(),1);
		
	for(int k=0; k<m; k++)
	{
		/* Fill d = [B * A][k] */
		BlasSet(n,0.0,d.Begin(),1);
		for(int l=0; l<m; l++)
		{
			double Temp = B(k,l);
			BlasAxpy(n,Temp,&A(l,0),1,d.Begin(),1);
		}
		
		/* C += A(k,i) * d */
		for(int i=0; i<n; i++)
		{
			double Temp = A(k,i);
			BlasAxpy(i+1,Temp,d.Begin(),1,&C(i,0),1);
		}
	}
}

/*

Compute the product, C = A^T * B * A

where,

C is a SymmetricMatrix
A is a Matrix
B is a DiagonalMatrix

*/

template <class T>
void ProdAtBA(SymmetricMatrix<T> &C, const Matrix<T> &A, const DiagonalMatrix<T> &B)
{
	int m = A.Rows();
	int n = A.Cols();
		
	if(B.Dim() != m || C.Dim() != n)
	{
		Print("Error in ProdAtBA(SymmetricMatrix,Matrix,DiagonalMatrix)");
		Print("C = A^T * B * A");
		Print("Incompadible Dimensions");
		Print("C.Name",A.Name);
		Print("A.Name",B.Name);
		Print("B.Name",C.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	for(int i=0; i<n; i++) for(int j=0; j<=i; j++)
	{
		C(i,j) = Zero;
		for(int k=0; k<n; k++) C(i,j) += A(j,k) * B(k) * A(k,i);
	}
}

/*

Compute the product A^T * B * A where B is a BlockMatrix and A is a Matrix
using,

[ A1^T A2^T ... Am^T ]  *  [ B1	          ]   [ A1 ]
                           [    B2        ] * [ A2 ]
                           [       ...    ]   [ .. ]
                           [           Bm ]   [ Am ]
                     
 
 = A1^T * B1 * A1 + A2^T * B2 * A2 + ... + Am^T * Bm * Am

*/

#ifdef _GenMat_BlockMatrix
	template <class T>
	void ProdAtBA(Matrix<T> &C, const Matrix<T> &A, const BlockMatrix<T> &B)
	{
		int b,i,j,k;
		int m = B.Blocks();
		int n = A.Cols();
		int Sm = 0;
		
		if(C.Rows() != n || C.Cols() != n || B.Dim() != A.Rows())
		{
			Print("Error in ProdAtBA(C,A,B)");
			HaltExecution();
		}
		
		for(i=0; i<n; i++) for(j=0; j<n; j++) C(i,j) = 0;
		for(b=0; b<m; b++)
		{
			int r = B.Block(i);
			Matrix<T> D(r,n);
			for(i=0; i<r; i++) for(j=0; j<n; j++)
			{
				// Compute D(b) = B(b) * A(b)
				D(i,j) = Zero;
				for(k=0; k<r; k++) D(i,j) += B(b,i,k) * A(Sm+k,j);
			}
			
			// Accumulate, C += A(b)^T * D(b)
			for(i=0; i<n; i++) for(j=0; j<n; j++) for(k=0; k<r; k++) C(i,j) += A(Sm+k,i) * D(k,j);
			Sm += r;
		}
	}
#endif

/*

Compute the product A^T * B * A where B is a BlockMatrix and A is a Matrix
using,

[ A1^T A2^T ... Am^T ]  *  [ B1	          ]   [ A1 ]
                           [    B2        ] * [ A2 ]
                           [       ...    ]   [ .. ]
                           [           Bm ]   [ Am ]
                     
 
 = A1^T * B1 * A1 + A2^T * B2 * A2 + ... + Am^T * Bm * Am

*/

#ifdef _GenMatUtil_BlockMatrix
	template <class T>
	void ProdAtBA(SymmetricMatrix<T> &C, const Matrix<T> &A, const BlockMatrix<T> &B)
	{
		int b,i,j,k;
		int m = B.Blocks();
		int n = A.Cols();
		int Sm = 0;
		
		if(C.Dim() != n)
		{
			Print("Error in ProdAtBA_(C,A,B)");
			HaltExecution();
		}
		
		for(i=0; i<n; i++) for(j=0; j<=i; j++) C(i,j) = 0;
		for(b=0; b<m; b++)
		{
			int r = B.Block(i);
			Matrix<T> D(r,n);
			for(i=0; i<r; i++) for(j=0; j<n; j++)
			{
				// Compute D(b) = B(b) * A(b)
				D(i,j) = 0.0;
				for(k=0; k<r; k++) D(i,j) += B(b,i,k) * A(Sm+k,j);
			}
			
			// Accumulate, C += A(b)^T * D(b)
			for(i=0; i<n; i++) for(j=0; j<=i; j++) for(k=0; k<r; k++) C(i,j) += A(Sm+k,i) * D(k,j);
			Sm += r;
		}
	}
#endif

#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdAtBA(SymmetricMatrix<T> &C, const SparseMatrix<T> &A, const SymmetricMatrix<T> &B)
	{
		int m = A.Rows();
		int n = A.Cols();
			
		if(B.Dim() != m || C.Dim() != n)
		{
			Print("Error in ProdAtBA(SymmetricMatrix,SparseMatrix,SymmetricMatrix)");
			Print("Incompadible Dimensions");
			Print("C = A^T * B * A");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/*
		
		Compute C = A^T * B * A efficiently using only an
		additional vector of storage (i.e. we do not form
		D = B * A). 
		
		This routine is very poor for out-of-core usage
		since B is not accesces sequentially. This can be fixed
		by storing B as a Matrix (rather than a Symmetric Matrix)
		
		*/

		/* 
		
		Old Code:
		
		Matrix<T> D(m,n);
		ProdAB(D,B,A);
		ProdAtB_(C,A,D);
		
		*/
		
		/* Temporary Storage */
		Vector<double> d(n);
		d.Name = "ProdAtBA::d";
			
		/* C = 0.0 */
		Set(C,Zero);
		
		Timer tm1(0),tm2(0);
		
		for(int k=0; k<m; k++)
		{
			/* Fill d = [B * A][k] */

			tm1.Start();

			Set(d,Zero);
			
			const int *c    = &A.c_[0];		// Pointer to sparse matrix columns
			const double *v = &A.v_[0];		// Pointer to sparse matrix vales
		
			/*
			
			This part requires constant reaccesing of the disk.
			If particular, the line,
			
			double  Temp = B(k,l)
			
			is causing problems
					
			*/
			
			//const double *p = &B(k,0);
			//double *q = &b[0];
			
			//for(int l=0; l<=k; l++) *q++ = *p++;
			//for(int l=k+1; l<m; l++)
			//{
			//	*q++ = *p;
			//	p += l;
			//}
			
			//q = &b[0];
			//const double *q = &B(k,0);
				
			for(int l=0; l<m; l++)
			{
				int Low  = A.r_[l];
				int High = A.r_[l+1];
				double Temp = B(k,l);
				for(int j=Low; j<High; j++) d[*c++] += Temp * (*v++);
			}
			
			tm1.Pause();
						
			/* Compute C += A^T * d */
			
			tm2.Start();
			
			for(int i=A.r_[k]; i<A.r_[k+1]; i++)
			{
				int Idx = A.c_[i];
				double Temp = A.v_[i];
				BlasAxpy(Idx+1,Temp,&d[0],1,&C(Idx,0),1);
			}
			
			tm2.Pause();
		}
		
		Print("Timer for Part 1",tm1.CpuTimeUsed);
		Print("Timer for Part 2",tm2.CpuTimeUsed);
	}
#endif

#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdAtBA(Matrix<T> &C, const SparseMatrix<T> &A, const Matrix<T> &B)
	{
		int m = A.Rows();
		int n = A.Cols();
			
		if(B.Rows() != m || B.Cols() != m || C.Rows() != n || C.Cols() != n)
		{
			Print("Error in ProdAtBA(Matrix,SparseMatrix,Matrix)");
			Print("Incompadible Dimensions");
			Print("C = A^T * B * A");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/* Temporary Storage */
		Vector<double> d(n);
		d.Name = "ProdAtBA::d";
		
		/* C = 0.0 */
		Set(C,Zero);
		
		for(int k=0; k<m; k++)
		{
			/* Fill d = [B * A][k] */

			Set(d,Zero);
			
			const int *c    = &A.c_[0];		// Pointer to sparse matrix columns
			const double *v = &A.v_[0];		// Pointer to sparse matrix vales
		
			/*
			
			This part requires constant reaccesing of the disk.
			If particular, the line,
			
			double  Temp = B(k,l)
			
			is causing problems
					
			*/
			
			const double *q = &B(k,0);
				
			for(int l=0; l<m; l++)
			{
				int Low  = A.r_[l];
				int High = A.r_[l+1];
				double Temp = *q++;
				for(int j=Low; j<High; j++) d[*c++] += Temp * (*v++);
			}
						
			/* Compute C += A^T * d */
			
			for(int i=A.r_[k]; i<A.r_[k+1]; i++)
			{
				int Idx = A.c_[i];
				double Temp = A.v_[i];
				BlasAxpy(Idx+1,Temp,&d[0],1,&C(Idx,0),1);
			}
		}
	}
#endif

template <class T>
void ProdAtBA_(SymmetricMatrix<T> &C, const Matrix<T> &A, const Matrix<T> &B)
{
	int m = A.Rows();
	int n = A.Cols();
	
	if(B.Rows() != m || B.Cols() != m || C.Dim() != n)
	{
		Print("Error in ProdAtBA_(SymmetricMatrix,Matrix,Matrix)");
		Print("Incompadible Dimensions");
		Print("C = A^T * B * A");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}
	
	/*
	
	Compute C = A^T * B * A efficiently using only an
	additional vector of storage (i.e. we do not form
	D = B * A).
	
	*/
	
	/* Temporary Storage */
	
	/*
	Vector<double> d(n);
	d.Name = "ProdAtBA::d";
		
	/* C = 0.0 */
	/*
	Set(C,Zero);
	
	for(int k=0; k<m; k++)
	{
		/* Fill d = [B * A][k] */
	/*
		Set(d,Zero);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) d(i) += B(i,j) * A(j,k);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) C(i,j) += A(j,i) * d(j);
	}
	*/
	
	/* C = A^T * B * A */
	
	Set(C,Zero);
	
	Matrix<double> D(m,n,Zero);
	for(int i=0; i<m; i++) for(int j=0; j<m; j++) for(int k=0; k<n; k++)  D(i,k) += B(i,j) * A(j,k);
	for(int i=0; i<n; i++) for(int j=0; j<m; j++) for(int k=0; k<=i; k++) C(i,k) += A(j,i) * D(j,k);
}

/*
template <class T>
void ProdAtBA_(Matrix<T> &C, const Matrix<T> &A, const Matrix<T> &B)
{
	int m = A.Rows();
	int n = A.Cols();
	
	if(B.Rows() != m || B.Cols() != m || C.Rows() != n || C.Cols() != n)
	{
		Print("Error in ProdAtBA_(Matrix,Matrix,Matrix)");
		Print("Incompadible Dimensions");
		Print("C = A^T * B * A");
		Print("C.Name",C.Name);
		Print("A.Name",A.Name);
		Print("B.Name",B.Name);
		Print("C.Size()",C.Size());
		Print("A.Size()",A.Size());
		Print("B.Size()",B.Size());
		HaltExecution();
	}

	Print("Warning: I Haven't Verified the this routine works properly!!!");
	PauseExecution();

	/*
	
	Compute C = A^T * B * A efficiently using only an
	additional vector of storage (i.e. we do not form
	D = B * A).
	
	*/
	
	/* Temporary Storage */
	
	/*
	Vector<double> d(n);
	d.Name = "ProdAtBA::d";
		
	/* C = 0.0 */
	/*
	Set(C,Zero);
	
	for(int k=0; k<m; k++)
	{
		/* Fill d = [B * A][k] */
	/*
		Set(d,Zero);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) d(i) += B(i,j) * A(j,k);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++) C(i,j) += A(j,i) * d(j);
	}
	*/
	
	/* C = A^T * B * A */
/*	
	Set(C,Zero);	
	Matrix<double> D(m,n,Zero);
	for(int i=0; i<m; i++) for(int j=0; j<m; j++) for(int k=0; k<n; k++)  D(i,k) += B(i,j) * A(j,k);
	for(int i=0; i<n; i++) for(int j=0; j<m; j++) for(int k=0; k<=i; k++) C(i,k) += A(j,i) * D(j,k);
	for(int i=0; i<n; i++) for(int k=0; k<=i; k++) C(k,i) = C(i,k);
}
*/

#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdAtBA_(SymmetricMatrix<T> &C, const SparseMatrix<T> &A, const Matrix<T> &B)
	{
		int m = A.Rows();
		int n = A.Cols();
			
		if(B.Rows() != m || B.Cols() != m || C.Dim() != n)
		{
			Print("Error in ProdAtBA_(SymmetricMatrix,SparseMatrix,Matrix)");
			Print("Incompadible Dimensions");
			Print("C = A^T * B * A");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/*
		
		Compute C = A^T * B * A efficiently using only an
		additional vector of storage (i.e. we do not form
		D = B * A). 
		
		This routine is preferable to,
		
			ProdAtBA_(SymmetricMatrix,SparseMatrix,Matrix)
		
		for out-of-core usage since B is accesed sequentialy
		
		*/
		
		/* Temporary Storage */
		Vector<double> d(n);
		d.Name = "ProdAtBA::d";
		
		/* C = 0.0 */
		Set(C,Zero);
		
		Timer tm1(0),tm2(0);
		
		for(int k=0; k<m; k++)
		{
			/* Fill d = [B * A][k] */

			tm1.Start();

			Set(d,Zero);
			
			const int    *c = &A.c_[0];		// Pointer to sparse matrix columns
			const double *v = &A.v_[0];		// Pointer to sparse matrix vales
		
			/*
			
			This part requires constant reaccesing of the disk.
			If particular, the line,
			
			double  Temp = B(k,l)
			
			is causing problems
					
			*/
			
			const double *q = &B(k,0);
				
			for(int l=0; l<m; l++)
			{
				int Low  = A.r_[l];
				int High = A.r_[l+1];
				double Temp = *q++;
				for(int j=Low; j<High; j++) d[*c++] += Temp * (*v++);
			}
			
			tm1.Pause();
						
			/* Compute C += A^T * d */
			
			tm2.Start();
			
			for(int i=A.r_[k]; i<A.r_[k+1]; i++)
			{
				int Idx = A.c_[i];
				double Temp = A.v_[i];
				BlasAxpy(Idx+1,Temp,&d[0],1,&C(Idx,0),1);
			}
			
			tm2.Pause();
		}
		
		Print("Timer for Part 1",tm1.CpuTimeUsed);
		Print("Timer for Part 2",tm2.CpuTimeUsed);
	}
#endif

#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdAtBA_(Matrix<T> &C, const SparseMatrix<T> &A, const Matrix<T> &B)
	{
		int m = A.Rows();
		int n = A.Cols();
			
		if(B.Rows() != m || B.Cols() != m || C.Rows() != n || C.Cols() != n)
		{
			Print("Error in ProdAtBA_(Matrix,SparseMatrix,Matrix)");
			Print("Incompadible Dimensions");
			Print("C = A^T * B * A");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/*
		
		Compute C = A^T * B * A efficiently using only an
		additional vector of storage (i.e. we do not form
		D = B * A). 
		
		This routine is preferable to,
		
			ProdAtBA_(SymmetricMatrix,SparseMatrix,Matrix)
		
		for out-of-core usage since B is accesed sequentialy
		
		*/
		
		/* Temporary Storage */
		Vector<double> d(n);
		d.Name = "ProdAtBA::d";
		
		/* C = 0.0 */
		Set(C,Zero);
		
		Timer tm1(0),tm2(0);
		
		for(int k=0; k<m; k++)
		{
			/* Fill d = [B * A][k] */

			tm1.Start();

			Set(d,Zero);
			
			const int *c    = &A.c_[0];		// Pointer to sparse matrix columns
			const double *v = &A.v_[0];		// Pointer to sparse matrix vales
		
			/*
			
			This part requires constant reaccesing of the disk.
			If particular, the line,
			
			double  Temp = B(k,l)
			
			is causing problems
					
			*/
			
			const double *q = &B(k,0);
				
			for(int l=0; l<m; l++)
			{
				int Low  = A.r_[l];
				int High = A.r_[l+1];
				double Temp = *q++;
				for(int j=Low; j<High; j++) d[*c++] += Temp * (*v++);
			}
			
			tm1.Pause();
						
			/* Compute C += A^T * d */
			
			tm2.Start();
			
			for(int i=A.r_[k]; i<A.r_[k+1]; i++)
			{
				int Idx = A.c_[i];
				double Temp = A.v_[i];
				BlasAxpy(Idx+1,Temp,&d[0],1,&C(Idx,0),1);
			}
			
			tm2.Pause();
		}
		
		/* Fill In the Upper Triangle */
		for(int i=0; i<n; i++) for(int j=0; j<i; j++) C(j,i) = C(i,j);
		
		Print("Timer for Part 1",tm1.CpuTimeUsed);
		Print("Timer for Part 2",tm2.CpuTimeUsed);
	}
#endif

#if defined(BlasMatrix) && defined(SparseMatrix)
	template <class T>
	void ProdAtBA_(BlasMatrix<T> &C, const SparseMatrix<T> &A, const Matrix<T> &B)
	{
		int m = A.Rows();
		int n = A.Cols();
			
		if(B.Rows() != m || B.Cols() != m || C.Rows() != n || C.Cols() != n)
		{
			Print("Error in ProdAtBA_(Matrix,SparseMatrix,Matrix)");
			Print("Incompadible Dimensions");
			Print("C = A^T * B * A");
			Print("C.Name",C.Name);
			Print("A.Name",A.Name);
			Print("B.Name",B.Name);
			Print("C.Size()",C.Size());
			Print("A.Size()",A.Size());
			Print("B.Size()",B.Size());
			HaltExecution();
		}

		/*
		
		Compute C = A^T * B * A efficiently using only an
		additional vector of storage (i.e. we do not form
		D = B * A). 
		
		This routine is preferable to,
		
			ProdAtBA_(SymmetricMatrix,SparseMatrix,Matrix)
		
		for out-of-core usage since B is accesed sequentialy
		
		*/
		
		/* Temporary Storage */
		Vector<double> d(n);
		d.Name = "ProdAtBA::d";
		
		/* C = 0.0 */
		Set(C,Zero);
		
		Timer tm1(0),tm2(0);
		
		for(int k=0; k<m; k++)
		{
			/* Fill d = [B * A][k] */

			tm1.Start();

			Set(d,Zero);
			
			const int *c    = &A.c[0];		// Pointer to sparse matrix columns
			const double *v = &A.v[0];		// Pointer to sparse matrix vales
		
			/*
			
			This part requires constant reaccesing of the disk.
			If particular, the line,
			
			double  Temp = B(k,l)
			
			is causing problems
					
			*/
			
			const double *q = &B(k,0);
				
			for(int l=0; l<m; l++)
			{
				int Low  = A.r[l];
				int High = A.r[l+1];
				double Temp = *q++;
				for(int j=Low; j<High; j++) d[*c++] += Temp * (*v++);
			}
			
			tm1.Pause();
						
			/* Compute C += A^T * d */
			
			tm2.Start();
			
			for(int i=A.r[k]; i<A.r[k+1]; i++)
			{
				int Idx = A.c[i];
				double Temp = A.v[i];
				BlasAxpy(Idx+1,Temp,&d[0],1,&C(Idx,0),1);
			}
			
			tm2.Pause();
		}
		
		/* Fill In the Upper Triangle */
		for(int i=0; i<n; i++) for(int j=0; j<i; j++) C(j,i) = C(i,j);
		
		Print("Timer for Part 1",tm1.CpuTimeUsed);
		Print("Timer for Part 2",tm2.CpuTimeUsed);
	}
#endif

#endif
