
#ifndef _GenMatUtil_ProdAtX
#define _GenMatUtil_ProdAtX

	
/* Compute the product, y = A^T * x */
template <class T>
void ProdAtX(Vector<T> &y, const Matrix<T> &A, const Vector<T> &x)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != m || y.Size() != n) 
	{
		Print("Error in Vector = ProdAtX(Matrix,Vector)");
		Print("y = A^T * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	for(int i=0; i<n; i++)
	{
		y[i] = 0;
		for(int j=0; j<m; j++) y[i] += A(j,i) * x[j];
	}
}

/*

Compute the product, y = A^T * x

*/

#ifdef _GenMat_SparseMatrixRcv
	template <class T>
	void ProdXA(Vector<T> &y, const SparseMatrixRcv<T> &A, const Vector<T> &x)
	{
		int m   = A.Rows();
		int n   = A.Cols();
		int len = A.Length();
		
		if(y.Size() != m || x.Size() != n)
		{
			Print("Error in ProdXA(Vector,SparseMatrixRcv,Vector)");
			Print("y = A^T * x");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}	
		
		Set(y,0.0);
		for(int i=0; i<len; i++) y[A.Col(i)] += A.Val(i) * x[A.Row(i)];
	}
#endif

/* Pre-multiply a Vector by a SparseMatrix (easy routine) */
#ifdef _GenMat_SparseMatrix
	template <class T>
	void ProdAtX(Vector<T> &y, const SparseMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		if(x.Size() != m || y.Size() != n) 
		{
			Print("Error in ProdAtX(Vector,SparseMatrix,Vector)");
			Print("y = A^T * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		Set(y,Zero);
		for(int i=0; i<m; i++) for(int j=A.RowLoc(i); j<A.RowLoc(i+1); j++)
			y[A.Col(j)] += A.Val(j) * x[i];
	}
#endif

/* Pre-multiply a Vector by a SparseMatrix (easy routine) */
#ifdef _GenMat_SparseUpdatedMatrix
	template <class T>
	void ProdAtX(Vector<T> &y, const SparseUpdatedMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		int k = A.Updates();
		if(x.Size() != m || y.Size() != n) 
		{
			Print("Error in ProdAtX(Vector,SparseMatrix,Vector)");
			Print("y = A^T * x");
			Print("Incompatible Dimensions");
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		ProdAtX(y,A.S,x);
		Vector<T> UtX(k,Zero);
		//for(int i=0; i<m; i++) for(int j=0; j<k; j++) UtX(j) += A.U(i,j) * x(i);
		BlasGemv(True,m,k,One,A.U.Begin(),k,x.Begin(),1,Zero,UtX.Begin(),1);
		for(int j=0; j<k; j++)
		{
			if(A.Pos(j))
			{
				//for(int i=0; i<n; i++) y(i) += A.V(i,j) * UtX(j);
				BlasAxpy(n,UtX(j),&A.V(0,j),k,y.Begin(),1);
			}
			else
			{
				//for(int i=0; i<n; i++) y(i) -= A.V(i,j) * UtX(j);
				BlasAxpy(n,-UtX(j),&A.V(0,j),k,y.Begin(),1);
			}
		}	
	}
#endif

/* Compute the product, y = A^T * x */
template <class T>
void ProdAtX(Vector<T> &y, const LowerMatrix<T> &A, const Vector<T> &x)
{
	/* Check for Incompadible Dimensions */
	int n = A.Dim();
	if(x.Size() != n || y.Size() != n) 
	{
		Print("Error in ProdAtX(Vector,LowerMatrix,Vector)");
		Print("y = A^T * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	for(i=0; i<n; i++)
	{
		y[i] = 0;
		for(j=0; j<=i; j++) y[i] += A(j,i) * x[j];
	}
}

#ifdef _GenMat_BandMatrix
	template <class T>
	void ProdAtX(Vector<T> &y, const BandMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		int kl = A.Lower();
		int ku = A.Upper();
		if(x.Size() != m || y.Size() != n)
		{
			Print("Error in Vector = BandMatrix * Vector");
			Print("ProdAtX(y,A,x)");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		Set(y,Zero);
		BlasGbmv(true,m,n,kl,ku,One,A.Begin(),kl+ku+1,x.Begin(),1,Zero,y.Begin(),1);
	}
#endif

#ifdef _GenMat_LowerBandMatrix
	template <class T>
	void ProdAtX(Vector<T> &y, const LowerBandMatrix<T> &A, const Vector<T> &x)
	{
		/* Check for Incompadible Dimensions */
		int m = A.Rows();
		int n = A.Cols();
		int kl = A.Lower();
		int ku = 0;
		if(x.Size() != m || y.Size() != n)
		{
			Print("Error in Vector = LowerBandMatrix * Vector");
			Print("ProdAtX(y,A,x)");
			Print("Incompatible Dimensions");
			Print("y.Name",y.Name);
			Print("A.Name",A.Name);
			Print("x.Name",x.Name);
			Print("y.Size()",y.Size());
			Print("A.Size()",A.Size());
			Print("x.Size()",x.Size());
			HaltExecution();
		}
		
		/* Preform Calculations */
		Set(y,Zero);
		BlasGbmv(True,m,n,kl,ku,One,A.Begin(),kl+ku+1,x.Begin(),1,Zero,y.Begin(),1);
	}
#endif

/* Compute the product, y = A^T * x */
template <class T>
void ProdAtX(Vector<T> &y, const GeneralMatrix<T> &A, const Vector<T> &x)
{
	/* Check for Incompadible Dimensions */
	int m = A.Rows();
	int n = A.Cols();
	if(x.Size() != m || y.Size() != n) 
	{
		Print("Error in Vector = ProdAtX(GeneralMatrix,Vector)");
		Print("y = A^T * x");
		Print("Incompatible Dimensions");
		Print("y.Name",y.Name);
		Print("A.Name",A.Name);
		Print("x.Name",x.Name);
		Print("y.Size()",y.Size());
		Print("A.Size()",A.Size());
		Print("x.Size()",x.Size());
		HaltExecution();
	}
	
	/* Preform Calculations */
	for(int i=0; i<n; i++)
	{
		y[i] = 0;
		for(int j=0; j<m; j++) y[i] += A(j,i) * x[j];
	}
}


#endif
