
#ifndef _Opt_Projection
#define _Opt_Projection



Matrix<double> Projection(const Matrix<double> &A)
{
	/*
	
	Given a matrix A, find a matrix Z such that
	Z * w = 0 and Z * w spans that null set of A.
	
	Z will have dimension n x r where r will be the
	dimension of the null space of Z, which may be zero.
	
	Note: Perhaps this routine should be rewritten to
	return Z as a sparse matrix.
	
	*/
	
	/*
	
	The first step is to use the LU decomposition
	to find linear dependencies in the columns of A
	
	*/
	
	int n = A.Cols();
	LUDecompObj<double> lu(A);

	Matrix<double> U = lu.GetU();	U.Name = "Projection::U";
	int m = U.Rows();
								
	int NumBasic = 0;
	Vector<int> Basic(n,0);		Basic.Name = "Projection::Basic";
	Vector<int> BasicRow(m,0);	BasicRow.Name = "Projection::BasicRow";
	
	for(int i=0; i<m; i++)
	{
		int j = 0;
		while((U(i,j) == Zero || Basic[j] == 1) && j < n - 1) j++;
		if(U(i,j) != Zero)
		{
			Basic[j] = 1;
			BasicRow[i] = 1;
			NumBasic++;
		}
	}
	
	/* Check if the dimension of the null space is zero */
	
	if(NumBasic == n)
	{
		Matrix<double> Z(n,0);
		Z.Name = "Projection::Z";
		return Z;
	}
	
	int Curr = 0;
	Vector<int> BasicRowIdx(NumBasic);		BasicRowIdx.Name = "Projection::BasicRowIdx";
	for(int i=0; i<m; i++)
	{
		if(BasicRow[i])
		{
			BasicRowIdx[Curr] = i;
			Curr++;
		}
	}
	
	/* Partician A */
	
	Matrix<double> A1(NumBasic,NumBasic);		A1.Name = "Projection::A1";
	Matrix<double> A2(NumBasic,n-NumBasic);		A2.Name = "Projection::A2";
	int CurrBasic = 0;
	int CurrNon = 0;
	
	for(int j=0; j<n; j++)
	{
		if(Basic[j])
		{
			for(int i=0; i<NumBasic; i++) A1(i,CurrBasic) = A(BasicRowIdx[i],j);
			CurrBasic++;
		}
		else
		{
			for(int i=0; i<NumBasic; i++) A2(i,CurrNon) = A(BasicRowIdx[i],j);
			CurrNon++;
		}
	}
	
	int Info = Invert(A1);
	if(Info)
	{
		Print("Error in Projection");
		Print("A1 is not invertable");
		Print("A",A);
		Print("A1",A1);
		Print("A2",A2);
		HaltExecution();
	}
	
	Matrix<double> A1A2(A1.Rows(),A2.Cols());
	A1A2.Name = "Projection::A1A2";
	ProdAB(A1A2,A1,A2);
	
	/* Create the Projection Matrix, Z */
	
	int r = n - NumBasic;
	Matrix<double> Z(n,r);
	
	Z.Name = "Projection::Z";
	CurrBasic = 0;
	CurrNon = 0;
	for(int i=0; i<n; i++)
	{
		if(Basic[i])
		{
			for(int j=0; j<r; j++) Z(i,j) = -A1A2(CurrBasic,j);
			CurrBasic++;
		}
		else
		{
			for(int j=0; j<r; j++) Z(i,j) = One * (CurrNon == j);
			CurrNon++;
		}
	}
	
	return Z;
}

#ifdef _GenMat_SparseMatrix
	SparseMatrix<double> ProjectionSparse(const Matrix<double> &A)
	{
		/*
		
		Given a matrix A, find a matrix Z such that
		Z * w = 0 and Z * w spans that null set of A.
		
		Z will have dimension n x r where r will be the
		dimension of the null space of Z, which may be zero.
		
		*/
		
		/*
		
		The first step is to use the LU decomposition
		to find linear dependencies in the columns of A
		
		*/
		
		int n = A.Cols();
		LUDecompObj<double> lu(A);
		Matrix<double> U = lu.GetU();	U.Name = "ProjectionSparse::U";
		int m = U.Rows();
			
		int NumBasic = 0;
		Vector<int> Basic(n,0);					Basic.Name    = "ProjectionSparse::Basic";
		Vector<int> BasicRow(m,0);				BasicRow.Name = "ProjectionSparse::BasicRow";
		
		for(int i=0; i<m; i++)
		{
			int j = 0;
			while((U(i,j) == Zero || Basic[j] == 1) && j < n - 1) j++;
			if(U(i,j) != Zero)
			{
				Basic[j] = 1;
				BasicRow[i] = 1;
				NumBasic++;
			}
		}
		
		/* Check if the dimension of the null space is zero */
		
		if(NumBasic == n)
		{
			SparseMatrix<double> Z(n,0,0);
			Z.Name = "Projection::Z";
			return Z;
		}
			
		int Curr = 0;
		Vector<int> BasicRowIdx(NumBasic);		BasicRowIdx.Name = "ProjectionSparse::BasicRowIdx";
		for(int i=0; i<m; i++)
		{
			if(BasicRow[i])
			{
				BasicRowIdx[Curr] = i;
				Curr++;
			}
		}
		
		/* Partician A */
		
		Matrix<double> A1(NumBasic,NumBasic);		A1.Name = "Projection::A1";
		Matrix<double> A2(NumBasic,n-NumBasic);		A2.Name = "Projection::A2";
		int CurrBasic = 0;
		int CurrNon = 0;

		for(int j=0; j<n; j++)
		{
			if(Basic[j])
			{
				for(int i=0; i<NumBasic; i++) A1(i,CurrBasic) = A(BasicRowIdx[i],j);
				CurrBasic++;
			}
			else
			{
				for(int i=0; i<NumBasic; i++) A2(i,CurrNon) = A(BasicRowIdx[i],j);
				CurrNon++;
			}
		}
		
		Print("A1 Bef",A1);
			
		int Info = Invert(A1);
		if(Info)
		{
			Print("Error in ProjectionSparse");
			Print("A1 is Not Invertable");
			Print("A",A);
			Print("A1",A1);
			Print("A2",A2);
			Print("Eigenvalues(A1)",Eigenvalues(A1));
			HaltExecution();
		}
		Matrix<double> A1A2 = A1 * A2;
		A1A2.Name = "ProjectionSparse::A1A2";

		/* Create the projection Matrix, Z */
		
		int r = n - NumBasic;
		SparseMatrix<double> Z(n,r,(n-r+1)*r);
		for(int i=0; i<Z.Length(); i++) Z.c[i] = -1;
		Z.Name = "ProjectionSparse::Z";
		
		CurrBasic = 0;
		CurrNon = 0;
		
		int Curr1 = 0;
		
		Z.r[0] = 0;
		for(int i=0; i<n; i++)
		{
			if(Basic[i])
			{
				for(int j=0; j<r; j++)
				{
					Z.c[Curr1] = j;
					Z.v[Curr1] = -A1A2(CurrBasic,j);
					Curr1++;	
				}
				CurrBasic++;
			}
			else
			{
				Z.c[Curr1] = CurrNon;
				Z.v[Curr1] = One;
				Curr1++;			
				CurrNon++;
			}
			Z.r[i+1] = Curr1;
		}
		
		return Z;
	}
#endif

#ifdef _GenMat_SparseMatrix
	SparseMatrix<double> ProjectionSparse(const SparseMatrix<double> &A)
	{
		/*
		
		Given a matrix A, find a matrix Z such that
		Z * w = 0 and Z * w spans that null set of A.
		
		Z will have dimension n x r where r will be the
		dimension of the null space of Z, which may be zero.
		
		*/
		
		/*
		
		The first step is to use the LU decomposition
		to find linear dependencies in the columns of A
		
		*/
		
		int n = A.Cols();
		SparseMatrixCC<double> L;
		SparseMatrixCC<double> U;
		Vector<int> P;
		Vector<int> Q;
		
		Print("Y.1");
		
		int Info = LUDecomp(ToSparseMatrixCC<double>(A),L,U,P,Q);
		
		Print("Y.2");
		
		int m = U.Rows();
			
		int NumBasic = 0;
		Vector<int> Basic(n,0);					Basic.Name    = "ProjectionSparse::Basic";
		Vector<int> BasicRow(m,0);				BasicRow.Name = "ProjectionSparse::BasicRow";
		
		for(int i=0; i<m; i++)
		{
			int j = 0;
			while((U(i,j) == Zero || Basic[j] == 1) && j < n - 1) j++;
			if(U(i,j) != Zero)
			{
				Basic[j] = 1;
				BasicRow[i] = 1;
				NumBasic++;
			}
		}
		
		/* Check if the dimension of the null space is zero */
		
		if(NumBasic == n)
		{
			SparseMatrix<double> Z(n,0,0);
			Z.Name = "Projection::Z";
			return Z;
		}
			
		int Curr = 0;
		Vector<int> BasicRowIdx(NumBasic);		BasicRowIdx.Name = "ProjectionSparse::BasicRowIdx";
		for(int i=0; i<m; i++)
		{
			if(BasicRow[i])
			{
				BasicRowIdx[Curr] = i;
				Curr++;
			}
		}
		
		/* Partician A */
		
		Matrix<double> A1(NumBasic,NumBasic);		A1.Name = "Projection::A1";
		Matrix<double> A2(NumBasic,n-NumBasic);		A2.Name = "Projection::A2";
		int CurrBasic = 0;
		int CurrNon = 0;

		for(int j=0; j<n; j++)
		{
			if(Basic[j])
			{
				for(int i=0; i<NumBasic; i++) A1(i,CurrBasic) = A(BasicRowIdx[i],j);
				CurrBasic++;
			}
			else
			{
				for(int i=0; i<NumBasic; i++) A2(i,CurrNon) = A(BasicRowIdx[i],j);
				CurrNon++;
			}
		}
		
		Print("A1 Bef",A1);
			
		Info = Invert(A1);
		if(Info)
		{
			Print("Error in ProjectionSparse");
			Print("A1 is Not Invertable");
			Print("A",A);
			Print("A1",A1);
			Print("A2",A2);
			Print("Eigenvalues(A1)",Eigenvalues(A1));
			HaltExecution();
		}
		Matrix<double> A1A2 = A1 * A2;
		A1A2.Name = "ProjectionSparse::A1A2";

		/* Create the projection Matrix, Z */
		
		int r = n - NumBasic;
		SparseMatrix<double> Z(n,r,(n-r+1)*r);
		for(int i=0; i<Z.Length(); i++) Z.c[i] = -1;
		Z.Name = "ProjectionSparse::Z";
		
		CurrBasic = 0;
		CurrNon = 0;
		
		int Curr1 = 0;
		
		Z.r[0] = 0;
		for(int i=0; i<n; i++)
		{
			if(Basic[i])
			{
				for(int j=0; j<r; j++)
				{
					Z.c[Curr1] = j;
					Z.v[Curr1] = -A1A2(CurrBasic,j);
					Curr1++;	
				}
				CurrBasic++;
			}
			else
			{
				Z.c[Curr1] = CurrNon;
				Z.v[Curr1] = One;
				Curr1++;			
				CurrNon++;
			}
			Z.r[i+1] = Curr1;
		}
		
		return Z;
	}
#endif

#if defined(SimpleConstraintMatrix) && defined(SimpleConstraintProjectionMatrix)
	SimpleConstraintProjectionMatrix<double> Projection(const SimpleConstraintMatrix<double> &A)
	{
		int m = A.Rows();
		int n = A.Cols();
		SimpleConstraintProjectionMatrix<double> Z(m,n,A.Cons);
		return Z;
	}
#endif

#endif
