
#ifndef _GenMat_RowVector
#define _GenMat_RowVector

/*
Taken from Template Numerical Toolkit (TNT)

Templated one-dimensional, numerical array which
looks like a conventional C array. 
Elements are accessed via the familiar A[i] notation. 

<p>
Array assignment is by reference (i.e. shallow assignment).
That is, B=A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.Copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.
	
<p>
The indexing and layout of this array object makes
it compatible with C and C++ algorithms that utilize
the familiar C[i] notation.  This includes numerous
textbooks, such as Numercial Recipes, and various
public domain codes.
	
<p>
This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.

*/
	
template <class T>
class RowVector : public GeneralVector<T>
{
	//private:
  	public:
  
    T* v;                  
    LongInt n;
    int *RefCount;
	
	void Initialize(LongInt n);
    //void Copy_(T* p, const T*  q, int len) const;
    //void Set_(const T& val);
    void Destroy();
	//inline const T* Begin_() const;
	//inline T* Begin_();

	public:

	         RowVector();
	explicit RowVector(LongInt n);
	         RowVector(LongInt n,  T *a);
	         RowVector(LongInt n, const T &a);
	inline   RowVector(const RowVector &A);
	inline   RowVector & operator=(const T &a);
	inline   RowVector & operator=(const RowVector &A);
	inline   RowVector & Ref(const RowVector &A);
	         RowVector Copy() const;
		     RowVector & Inject(const RowVector & A);
	inline   T& operator()(LongInt i);
	inline   const T& operator()(LongInt i) const;
	inline   T& operator[](LongInt i);
	inline   const T& operator[](LongInt i) const;
	inline 	 int Dim() const;
	inline   LongInt Size() const;
	inline   int Type() const;
	inline   LongInt Length() const;
	inline   int Increment() const;
	inline   int GetRefCount() const;
             ~RowVector();
    
    inline   void operator += (const T &c);
    inline   void operator -= (const T &c);         
    inline   void operator *= (const T &c);
    inline   void operator /= (const T &c);
    
    inline   void operator += (const RowVector<T> &x);
    inline   void operator -= (const RowVector<T> &x);
    inline   void operator *= (const RowVector<T> &x);
    inline   void operator /= (const RowVector<T> &x);
    
    		 T* Begin() const;
			 T* End()   const;
			 T* Loc(LongInt i) const;
			 void Resize(LongInt n);
			 void Resize(LongInt n, const T &a);
};

/*

Null constructor. Creates a 0-length (NULL) array
(Reference count is also zero)

*/

template <class T>
RowVector<T>::RowVector() : v(0), n(0), RefCount(0), Name("NoName")
{
	RefCount = new int;
	*RefCount = 1;
}

/*

Copy constructor. Array data is NOT copied, but shared.
Thus, in Vector B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
Vector B(A.Copy()), or B = A.Copy(), instead.

*/

template <class T>
RowVector<T>::RowVector(const RowVector<T> &A)
{
	v = A.v;
	n = A.n;
	RefCount = A.RefCount;
	Name = A.Name;
	(*RefCount)++;
}

/*

Create a new array (Vector) of length n,
WITHOUT initializing array elements.
To create an initialized array of constants, see Vector(n,Val).

<p>
This version avoids the O(n) initialization overhead and
is used just before manual assignment.

@param n the dimension (length) of the new matrix.

*/

template <class T>
RowVector<T>::RowVector(LongInt n)
{
	this->v = 0;
	this->n = n;
	this->RefCount = 0;
	Name = "NoName";
	Initialize(n);
	RefCount = new int;
	*RefCount = 1;
}

/*

Create a new array of length n, initializing array
elements to constant specified by argument.  Most
often used to create an array of zeros, as in A(n,0.0).

n	-	Dimension (length) of the new Vector.  
Val	-	The constant value to set all elements of the new array to.

*/

template <class T>
RowVector<T>::RowVector(LongInt n, const T &Val)
{
	this->v = 0;
	this->n = n;
	this->RefCount = 0;
	this->Name = "NoName";
	Initialize(n);
	BlasSet(n,Val,v,1);
	RefCount = new int;
	*RefCount = 1;
}

/*

Create a new n-length array, as a view of an existing one-dimensional
C array.  (Note that the storage for this pre-existing array will
never be destroyed by the VectorRef class.)

@param n the dimension (length) of the new Vector.
@param a the one dimensional C array to use as data storage for
the array. 

*/

template <class T>
RowVector<T>::RowVector(LongInt n, T *a)
{
	v = a;
	this->n = n;
	this->Name = "NoName";
	this->RefCount = 0;
	RefCount = new int;
	*RefCount = 2;		// This avoid destorying original data
}

/*

A[i] indexes the ith element of A.  The first element is
A[0]. If _CheckBounds is defined, then the index is
checked that it falls within the array bounds.

*/

template <class T>
inline T& RowVector<T>::operator[](LongInt i) 
{ 
	#ifdef _CheckBounds
		if(i < 0 || i >= n)
		{
			Print("Error in RowVector");
			Print("Index out of bounds");
			Print("Name",Name);
			Print("Current Index",i);
			Print("Size",n);
			HaltExecution();
		}
	#endif
	
	return v[i];
}

/*

A[i] indexes the ith element of A.  The first element is
A[0]. If _CheckBounds is defined, then the index is
checked that it fall within the array bounds.

*/

template <class T>
inline const T& RowVector<T>::operator[](LongInt i) const 
{ 
	#ifdef _CheckBounds
		if(i < 0 || i >= n)
		{
			Print("Error in Vector");
			Print("Index out of bounds");
			Print("Name",Name);
			Print("Current Index",i);
			Print("Size",n);
			HaltExecution();
		}
	#endif
	
	return v[i];
}

/*

A[i] indexes the ith element of A.  The first element is
A[0]. If _CheckBounds is defined, then the index is
checked that it falls within the array bounds.

*/

template <class T>
inline T& RowVector<T>::operator()(LongInt i) 
{ 
	#ifdef _CheckBounds
		if(i < 0 || i >= n)
		{
			Print("Error in RowVector");
			Print("Index out of bounds");
			Print("Name",Name);
			Print("Current Index",i);
			Print("Size",n);
			HaltExecution();
		}
	#endif
	
	return v[i];
}

/*

A[i] indexes the ith element of A.  The first element is
A[0]. If _CheckBounds is defined, then the index is
checked that it fall within the array bounds.

*/

template <class T>
inline const T& RowVector<T>::operator()(LongInt i) const 
{ 
	#ifdef _CheckBounds
		if(i < 0 || i >= n)
		{
			Print("Error in RowVector");
			Print("Index out of bounds");
			Print("Name",Name);
			Print("Current Index",i);
			Print("Size",n);
			HaltExecution();
		}
	#endif
	
	return v[i];
}

/* Assign all elemnts of A to a constant scalar */
template <class T>
RowVector<T> & RowVector<T>::operator=(const T &a)
{
	BlasSet(n,a,v,1);
	return *this;
}

/* 

Create a new of existing matrix. Used in B = A.copy()
or in the construction of B, e.g. Vector B(A.copy()), 
to create a new array that does not share data.

*/

template <class T>
RowVector<T> RowVector<T>::Copy() const
{
	RowVector A(n);
	A.Name = Name;
	BlasCopy(n,v,1,A.v,1);
	return A;
}

/*

Copy the elements to from one array to another, in place.
That is B.Inject(A), both A and B must conform (i.e. have
identical row and column dimensions).

This differs from B = A.Copy() in that references to B
before this assignment are also affected.  That is, if
we have 
<pre>
Vector A(n);
Vector C(n);
Vector B(C);        // elements of B and C are shared. 

</pre>
then B.Inject(A) affects both and C, while B = A.Copy() creates
 a new array B which shares no data with C or A.

@param A the array from which elements will be copied
@return an instance of the modifed array. That is, in B.Inject(A),
it returns B.  If A and B are not conformat, no modifications to 
B are made.

*/

template <class T>
RowVector<T> & RowVector<T>::Inject(const RowVector &A)
{
	if(A.n == n) BlasCopy(n,A.v,1,v,1);
	return *this;
}

/*

Create a reference (shallow assignment) to another existing array.
In B.Ref(A), B and A shared the same data and subsequent changes
to the array elements of one will be reflected in the other.
<p>
This is what operator= calls, and B = A and B.Ref(A) are equivalent
operations.

@return The new referenced array: in B.Ref(A), it returns B.

*/

template <class T>
RowVector<T> & RowVector<T>::Ref(const RowVector<T> &A)
{
	if(this != &A)
	{
		(*RefCount)--;
		if(*RefCount < 1) Destroy();
		n = A.n;
		v = A.v;
		Name = A.Name;
		RefCount = A.RefCount;
		(*RefCount)++ ;
	}
	return *this;
}

/* B = A is shorthand notation for B.Ref(A) */
template <class T>
RowVector<T> & RowVector<T>::operator=(const RowVector<T> &A)
{
	return Ref(A);
}

/*

Return the dimension (number of elements) of the Vector
This is equivalent to Size()

*/

template <class T>
inline int RowVector<T>::Dim() const
{
	return n;
}

/*

@return the dimension (number of elements) of the array
This is equivalent to Dim()

*/

template <class T>
inline LongInt RowVector<T>::Size() const
{
	return n;
}

template <class T>
inline int RowVector<T>::Type() const
{
	return MatrixType::RowVector;
}

template <class T>
inline void RowVector<T>::operator += (const T &c)
{
	for(LongInt i=0; i<n; i++) v[i] += c;
}

template <class T>
inline void RowVector<T>::operator -= (const T &c)
{
	for(LongInt i=0; i<n; i++) v[i] -= c;
}

template <class T>
inline void RowVector<T>::operator *= (const T &c)
{
	for(LongInt i=0; i<n; i++) v[i] *= c;
}

template <class T>
inline void RowVector<T>::operator /= (const T &c)
{
	for(LongInt i=0; i<n; i++) v[i] /= c;
}

template <class T>
inline void RowVector<T>::operator += (const RowVector<T> &x)
{
	for(LongInt i=0; i<n; i++) v[i] += x[i];
}

template <class T>
inline void RowVector<T>::operator -= (const RowVector<T> &x)
{
	for(LongInt i=0; i<n; i++) v[i] -= x[i];
}

template <class T>
inline void RowVector<T>::operator *= (const RowVector<T> &x)
{
	for(LongInt i=0; i<n; i++) v[i] *= x[i];
}

template <class T>
inline void RowVector<T>::operator /= (const RowVector<T> &x)
{
	for(LongInt i=0; i<n; i++) v[i] /= x[i];
}

/*

Return the number of arrays that share the same storage area
as this one (Must be at least one)

*/

template <class T>
inline int RowVector<T>::GetRefCount() const
{
	return *RefCount;
}

/* Destructor */
template <class T>
RowVector<T>::~RowVector()
{
	(*RefCount) --;
	if(*RefCount < 1) Destroy();
}

/* Private internal functions */

template <class T>
void RowVector<T>::Initialize(LongInt n)
{
	try
	{
		v = new T[n];
	}
	catch(bad_alloc ba)
	{
		Print("Error in RowVector");
		Print("Error Allocating Memory");
		Print("Name",Name);
		Print("Size",n);
		Print("Memory per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",n*sizeof(T)/1e6);
		HaltExecution();
	}
	this->n = n;
}


//template <class T>
//void RowVector<T>::Set_(const T& a)
//{
//	T *begin = &(v_[0]);
//	T *end = begin + n_;
//	for (T* p=begin; p<end; p++) *p = a;
//}

//template <class T>
//void RowVector<T>::Copy_(T* p, const T* q, int len) const
//{
//	T *end = p + len;
//	while (p < end) *p++ = *q++;
//}

template <class T>
void RowVector<T>::Destroy()
{
	if (v != 0) { delete[] (v); }
	if (RefCount != 0) delete RefCount;
}

/* @returns location of first element, i.e. A[0] (mutable) */
/*
template <class T>
const T* RowVector<T>::Begin_() const { return &(v_[0]); }
*/

/* @returns location of first element, i.e. A[0] (mutable) */
//template <class T>
//T* RowVector<T>::Begin_()
//{
//	return &(v_[0]);
//}

/* Michael's Functions */

// Begin
template <class T>
T* RowVector<T>::Begin() const
{
	return &(v[0]);
}

// End
template <class T>
T* RowVector<T>::End() const
{
	return &(v[0]) + n;
}

template <class T>
T* RowVector<T>::Loc(LongInt i) const
{
	return &(v[0]) + i;
}

// Resize a vector, deleting all previous information
template <class T>
void RowVector<T>::Resize(LongInt n)
{
	if (v != 0) delete[] (v);
	v = new T[n];
	this->n = n;
	return;
}

/* Resize a vector, deleting all previous information */
template <class T>
void RowVector<T>::Resize(LongInt n, const T &a)
{
	if(v != 0) delete[] (v);
	v = new T[n];
	this->n = n;
	Set(a);
	return;
}

template <class T>
inline LongInt RowVector<T>::Length() const
{
	return n;
}

template <class T>
inline int RowVector<T>::Increment() const
{
	return 1;
}

#endif
