
#ifndef _GenMatUtil_SVDecompObj
#define _GenMatUtil_SVDecompObj

/*
Singular Value Decomposition.

<P>
For an m-by-n matrix A with m >= n, the singular value decomposition is
an m-by-n orthogonal matrix U, an n-by-n diagonal matrix S, and
an n-by-n orthogonal matrix V so that A = U * S * V^T.
<P>
The singular values, sigma[k] = S[k][k], are ordered so that
sigma[0] >= sigma[1] >= ... >= sigma[n-1].
<P>
The singular value decompostion always exists, so the constructor will
never fail.  The matrix condition number and the effective numerical
rank can be computed from this decomposition.

<p>
(Adapted from JAMA, a Java Matrix Library, developed by jointly
by the Mathworks and NIST; see  http://math.nist.gov/javanumerics/jama).

MP: I updated this code, but it still uses non-unit strides to often

*/

template <class Real>
class SVDecompObj
{
	Matrix<Real> A;
	int m, n;
	int nu;

	bool WantU;
	bool WantV;

	public:

	Matrix<Real> U, V;
	Vector<Real> s;

	void Factor()
	{
		int i=0, j=0, k=0;
		Vector<Real> e(n);
		Vector<Real> Work(Max(m,n));

   		/*

   		Reduce A to bidiagonal form, storing the diagonal elements
   		in s and the super-diagonal elements in e.

   		*/
   		
   		int nct = min(m-1,n);
		int nrt = max(0,min(n-2,m));
		for(k=0; k<max(nct,nrt); k++)
		{
			
			if(k < nct)
			{
				/*

				Compute the transformation for the k-th column and
				place the k-th diagonal in s[k].
				Compute 2-norm of k-th column without under/overflow.

				*/

				s[k] = 0;
				for(i=k; i<m; i++) s[k] = hypot(s[k],A(i,k));
				if(s[k] != Zero)
				{
					if(A(k,k) < Zero) s[k] = -s[k];
					for(i=k; i<m; i++) A(i,k) /= s[k];
					A(k,k) += One;
				}
				s[k] = -s[k];
			}

			/* Apply the transformation. */

			/*
			for(j=k+1; j<n; j++)
			{
            	if((k < nct) && (s[k] != 0.0))
            	{
            		double t = 0;
               		for(i=k; i<m; i++) t += A(i,k) * A(i,j);
               		t = -t / A(k,k);
               		for(i=k; i<m; i++) A(i,j) += t * A(i,k);
				}
			}
			*/

			if((k < nct ) && (k < n - 1) && (s[k] != Zero))
           	{
           		BlasSet(n-k-1,Zero,&Work[k+1],1);
           		for(i=k; i<m; i++) BlasAxpy(n-k-1,A(i,k),&A(i,k+1),1,&Work[k+1],1);
				BlasScal(n-k-1,-One / A(k,k),&Work[k+1],1);
				for(i=k; i<m; i++) BlasAxpy(n-k-1,A(i,k),&Work[k+1],1,&A(i,k+1),1);;
			}

			/*

			Place the k-th row of A into e for the
			subsequent calculation of the row transformation.

			*/

			for(j=k+1; j<n; j++) e[j] = A(k,j);

			if(WantU & (k < nct))
			{
				/*

				Place the transformation in U for subsequent back
				multiplication.

				*/

				for(i=k; i<m; i++) U(i,k) = A(i,k);
			}
			if(k < nrt)
			{
            	/*

            	Compute the k-th row transformation and place the
            	k-th super-diagonal in e[k].
            	Compute 2-norm without under/overflow.

            	*/

            	e[k] = 0;
            	for(i=k+1; i<n; i++) e[k] = hypot(e[k],e[i]);
				if(e[k] != Zero)
				{
               		if(e[k+1] < Zero) e[k] = -e[k];
               		for(i=k+1; i<n; i++) e[i] /= e[k];
					e[k+1] += One;
				}
				e[k] = -e[k];
				if((k+1 < m) & (e[k] != Zero))
				{
					/* Apply the transformation. */

					/*
               		for(i=k+1; i<m; i++) work[i] = 0.0;
               		for(j=k+1; j<n; j++) for(i=k+1; i<m; i++) work[i] += e[j] * A(i,j);
               		for(j=k+1; j<n; j++)
               		{
                  		double t = -e[j] / e[k+1];
						for(i=k+1; i<m; i++) A(i,j) += t * work[i];
               		}
               		*/

               		for(i=k+1; i<m; i++) Work[i] = BlasDot(n-k-1,&e[k+1],1,&A(i,k+1),1);
               		for(i=k+1; i<m; i++)
               		{
               			double Temp = -Work[i] / e[k+1];
               			BlasAxpy(n-k-1,Temp,&e[k+1],1,&A(i,k+1),1);
               		}
            	}
            	if(WantV)
            	{
            		/*

            		Place the transformation in V for subsequent
            		back multiplication.

            		*/

 					for(i=k+1; i<n; i++) V(i,k) = e[i];
				}
			}
		}
		
		/* Set up the final bidiagonal matrix or order p. */
		int p = min(n,m+1);
		if(nct < n) s[nct] = A(nct,nct);
      	if(m < p) s[p-1] = Zero;
		if(nrt+1 < p) e[nrt] = A(nrt,p-1);
		e[p-1] = Zero;

		/* If required, generate U. */
		if(WantU)
		{
			for(j=nct; j<nu; j++)
			{
				for(i=0; i<m; i++) U(i,j) = Zero;
            	U(j,j) = One;
            }
            for(k=nct-1; k>=0; k--)
            {
            	if(s[k] != Zero)
            	{
               		for(j=k+1; j<nu; j++)
               		{
                  		//double t = 0;
                  		//for(i=k; i<m; i++) t += U(i,k) * U(i,j);
                  		double t = BlasDot(m-k,&U(k,k),nu,&U(k,j),nu);
                  		t = -t / U(k,k);
                  		//for(i=k; i<m; i++) U(i,j) += t * U(i,k);
                  		BlasAxpy(m-k,t,&U(k,k),nu,&U(k,j),nu);
               		}
               		for(i=k; i<m; i++) U(i,k) = -U(i,k);
               		U(k,k) = One + U(k,k);
               		for(i=0; i<k-1; i++) U(i,k) = Zero;
            	}
            	else
            	{
           			for(i=0; i<m; i++) U(i,k) = Zero;
            		U(k,k) = One;
            	}
         	}
      	}

		/* If required, generate V. */	
		if(WantV)
		{
        	for(k=n-1; k>=0; k--)
        	{
            	if((k < nrt) & (e[k] != Zero))
            	{
               		for(j=k+1; j<nu; j++)
               		{
                  		//double t = 0;
                  		//for(i=k+1; i<n; i++) t += V(i,k) * V(i,j);
                  		double t = BlasDot(n-k-1,&V(k+1,k),n,&V(k+1,j),n);
                  		t = -t / V(k+1,k);
                  		//for(i=k+1; i<n; i++) V(i,j) += t * V(i,k);
                  		BlasAxpy(n-k-1,t,&V(k+1,k),n,&V(k+1,j),n);
               		}
            	}
            	for(i=0; i<n; i++) V(i,k) = Zero;
            	V(k,k) = One;
         	}
      	}

      	/* Main iteration loop for the singular values. */
		int pp = p - 1;
     	int iter = 0;
     	double eps = pow(2.0,-52.0);
     	while(p > 0)
     	{     		
       		int k = 0;
			int kase = 0;

    		/* Here is where a test for too many iterations would go. */
    		if(iter > n + 10000) // MP's Code
    		{
    			Print("Error in SVDecompObj::Factor");
    			Print("Too Many Iterations");
    			Print("This sometimes happens with many identical rows");
    			HaltExecution();    		
    		}

			/*

			This section of the program inspects for
			negligible elements in the s and e arrays.  On
			completion the variables kase and k are set as follows.

			kase = 1     if s(p) and e[k-1] are negligible and k<p
			kase = 2     if s(k) is negligible and k<p
			kase = 3     if e[k-1] is negligible, k<p, and
			             s(k), ..., s(p) are not negligible (qr step).
			kase = 4     if e(p-1) is negligible (convergence).

			*/

			for(k=p-2; k>=-1; k--)
			{
				if(k == -1) break;
            	if(abs(e[k]) <= eps * (abs(s[k]) + abs(s[k+1])))
            	{
            		e[k] = Zero;
            		break;
           		}
         	}
         	if(k == p - 2)
         	{
            	kase = 4;
         	}
         	else
         	{
           		int ks;
				for(ks=p-1; ks>=k; ks--)
				{
               		if(ks == k) break;
               		double t = (ks != p ? abs(e[ks]) : Zero) +  (ks != k+1 ? abs(e[ks-1]) : Zero);
               		if(abs(s[ks]) <= eps * t)
               		{
                  		s[ks] = Zero;
                 		break;
               		}
            	}
            	if(ks == k)
            	{
               		kase = 3;
            	}
            	else if(ks == p-1)
            	{
              		kase = 1;
            	}
            	else
            	{
           			kase = 2;
             		k = ks;
           		}
        	}
         	k++;

			/* Perform the task indicated by kase. */

         	switch(kase)
         	{

            	/* Deflate negligible s(p). */

           		case 1:
           		{
					double f = e[p-2];
               		e[p-2] = Zero;
               		for(j=p-2; j>=k; j--)
               		{
                 		double t = hypot(s[j],f);
						double cs = s[j] / t;
						double sn = f / t;
						s[j] = t;
						if(j != k)
						{
							f = -sn * e[j-1];
							e[j-1] = cs * e[j-1];
                  		}
                  		if(WantV)
                  		{
                  			/*
                     		for(i=0; i<n; i++)
                     		{
                        		t = cs * V(i,j) + sn * V(i,p-1);
                        		V(i,p-1) = -sn * V(i,j) + cs * V(i,p-1);
                        		V(i,j) = t;
                    		}
                    		*/
                    		BlasSet(n,Zero,Work.Begin(),1);
                    		BlasAxpy(n, cs,&V(0,  j),n,Work.Begin(),1);
                    		BlasAxpy(n, sn,&V(0,p-1),n,Work.Begin(),1);
                    		BlasScal(n,cs,&V(0,p-1),n);
                    		BlasAxpy(n,-sn,&V(0,  j),n,&V(0,p-1)   ,n);
							BlasCopy(n,Work.Begin(),1,&V(0,j),n);
                 		}
              		}
           		}
            	break;

            	/* Split at negligible s(k). */

            	case 2:
            	{
               		double f = e[k-1];
               		e[k-1] = Zero;
               		for(j=k; j<p; j++)
               		{
                  		double t = hypot(s[j],f);
                  		double cs = s[j] / t;
                  		double sn = f / t;
                		s[j] = t;
                  		f = -sn * e[j];
                  		e[j] = cs * e[j];

                  		if(WantU)
                  		{
                  			/*
                    		for(i=0; i<m; i++)
                    		{
                        		t = cs * U(i,j) + sn * U(i,k-1);
                        		U(i,k-1) = -sn * U(i,j) + cs * U(i,k-1);
                        		U(i,j) = t;
                     		}
                     		*/
                     		BlasSet (m,Zero,Work.Begin(),1);
                    		BlasAxpy(m, cs,&U(0,  j),nu,Work.Begin(),1);
                    		BlasAxpy(m, sn,&U(0,k-1),nu,Work.Begin(),1);
                    		BlasScal(m,cs,&U(0,k-1),nu);
                    		BlasAxpy(m,-sn,&U(0,  j),nu,&U(0,k-1)   ,nu);
                    		BlasCopy(m,Work.Begin(),1,&U(0,j),nu);
                  		}
               		}
            	}
            	break;

				/* Perform one qr step. */

           		case 3:
           		{

           			/* Calculate the shift. */

             		double scale = max(max(max(max(
                       abs(s[p-1]),abs(s[p-2])),abs(e[p-2])),
                       abs(s[k])),abs(e[k]));
               		double sp = s[p-1] / scale;
               		double spm1 = s[p-2] / scale;
               		double epm1 = e[p-2] / scale;
               		double sk = s[k] / scale;
               		double ek = e[k] / scale;
               		double b = ((spm1 + sp) * (spm1 - sp) + epm1 * epm1) / 2.0;
               		double c = (sp * epm1) * (sp * epm1);
               		double shift = Zero;
               		if((b != Zero) | (c != Zero))
               		{
                  		shift = sqrt(b * b + c);
                  		if(b < Zero) shift = -shift;
               			shift = c / (b + shift);
               		}
               		double f = (sk + sp) * (sk - sp) + shift;
               		double g = sk * ek;

              		/* Chase zeros. */

              		for(j=k; j<p-1; j++)
              		{
               			double t = hypot(f,g);
                  		double cs = f / t;
                 		double sn = g / t;
                 		if(j != k) e[j-1] = t;
                  		f = cs * s[j] + sn * e[j];
                  		e[j] = cs * e[j] - sn * s[j];
                  		g = sn * s[j+1];
                  		s[j+1] = cs * s[j+1];
                  		if(WantV)
                  		{
                  			/*
                     		for(i=0; i<n; i++)
                     		{
                     			t = cs * V(i,j) + sn * V(i,j+1);
                        		V(i,j+1) = -sn * V(i,j) + cs * V(i,j+1);
                        		V(i,j) = t;
                       		}
                       		*/

                     		BlasSet(n,Zero,Work.Begin(),1);
							BlasAxpy(n, cs,&V(0,  j),n,Work.Begin(),1);
							BlasAxpy(n, sn,&V(0,j+1),n,Work.Begin(),1);
	                    	BlasScal(n,cs,&V(0,j+1),n);
							BlasAxpy(n,-sn,&V(0,  j),n,&V(0,j+1)   ,n);
							BlasCopy(n,Work.Begin(),1,&V(0,j),n);
                 		}
                  		t = hypot(f,g);
                  		cs = f / t;
                  		sn = g / t;
                  		s[j] = t;
                  		f = cs * e[j] + sn * s[j+1];
                  		s[j+1] = -sn * e[j] + cs * s[j+1];
                  		g = sn * e[j+1];
                  		e[j+1] = cs * e[j+1];
						if(WantU && (j < m-1))
						{
							/*
                  			for(i=0; i<m; i++)
                  			{
								t = cs * U(i,j) + sn * U(i,j+1);
								U(i,j+1) = -sn * U(i,j) + cs * U(i,j+1);
								U(i,j) = t;
							}
							*/

							BlasSet (m,Zero,Work.Begin(),1);
                    		BlasAxpy(m, cs,&U(0,  j),nu,Work.Begin(),1);
                    		BlasAxpy(m, sn,&U(0,j+1),nu,Work.Begin(),1);
                    		BlasScal(m,cs,&U(0,j+1),nu);
                    		BlasAxpy(m,-sn,&U(0,  j),nu,&U(0,j+1),nu);
                    		BlasCopy(m,Work.Begin(),1,&U(0,j),nu);
						}
					}
       				e[p-2] = f;
        			iter = iter + 1;
          		}
            	break;

	            /* Convergence. */

    	        case 4:
       	     	{

					/* Make the singular values positive. */

					if(s[k] <= Zero)
					{
						s[k] = (s[k] < Zero ? -s[k] : Zero);
						if(WantV)
						{
       		             	for(i=0; i<=pp; i++) V(i,k) = -V(i,k);
               		  	}
           		 	}

  	   				/* Order the singular values. */

  					while(k < pp)
  					{
       		         	if(s[k] >= s[k+1]) break;
             	    	double t = s[k];
              		   	s[k] = s[k+1];
                	 	s[k+1] = t;
               		  	if(WantV && (k < n-1))
                	 	{
                	 		for(i=0; i<n; i++)
                   		 	{
                      		  	t = V(i,k+1);
                      		  	V(i,k+1) = V(i,k);
                      		  	V(i,k) = t;
                    	 	}
                 	 	}
                	  	if(WantU && (k < m-1))
                 	 	{
                  		  	for(i=0; i<m; i++)
                    		{
                      		  	t = U(i,k+1);
                       		 	U(i,k+1) = U(i,k);
                      		  	U(i,k) = t;
                   		  	}
                	  	}
               			k++;
             	 	}
            	  	iter = 0;
            	 	p--;
         	 	}
        		break;
    	  	}
		}
	}

	SVDecompObj(const Matrix<Real> &A)
	{
		if(IsNan(A))
		{
			Print("Error in SVDecompObj(Matrix)");
			Print("A has NaN entry");
			Print("A.Name",A.Name);
			Print("IsNan(A)",IsNan(A));
			HaltExecution();
		}
		
		m = A.Rows();
		n = A.Cols();
		nu = Min(m,n);
		s = Vector<Real>(Min(m+1,n));
		U = Matrix<Real>(m,nu,Real(0));
		V = Matrix<Real>(n,n);
		this->A = A.Copy();
		WantU = True;
		WantV = True;
		if(m > 0 && n > 0) Factor();
	}

	SVDecompObj(const Matrix<Real> &A, bool WantU, bool WantV)
	{
		if(IsNan(A))
		{
			Print("Error in SVDecompObj(Matrix,bool,bool)");
			Print("A has NaN entry");
			Print("A.Name",A.Name);
			Print("IsNan(A)",IsNan(A));
			HaltExecution();
		}
	
		m = A.Rows();
		n = A.Cols();
		this->WantU = WantU;
		this->WantV = WantV;
		nu = Min(m,n);
		s = Vector<Real>(Min(m+1,n));
		U = Matrix<Real>(m,nu,Real(0));
		V = Matrix<Real>(n,n);
		this->A = A.Copy();
		if(m > 0 && n > 0) Factor();
	}

	SVDecompObj(const SymmetricMatrix<Real> &A)
	{
		if(IsNan(A))
		{
			Print("Error in SVDecompObj(SymmetricMatrix)");
			Print("A has NaN entry");
			Print("A.Name",A.Name);
			Print("IsNan(A)",IsNan(A));
			HaltExecution();
		}
	
		m = A.Rows();
		n = A.Cols();
		nu = Min(m,n);
		s = Vector<Real>(Min(m+1,n));
		U = Matrix<Real>(m,nu,Real(0));
		V = Matrix<Real>(n,n);
		this->A = Matrix<Real>(m,n);
		WantU = True;
		WantV = True;
		Copy(this->A,A);
		if(m > 0 && n > 0) Factor();
	}

	SVDecompObj(const SymmetricMatrix<Real> &A, bool WantU, bool WantV)
	{
		if(IsNan(A))
		{
			Print("Error in SVDecompObj(SymmetricMatrix,bool,bool)");
			Print("A has NaN entry");
			Print("A.Name",A.Name);
			Print("IsNan(A)",IsNan(A));
			HaltExecution();
		}
	
		m = A.Rows();
		n = A.Cols();
		nu = Min(m,n);
		s = Vector<Real>(Min(m+1,n));
		U = Matrix<Real>(m,nu,Real(0));
		V = Matrix<Real>(n,n);
		this->A = Matrix<Real>(m,n);
		this->WantU = WantU;
		this->WantV = WantV;
		Copy(this->A,A);
		if(m > 0 && n > 0) Factor();
	}

	SVDecompObj(const SVDecompObj &svd)
	{
		A = svd.A;
		U = svd.A;
		V = svd.A;
		s = svd.s;
		m = svd.m;
		n = svd.n;
		nu = svd.nu;
		WantU = svd.WantU;
		WantV = svd.WantV;
	}

	SVDecompObj() {}

	~SVDecompObj() {}

	Matrix<Real> GetU()
	{
		//int minm = min(m+1,n);
		int minm = Min(m,n);
		Matrix<Real> A(m,minm);
		for(int i=0; i<m; i++) for(int j=0; j<minm; j++) A(i,j) = U(i,j);
		return A;
	}

	/* Return the right singular vectors */
	Matrix<Real> GetV()
	{
   		return V;
   	}

	/** Return the one-dimensional array of singular values */
	Vector<Real> GetSingularValues()
	{
		return s;
	}

   	/*

	Return the diagonal matrix of singular values
   	@return     S

   	*/

  	Matrix<Real> GetS()
	{
		int minm = Min(m,n);
   		Matrix<Real> S(m,n,Zero);
   		for(int i=0; i<minm; i++) S(i,i) = s[i];
      	return S;
  	}

   /** Two norm  (max(S)) */

	double norm2()
	{
		return s[0];
	}

   /** Two norm of condition number (max(S)/min(S)) */

	double Cond()
	{
		return s[0] / s[min(m,n)-1];
	}

	/*

	Effective numerical matrix rank
	@return     Number of nonnegligible singular values.

	*/

	int Rank()
	{
		if(Max(m,n) == 0) return 0;
		double eps = pow(2.0,-52.0);
		double tol = max(m,n) * s[0] * eps;
		int r = 0;
		for(int i=0; i<s.Size(); i++) if(s[i] > tol) r++;
        return r;
	}

	int Rank(double eps)
	{
		if(Max(m,n) == 0) return 0;
		double tol = max(m,n) * s[0] * eps;
		int r = 0;
		for(int i=0; i<s.Size(); i++) if(s[i] > tol) r++;
        return r;
	}

	Matrix<double> ColumnBasis()
	{
		double eps = pow(2.0,-52.0);
		double tol = max(m,n) * s[0] * eps;
		int r = 0;
		for(int i=0; i<s.Size(); i++) if(s[i] > tol) r++;
		Matrix<double> B(m,r);
		B.Name = "SVD::ColumnBasis()::B";
		for(int i=0; i<m; i++) for(int j=0; j<r; j++) B(i,j) = U(i,j);
		return B;
	}

	Matrix<double> ColumnBasis(double eps)
	{
		double tol = max(m,n) * s[0] * eps;
		int r = 0;
		for(int i=0; i<s.Size(); i++) if(s[i] > tol) r++;
		Matrix<double> B(m,r);
		B.Name = "SVD::ColumnBasis(eps)::B";
		for(int i=0; i<m; i++) for(int j=0; j<r; j++) B(i,j) = U(i,j);
		return B;
	}

	Matrix<double> RowBasis()
	{
		double eps = pow(2.0,-52.0);
		double tol = max(m,n) * s[0] * eps;
		int r = 0;
		for(int i=0; i<s.Size(); i++) if(s[i] > tol) r++;
		Matrix<double> B(r,n);
		B.Name = "SVD::RowBasis()::B";
		for(int i=0; i<r; i++) for(int j=0; j<n; j++) B(i,j) = V(j,i);
		return B;
	}

	Matrix<double> RowBasis(double eps)
	{
		double tol = max(m,n) * s[0] * eps;
		int r = 0;
		for(int i=0; i<s.Size(); i++) if(s[i] > tol) r++;
		Matrix<double> B(r,n);
		B.Name = "SVD::ColumnBasis(eps)::B";
		for(int i=0; i<r; i++) for(int j=0; j<n; j++) B(i,j) = V(j,i);
		return B;
	}

	Matrix<double> NullSpace()
   	{
   		int sing = 0;
   		int i,j,k;
   		for(i=0; i<min(m+1,n); i++) if(s[i] == 0.0) sing++;
  		Matrix<double> Z(n,sing);

   		Print("U",U);
   		Print("V",V);
   		Print("s",s);
   		Print("sing",sing);

   		k=0;
   		for(j=0; j<n; j++)
   		{
   			if(s[j] == Zero)
   			{
   				for(i=0; i<n; i++) Z(i,k) = V(i,j);
   				k++;
   			}
   		}

   		Print("Z",Z);

   		return Z;
   	}

   	Matrix<double> RangeSpace()
   	{
   		/*

   		The range of the linear transformation is the space spanned
   		by the columns of U corresponding to non-sigular values

   		*/

   		int nsing = 0;
   		int i,j,k;
   		for(i=0; i<min(m+1,n); i++) if(s[i] != 0.0) nsing++;
   		Matrix<double> Z(m,nsing);

   		Print("m",m);
   		Print("n",n);
   		Print("U",U);
   		Print("V",V);
   		Print("s",s);
   		Print("nsing",nsing);

   		k=0;
   		for(j=0; j<n; j++)
   		{
   			if(s[j] != Zero)
   			{
   				for(i=0; i<m; i++) Z(i,k) = U(i,j);
   				k++;
   			}
   		}
   		return Z;
   	}

   	/*
   	Vector<double> Solve(const Vector<double> &x)
   	{
   		Matrix<double> U2 = U.Copy();
		Vector<double> y = Inverse(Transpose(U2) * U2) * Transpose(U2) * x;
		Print("Solve::y",y);
		PauseExecution();
		return y;
   	}
	*/

   	Matrix<double> Inverse()
   	{
		if(m != n)
		{
			Print("Error in SVDecompObj::Inverse");
			Print("Matrix is Not Square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}

		/* A^-1 = V * D^-1 * U^T */

		Matrix<double> AInv(n,n);

		/* This part could probably be rewritten to avoid using a new matrix
		by allowing crease a routine to multiple such things */

		Matrix<double> DInvUT(n,n);
		for(int i=0; i<n; i++) for(int j=0; j<n; j++)
			DInvUT(i,j) = U(j,i) / s(i);

		ProdAB(AInv,V,DInvUT);

		return AInv;
	}

	/*
	Matrix<double> PDInverse(double Tol)
   	{

   		/* Compute the postive definite matrix which is 'close' to being an inverse */
   	/*
		if(m != n)
		{
			Print("Error in SVDecomp::PDInverse(Real Tol)");
			Print("Matrix is not square");
			Print("m",m);
			Print("n",n);
			HaltExecution();
		}

		/* A^-1 = V * D^-1 * U^T */
	/*
		Matrix<double> AInv(n,n);

		/* This part could probably be rewritten to avoid using a new matrix
		by allowing crease a routine to multiple such things */
	/*
		Matrix<double> DInvUT(n,n);
		for(int j=0; j<n; j++)
		{
			double Temp;
			if(s(i) <= Zero) Temp = Tol
			else Temp = Max(One / s(i),Tol)
			for(int i=0; i<n; i++) DInvUT(i,j) = Temp * U(j,i);
		}

		ProdAB(AInv,V,DInvUT);

		return AInv;
	}
	*/

	/*
	SymmetricMatrix<double> InverseSymmetric()
	{
		if(




	}
	*/
};

int Rank(const Matrix<double> &A);
int Rank(const Matrix<double> &A)
{
	if(Min(A.Size()) == 0) return 0;
	SVDecompObj<double> svd(A);
	return svd.Rank();
}

int Rank(const SymmetricMatrix<double> &A);
int Rank(const SymmetricMatrix<double> &A)
{
	if(A.Dim() == 0) return 0;
	SVDecompObj<double> svd(A);
	return svd.Rank();
}

int Rank(const Matrix<double> &A, double Tol);
int Rank(const Matrix<double> &A, double Tol)
{
	if(Min(A.Size()) == 0) return 0;
	SVDecompObj<double> svd(A);
	return svd.Rank(Tol);
}

Vector<double> SingularValues(const Matrix<double> &A);
Vector<double> SingularValues(const Matrix<double> &A)
{
	SVDecompObj<double> svd(A);
	return svd.GetSingularValues();
}

#endif
