
#ifndef _Opt_Solve1D
#define _Opt_Solve1D

/*

Find solutions to the non-linear equation, f(x) = 0,
in the range (x1,x2).

Option:

PrintLevel	=	0: Print Nothing
			=	1: Print Results and Errors
			=	2: Print One Line Per Iteration
      
Method		=	1: Bisection Method
			=	2: Ridder's Method
			=	3: Secant Method
			=	4: Brent's Method
			=	5: Newton's Method
			=	6: False Position Method
			


Error Codes:
0	-	Successful
1	-	Root is Not Bracketed
2	-	Too Many Iterations
3	-	Jumped Out of Brackets

*/

class Solve1D
{

	public:

	double (*Func)(double); // Objective Function
	void (*DFunc)(double, double &, double &); // Objective Function and it's Derivative
	
	int MaxIter;		// Maximum Number of Iterations
	double TolX;		// Tolerance for X
	double TolD;		// Tolerance for D (not used yet)
	int PrintLevel;
	int DerivOpt;
	int Method;
	double DerivStep;
	int DerivType;
		
	double x;	// Optimal Value
	double f;	// Optimal Function Value
	
	double x1,x2;
	double f1,f2;
	
	double xCurr,fCurr;
	
	double Lambda;
	
	int Iter;
	
	/* Constructor: No Analytical Derivative Provided */
	Solve1D(double (*Func)(double)) : Func(Func)
	{
		MaxIter = 100;
		TolX    = 0.00000001;
		TolD    = 0.0001;
		PrintLevel = 0;
		DerivOpt = 0;
		Method = 1;
		Lambda = 1.0;
		DerivStep = MachEps2;
		DerivType = 1;
		Iter = 0;
	}
	
	/* Constructor: Analytical Derivative Provided */
	Solve1D(void (*DFunc)(double, double &, double &)) : DFunc(DFunc)
	{
		MaxIter = 100;
		TolX    = 0.00000001;
		TolD    = 0.0001;
		PrintLevel = 0;
		DerivOpt = 1;
		Method = 1;
		Lambda = 1.0;
		DerivStep = MachEps2;
		DerivType = 1;
		Iter = 0;
	}
	
	~Solve1D() {}
	
	/* Solve a Bracket Minimum */
	int Solve(double xLow, double xHigh)
	{
	
		if(xLow >= xHigh)
		{
			Print("Error in Solve1D::Solve(xLow,xHigh)");
			HaltExecution();
		}
	
		double FLow  = Func(xLow);
		double FHigh = Func(xHigh);
		
		if(FLow * FHigh >= Zero)
		{
			if(PrintLevel >= 1)
			{
				Print("Unsuccesfully Exiting Solve1D");
				Print("Root is Not Bracketed");
				Print("xLow",xLow,"FLow",FLow);
				Print("xHigh",xHigh,"FHigh",FHigh);
			}
			return 1;
		}
		
		/* Call the Solver */
		     if(Method == 1) return SolveBisection(xLow,xHigh);
		else if(Method == 2) return SolveRidder(xLow,xHigh);
		else if(Method == 3) return SolveSecant(xLow,xHigh);
		else if(Method == 4) return SolveBrent(xLow,xHigh);
		else if(Method == 5) return SolveNewton(xLow,xHigh);
		else				 return SolveFalsePosition(xLow,xHigh);
	}

	/* Find All Solution of f(x) = 0 Usign a Grid of Points */
	Vector<double> SolveAll(const Vector<double> &XGrid)
	{
	
		if(!IsSorted(XGrid))
		{
			Print("Error in SolveAll");
			HaltExecution();
		}
	
		Stack<double> Ret;
		
		int K = XGrid.Size();
		Vector<double> FGrid(K);
		for(int k=0; k<K; k++)
		{
			FGrid(k) = Func(XGrid(k));
		}
		
		for(int k=1; k<K; k++)
		{
			if(FGrid(k) * FGrid(k-1) <= Zero)
			{
				/*
				Print("here-k",k);
				Print("XGrid(k-1)",XGrid(k-1));
				Print("XGrid(k)",XGrid(k));
				Print("FGrid(k-1)",FGrid(k-1));
				Print("FGrid(k)",FGrid(k));
				Print("Method",Method);
				*/
				
				     if(Method == 1) SolveBisection    (XGrid(k-1),XGrid(k));
				else if(Method == 2) SolveRidder       (XGrid(k-1),XGrid(k));
				else if(Method == 3) SolveSecant       (XGrid(k-1),XGrid(k));
				else if(Method == 4) SolveBrent        (XGrid(k-1),XGrid(k));
				else if(Method == 5) SolveNewton       (XGrid(k-1),XGrid(k));
				else                 SolveFalsePosition(XGrid(k-1),XGrid(k));
				
				if(abs(f) < 0.00000001) Ret.Push(x);
			}		
		}
						
		return ListValues(Ret);
	}	
	
	/* Bisection Method */
	int SolveBisection(double x1, double x2)
	{
		double f1   = Func(x1);
		double fmid = Func(x2);
		
		double xCurr,dx;
		if(f1 < Zero)
		{
			dx = x2 - x1;
			xCurr = x1;
		}
		else
		{
			dx = x1 - x2;
			xCurr = x2;
		}
		
		if(f1 * fmid > Zero)
		{
			if(PrintLevel >= 1)
			{
				Print("Error in SolveBisection");
				Print("Root is Not Bracketed");
				Print("x1",x1);
				Print("x2",x2);
				Print("f1",f1);
				Print("f2",fmid);
				Print("");
			}					
			return 1;
		}
		
		for(int Iter=1; Iter<=MaxIter; Iter++)
		{
			dx *= Half;
			double xmid = xCurr + dx;
			fmid = Func(xmid);
						
			if(PrintLevel >= 2) Print("Iter",Iter,"xmid",xmid,"fmid",fmid,"dx",dx);
			
			if(fmid <= Zero) xCurr = xmid;
			
			if(fabs(dx) < TolX || fmid == Zero)
			{
				x = xCurr;
				f = fmid;
				if(PrintLevel >= 1) Print("Succesfully Exiting Solve Bisection");
				if(PrintLevel >= 1) Print("Number of Iterations",Iter);
				this->Iter = Iter;
				return 0;
			}
		}
		
		if(PrintLevel >= 1)
		{
			Print("Unsuccesfully Exiting SolveBisection: Too Many Iterations");
			Print("MaxIter",MaxIter);
		}
		
		return 2;
	}
	
	/* Brent's Method */
	int SolveBrent(double x1, double x2)
	{
		double d,e,p,q,r,s,xm;
		double Eps = 3.0e-8;
		
		/* Check if Root is Bracketed */
		double a  = x1;
		double fa = Func(a);
		double b  = x2;
		double fb = Func(b);
		if(fa * fb > Zero)
		{
			if(PrintLevel >= 1) Print("Unsuccesfully Exiting Solve1D::SolveBrent: Root Is Not Bracketed");
			return 1;
		}
		
		double c  = x2;
		double fc = fb;
		for(int Iter=1; Iter<=MaxIter; Iter++)
		{
			if((fb > Zero && fc > Zero) || (fb < Zero && fc < Zero))
			{
				c = a;
				fc = fa;
				e = d = b - a;
			}
			if(abs(fc) < abs(fb))
			{
				a = b;
				b = c;
				c = a;
				fa = fb;
				fb = fc;
				fc = fa;
			}
			double Tol1 = Two * Eps * abs(b) + Half * TolX;
			xm = Half * (c - b);
			if(fabs(xm) <= Tol1 || fb == Zero)
			{
				x = b;
				f = fb;
				this->Iter = Iter;	
				return 0;
			}
			if(abs(e) >= Tol1 && abs(fa) > abs(fb))
			{
				s = fb / fa;
				if(a == c)
				{
					p = Two * xm * s;
					q = One - s;
				}
				else
				{
					q = fa / fc;
					r = fb / fc;
					p = s * (Two * xm * q * (q - r) - (b - a) * (r - One));
					q = (q - One) * (r - One) * (s - One);
				}
				if(p > Zero) q = -q;
				p = abs(p);
				double Min1 = Three * xm * q - abs(Tol1 * q);
				double Min2 = abs(e * q);
				if(Two * p < (Min1 < Min2 ? Min1:Min2))
				{
					e = d;
					d = p / q;
				}
				else
				{
					d = xm;
					e = d;
				}
			}
			else
			{
				d = xm;
				e = d;
			}
			a = b;
			fa = fb;
			if(abs(d) > Tol1) b += d;
			else	          b += Sign(Tol1,xm);
			
			fb = Func(b);
			
			if(PrintLevel >= 2) Print("Iter",Iter,"x",b,"f",fb,"Diff",abs(xm));
		}
		if(PrintLevel >= 1) Print("Unsucessfully Exiting SolveBrent: Too Many Iterations");
		return 2;
	}
	
	/* Ridder's Method */
	int SolveRidder(double x1, double x2)
	{
		double xCurr,fh,fl,fm,fnew,s,xh,xl,xm,xnew;
		double Unused = -1.11e30;
		
		fl = Func(x1);
		fh = Func(x2);
		
		if(fl * fh >= Zero)
		{
			if(PrintLevel >= 1)
			{
				Print("Error in SolveRidder");
				Print("Root is Not Bracketed");
				Print("x1",x1);
				Print("x2",x2);
				Print("f1",fl);
				Print("f2",fh);
				Print("");
			}					
			return 1;
		}
		
		if(fl == Zero)
		{
			x = x1;
			f = fl;
			this->Iter = Iter;
			return 0;
		}
		else if(fh == Zero)
		{
			x = x2;
			f = fh;
			this->Iter = Iter;
			return 0;
		}
		
		xl = x1;
		xh = x2;
		xCurr = Unused;
		for(int Iter=1; Iter<=MaxIter; Iter++)
		{
			xm = 0.5 * (xl + xh);
			
			fm = Func(xm);
			
			if(PrintLevel >= 2) Print("Iter",Iter,"xm",xm,"fm",fm);
		
			s = sqrt(fm * fm - fl * fh);
			if(s == Zero)
			{
				x = xCurr;
				f = fnew;
				this->Iter = Iter;
				return 0;			
			}
			xnew = xm + (xm - xl) * ((fl >= fh ? 1.0 : -1.0) * fm / s);
			
			if(fabs(xnew - xCurr) <= TolX)
			{
				x = xCurr;
				f = fnew;
				this->Iter = Iter;
				return 0;
			}
			xCurr = xnew;
			
			fnew = Func(xCurr);
				
			if(fnew == Zero)
			{
				x = xCurr;
				f = fnew;
				this->Iter = Iter;
				return 0;
			}
								
			if(Sign(fm,fnew) != fm)
			{
				xl = xm;
				fl = fm;
				xh = xCurr;
				fh = fnew;
			}
			else if(Sign(fl,fnew) != fl)
			{
				xh = xCurr;
				fh = fnew;
			}
			else if(Sign(fh,fnew) != fh)
			{
				xl = xCurr;
				fl = fnew;
			}
			
			if(fabs(xh - xl) <= TolX)
			{
				x = xCurr;
				f = fnew;
				this->Iter = Iter;
				return 0;				
			} 
		}
			
		Print("Unsuccesfully Exiting SolveRidder: Too Many Iterations");
		return 2;
	}
	
	/* Secant Method */
	int SolveSecant(double x1, double x2)
	{
		
		double fPrev = Func(x1);
		double fCurr = Func(x2);

		double xPrev,xCurr;
		if(abs(fPrev) < abs(fCurr))
		{
			xCurr = x1;
			xPrev = x2;
			Swap(fCurr,fPrev);
		}
		else
		{
			xPrev = x1;
			xCurr = x2;
		}
		
		for(int Iter=1; Iter<=MaxIter; Iter++)
		{
			double dx = (xPrev - xCurr) * fCurr / (fCurr - fPrev);

			xPrev = xCurr;
			fPrev = fCurr;
			xCurr += dx;

			if(PrintLevel >= 2) Print("Iter",Iter,"x",xPrev,"f",fPrev);
						
			if((x1 - xCurr) * (xCurr - x2) < Zero)
			{
				if(PrintLevel >= 1) Print("Unsuccesfully Exiting Solve1D::Secant: Jumped Out of Brakets");
				if(PrintLevel >= 2) Print("x1",x1,"x2",x2);
				if(PrintLevel >= 2) Print("xCurr",xCurr);
				return 3;
			}
			
			fCurr = Func(xCurr);
						
			if(fabs(dx) < TolX || fCurr == Zero)
			{
				x = xCurr;
				f = fCurr;
				if(PrintLevel >= 1) Print("Succesfully Exiting SolveSecant");
				this->Iter = Iter;
				return 0;
			}
		}
		
		Print("Unsuccesfully Exiting SolveSecant: Too Many Iterations");
		return 2;
	}
	
	/* Newton's Method */
	int SolveNewton(double x1, double x2)
	{
		if(DerivOpt == 0)
		{
			/* Numerical Derivatives */
			double xCurr = Half * (x1 + x2);
			for(int Iter=1; Iter<=MaxIter; Iter++)
			{
				double fCurr,dfCurr;
				Deriv1(Func,xCurr,fCurr,dfCurr,DerivStep,DerivType);
				double dx = Lambda * fCurr / dfCurr;
				
				if(PrintLevel >= 2) Print("Iter",Iter,"x",xCurr,"f",fCurr,"dx",dx);
				
				xCurr -= dx;
				if((x1 - xCurr) * (xCurr - x2) < Zero)
				{
					Print("Error in Solve1D::Newton()");
					Print("Jumped Out of Brackets");
					return 3;
				}
				if(fabs(dx) < TolX)
				{
					x = xCurr;
					f = fCurr;
					this->Iter = Iter;
					return 0;
				}
			}
			Print("Unsuccesfully Exiting Solve1D::Newton(): Too Many Iterations");
			return 2;
		}
		else
		{
			/* Analytical Derivatives */
			double xCurr = Half * (x1 + x2);
			for(int Iter=1; Iter<=MaxIter; Iter++)
			{
				double fCurr,dfCurr;
				DFunc(xCurr,fCurr,dfCurr);
				double dx = fCurr / dfCurr;
				
				if(PrintLevel >= 2) Print("Iter",Iter,"x",xCurr,"f",fCurr,"dx",dx);
				
				xCurr -= Lambda * dx;
				if((x1 - xCurr) * (xCurr - x2) < Zero)
				{
					Print("Error in Solve1D::Newton()");
					Print("Jumped Out of Brackets");
					return 3;
				}
				if(fabs(dx) < TolX)
				{
					x = xCurr;
					f = fCurr;
					this->Iter = Iter;
					return 0;
				}
			}
			Print("Unsuccesfully Exiting Solve1D::Newton(): Too Many Iterations");
			return 2;
		}
	}
	
	/* False Position Method */
	int SolveFalsePosition(double x1, double x2)
	{

		double del,f,rtf;
		
		double fl = Func(x1);
		double fh = Func(x2);
		if(fl * fh > Zero)
		{
			Print("Error in SolveFalsePosition");
			Print("Root must be bracketed");
			HaltExecution();
		}
		
		double xl,xh;
		if(fl < Zero)
		{
			xl = x1;
			xh = x2;
		}
		else
		{
			xl = x2;
			xh = x1;
			Swap(fl,fh);
		}
	
		double dx = xh - xl;
		for(int Iter=1; Iter<=MaxIter; Iter++)
		{
			rtf = xl + dx * fl / (fl-fh);
			f = Func(rtf);
			
			if(PrintLevel >= 2) Print("Iter",Iter,"x",rtf,"f",f);
			
			if(f < Zero)
			{
				del = xl - rtf;
				xl = rtf;
				fl = f;
			}
			else
			{
				del = xh - rtf;
				xh = rtf;
				fh = f;
			}
			dx = xh - xl;
			if(fabs(del) < TolX || f == Zero)
			{
				x = rtf;
				this->Iter = Iter;
				return 0;
			}
		}
		
		Print("Error in Solve1D::FalsePosition: Too Many Iterations");
		return 2;
	}

};		
		
#endif
