
#ifndef _Opt_SolveNewton2
#define _Opt_SolveNewton2


/*

Class for solving nonlinear equations using Newton's method
or Broydens method with line search

Parameters:

Func:		Vector-valued funciton to be solved
dFunc:		Derivative of the funciton
x:			Initial value
TolX:		Tolerance in x
TolF:		Tolerance in vfunc
MaxIter:	Maximum number of iterations
DerivStep:	Step size for numerical derivatives
DerivType:	Numerical derivative type
Opt:		Option
				1	-	Newton's method
				2	-	Broyden's method, jac
				3	-	Broyden's method, identity
				4	-	Broyden's method, jac, recompute
				5	-	Broyden's method, identity, recompute
Lambda:		Intial step size
Fac:			Lambda reduction factor
Steps:		Number of steps to take
DiffTol:		Requierd reduction in fucntion value
PrintLevel	Amount of output to print
				0	-	No output
				1	-	No matrices
				2	-	Matrices

*/

template <class MatrixType>
class SolveNewton
{

	public:

	/* Main Variables */

	int n;							// Number of Variables
	double g;						// Objective Value for Line Search
	Vector<double> x;				// Point to Evaluate Function
	Vector<double> f;				// Function Value
	MatrixType df;		// Jacobian of objectivbe function
	MatrixType dfInv;	// Inverse of jacobian
	double gPrev;					// Previous Objective Value for Line Search
	Vector<double> xPrev;			// Previous Point
	Vector<double> fPrev;			// Previous Function Value
	double gDiff;					// Decrease after line search

	/* Pointers to Functions */

	int (*Func1)(const Vector<double> &, Vector<double> &);
	//int (*Func2)(const Vector<double> &, Vector<double> &, MatrixType &);
	int (*Func2)(const Vector<double> &, Vector<double> &, GeneralMatrix<double> &);

	/* Tolerances */

	int MaxIter;
	double TolX;
	double TolF;
	double TolDiff;
	double xMax,fMax;

	/* Numerical Derivative Parameters */

	double DerivStep;
	int DerivType;

	/* Line Search Parameters */

	double RhoCurr;
	double RhoFac;
	double MinStep;

	/* Level of Output to Print */

	int PrintLevel;
	int DerivPrintLevel;

	/* Count Function Evaluations */

	int FuncEval;
	int DFuncEval;

	/* Scale the Variables */

	double fScale;
	Vector<double> xScale;

	/* Options */

	int Method;
	int EvalOpt;		// 1 = Func1, 2 = Func2
	int NormType;

	/* Auxilary variables */

	int Tst;//,Chk;
	int Iter,Step;
	int Restart;
	double Den;
	Vector<double> p;
	Vector<double> a;
	Vector<double> b;
	Vector<double> TypX;

	/* Constructor for Numerical Derivatives */

	SolveNewton(int (*Func1)(const Vector<double> &, Vector<double> &), int n) : Func1(Func1), n(n)
	{
		x     = Vector<double>(n,Zero);	x.Name     = "SolveNewton::x";
		xPrev = Vector<double>(n);		xPrev.Name = "SolveNewton::xPrev";
		f     = Vector<double>(n);		f.Name     = "SolveNewton::f";
		fPrev = Vector<double>(n);		fPrev.Name = "SolveNewton::fPrev";
		df    = MatrixType(n,n);		df.Name    = "SolveNewton::df";
		dfInv = MatrixType(n,n);		dfInv.Name = "SolveNewton::dfInv";
		p     = Vector<double>(n);		p.Name     = "SolveNewton::p";
		a     = Vector<double>(n);		a.Name     = "SolveNewton::a";
		b     = Vector<double>(n);		b.Name     = "SolveNewton::b";
		TypX  = Vector<double>(n,Zero);	TypX.Name  = "SolveNewton::TypX";

		TolX            = 0.000000000001;
		TolF            = 0.00000001;
		TolDiff         = 0.0;
		MaxIter         = 1000;
		DerivStep       = 0.000001;
		DerivType       = 4;
		RhoFac          = 0.5;
		MinStep         = 1.0e-16;
		PrintLevel      = 1;
		DerivPrintLevel = 0;
		FuncEval        = 0;
		Method          = 1;
		EvalOpt         = 1;
		NormType        = 1;
	}

	/* Constuctor for Analytic Gradient */

	//SolveNewton(int (*Func2)(const Vector<double> &, Vector<double> &, MatrixType &),
	//	Matrix<double>(*dFunc)(const Vector<double> &), int n)
	//	: Func2(Func2), n(n)
	SolveNewton(int (*Func2)(const Vector<double> &, Vector<double> &, GeneralMatrix<double> &),
				int n
			   ) : Func2(Func2), n(n)
	{
		x     = Vector<double>(n,Zero);		x.Name     = "SolveNewton::x";
		xPrev = Vector<double>(n);			xPrev.Name = "SolveNewton::xPrev";
		f     = Vector<double>(n);			f.Name     = "SolveNewton::f";
		fPrev = Vector<double>(n);			fPrev.Name = "SolveNewton::fPrev";
		df    = MatrixType<double>(n,n);	df.Name    = "SolveNewton::df";
		dfInv = MatrixType<double>(n,n);	dfInv.Name = "SolveNewton::dfInv";
		p     = Vector<double>(n);			p.Name     = "SolveNewton::p";
		a     = Vector<double>(n);			a.Name     = "SolveNewton::a";
		b     = Vector<double>(n);			b.Name     = "SolveNewton::b";
		TypX  = Vector<double>(n,Zero);		TypX.Name  = "SolveNewton::TypX";

		TolX            = 0.000000000001;
		TolF            = 0.00000001;
		TolDiff         = 0.0;
		MaxIter         = 1000;
		DerivStep		= 0.000001;
		DerivType 	    = 4;
		RhoFac          = 0.5;
		MinStep         = 1.0e-16;
		PrintLevel      = 1;
		DerivPrintLevel = 0;
		FuncEval        = 0;
		DFuncEval       = 0;
		Method          = 1;
		EvalOpt         = 2;
		NormType        = 1;
	}

	/*

	Constructor 3: Use finite differences for gradient, but matrix has special strcuture

	*/

	/*
	SolveNewton(int (*Func1)(const Vector<double> &, Vector<double> &), int n) : Func1(Func1), n(n)
	{
			x     = Vector<double>(n,Zero);	x.Name     = "SolveNewton::x";
			xPrev = Vector<double>(n);		xPrev.Name = "SolveNewton::xPrev";
			f     = Vector<double>(n);		f.Name     = "SolveNewton::f";
			fPrev = Vector<double>(n);		fPrev.Name = "SolveNewton::fPrev";
			df    = MatrixType(n,n);		df.Name    = "SolveNewton::df";
			dfInv = MatrixType(n,n);		dfInv.Name = "SolveNewton::dfInv";
			p     = Vector<double>(n);		p.Name     = "SolveNewton::p";
			a     = Vector<double>(n);		a.Name     = "SolveNewton::a";
			b     = Vector<double>(n);		b.Name     = "SolveNewton::b";
			TypX  = Vector<double>(n,Zero);	TypX.Name  = "SolveNewton::TypX";

			TolX            = 0.000000000001;
			TolF            = 0.00000001;
			TolDiff         = 0.0;
			MaxIter         = 1000;
			DerivStep       = 0.000001;
			DerivType       = 4;
			RhoFac          = 0.5;
			MinStep         = 1.0e-16;
			PrintLevel      = 1;
			DerivPrintLevel = 0;
			FuncEval        = 0;
			Method          = 1;
			EvalOption      = 0;
			NormType        = 1;
	}
	*/

	/* Compute the Objective Function and the Jacobian */

	void ComputeJacobian()
	{
		if(Method == 3 && Restart)
		{
			/* Set Jacobian to Identity */

			if(PrintLevel >= 1) Print("Setting Jacobian to Identity Matrix");

			/* Evaluate Function */

			if(EvalOpt == 1)
			{
				int Info = Func1(x,f);
				FuncEval++;
			}
			else
			{
				int Info = Func2(x,f,df);
				FuncEval++;
			}

			/* Evaluate Norm */

			     if(NormType == 1) g = NormL1(f);
			else if(NormType == 2) g = NormL2(f);
			else                   g = NormLInf(f);

			/* Inialize Jacobian to Identity */

			for(int i=0; i<n; i++) for(int j=0; j<n; j++) dfInv(i,j) = One * (i == j);
			for(int i=0; i<n; i++) p(i) = -f(i);

		}
		else if(Method == 1)
		{

			/* Using Newton's Method */

			Print("Using Newton's Method");

			/* Evaluate Function */

			if(EvalOpt == 1)
			{
				
				/* Compute Jacobian using Finite Differences */
					
				if(PrintLevel >= 1) Print("Computing Jacobian Using Finite Differences");
				
				int Info = Jacobian(Func1,x,f,df,DerivStep,DerivType,TypX);
				if(DerivType == 3 || DerivType == 6) FuncEval += (2 * n + 1);
				else                                 FuncEval += (n + 1);
			}
			else
			{
				int Info = Func2(x,f,df);
			}

			/* Evaluate Norm */

			     if(NormType == 1) g = NormL1(f);
			else if(NormType == 2) g = NormL2(f);
			else                   g = NormLInf(f);
		
			/*
			
			Solve the system,

			df * p = -f

			for p, taking advantage of the special structure of df

			*/

			LinSolve(p,df,f);
			p *= -One;
		}
		else if(Method == 2 && Restart)
		{
			if(EvalOpt == 1)
			{
				/* Compute Jacobian using Finite Differences */

				if(PrintLevel >= 1) Print("Computing Jacobian Using Finite Differences");
				int Info = Jacobian(Func1,x,f,df,DerivStep,DerivType,TypX);
				if(DerivType == 3 || DerivType == 6) FuncEval += (2 * n + 1);
				else                                 FuncEval += (n + 1);
			}
			else
			{
				/* Comoute Analytical Jacobian */				
			
				if(PrintLevel >= 1) Print("Computing Analytical Jacobian");
				int Info = Func2(x,f,df);
				FuncEval++;
			}
			
			/* Evaluate Norm */

			     if(NormType == 1) g = NormL1(f);
			else if(NormType == 2) g = NormL2(f);
			else                   g = NormLInf(f); 

			
			Copy(dfInv,df);
			int Info = Invert(dfInv);
			if(Info)
			{
				Print("Error in SolveNewton");
				Print("Singular Jacobian");
				if(PrintLevel >= 3) Print("df",df);
				if(PrintLevel >= 3) Print("dfInv",dfInv);
				HaltExecution();
			}

			for(int i=0; i<n; i++)
			{
				p[i] = Zero;
				for(int j=0; j<n; j++) p[i] -= dfInv(i,j) * f[j];
			}
		}
		else
		{

			/* Update Jacobian using Broyden's Method */

			if(PrintLevel >= 1) Print("Updating Jacobian Using Broyden's Method");

			/* Evaluate Function */

			if(EvalOpt == 1)
			{
				int Info = Func1(x,f);
				FuncEval++;
			}
			else
			{
				int Info = Func2(x,f,df);
				FuncEval++;
			}
					

			/* Evaluate Norm */

			     if(NormType == 1) g = NormL1(f);
			else if(NormType == 2) g = NormL2(f);
			else                   g = NormLInf(f);

			/* Broyden's Update */

			Den = Zero;
			for(int i=0; i<n; i++)
			{
				a[i] = x[i] - xPrev[i];
				b[i] = f[i] - fPrev[i];
				for(int j=0; j<n; j++) Den += (x[i] - xPrev[i]) * dfInv(i,j) * (f[j] - fPrev[j]);
				for(int j=0; j<n; j++)
				{
					a[i] += dfInv(i,j) * (f[j] - fPrev[j]);
					b[i] += dfInv(j,i) * (x[j] - xPrev[j]);
				}
			}
			for(int i=0; i<n; i++) for(int j=0; j<n; j++) dfInv(i,j) += a[i] * b[j] / Den;

			for(int i=0; i<n; i++)
			{
				p[i] = Zero;
				for(int j=0; j<n; j++) p[i] -= dfInv(i,j) * f[j];
			}
		}
	}

	/* Preform a Line Search */

	void LineSearch()
	{

		if(PrintLevel >= 1) Print("gPrev",gPrev);
		
		// Temp Code
		Print("p",p);
		
		for(int i=0; i<n; i++) x[i] += p[i];

		/* Evaluate Function */

		if(EvalOpt == 1) Func1(x,f);
		else             Func2(x,f,df);
		FuncEval++;

		/* Evaluate Norm */

		     if(NormType == 1) g = NormL1(f);
		else if(NormType == 2) g = NormL2(f);
		else                   g = NormLInf(f);


		if(PrintLevel >= 2) Print("Step",0,"gCurr",g,"RhoCurr",RhoCurr);

		Step = 1;
		RhoCurr = One;
		xMax = RhoCurr * NormLInf(p);
		while(!(g < gPrev) && xMax >= MinStep)
		{
			for(int i=0; i<n; i++) x[i] -= RhoCurr * p[i];
			RhoCurr *= RhoFac;
			for(int i=0; i<n; i++) x[i] += RhoCurr * p[i];

			/* Evaluate Function */

			if(EvalOpt == 1) Func1(x,f);
			else             Func2(x,f,df);
			FuncEval++;

			/* Evaluate Norm */

			     if(NormType == 1) g = NormL1(f);
			else if(NormType == 2) g = NormL2(f);
			else                   g = NormLInf(f);

			if(PrintLevel >= 2) Print("Step",Step,"gCurr",g,"RhoCurr",RhoCurr);
			xMax = RhoCurr * NormLInf(p);
			Step++;
		}
		if(PrintLevel >= 1) Print("Step",Step);
		if(PrintLevel >= 1) Print("gNew",g);
	}

	int TestConverge()
	{
		gDiff = g - gPrev;
		fMax = NormLInf(f);
		xMax = RhoCurr * NormLInf(p);

		if(PrintLevel >= 1) Print("RhoCurr",RhoCurr);
		if(PrintLevel >= 1) Print("gDiff",gDiff);
		if(PrintLevel >= 1) Print("fMax",fMax);
		if(PrintLevel >= 1) Print("xMax",xMax);

		if(!(gDiff < TolDiff))
		{
			if(Restart == 1)
			{
				if(PrintLevel >= 1) Print("Unsuccessfully Exiting SolveNewton: Insufficient Decrease");
				if(PrintLevel >= 1) Print("Number of Iterations",Iter);
				if(PrintLevel >= 1) Print("Number of Function Evaluations",FuncEval);
				Copy(x,xPrev);
				return 1;
			}
			else
			{
				
				/* Try Resarting */
				
				if(PrintLevel >= 1) Print("Insufficient Decrease: Recomputing Jacobian");
				Copy(x,xPrev);
				Restart = 1;
			}
		}
		else if(xMax < TolX)
		{
			if(Restart == 1)
			{
				if(PrintLevel >= 1) Print("Unsucessfully Exiting SolveNewton: Insufficient Change in X");
				if(PrintLevel >= 1) Print("Number of Iterations",Iter);
				if(PrintLevel >= 1) Print("Number of Function Evaluations",FuncEval);
				return 2;
			}
			else
			{

				/* Try Resarting */

				if(PrintLevel >= 1) Print("Insufficient Change in X: Recomputing Hessian");
				Restart = 1;
				Copy(x,xPrev);
			}
		}
		else Restart = 0;

		if(fMax < TolF)
		{
			if(PrintLevel >= 1) Print("Succesfully Exiting SolveNewton: F within tolerance");
			if(PrintLevel >= 1) Print("Number of Iterations",Iter);
			if(PrintLevel >= 1) Print("Number of Function Evaluations",FuncEval);
			return 0;
		}

		return -1;
	}

	int Solve()
	{
		if(PrintLevel >= 1)
		{
			Print("Calling SolveNewton");
			Print("Method",Method);
			Print("EvalOpt",EvalOpt);
			Print("NormType",NormType);
		}

		Restart = 1;

		for(Iter=1; Iter<=MaxIter; Iter++)
		{

			/* Evaluate New Point */

			if(PrintLevel >= 1) Print("Iter",Iter);
			ComputeJacobian();
			if(PrintLevel >= 1) Print("g",g);
			if(PrintLevel >= 1) Print("x",x);
			if(PrintLevel >= 1) Print("f",f);
			if(PrintLevel >= 3) Print("df",df);
			if(PrintLevel >= 3) Print("dfInv",dfInv);

			/* Save Information */

			gPrev = g;
			Copy(fPrev,f);
			Copy(xPrev,x);

			/* Line search */

			LineSearch();

			/* Test for Convergence */

			Tst = TestConverge();
			if(Tst >= 0) return Tst;

			if(PrintLevel >= 1) Print("");

		}

		if(PrintLevel >= 1) Print("Unsucessfully Exiting SolveNewton: Too Many Iterations");
		if(PrintLevel >= 1) Print("Number of Iterations",MaxIter);
		if(PrintLevel >= 1) Print("Number of Function Evaluations",FuncEval);
		return 6;
	}
};

#endif
