
#ifndef _GenMat_SymmetricMatrix
#define _GenMat_SymmetricMatrix

/*

Templated numerical array resenting a symmetric
matrix. Storage corresponds to C (row-major) ordering.
Elements are accessed via A(i,j) notation.

Array assignment is by reference (i.e. shallow assignment).
That is, B = A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.Copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.

This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.

Adapted from the Template Numerical Toolkit (TNT)

*/

template <class T>
class SymmetricMatrix : public GeneralAssignableMatrix<T>
{
	public:

	int n;				// Dimension of matrix
  	int len;			// Size of underlying array
    int *RefCount;		// Check for uneeded arrays
	T*  p; 				// Pointer to underlying array

	void Initialize();
    void Destroy();

	public:

  	string Name;		// Name of the SymmetricMatrix

	       SymmetricMatrix();
	       SymmetricMatrix(int n);
	       SymmetricMatrix(int n, const T &a);
	       SymmetricMatrix(int n, T *a);
	       SymmetricMatrix(int n, const T *a);
	inline SymmetricMatrix(const SymmetricMatrix &A);
	inline SymmetricMatrix & operator=(const T &a);
	inline SymmetricMatrix & operator=(const SymmetricMatrix &A);
	inline SymmetricMatrix & Ref(const SymmetricMatrix &A);
	       SymmetricMatrix Copy() const;
		   SymmetricMatrix & Inject(const SymmetricMatrix &A);
	inline T& operator[](int i);
	inline const T& operator[](int i) const;
	inline T& operator()(int i, int j);
	inline const T& operator()(int i, int j) const;
	inline int Type() const;
	inline int Dim() const;
	inline int Rows() const;
	inline int Cols() const;
	inline int Length() const;
	inline double MemSize() const;
	inline T* Begin();
	inline const T* Begin() const;
	inline T* End();
	const inline T* End() const;
	inline const Vector<int> Size() const;
	inline int GetRefCount() const;
               ~SymmetricMatrix();

	inline void operator += (const T &c);
    inline void operator -= (const T &c);
    inline void operator *= (const T &c);
    inline void operator /= (const T &c);

    inline void operator += (const SymmetricMatrix<T> &A);
    inline void operator -= (const SymmetricMatrix<T> &A);
    inline void operator *= (const SymmetricMatrix<T> &A);
    inline void operator /= (const SymmetricMatrix<T> &A);
};


/* Public Funcitons */

/*

Copy constructor

Array data is NOT copied, but shared.
Thus, in Matrix B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
Matrix B(A.copy()), or B=A.copy(), instead.

*/

template <class T>
SymmetricMatrix<T>::SymmetricMatrix(const SymmetricMatrix<T> &A) : p(A.p), n(A.n), len(A.len),
	RefCount(A.RefCount), Name(A.Name)
{
	(*RefCount)++;
}

/*

Constructor 1:

Create a new (n x n) SymMatrix object, WITHOUT initializing
array elements. This version avoids the O(n^2) initialization
overhead and is used just before manual assignment.

@param n:	The dimension of the new SymMatrix.

*/

template <class T>
SymmetricMatrix<T>::SymmetricMatrix(int n) : p(0), n(n), RefCount(0)
{
	len = n * (n + 1) / 2;
	if(n > 0) Initialize();
	Name = "NoName";
	RefCount = new int;
	*RefCount = 1;
}

/*

Constructor 2:

Create a new (m x n) array, initializing array elements to
constant specified by argument.

@param n:	The dimension of the new SymMatrix.
@param val:	The constant value to set all elements of the new array to.

*/

template <class T>
SymmetricMatrix<T>::SymmetricMatrix(int n, const T &a) : p(0), n(n), RefCount(0)
{
	len = n * (n + 1) / 2;
	if(n > 0) Initialize();
	BlasSet(len,a,p,1);
	Name = "NoName";
	RefCount = new int;
	*RefCount = 1;
}

/*

Constructor 3:

Create a new (m x n) array,  as a view of an existing one-dimensional
array stored in <b>C order</b>, i.e. right-most dimension varying fastest.
(Often referred to as "row-major" ordering.)
Note that the storage for this pre-existing array will
never be garbage collected by the Array2D class.

@param n the dimension of the new SymMatrix.
@param a the one dimensional C array to use as data storage for the array.

*/

template <class T>
SymmetricMatrix<T>::SymmetricMatrix(int n, T *a) : p(0), n(n), RefCount(0)
{
	len = n * (n + 1) / 2;
	p = a;
	Name = "NoName";
	RefCount = new int;
	*RefCount = 2;		// This avoid destorying original data
}

template <class T>
SymmetricMatrix<T>::SymmetricMatrix(int n, const T *a) : p(a), n(n), RefCount(0)
{
	len = n * (n + 1) / 2;
	//p_ = a;
	Name = "NoName";
	RefCount_ = new int;
	*RefCount_ = 2;		// This avoid destorying original data
}

/* Return pointer to an element of the underlying array of the SymmetricMatrix */
template <class T>
inline T& SymmetricMatrix<T>::operator[](int i)
{
	return p[i];
}

/* Return pointer to an element of the underlying array of the SymmetricMatrix */
template <class T>
inline const T& SymmetricMatrix<T>::operator[](int i) const
{
	return p[i];
}

/* Access elements of the Matrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline T& SymmetricMatrix<T>::operator()(int i, int j)
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n || j < 0 || j >= n)
		{
			Print("Error in SymmetricMatrix");
			Print("Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("n",n);
			HaltExecution();
		}
	#endif

	if(i >= j) return p[i*(i+1)/2+j];
	else       return p[j*(j+1)/2+i];
}

/* Access elements of the SymmetricMatrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline const T& SymmetricMatrix<T>::operator()(int i, int j) const
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n || j < 0 || j >= n)
		{
			Print("Error in SymmetricMatrix");
			Print("Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("n",n);
			HaltExecution();
		}
	#endif

	if(i >= j) return p[i*(i+1)/2+j];
	else       return p[j*(j+1)/2+i];
}

/* Assigns all elemnts of A to a constant scalar */
template <class T>
SymmetricMatrix<T> & SymmetricMatrix<T>::operator=(const T &a)
{
	BlasSet(len,a,p,1);
	return *this;
}

/*

Create a new copy of existing Matrix.
Used in B = A.copy() or in the construction
of B, e.g. SymmetricMatrix B(A.Copy()), to create a
new array that does not share data.

*/

template <class T>
SymmetricMatrix<T> SymmetricMatrix<T>::Copy() const
{
	SymmetricMatrix A(n);
	BlasCopy(len,p,1,A.p,1);
	return A;
}

/*

Copy the elements to from one array to another, in place.
That is B.Inject(A), both A and B must conform (i.e. have
identical row and column dimensions).

This differs from B = A.Copy() in that references to B
before this assignment are also affected.  That is, if
we have

<pre>
Matrix A(m,n);
Matrix C(m,n);
Matrix B(C);        // elements of B and C are shared.

<pre>
then B.inject(A) affects both and C, while B = A.copy() creates
a new array B which shares no data with C or A.

@param A the Matrix from elements will be copied
@return an instance of the modifed array. That is, in B.inject(A),
it returns B.  If A and B are not conformat, no modifications to
B are made.

*/

template <class T>
SymmetricMatrix<T> & SymmetricMatrix<T>::Inject(const SymmetricMatrix<T> &A)
{
	if(A.n == n)
	BlasCopy(len,A.v,1,v,1);
	return *this;
}

/*

Create a reference (shallow assignment) to another existing array.
In B.ref(A), B and A shared the same data and subsequent changes
to the array elements of one will be reflected in the other.

<p>
This is what operator= calls, and B = A and B.ref(A) are equivalent
operations.

@return The new referenced array: in B.ref(A), it returns B.

*/

template <class T>
SymmetricMatrix<T> & SymmetricMatrix<T>::Ref(const SymmetricMatrix<T> &A)
{
	if(this != &A)
	{
		(*RefCount)--;
		if(*RefCount < 1) Destroy();
		n = A.n;
		len = A.len;
		p = A.p;
		Name = A.Name;
		RefCount = A.RefCount;
		(*RefCount)++ ;
	}
	return *this;
}

/* B = A is shorthand notation for B.Ref(A) */
template <class T>
SymmetricMatrix<T> & SymmetricMatrix<T>::operator=(const SymmetricMatrix<T> &A)
{
	return Ref(A);
}

/* Return the type of the SymmetricMatrix */
template <class T>
inline int SymmetricMatrix<T>::Type() const
{
	return MatrixType::SymmetricMatrix;
}

/* Return the number of rows in the SymmetricMatrix */
template <class T>
inline int SymmetricMatrix<T>::Dim() const
{
	return n;
}

/* Return the number of rows in the SymmetricMatrix */
template <class T>
inline int SymmetricMatrix<T>::Rows() const
{
	return n;
}

/* Return the number of columns in the SymmetricMatrix */
template <class T>
inline int SymmetricMatrix<T>::Cols() const
{
	return n;
}

/* Return the length of the underlying array */
template <class T>
inline int SymmetricMatrix<T>::Length() const
{
	return len;
}

/* Return the Size of a Matrix in a 2-element Vector */
template <class T>
inline const Vector<int> SymmetricMatrix<T>::Size() const
{
	Vector<int> sz(2);
	sz[0] = n;
	sz[1] = n;
	return sz;
}

/* Return the Number of Megabytes Used to Store a Metrix */
template <class T>
inline double SymmetricMatrix<T>::MemSize() const
{
	return len * sizeof(T) / 1e6;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* SymmetricMatrix<T>::Begin()
{
	return &p[0];
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
inline const T* SymmetricMatrix<T>::Begin() const
{
	return &p[0];
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* SymmetricMatrix<T>::End()
{
	return &p[0] + len;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
const T* SymmetricMatrix<T>::End() const
{
	return &p[0] + len;
}

/*

@return the number of arrays that share the same storage area
as this one.  (Must be at least one.)

*/

template <class T>
inline int SymmetricMatrix<T>::GetRefCount() const
{
	return *RefCount;
}

/* Destructor */
template <class T>
SymmetricMatrix<T>::~SymmetricMatrix()
{
	(*RefCount)--;
	if(*RefCount < 1) Destroy();
}

/* Private internal functions */

/* Allocate memory for the Matrix */
template <class T>
void SymmetricMatrix<T>::Initialize()
{
	try
	{
		p = new T[len];
	}
	catch(bad_alloc ba)
	{
		Print("Error in SymmetricMatrix");
		Print("Unable to Allocate Memory");
		Print("len",len);
		Print("Memory per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",len*sizeof(T)/2e6);
		HaltExecution();
	}
}

/* Delete the Matrix from memory */
template <class T>
void SymmetricMatrix<T>::Destroy()
{
	if(p != 0) delete[] p;
	if(RefCount != 0) delete RefCount;
}

/* Create a null (0x0) array */
template <class T>
SymmetricMatrix<T>::SymmetricMatrix() : p(0), n(0), len(0), Name("NoName")
{
	RefCount = new int;
	*RefCount = 1;
}

template <class T>
inline void SymmetricMatrix<T>::operator += (const T &c)
{
	for(int i=0; i<len; i++) p[i] += c;
}

template <class T>
inline void SymmetricMatrix<T>::operator -= (const T &c)
{
	for(int i=0; i<len; i++) p[i] -= c;
}

template <class T>
inline void SymmetricMatrix<T>::operator *= (const T &c)
{
	for(int i=0; i<len; i++) p[i] *= c;
}

template <class T>
inline void SymmetricMatrix<T>::operator /= (const T &c)
{
	for(int i=0; i<len; i++) p[i] /= c;
}

template <class T>
inline void SymmetricMatrix<T>::operator += (const SymmetricMatrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	if(n != A.n)
	{
		Print("Error in  SymmetricMatrix += SymmetrixMatrix");
		Print("Incompadible Dimensions");
		Print("n",n);
		Print("A.n",A.n);
		HaltExecution();
	}

	for(int i=0; i<len; i++) p[i] += A.p[i];
}

template <class T>
inline void SymmetricMatrix<T>::operator -= (const SymmetricMatrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	if(n != A.n)
	{
		Print("Error in  SymmetricMatrix -= SymmetrixMatrix");
		Print("Incompadible Dimensions");
		Print("n",n);
		Print("A.n",A.n);
		HaltExecution();
	}

	for(int i=0; i<len; i++) p[i] -= A.p[i];
}

template <class T>
inline void SymmetricMatrix<T>::operator *= (const SymmetricMatrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	if(n != A.n)
	{
		Print("Error in  SymmetricMatrix *= SymmetrixMatrix");
		Print("Incompadible Dimensions");
		Print("n_",n_);
		Print("A.n_",A.n_);
		HaltExecution();
	}

	for(int i=0; i<len; i++) p[i] *= A.p[i];
}

template <class T>
inline void SymmetricMatrix<T>::operator /= (const SymmetricMatrix<T> &A)
{
	/* Check for Incompadible Dimensions */
	if(n != A.n)
	{
		Print("Error in  SymmetricMatrix /= SymmetrixMatrix");
		Print("Incompadible Dimensions");
		Print("n",n);
		Print("A.n",A.n);
		HaltExecution();
	}

	for(int i=0; i<len; i++) p[i] /= A.p[i];
}

#endif
