
#ifndef _GenMat_SymmetricMatrixLU
#define _GenMat_SymmetricMatrixLU

/*

Symmetric matrix stored in lower-unpacked storage.

Templated numerical array resenting a symmetric
matrix. Storage corresponds to C (row-major) ordering.
Elements are accessed via A(i,j) notation.

Array assignment is by reference (i.e. shallow assignment).
That is, B = A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.Copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.

This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.

Adapted from the Template Numerical Toolkit (TNT)

*/

template <class T>
class SymmetricMatrixLU : public GeneralMatrix<T>
{
	private:

	T*  p_;
    T** v_;				// Array of pointers to the start of each vector
	int n_;				// Columns
  	int len_;
    int *RefCount_;		// Check for uneeded arrays

	void Initialize_(int n);
    void CopyTri_(T* p, const T*  q, int len) const;
    //void Set_(const T& val);
    void Destroy_();
	inline const T* Begin_() const;
	inline T* Begin_();

	public:

  	string Name;		// Name of the SymmetricMatrix

    typedef T value_type;

	       SymmetricMatrixLU();
	       SymmetricMatrixLU(int n);
	       SymmetricMatrixLU(int n,  T *a);
	       SymmetricMatrixLU(int n, const T &a);
	inline SymmetricMatrixLU(const SymmetricMatrixLU &A);
	inline SymmetricMatrixLU & operator=(const T &a);
	inline SymmetricMatrixLU & operator=(const SymmetricMatrixLU &A);
	inline SymmetricMatrixLU & Ref(const SymmetricMatrixLU &A);
	       SymmetricMatrixLU Copy() const;
		   SymmetricMatrixLU & Inject(const SymmetricMatrixLU &A);
	inline T& operator[](int i);
	inline const T& operator[](int i) const;
	//inline T* operator()(int i); /* Pointer Operator */
	inline T& operator()(int i, int j);
	inline const T& operator()(int i, int j) const;
	inline int Type() const;
	inline int Dim() const;
	inline int Rows() const;
	inline int Cols() const;
	inline int Length() const;
	inline T* Begin();
	inline const T* Begin() const;
	inline T* End();
	inline const T* End() const;
	inline const Vector<int> Size() const;
	inline int RefCount() const;
               ~SymmetricMatrixLU();
};


/* Public Funcitons */

/*

Copy constructor

Array data is NOT copied, but shared.
Thus, in Matrix B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
Matrix B(A.copy()), or B=A.copy(), instead.

*/

template <class T>
SymmetricMatrixLU<T>::SymmetricMatrixLU(const SymmetricMatrixLU<T> &A) : p_(A.p_), v_(A.v_), n_(A.n_),
	RefCount_(A.RefCount_), Name(A.Name)
{
	len_ = Sqr(n_);
	(*RefCount_)++;
}

/*

Constructor 1:

Create a new (n x n) SymMatrix object, WITHOUT initializing
array elements. This version avoids the O(n^2) initialization
overhead and is used just before manual assignment.

@param n:	The dimension of the new SymMatrix.

*/

template <class T>
SymmetricMatrixLU<T>::SymmetricMatrixLU(int n) : p_(0), v_(0), n_(n), RefCount_(0)
{
	len_ = Sqr(n_);
	if(n > 0) Initialize_(n);
	Name = "NoName";
	RefCount_ = new int;
	*RefCount_ = 1;
}

/*

Constructor 2:

Create a new (m x n) array, initializing array elements to
constant specified by argument.

@param n:	The dimension of the new SymMatrix.
@param val:	The constant value to set all elements of the new array to.

*/

template <class T>
SymmetricMatrixLU<T>::SymmetricMatrixLU(int n, const T &a) : p_(0), v_(0), n_(n), RefCount_(0)
{
	len_ = Sqr(n_);
	if(n > 0) Initialize_(n);
	BlasSetTri(0,n,a,p_);
	Name = "NoName";
	RefCount_ = new int;
	*RefCount_ = 1;
}

/*

Constructor 3:

Create a new (m x n) array,  as a view of an existing one-dimensional
array stored in <b>C order</b>, i.e. right-most dimension varying fastest.
(Often referred to as "row-major" ordering.)
Note that the storage for this pre-existing array will
never be garbage collected by the SymmetricMatrixLU class.

@param n the dimension of the new SymMatrix.
@param a the one dimensional C array to use as data storage for the array.

*/

template <class T>
SymmetricMatrixLU<T>::SymmetricMatrixLU(int n, T *a) : v_(0), p_(0), n_(n), RefCount_(0), Name("NoName")
{
	len_ = Sqr(n_);
	p_ = a;
	v_ = new T*[n];
	for(int i=0; i<n; i++)
	{
		v_[i] = p_;
		p_ += n;
	}
	p_ = v_[0];
	RefCount_ = new int;
	*RefCount_ = 2;		// This avoid destorying original data
}

/* Return pointer to an element of the underlying array of the SymmetricMatrixLU */
template <class T>
inline T& SymmetricMatrixLU<T>::operator[](int i)
{
	return p_[i];
}

/* Return pointer to an element of the underlying array of the SymmetricMatrixLU */
template <class T>
inline const T& SymmetricMatrixLU<T>::operator[](int i) const
{
	return p_[i];
}

/* Access elements of the SymmetricMatrixLU. Use _BoundsCheck for bounds-checking */
template <class T>
inline T& SymmetricMatrixLU<T>::operator()(int i, int j)
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n_ || j < 0 || j >= n_)
		{
			Print("Error in SymmetricMatrixLU(int,int)");
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("n",n_);
			HaltExecution();
		}
	#endif

	if(i >= j) return v_[i][j];
	else       return v_[j][i];
}

/* Access elements of the SymmetricMatrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline const T& SymmetricMatrixLU<T>::operator()(int i, int j) const
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n_ || j < 0 || j >= n_)
		{
			Print("Error in SymmetricMatrixLU(int,int)");
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("n",n_);
			HaltExecution();
		}
	#endif

	if(i >= j) return v_[i][j];
	else       return v_[j][i];
}

/* Assigns all elemnts of A to a constant scalar */
template <class T>
SymmetricMatrixLU<T> & SymmetricMatrixLU<T>::operator=(const T &a)
{
	Set_(a);
	return *this;
}

/*

Create a new copy of existing Matrix.
Used in B = A.copy() or in the construction
of B, e.g. Array2D B(A.copy()), to create a
new array that does not share data.

*/

template <class T>
SymmetricMatrixLU<T> SymmetricMatrixLU<T>::Copy() const
{
	SymmetricMatrixLU A(n_);
	BlasCopy(len_,p_,1,A.p_,1);
	return A;
}

/*

Copy the elements to from one array to another, in place.
That is B.Inject(A), both A and B must conform (i.e. have
identical row and column dimensions).

This differs from B = A.Copy() in that references to B
before this assignment are also affected.  That is, if
we have

<pre>
Matrix A(m,n);
Matrix C(m,n);
Matrix B(C);        // elements of B and C are shared.

<pre>
then B.inject(A) affects both and C, while B = A.copy() creates
a new array B which shares no data with C or A.

@param A the Matrix from elements will be copied
@return an instance of the modifed array. That is, in B.inject(A),
it returns B.  If A and B are not conformat, no modifications to
B are made.

*/

/*
template <class T>
SymmetricMatrixLU<T> & SymmetricMatrixLU<T>::Inject(const SymmetricMatrixLU &A)
{
	if(A.n_ == n_)
	CopyTri_(Begin_(),A.Begin_(),len_);
	return *this;
}
*/

/*

Create a reference (shallow assignment) to another existing array.
In B.ref(A), B and A shared the same data and subsequent changes
to the array elements of one will be reflected in the other.

<p>
This is what operator= calls, and B = A and B.ref(A) are equivalent
operations.

@return The new referenced array: in B.ref(A), it returns B.

*/

template <class T>
SymmetricMatrixLU<T> & SymmetricMatrixLU<T>::Ref(const SymmetricMatrixLU<T> &A)
{
	if(this != &A)
	{
		(*RefCount_)--;
		if(*RefCount_ < 1) Destroy_();
		n_ = A.n_;
		len_ = A.len_;
		p_ = A.p_;
		v_ = A.v_;
		Name = A.Name;
		RefCount_ = A.RefCount_;
		(*RefCount_)++ ;
	}
	return *this;
}

/* B = A is shorthand notation for B.Ref(A) */
template <class T>
SymmetricMatrixLU<T> & SymmetricMatrixLU<T>::operator=(const SymmetricMatrixLU<T> &A)
{
	return Ref(A);
}

/* Return the type of the SymmetricMatrix ( = 9 ) */
template <class T>
inline int SymmetricMatrixLU<T>::Type() const
{
	return MatrixType::SymmetricMatrixLU;
}

/* Return the number of rows in the SymmetricMatrix */
template <class T>
inline int SymmetricMatrixLU<T>::Dim() const { return n_; }

/* Return the number of rows in the SymmetricMatrix */
template <class T>
inline int SymmetricMatrixLU<T>::Rows() const { return n_; }

/* Return the number of columns in the SymmetricMatrix */
template <class T>
inline int SymmetricMatrixLU<T>::Cols() const { return n_; }

/* Return the length of the underlying array */
template <class T>
inline int SymmetricMatrixLU<T>::Length() const { return len_; }

/* Return the Size of a SymmetricMatrixLU in a 2-element Vector */
template <class T>
inline const Vector<int> SymmetricMatrixLU<T>::Size() const
{
	Vector<int> sz(2);
	sz[0] = n_;
	sz[1] = n_;
	return sz;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* SymmetricMatrixLU<T>::Begin()
{
	return &p_[0];
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
const T* SymmetricMatrixLU<T>::Begin() const
{
	return &p_[0];
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* SymmetricMatrixLU<T>::End()
{
	return &p_[0] + len_;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
const T* SymmetricMatrixLU<T>::End() const
{
	return &p_[0] + len_;
}

/*

@return the number of arrays that share the same storage area
as this one.  (Must be at least one.)

*/

template <class T>
inline int SymmetricMatrixLU<T>::RefCount() const
{
	return *RefCount_;
}

/* Destructor */
template <class T>
SymmetricMatrixLU<T>::~SymmetricMatrixLU()
{
	(*RefCount_)--;
	if(*RefCount_ < 1) Destroy_();
}

/* Private internal functions */

/* Allocate memory for the Matrix */
template <class T>
void SymmetricMatrixLU<T>::Initialize_(int n)
{
	try
	{
		p_ = new T[n*n];
		v_ = new T*[n];
		for(int i=0; i<n; i++)
		{
			v_[i] = p_;
			p_ += n;
		}
		p_ = v_[0];
		n_ = n;
	}
	catch(bad_alloc ba)
	{
		Print("Error in SymmetricMatrixLU");
		Print("Size of Array",len_);
		Print("Memory Per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",len_*sizeof(T)/2e6);
		HaltExecution();
	}
}

/* Set all the elements of a Matrix equal to a specified constant */
/*
template <class T>
void SymmetricMatrixLU<T>::Set_(const T& a)
{
	for(int i=0; i<n; i++) for(T* q=p_[i]; q<p_[i]+i; q++) *q = a;
}
*/

/* Copy the elements of one Matrix to another Matrix */
/*
template <class T>
void SymmetricMatrixLU<T>::Copy_(T* q, const T* r, int len) const
{
	for(int i=0; i<n; i++)
	{
		q = p_[i];
		while(q < p_[i] + i) *q++ = *r++;
	}
}
*/

/* Delete the Matrix from memory */
template <class T>
void SymmetricMatrixLU<T>::Destroy_()
{
	if(v_ != 0)
	{
		delete[] p_;
		delete[] v_;
	}
	if(RefCount_ != 0) delete RefCount_;
}

/* Return the location of first element of the underlying array. */
template <class T>
T* SymmetricMatrixLU<T>::Begin_()
{
	return &p_[0];
}

/* Return the location of first element of the underlying array. */
template <class T>
const T* SymmetricMatrixLU<T>::Begin_() const
{
	return &p_[0];
}


/* Create a null (0x0) array */
template <class T>
SymmetricMatrixLU<T>::SymmetricMatrixLU() : p_(0), v_(0), n_(0), len_(0), Name("No Name")
{
	RefCount_ = new int;
	*RefCount_ = 1;
}

#endif