
#ifndef _GenMat_UpperMatrix
#define _GenMat_UpperMatrix

/*

Templated numerical array resenting a upper-triangular
matrix. Storage corresponds to C (row-major) ordering.
Elements are accessed via A(i,j) notation.

Array assignment is by reference (i.e. shallow assignment).
That is, B = A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.Copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.

This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.

Adapted from the Template Numerical Toolkit (TNT)

*/

template <class T>
class UpperMatrix : public GeneralMatrix<T>
{
	//private:
	public:
    T*  p_;
    T** v_;					// Array of pointers to the start of each vector
	int n_;					// Dimension
	int len_;				// Length of underlying array
    int *RefCount_;			// Check for uneeded arrays
	double* ZeroPointer;	// Pointer to the value 0.0

	void Initialize_(int n);
    void Copy_(T* p, const T*  q, int Len) const;
    void Set_(const T& val);
    void Destroy_();

	public:
   	string Name;			// Name of the Matrix

    typedef T value_type;

	       UpperMatrix();
	       UpperMatrix(int n);
	       UpperMatrix(int n, T *a);
	       UpperMatrix(int n, const T &a);
	inline UpperMatrix(const UpperMatrix &A);
	inline UpperMatrix & operator=(const T &a);
	inline UpperMatrix & operator=(const UpperMatrix &A);
	inline UpperMatrix & Ref(const UpperMatrix &A);
	       UpperMatrix Copy() const;
	       UpperMatrix Conform() const;
		   UpperMatrix & Inject(const UpperMatrix & A);
	inline T& operator[](int i);
	inline const T& operator[](int i) const;
	inline T* operator()(int i);
	inline T& operator()(int i, int j);
	inline const T& operator()(int i, int j) const;
	inline int Type() const;
	inline int Dim() const;
	inline int Rows() const;
	inline int Cols() const;
	inline int Length() const;
	inline const Vector<int> Size() const;
	inline int GetRefCount() const;
	inline T* Begin();
	inline const T* Begin() const;
	inline T* End();
	inline const T* End() const;
               ~UpperMatrix();
};

/* Public Funcitons */

/*

Copy constructor

Array data is NOT copied, but shared.
Thus, in UpperMatrix B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
UpperMatrix B(A.Copy()), or B = A.Copy(), instead.

*/

template <class T>
UpperMatrix<T>::UpperMatrix(const UpperMatrix<T> &A) : p_(A.p_), v_(A.v_), n_(A.n_), RefCount_(A.RefCount_),
	Name(A.Name)
{
	len_ = (n_ + 1) * n_ / 2;
	(*RefCount_)++;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/*

Constructor 1:

Create a new (n x n) UpperMatrix object, WITHOUT initializing
array elements. This version avoids the O(n^2) initialization
overhead and is used just before manual assignment.

@param n:	The dimension of the new UpperMatrix.

*/

template <class T>
UpperMatrix<T>::UpperMatrix(int n) : p_(0), v_(0), n_(n), RefCount_(0), Name("NoName")
{
	if(n > 0) Initialize_(n);
	len_ = (n_ + 1) * n_ / 2;
	RefCount_ = new int;
	*RefCount_ = 1;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/*

Constructor 2:

Create a new (n x n) UpperMatrix, initializing array elements to
constant specified by argument.

@param n:	The dimension of the new UpperMatrix.
@param Val:	The constant value to set all elements of the new array to.

*/

template <class T>
UpperMatrix<T>::UpperMatrix(int n, const T &Val) : p_(0), v_(0), n_(n), RefCount_(0), Name("NoName")
{
	len_ = (n_ + 1) * n_ / 2;
	if(n > 0) Initialize_(n);
	Set_(Val);
	RefCount_ = new int;
	*RefCount_ = 1;
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/*

Constructor 3:

Create a new (m x n) array,  as a view of an existing one-dimensional
array stored in <b>C order</b>, i.e. right-most dimension varying fastest.
(Often referred to as "row-major" ordering.)
Note that the storage for this pre-existing array will
never be garbage collected by the Array2D class.

@param n:	The dimension of the new UpperMatrix object
@param a:	The one dimensional C array to use as data storage for the UpperMatrix

*/

template <class T>
UpperMatrix<T>::UpperMatrix(int n, T *a) : p_(0), v_(0), n_(n), RefCount_(0), Name("NoName")
{
	len_ = (n_ + 1) * n_ / 2;
	p_ = a;
	v_ = new T*[n];
	for(int i=0; i<n; i++)
	{
		v_[i] = p_;
		p_ += (n - i - 1);
	}
	p_ = v_[0];
	RefCount_ = new int;
	*RefCount_ = 2;		// this avoid destorying original data
	ZeroPointer = new T;
	*ZeroPointer = 0;
}

/* ... */
template <class T>
inline T& UpperMatrix<T>::operator[](int i)
{
	return p_[i];
}

/* ... */
template <class T>
inline const T& UpperMatrix<T>::operator[](int i) const
{
	return p_[i];
}

/* Return pointer to a Row of a Matrix */
template <class T>
inline T* UpperMatrix<T>::operator()(int i)
{
	#ifdef _CheckBounds
		if(i < 0 && i >= n_)
		{
			Print("Error in UpperMatrix");
			Print("Get Row out of bounds");
			Print("Current Index",i );
			Print("High Index",n_-1);
			Print("Halting Execution");
			exit(0);
		}
	#endif

	return v_[i];
}

/* Access elements of the UpperMatrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline T& UpperMatrix<T>::operator()(int i, int j)
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n_ || j < 0 || j >= n_)
		{
			Print("Error in UpperMatrix");
			Print("Index out of bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("n",n_);
			Print("Halting Execution");
			exit(0);
			return v_[0][0];
		}
	#endif

	if(i <= j) return v_[i][j];
	else
	{
		*ZeroPointer = 0;
		return *ZeroPointer;
	}
}

/* Access elements of the UpperMatrix. Use _BoundsCheck for bounds-checking */
template <class T>
inline const T& UpperMatrix<T>::operator()(int i, int j) const
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n_ || j < 0 || j >= n_)
		{
			Print("Error in UpperMatrix");
			Print("Index out of bounds");
			Print("Name",Name);
			Print("i",i);
			Print("j",j);
			Print("n",n_);
			Print("Halting Execution");
			exit(1);
			return v_[0][0];
		}
	#endif

	if(i <= j) return v_[i][j];
	else
	{
		*ZeroPointer = 0;
		return *ZeroPointer;
	}
}

/* Assigns all elemnts of A to a constant scalar */
template <class T>
UpperMatrix<T> & UpperMatrix<T>::operator=(const T &a)
{
	Set_(a);
	return *this;
}

/*

Create a new copy of existing UpperMatrix.
Used in B = A.Copy() or in the construction
of B, e.g. UpperMatrix B(A.Copy()), to create a
new array that does not share data.

*/

template <class T>
UpperMatrix<T> UpperMatrix<T>::Copy() const
{
	UpperMatrix A(n_);
	Copy_(A.Begin(),Begin(),n_*(n_+1)/2);
	return A;
}

/*

Create a new UpperMatrix with same dimensions
as existing UpperMatrix.

*/

template <class T>
UpperMatrix<T> UpperMatrix<T>::Conform() const
{
	UpperMatrix A(n_);
	return A;
}


/*

Copy the elements to from one array to another, in place.
That is B.Inject(A), both A and B must conform (i.e. have
identical row and column dimensions).

This differs from B = A.Copy() in that references to B
before this assignment are also affected.  That is, if
we have

<pre>
Matrix A(m,n);
Matrix C(m,n);
Matrix B(C);        // elements of B and C are shared.

<pre>
then B.inject(A) affects both and C, while B = A.copy() creates
a new array B which shares no data with C or A.

@param A the Matrix from elements will be copied
@return an instance of the modifed array. That is, in B.inject(A),
it returns B.  If A and B are not conformat, no modifications to
B are made.

*/

template <class T>
UpperMatrix<T> & UpperMatrix<T>::Inject(const UpperMatrix<T> &A)
{
	if(A.n_ == n_)
	//Copy_(Begin_(),A.Begin_(),n_*(n_+1)/2);
	BlasCopy(len_,A.v_,1,v_,1);
	return *this;
}

/*

Create a reference (shallow assignment) to another existing array.
In B.ref(A), B and A shared the same data and subsequent changes
to the array elements of one will be reflected in the other.

<p>
This is what operator= calls, and B = A and B.ref(A) are equivalent
operations.

@return The new referenced array: in B.ref(A), it returns B.

*/

template <class T>
UpperMatrix<T> & UpperMatrix<T>::Ref(const UpperMatrix<T> &A)
{
	if(this != &A)
	{
		(*RefCount_)--;
		if(*RefCount_ < 1) Destroy_();
		n_ = A.n_;
		v_ = A.v_;
		Name = A.Name;
		RefCount_ = A.RefCount_;
		ZeroPointer = A.ZeroPointer;
		(*RefCount_)++ ;
	}
	return *this;
}

/* B = A is shorthand notation for B.Ref(A) */
template <class T>
UpperMatrix<T> & UpperMatrix<T>::operator=(const UpperMatrix<T> &A)
{
	return Ref(A);
}

/* Return the Type of the UpperMatrix ( = 5 ) */
template <class T>
inline int UpperMatrix<T>::Type() const
{
	return MatrixType::UpperMatrix;
}

/* @return the number of rows in the Matrix */
template <class T>
inline int UpperMatrix<T>::Dim() const { return n_; }

/* @return the number of rows in the Matrix */
template <class T>
inline int UpperMatrix<T>::Rows() const { return n_; }

/* @return the number of columns in the Matrix */
template <class T>
inline int UpperMatrix<T>::Cols() const { return n_; }

/* Return the Size of a Matrix in a 2-element Vector */
template <class T>
inline const Vector<int> UpperMatrix<T>::Size() const
{
	Vector<int> sz(2);
	sz[0] = n_;
	sz[1] = n_;
	return sz;
}

/* Return the length of the underlying array */
template <class T>
inline int UpperMatrix<T>::Length() const { return len_; }

/*

@return the number of arrays that share the same storage area
as this one.  (Must be at least one.)

*/

template <class T>
inline int UpperMatrix<T>::GetRefCount() const
{
	return *RefCount_;
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* UpperMatrix<T>::Begin()
{
	return &(v_[0][0]);
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
const T* UpperMatrix<T>::Begin() const
{
	return &(v_[0][0]);
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
T* UpperMatrix<T>::End()
{
	return &(v_[0][0]);
}

/* Returns location of first element, i.e. A[0][0] (mutable) */
template <class T>
const T* UpperMatrix<T>::End() const
{
	return &(v_[0][0]);
}

/* Destructor */
template <class T>
UpperMatrix<T>::~UpperMatrix()
{
	(*RefCount_)--;
	if(*RefCount_ < 1) Destroy_();
}

/* Private internal functions */

/* Allocate memory for the Matrix */
template <class T>
void UpperMatrix<T>::Initialize_(int n)
{
	try
	{
		p_ = new T[n*(n+1)/2];
		v_ = new T*[n];
		for(int i=0; i<n; i++)
		{
			v_[i] = p_;
			p_ += (n - i - 1);
		}
		p_ = v_[0];
		n_ = n;
		ZeroPointer = new T;
		*ZeroPointer = 0;
	}
	catch(bad_alloc ba)
	{
		Print("Error in UpperMatrix: Error Allocating Memory");
		Print("Size",n*(n+1));
		Print("Memory per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",n*(n+1)*sizeof(T)/2e6);
		HaltExecution();
	}
}

/* Set all the elements of a Matrix equal to a specified constant */
template <class T>
void UpperMatrix<T>::Set_(const T& a)
{
	T *begin = &v_[0][0];
	T *end = begin + n_ * (n_ + 1) / 2;
	for(T* p=begin; p<end; p++) *p = a;
}

/* Copy the elements of one Matrix to another Matrix */
template <class T>
void UpperMatrix<T>::Copy_(T* p, const T* q, int Len) const
{
	T *End = p + Len;
	while(p < End) *p++ = *q++;
}

/* Delete the Matrix from memory */
template <class T>
void UpperMatrix<T>::Destroy_()
{
	if(v_ != 0)
	{
		delete[] (v_[0]);
		delete[] (v_);
	}
	if(RefCount_ != 0) delete RefCount_;
	if(ZeroPointer != 0) delete ZeroPointer;
}

/* Create a null (0x0) array */
template <class T>
UpperMatrix<T>::UpperMatrix() : p_(0), v_(0), n_(0), Name("Name")
{
	RefCount_ = new int;
	*RefCount_ = 1;
}

#endif
