
#ifndef _Stat_Var
#define _Stat_Var


	// Compute the Sample Variance
	double Var(const GeneralVector<double> &X);
	SymmetricMatrix<double> Var(const Matrix<double> &X);
	SymmetricMatrix<double> Var(const Matrix<double> &X, const Matrix<bool> &XMiss);
	double VarWeighted(const GeneralVector<double> &X, const GeneralVector<double> &W);
	


/* Compute the Sample Variance */
double Var(const GeneralVector<double> &X)
{
	int N = X.Size();
	double Mu = Mean(X);
	
	double Sigma = Zero;
	for(int n=0; n<N; n++) Sigma += Sqr(X[n] - Mu);
	Sigma /= (N - One);
	
	return Sigma;
}

/* Compute the Sample Variance */
DataPoint<double> Var(const GeneralVector< DataPoint<double> > &X)
{
	int N = X.Size();
	DataPoint<double> Mu = Mean(X);
	if(Mu.IsMissing()) return DataPoint<double>();
	
	int N1 = 0;
	DataPoint<double> Sigma = Zero;
	for(int n=0; n<N; n++)
	{
		if(!X(n).IsMissing())
		{
			Sigma += Sqr(X(n).GetData() - Mu.GetData());
			N1++;
		}
	}
	if(N1 < 2) return DataPoint<double>();
	Sigma /= (N1 - One);
	return Sigma;
}

/* Compute the Weighted Sample Variance */
double VarWeighted(const GeneralVector<double> &X, const GeneralVector<double> &W)
{
	int N = X.Size();
	double Mu = MeanWeighted(X,W);
	
	double Num = Zero;
	double Den = Zero;
	for(int n=0; n<N; n++)
	{
		Num += W(n) * Sqr(X(n) - Mu);
		Den += W(n);
	}
	
	return Num / Den;
}

/* Compute the sample variance for vector-valued data */
SymmetricMatrix<double> Var(const Matrix<double> &X)
{
	int N = X.Rows();
	int K = X.Cols();
	SymmetricMatrix<double> Ret(K);

	/* Compute sample mean */
	Vector<double> Mu(K,Zero);
	for(int n=0; n<N; n++) for(int k=0; k<K; k++) Mu[k] += X(n,k);
	for(int k=0; k<K; k++) Mu[k] /= (double)N;
	
	/* Compute sample variance */
	for(int k1=0; k1<K; k1++) for(int k2=0; k2<=k1; k2++)
	{
		double Pr = Zero;
		for(int n=0; n<N; n++) Pr += X(n,k1) * X(n,k2);
		Pr /= (double)N;
		Ret(k1,k2) = Pr - Mu[k1] * Mu[k2];
	}
	return Ret;
}

/* Compute the sample variance for vector-valued data with missing values */
SymmetricMatrix<double> Var(const Matrix<double> &X, const Matrix<bool> &XMiss)
{
	int N = X.Rows();
	int K = X.Cols();
	SymmetricMatrix<double> Ret(K);

	for(int k1=0; k1<K; k1++) for(int k2=0; k2<=k1; k2++)
	{
		double Mu1 = Zero;
		double Mu2 = Zero;
		double Pr  = Zero;
		double Num = Zero;
		
		for(int n=0; n<N; n++)
		{
			if(XMiss(n,k1) == 0 && XMiss(n,k2) == 0)
			{
				double Temp1 = X(n,k1);
				double Temp2 = X(n,k2);
				Num += One;
				Mu1 += Temp1;
				Mu2 += Temp2;
				Pr += Temp1 * Temp2;
			}
		}
		
		if(Num == Zero)
		{
			Print("Error in Var(Matrix,Matrix)");
			Print("No Relevant Observations");
			Print("k1",k1);
			Print("k2",k2);
			HaltExecution();
		}
		
		Mu1 /= Num;
		Mu2 /= Num;
		Pr /= Num;
		Ret(k1,k2) = Pr - Mu1 * Mu2;
	}
	return Ret;
}

/* Compute the sample variance for vector-valued data with missing values
Missing covariances are imputed based on average correlation */
SymmetricMatrix<double> VarImp(const Matrix<double> &X, const Matrix<bool> &XMiss)
{
	int N = X.Rows();
	int K = X.Cols();
	SymmetricMatrix<double> Ret(K,-One);
	
	for(int k1=0; k1<K; k1++) for(int k2=0; k2<=k1; k2++)
	{
		double Mu1 = Zero;
		double Mu2 = Zero;
		double Pr  = Zero;
		double Num = Zero;
		
		for(int n=0; n<N; n++)
		{
			if(XMiss(n,k1) == 0 && XMiss(n,k2) == 0)
			{
				double Temp1 = X(n,k1);
				double Temp2 = X(n,k2);
				Num += One;
				Mu1 += Temp1;
				Mu2 += Temp2;
				Pr += Temp1 * Temp2;
			}
		}
		
		if(Num != Zero)
		{
			Mu1 /= Num;
			Mu2 /= Num;
			Pr /= Num;
			Ret(k1,k2) = Pr - Mu1 * Mu2;
		}
	}
	
	double AvgSd = Zero;
	int NumSd = 0;
	for(int k=0; k<K; k++) if(Ret(k,k) != -One)
	{
		AvgSd += Sqrt(Ret(k,k));
		NumSd++;
	}
	AvgSd /= (double)NumSd;
	for(int k=0; k<K; k++) if(Ret(k,k) == -One || Ret(k,k) == Zero) Ret(k,k) = Sqr(AvgSd);
	
	double AvgCorr = Zero;
	int NumCorr = 0;
	for(int k1=0; k1<K; k1++) for(int k2=0; k2<k1; k2++)
	{
		if(Ret(k1,k2) != -One)
		{
			AvgCorr += Ret(k1,k2) / Sqrt(Ret(k1,k1) * Ret(k2,k2));
			NumCorr++;
		}
	}
	AvgCorr /= (double)NumCorr;
	for(int k1=0; k1<K; k1++) for(int k2=0; k2<k1; k2++)
	{
		if(Ret(k1,k2) == -One) Ret(k1,k2) = Sqrt(Ret(k1,k1) * Ret(k2,k2)) * AvgCorr;
	}
			
	return Ret;
}

/* Compute the sample variance for vector-valued data with missing values */
SymmetricMatrix< DataPoint<double> > Var(const Matrix< DataPoint<double> > &X)
{
	int N = X.Rows();
	int K = X.Cols();
	SymmetricMatrix< DataPoint<double> > Ret(K);

	for(int k1=0; k1<K; k1++) for(int k2=0; k2<=k1; k2++)
	{
		double Mu1 = Zero;
		double Mu2 = Zero;
		double Pr  = Zero;
		double Num = Zero;
		
		for(int n=0; n<N; n++)
		{
			if(!X(n,k1).IsMissing() && !X(n,k2).IsMissing())
			{
				double Temp1 = X(n,k1).GetData();
				double Temp2 = X(n,k2).GetData();
				Num += One;
				Mu1 += Temp1;
				Mu2 += Temp2;
				Pr += Temp1 * Temp2;
			}
		}
		
		if(Num == Zero)
		{
			Ret(k1,k2).SetToMissing();
		}
		else
		{
			Mu1 /= Num;
			Mu2 /= Num;
			Pr /= Num;
			Ret(k1,k2) = Pr - Mu1 * Mu2;
		}
	}
	return Ret;
}

/* Compute the sample variance for vector-valued data with missing values */
#ifdef _GenMat_DiagonalMatrix
	DiagonalMatrix<double> VarDiag(const Matrix<double> &X, const Matrix<bool> &XMiss)
	{
		int N = X.Rows();
		int K = X.Cols();
		DiagonalMatrix<double> Ret(K);

		for(int k=0; k<K; k++)
		{
			double Mu1 = Zero;
			double Mu2 = Zero;
			double Pr  = Zero;
			double Num = Zero;
			
			for(int n=0; n<N; n++)
			{
				if(!XMiss(n,k))
				{
					double Temp1 = X(n,k);
					double Temp2 = X(n,k);
					Num += One;
					Mu1 += Temp1;
					Mu2 += Temp2;
					Pr += Temp1 * Temp2;
				}
			}
			
			if(Num == Zero)
			{
				Print("Error in VarDiag(Matrix,Matrix)");
				Print("No Relevant Observations");
				Print("k",k);
				HaltExecution();
			}
			
			Mu1 /= Num;
			Mu2 /= Num;
			Pr /= Num;
			Ret(k) = Pr - Mu1 * Mu2;
		}
		return Ret;
	}

	/* Compute the sample variance for vector-valued data with missing values */
	DiagonalMatrix<double> VarDiag(const Matrix< DataPoint<double> > &X)
	{
		int N = X.Rows();
		int K = X.Cols();
		DiagonalMatrix<double> Ret(K);

		for(int k=0; k<K; k++)
		{
			double Mu1 = Zero;
			double Mu2 = Zero;
			double Pr  = Zero;
			double Num = Zero;
			
			for(int n=0; n<N; n++)
			{
				if(!X(n,k).IsMissing())
				{
					double Temp1 = X(n,k).GetData();
					double Temp2 = X(n,k).GetData();
					Num += One;
					Mu1 += Temp1;
					Mu2 += Temp2;
					Pr += Temp1 * Temp2;
				}
			}
			
			if(Num == Zero)
			{
				Print("Error in VarDiag(Matrix,Matrix)");
				Print("No Relevant Observations");
				Print("k",k);
				HaltExecution();
			}
			
			Mu1 /= Num;
			Mu2 /= Num;
			Pr /= Num;
			Ret(k) = Pr - Mu1 * Mu2;
		}
		return Ret;
	}
#endif

#endif