
#ifndef _GenMat_Vector
#define _GenMat_Vector

//bool TempGlob = false;

/*

Adapted from the Template Numerical Toolkit (TNT)

Templated one-dimensional, numerical array which
looks like a conventional C array.
Elements are accessed via the A[i] or A(i) notation.

Array assignment is by reference (i.e. shallow assignment).
That is, B = A implies that the A and B point to the
same array, so modifications to the elements of A
will be reflected in B. If an independent copy
is required, then B = A.Copy() can be used.  Note
that this facilitates returning arrays from functions
without relying on compiler optimizations to eliminate
extensive data copying.

The indexing and layout of this array object makes
it compatible with C and C++ algorithms that utilize
the familiar C[i] notation.  This includes numerous
textbooks, such as Numercial Recipes, and various
public domain codes.

This class employs its own garbage collection via
the use of reference counts.  That is, whenever
an internal array storage no longer has any references
to it, it is destoryed.

*/

template <class T>
class Vector : public GeneralVector<T>
{
	private:

	LongInt n;				// Size of vector
    T* p;				// Underlying array
    int *RefCount;		// Number of references

	void Initialize();
	void Destroy();
	
	public:

	         Vector();
	explicit Vector(LongInt n);
	         Vector(LongInt n, const T &a);
   	         Vector(LongInt n, T *a);
   	         //Vector(LongInt n, const T *a);
	inline   Vector(const Vector &A);
	inline   Vector & operator=(const T &a);
	inline   Vector & operator=(const Vector &A);
	inline   Vector & Ref(const Vector &A);
	         Vector Copy() const;
		     Vector & Inject(const Vector & A);
	inline   T& operator[](LongInt i);
	inline   const T& operator[](LongInt i) const;
	inline   T& operator()(LongInt i);
	inline   const T& operator()(LongInt i) const;
	inline 	 int Dim() const;
	inline   LongInt Size() const;
	inline   LongInt Length() const;
	inline   int Increment() const;
	inline   int Type() const;
	inline   int GetRefCount() const;
			 void Sort();
             ~Vector();

    inline   void operator += (const T &c);
    inline   void operator -= (const T &c);
    inline   void operator *= (const T &c);
    inline   void operator /= (const T &c);

    inline   void operator += (const Vector<T> &x);
    inline   void operator -= (const Vector<T> &x);
    inline   void operator *= (const Vector<T> &x);
    inline   void operator /= (const Vector<T> &x);

    		 T* Begin() const;
			 T* End()   const;
};

/*

Null constructor. Creates a 0-length (NULL) array
(Reference count is also zero)

*/

template <class T>
Vector<T>::Vector()
{
	p = 0;
	n = 0;
	RefCount = 0;
	Name = "NoName";
	RefCount = new int;
	*RefCount = 1;
}

/*

Copy constructor. Array data is NOT copied, but shared.
Thus, in Vector B(A), subsequent changes to A will
be reflected in B.  For an indepent copy of A, use
Vector B(A.Copy()), or B = A.Copy(), instead.

*/

template <class T>
Vector<T>::Vector(const Vector<T> &A)
{
	p = A.p;
	n = A.n;
	RefCount = A.RefCount;
	Name = A.Name;
	(*RefCount)++;
}

/*

Create a new array (Vector) of length n,
WITHOUT initializing array elements.
To create an initialized array of constants, see Vector(n,Val).

<p>
This version avoids the O(n) initialization overhead and
is used just before manual assignment.

@param n the dimension (length) of the new matrix.

*/

template <class T>
Vector<T>::Vector(LongInt n)
{
	this->n = n;
	Name = "NoName";
	
	if(n < 0)
	{
		Print("Error in Vector::Vector(int n)");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}

	Initialize();
	RefCount = new int;
	*RefCount = 1;
}

/*

Create a new array of length n, initializing array
elements to constant specified by argument.  Most
often used to create an array of zeros, as in A(n,0.0).

n	-	Dimension (length) of the new Vector.
Val	-	The constant value to set all elements of the new array to.

*/

template <class T>
Vector<T>::Vector(LongInt n, const T &a)
{
	this->n = n;
	Name = "NoName";

	if(n < 0)
	{
		Print("Error in Vector::Vector(int n, const T &a)");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}

	
	Initialize();
	BlasSet(n,a,p,1);
	RefCount = new int;
	*RefCount = 1;
}

/*

Create a new n-length array, as a view of an existing one-dimensional
C array.  (Note that the storage for this pre-existing array will
never be destroyed by the VectorRef class.)

@param n the dimension (length) of the new Vector.
@param a the one dimensional C array to use as data storage for
the array.

*/

template <class T>
Vector<T>::Vector(LongInt n, T *a)
{
	this->n = n;
	p = a;
	Name = "NoName";

	if(n < 0)
	{
		Print("Error in Vector::Vector(int n, T *a)");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}
	
	RefCount = new int;
	*RefCount = 2;		// This avoid destorying original data
}

/*

Create a new n-length array, as a view of an existing one-dimensional
C array.  (Note that the storage for this pre-existing array will
never be destroyed by the VectorRef class.)

@param n the dimension (length) of the new Vector.
@param a the one dimensional C array to use as data storage for
the array.

*/

/*
template <class T>
Vector<T>::Vector(LongInt n, const T *a) : p(a), n(n), RefCount(0)
{
	if(n < 0)
	{
		Print("Error in Vector::Vector(int n, T *a)");
		Print("n < 0");
		Print("n",n);
		HaltExecution();
	}

	Name = "NoName";
	RefCount = new int;
	*RefCount = 2;		// This avoid destorying original data
}
*/

/*

A[i] indexes the ith element of A.  The first element is
A[0]. If _CheckBounds is defined, then the index is
checked that it falls within the array bounds.

*/

template <class T>
inline T& Vector<T>::operator[](LongInt i)
{
	#ifdef _CheckBoundsBracket
		if(i < 0 || i >= n)
		{
			Print("Error in Vector::[]");
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("Current Index",i);
			Print("Size",n);
			HaltExecution();
		}
	#endif

	return p[i];
}

/*

A[i] indexes the ith element of A.  The first element is
A[0]. If _CheckBounds is defined, then the index is
checked that it fall within the array bounds.

*/

template <class T>
inline const T& Vector<T>::operator[](LongInt i) const
{
	#ifdef _CheckBoundsBracket
		if(i < 0 || i >= n)
		{
			Print("Error in Vector::[]");
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("Current Index",i);
			Print("Size",n);
			HaltExecution();
		}
	#endif

	return p[i];
}

/*

A[i] indexes the ith element of A.  The first element is
A[0]. If _CheckBounds is defined, then the index is
checked that it falls within the array bounds.

*/

template <class T>
inline T& Vector<T>::operator()(LongInt i)
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n)
		{
			Print("Error in Vector::()");
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("Current Index",i);
			Print("Size",n);
			HaltExecution();
		}
	#endif

	return p[i];
}

/*

A[i] indexes the ith element of A.  The first element is
A[0]. If _CheckBounds is defined, then the index is
checked that it fall within the array bounds.

*/

template <class T>
inline const T& Vector<T>::operator()(LongInt i) const
{
	#ifdef _CheckBounds
		if(i < 0 || i >= n)
		{
			Print("Error in Vector::()");;
			Print("Index Out of Bounds");
			Print("Name",Name);
			Print("Current Index",i);
			Print("Size",n);
			HaltExecution();
		}
	#endif

	return p[i];
}


/* Assign all elemnts of A to a constant scalar */
template <class T>
Vector<T> & Vector<T>::operator=(const T &a)
{
	BlasSet(n,a,p,1);
	return *this;
}

/*

Create a new of existing matrix. Used in B = A.copy()
or in the construction of B, e.g. Vector B(A.copy()),
to create a new array that does not share data.

*/

template <class T>
Vector<T> Vector<T>::Copy() const
{
	Vector A(n);
	A.Name = Name;
	BlasCopy(n,p,1,A.p,1);
	return A;
}

/*

Copy the elements to from one array to another, in place.
That is B.Inject(A), both A and B must conform (i.e. have
identical row and column dimensions).

This differs from B = A.Copy() in that references to B
before this assignment are also affected.  That is, if
we have
<pre>
Vector A(n);
Vector C(n);
Vector B(C);        // elements of B and C are shared.

</pre>
then B.Inject(A) affects both and C, while B = A.Copy() creates
 a new array B which shares no data with C or A.

@param A the array from which elements will be copied
@return an instance of the modifed array. That is, in B.Inject(A),
it returns B.  If A and B are not conformat, no modifications to
B are made.

*/

template <class T>
Vector<T> & Vector<T>::Inject(const Vector &A)
{
	if(A.n == n) Copy(Begin(),A.Begin(),n);
	return *this;
}

/*

Create a reference (shallow assignment) to another existing array.
In B.Ref(A), B and A shared the same data and subsequent changes
to the array elements of one will be reflected in the other.
<p>
This is what operator= calls, and B = A and B.Ref(A) are equivalent
operations.

@return The new referenced array: in B.Ref(A), it returns B.

*/

template <class T>
Vector<T> & Vector<T>::Ref(const Vector<T> &A)
{
	if(this != &A)
	{
		(*RefCount)--;
		if(*RefCount < 1) Destroy();
		n = A.n;
		p = A.p;
		Name = A.Name;
		RefCount = A.RefCount;
		(*RefCount)++ ;
	}
	return *this;
}

/* B = A is shorthand notation for B.Ref(A) */
template <class T>
Vector<T> & Vector<T>::operator=(const Vector<T> &A)
{
	return Ref(A);
}

/*

Return the dimension (number of elements) of the Vector
This is equivalent to Size()

*/

template <class T>
inline int Vector<T>::Dim() const
{
	return n;
}

/*

@return the dimension (number of elements) of the array
This is equivalent to Dim()

*/

template <class T>
inline LongInt Vector<T>::Size() const
{
	return n;
}

template <class T>
inline LongInt Vector<T>::Length() const
{
	return n;
}

template <class T>
inline int Vector<T>::Increment() const
{
	return 1;
}

template <class T>
inline int Vector<T>::Type() const
{
	return MatrixType::Vector;
}

template <class T>
inline void Vector<T>::operator += (const T &c)
{
	for(LongInt i=0; i<n; i++) p[i] += c;
}

template <class T>
inline void Vector<T>::operator -= (const T &c)
{
	for(LongInt i=0; i<n; i++) p[i] -= c;
}

template <class T>
inline void Vector<T>::operator *= (const T &c)
{
	for(LongInt i=0; i<n; i++) p[i] *= c;
}

template <class T>
inline void Vector<T>::operator /= (const T &c)
{
	for(LongInt i=0; i<n; i++) p[i] /= c;
}

template <class T>
inline void Vector<T>::operator += (const Vector<T> &x)
{
	for(LongInt i=0; i<n; i++) p[i] += x[i];
}

template <class T>
inline void Vector<T>::operator -= (const Vector<T> &x)
{
	for(LongInt i=0; i<n; i++) p[i] -= x[i];
}

template <class T>
inline void Vector<T>::operator *= (const Vector<T> &x)
{
	for(LongInt i=0; i<n; i++) p[i] *= x[i];
}

template <class T>
inline void Vector<T>::operator /= (const Vector<T> &x)
{
	for(LongInt i=0; i<n; i++) p[i] /= x[i];
}

/*

Return the number of arrays that share the same storage area
as this one (Must be at least one)

*/

template <class T>
inline int Vector<T>::GetRefCount() const
{
	return *RefCount;
}

/* Destructor */
template <class T>
Vector<T>::~Vector()
{
	//if(TempGlob) Print("Vec Dest, Name",Name);
	(*RefCount)--;
	if(*RefCount < 1) Destroy();
}

/* Private internal functions */

template <class T>
void Vector<T>::Initialize()
{
	try
	{
		p = new T[n];
	}
	catch(bad_alloc ba)
	{
		Print("Error in Vector");
		Print("Error Allocating Memory");
		Print("Name",Name);
		Print("Size",n);
		Print("Memory per Entry",sizeof(T));
		Print("Memory Needed in Megabytes",n*sizeof(T)/1e6);
		HaltExecution();
	}
	this->n = n;
}

template <class T>
void Vector<T>::Destroy()
{
	if(p != 0) delete[] p;
	if(RefCount != 0) delete RefCount;
}

/* Begin */
template <class T>
T* Vector<T>::Begin() const
{
	return &p[0];
}

/* End */
template <class T>
T* Vector<T>::End() const
{
	return &p[0] + n;
}

/* Sort */
template <class T>
void Vector<T>::Sort()
{
	sort(Begin(),End());
}

#endif
